/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.io.IOException;
import java.io.ObjectInputStream;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.jaxodraw.graph.JaxoGraph;
import net.sf.jaxodraw.gui.grid.JaxoDefaultGrid;
import net.sf.jaxodraw.gui.handle.JaxoDefaultHandle;
import net.sf.jaxodraw.gui.menu.popup.JaxoFBoxPopupMenu;
import net.sf.jaxodraw.gui.panel.JaxoColorChooser;
import net.sf.jaxodraw.gui.panel.JaxoMainPanel;
import net.sf.jaxodraw.gui.panel.edit.JaxoOptionsPanel;
import net.sf.jaxodraw.object.JaxoExtendedObject;
import net.sf.jaxodraw.object.JaxoFillColorObject;
import net.sf.jaxodraw.object.JaxoHandle;
import net.sf.jaxodraw.object.JaxoList;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.object.JaxoObjectFactory;
import net.sf.jaxodraw.object.group.JaxoGroup;
import net.sf.jaxodraw.object.text.JaxoPSText;
import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoGeometry;
import net.sf.jaxodraw.util.JaxoPrefs;
import net.sf.jaxodraw.util.JaxoUtils;
import net.sf.jaxodraw.util.Location;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** An instance of the canvas: responsible for all the painting.
 * @since 2.0
 */
public final class JaxoCanvas extends JComponent implements MouseListener,
    MouseMotionListener, Scrollable {

    private static final long serialVersionUID = 7526471155622776147L;

    private static final DataFlavor GRAPH_FLAVOR =
        new DataFlavor("application/x-jaxodraw-graph;class="
            + JaxoGraph.class.getName() + ";version=2", "JaxoDraw graph");

    // Padding added to the bounding box (in component coordinates) to determine
    // the component area to be repainted
    private static final Insets COMPONENT_REPAINT_PADDING =
        new Insets(5, 5, 5, 5);

    // Padding added to the bounding box (in image coordinates) to determine the
    // image area to be repainted
    //private static final Insets IMAGE_REPAINT_PADDING = new Insets(5, 5, 5, 5);
    // Padding added to graph bounding box to determine canvas size
    private static final int GRAPH_PADDING = 10;

    // Padding added to objects bounding boxes to determine whether
    // they are in the faint box
    private static final Insets FAINT_BOX_PADDING = new Insets(5, 5, 5, 5);
    private static final boolean PUT_DRAGGED_OBJECTS_TO_FOREGROUND = false;
    private static final boolean HOVERING_EDITED_OBJECTS = true;
    private static final int HANDLE_PAINT_OFF = 0; // don't paint handles
    private static final int HANDLE_PAINT_ON = 1; // paint handles
    private static final int HANDLE_PAINT_SELECTION = 2; // only paint handles of selected object
    private transient Clipboard fakeSystemClipboard;
    private final JaxoMainPanel thePanel; // Instance of the main panel
    private transient JaxoGraph canvasGraph; // Instance of the canvas graph
    private transient Image offScreenImage; // Image for double buffering
    private final Point last; // Last mouse position in graph coordinates
    private final Point lastOnScreen; // Last mouse position on screen
    private final Point fixed; // Fixed point in graph when resizing
    private int curHandle; // Current handle
    private transient JaxoHandle handle;

    /** Mouse location, in Graph coordinates. */
    private Point mouseLocation;

    // Faint Box variables
    private boolean fboxON;
    private final Point fboxOrigin; // Fixed point in graph, only valid when fboxON
    private final Point fboxLocation; // Moved point in graph, only valid when fboxON
    private int fboxArea; // Width * height, only valid when fboxON
    private static final Color FBOX_FILL_COLOR = new Color(1.f, 1.f, 1.f, .4f);
    private static final Color FBOX_LINE_COLOR = new Color(.5f, .5f, .5f, 1.f);
    private final JaxoFBoxPopupMenu fboxPopup;

    //////////////////////////////////
    private Color canvasBackground;

    /** indicates how/whether to paint the handles of the objects. */
    private int handlePaintMode;

    /** indicates whether to paint the visual aid of objects. */
    private boolean visualAidPainted;

    /** variables for multi-point objects. */
    private final List points;
    private final Point pointsAidLocation;
    private boolean pointsON;

    /** Resize groups by dragging variables. */
    private Rectangle rgbox;
    private float newRf = 1.0f;
    private int gHandle;
    private JaxoObject cachedGroup;
    private transient JaxoZoom zoom;
    private transient JaxoColorChooser colorChooser;
    private boolean antialiasEnabled;
    private boolean snappingToGrid;
    private boolean gridPainted;
    private transient JaxoDefaultGrid grid;
    private transient ChangeListener gridListener;
    private final Dimension minimumCanvasSize;
    private final Dimension maximumCanvasSize;

    private JaxoObject selectedObject;

    // Object that is not part of the background image, but painted
    // in the foreground - may (and now always is?) still be part of
    // the graph.
    // Typically, if it exists, it is equal to the selected object of
    // the graph. The extra reference is needed because the selected
    // object can be cleared anytime/outside, and the canvas needs to
    // know the state of the offscreen image (i.e. which 'hovering' object
    // is missing.
    private JaxoObject hoveringObject;
    private boolean selectInside;
    private transient MouseEvent lastDraggedEvent;
    private final Timer scrollTimer;

    // Cached bounds of the graph, which does /not/ include the hovering object.
    private transient Rectangle cachedGraphBounds;


    /** Constructs a new canvas within the specified panel.
     * @param mainPanel An instance of the main panel.
     */
    public JaxoCanvas(JaxoMainPanel mainPanel) {
        this.thePanel = mainPanel;

        setBackground(Color.gray);
        setForeground(Color.black);
        canvasBackground = Color.white;
        setOpaque(true);
        setFocusable(true);

        // property should not be null
        canvasGraph = new JaxoGraph();

        last = new Point();
        lastOnScreen = new Point();
        fixed = new Point();
        fboxOrigin = new Point();
        fboxLocation = new Point();
        pointsAidLocation = new Point();

        points = new ArrayList(4);

        handle = new JaxoDefaultHandle();

        minimumCanvasSize = new Dimension(0, 0);
        maximumCanvasSize = new Dimension(7 * 72, 5 * 72);

        Dimension d = new Dimension(minimumCanvasSize);
        Insets n = getInsets();
        JaxoGeometry.grow(d, n);
        setPreferredSize(d);

        this.gridListener = getNewGridListener();

        this.zoom = new JaxoZoom(this);
        zoom.setActive(false);

        this.addMouseMotionListener(this);
        this.addMouseListener(this);

        setupPointMouseAction();

        fboxPopup = new JaxoFBoxPopupMenu(thePanel);
        fboxPopup.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    stopFaintBox();
                }
            });

        colorChooser = new JaxoColorChooser(thePanel);
        colorChooser.setPermanent(true);

        thePanel.addPropertyChangeListener(new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent e) {
                    if (e.getPropertyName().equals("Jaxo.mode")) {
                        //Integer old = (Integer) e.getOldValue();
                        Integer mode = (Integer) e.getNewValue();
                        updateMarked(mode.intValue());
                        repaint(); // handles etc., should not be needed in theory
                        getZoom().setActive(mode.intValue() == JaxoConstants.ZOOM);
                    }
                }
            });

        // Scroll to make 'last' (the last known mouse position) visible,
        // and send an extra mouseDragged event because the location within the
        // component has changed.
        // This is similar to what setAutoscrolls(true) would do, but with control
        // over scrolling speed.
        scrollTimer =
            new Timer(250,
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        revalidateCanvasFast();

                        scrollRectToVisible(toComponentCoordinates(
                                new Rectangle(last.x - 10, last.y - 10, 20, 20)));

                        if (lastDraggedEvent != null) {
                            Point p = getLocationOnScreen();

                            mouseDragged(new MouseEvent(
                                    lastDraggedEvent.getComponent(),
                                    lastDraggedEvent.getID(),
                                    lastDraggedEvent.getWhen(),
                                    lastDraggedEvent.getModifiers(),
                                    lastOnScreen.x - p.x,
                                    lastOnScreen.y - p.y,
                                    lastDraggedEvent.getClickCount(),
                                    lastDraggedEvent.isPopupTrigger(),
                                    lastDraggedEvent.getButton()));
                        }
                    }
                });

        scrollTimer.setInitialDelay(0);
    }

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        this.zoom = new JaxoZoom(this);
        this.handle = new JaxoDefaultHandle();
        this.canvasGraph = new JaxoGraph();
        this.colorChooser = new JaxoColorChooser(thePanel);
        colorChooser.setPermanent(true);
        this.gridListener = getNewGridListener();
    }

    /** Origin of the actual canvas within the component.
     * @return Point
     */

    // TODO: When ComponentOrientation is right-to-left, should place
    // canvas in the upper-RIGHT corner?
    public Point getCanvasOrigin() {
        Insets n = getInsets();

        return new Point(n.left, n.top);
    }

    /** Part of the component that is covered by the actual canvas. This includes canvas background
     * and grid, -- i.e. a rectangle with size 'canvasSize' and origin 'canvasOrigin'.
     * @return Rectangle
     * @see #getCanvasSize
     * @see #getCanvasOrigin
     */
    public Rectangle getCanvasBounds() {
        Insets n = getInsets();
        Dimension d = getCanvasSize();

        return new Rectangle(n.left, n.top, d.width, d.height);
    }

    /** Point 'p' in component coordinates converted to
     *  graph coordinates (at the moment, only translated from canvas origin).
     * @param p Point
     * @return Point
     */
    private Point toGraphCoordinates(Point p) {
        Insets n = getInsets();

        return new Point(p.x - n.left, p.y - n.top);
    }

    /** Point 'p' in graph coordinates converted to
     *  component coordinates (at the moment, only translated by canvas origin).
     */
    private Point toComponentCoordinates(Point p) {
        Insets n = getInsets();

        return new Point(p.x + n.left, p.y + n.top);
    }

    private Rectangle toComponentCoordinates(Rectangle r) {
        Rectangle result = new Rectangle(r);

        result.setLocation(toComponentCoordinates(r.getLocation()));

        return result;
    }

    private Rectangle getScreenBounds(JaxoObject o) {
        Rectangle bounds = toComponentCoordinates(o.getBounds());

        Point p = getLocationOnScreen();

        return new Rectangle(p.x + bounds.x, p.y + bounds.y,
                bounds.width, bounds.height);
    }



    /** Mark background or grid invalid (but not the graph/objects). */
    private void markBackgroundInvalid() {
        markImageInvalid();
    }

    private ChangeListener getNewGridListener() {
        return new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    markBackgroundInvalid();
                }
            };
    }

    /** Mark the whole image as invalid. Currently this repaints
     * everything at once, could be changed to defer until later.
     * This is public because now the main panel does some things for the canvas.
     */
    public void markImageInvalid() {
        updateOffScreenImage(null);
    }

    /**
     * Mark the specified region as invalid.
     * Currently this is also synchronous.
     * @param boundingBox The region to update.
     */
    public void markImageInvalid(Rectangle boundingBox) {
        updateOffScreenImage(boundingBox);
    }

    /**
     * Refresh the canvas.
     */
    public void refresh() {
        dropHoveringObject();
        setSelectedObject(null);
        markImageInvalid();
    }

    /**
     * Drop the current hovering object (put it back to the Image,
     * or into void if it is not part of the current graph).
     */
    private void dropHoveringObject() {
        if (hoveringObject != null) {
            List l = canvasGraph.getObjectList();

            int index = l.lastIndexOf(hoveringObject);

            if (!l.isEmpty() && (index == (l.size() - 1))) {
                putObjectOnOffScreenImage(hoveringObject);
                hoveringObject = null;
            } else if (index == -1) {
                hoveringObject = null;
                // update the whole OSI, rescaling of groups doesn't work otherwise
                updateOffScreenImage(null);
            } else {
                Rectangle bounds = hoveringObject.getBounds();
                hoveringObject = null;
                updateOffScreenImage(bounds);
            }

            cachedGraphBounds = null;
        }
    }

    private void setHoveringObject(JaxoObject value) {
        if (hoveringObject != value) {
            if (hoveringObject != null) {
                dropHoveringObject();
            }

            hoveringObject = value;
            cachedGraphBounds = null;

            if (canvasGraph.getObjectList().contains(hoveringObject)) {
                // Assume that the bounding box is valid -- only if contained
                markImageInvalid(hoveringObject.getBounds());
            } else {
                repaintObject(hoveringObject);
            }
        }
    }

    /** Sets the specified JaxoObject as currently selected.
     * @param newOb The JaxoObject to be set selected.
     */
    private void setSelectedObject(JaxoObject newOb) {
        this.selectedObject = newOb;
    }

    /**
     * Background color in the region that is actually covered by the canvas
     * {@link #getCanvasSize}, which is the whole canvas, unless
     * the effective maximum size is smaller than the component size.
     * Note that this color may be overwritten by the grid.
     * @return The canvas background color.
     */
    public Color getCanvasBackground() {
        return canvasBackground;
    }

    /**
     * Sets the color of the background.
     * @param value The background color.
     */
    public void setCanvasBackground(Color value) {
        if (!canvasBackground.equals(value)) {
            canvasBackground = value;
            markBackgroundInvalid();
        }
    }

    /** Determines if points are snapped to the grid.
     *  This property is independent from whether the
     *  grid is painted ('isGridPainted').
     * @return True if snapping is currently enabled.
     */
    public boolean isSnappingToGrid() {
        return snappingToGrid;
    }

    /**
     * Switches snapping on/off.
     * @param value Whether to snap or not.
     */
    public void setSnappingToGrid(boolean value) {
        snappingToGrid = value;
    }

    /**
     * Should objects (but not handles) be painted with
     * antialiasing turned explicitly on?
     * @return True if antialising is enebled.
     */
    public boolean isAntialiasEnabled() {
        return antialiasEnabled;
    }

    /**
     * Sets the antialias property.
     * @param value True if antialising should be enabled.
     */
    public void setAntialiasEnabled(boolean value) {
        if (antialiasEnabled != value) {
            antialiasEnabled = value;

            markImageInvalid();
        }
    }

    /**
     * Whether the grid is painted.
     * @return True if the grid is visible.
     */
    public boolean isGridPainted() {
        return gridPainted;
    }

    /**
     * Switches the grid painting on/off.
     * @param value True if the grid should be painted.
     */
    public void setGridPainted(boolean value) {
        if (value != gridPainted) {
            gridPainted = value;

            if (grid != null) {
                markBackgroundInvalid();
            }
        }
    }

    /**
     * Grid to be used, may be null. A non-null grid is
     * painted based on 'isGridPainted' and potentially
     * used for snapping points.
     * If grid properties change, the canvas automatically updates
     * and repaints itself.
     * @return The grid used by this canvas.
     */
    public JaxoDefaultGrid getGrid() {
        return grid;
    }

    /**
     * Sets the grid to be used by this canvas.
     * @param value The new grid.
     */
    public void setGrid(JaxoDefaultGrid value) {
        if (value != grid) {
            if (grid != null) {
                grid.removeChangeListener(gridListener);
            }
            grid = value;
            if (grid != null) {
                grid.addChangeListener(gridListener);
            }

            if (gridPainted) {
                markBackgroundInvalid();
            }
        }
    }

    /** Updates the language on any localized sub-components. */
    public void updateLanguage() {
        // try to pull these out of canvas!
        fboxPopup.updateLanguage();
        colorChooser.updateLanguage();
    }

    /**
     * Set both graph and grid. This is an optimization to avoid
     * painting to the background image twice (because that is done
     * synchronously).
     * @param newGraph The new graph.
     * @param newGrid The new grid.
     */
    public void setCanvasGraphAndGrid(JaxoGraph newGraph,
        JaxoDefaultGrid newGrid) {
        if (gridPainted) {
            if (newGrid == grid) {
                setCanvasGraph(newGraph);
            } else {
                if (newGraph != canvasGraph) {
                    canvasGraph = newGraph;
                    hoveringObject = null;
                }

                if (grid != null) {
                    grid.removeChangeListener(gridListener);
                }

                grid = newGrid;

                if (grid != null) {
                    grid.addChangeListener(gridListener);
                }

                markImageInvalid();
                revalidateCanvas();
            }
        } else {
            setGrid(newGrid);
            setCanvasGraph(newGraph);
        }
    }

    /** Sets the current graph. Also clears the hovering object.
     * @param value The graph to be set.
     */
    public void setCanvasGraph(JaxoGraph value) {
        if (canvasGraph != value) {
            //Rectangle oldBounds = null;
            //Rectangle newBounds = null;
            //if (canvasGraph != null) {
            //    oldBounds = canvasGraph.getBounds();
            //}
            canvasGraph = value;

            //if (canvasGraph != null) {
            //    newBounds = canvasGraph.getBounds();
            //}
            //Rectangle damagedBounds =
            //    (oldBounds == null) ? newBounds
            //                        : (
            //        (newBounds == null) ? oldBounds : newBounds.union(oldBounds)
            //    );
            if (hoveringObject != null) {
                // Assume that the bounding box is valid
                //damagedBounds =
                //    damagedBounds.union(hoveringObject.getBounds());
                hoveringObject = null;
            }

            // markImageInvalid(damagedBounds);
            // LEGACY:
            markImageInvalid();

            if ((handlePaintMode != HANDLE_PAINT_OFF) || visualAidPainted) {
                repaint();
            }

            revalidateCanvas();
        }
    }

    /** Returns the current graph.
     * @return The current graph.
     */
    public JaxoGraph getCanvasGraph() {
        return this.canvasGraph;
    }

    /** Ensure the off screen image exists and is up-to-date. */
    private void ensureValidOffScreenImage() {
        // currently synchronous updates - so the only way it can
        // not be up-to-date is when it does not exist.
        if (offScreenImage == null) {
            createOffScreenImage();
            updateOffScreenImage(null);
        }
    }

    /**
     * Paint the Canvas.
     *
     * @param g The graphics context to paint to.
     */
    protected void paintComponent(Graphics g) {
        JaxoGraphics2D g2 = new JaxoGraphics2D((Graphics2D) g.create());

        Insets n = getInsets();

        ensureValidOffScreenImage();

        Dimension d = getSize();
        Dimension canvasSize = getCanvasSize();

        if (d.width != canvasSize.width || d.height != canvasSize.height) {
            g2.setColor(getBackground());
            g2.fillRect(0, 0, getWidth(), getHeight());
        }

        g2.translate(n.left, n.top);
        g2.clipRect(0, 0, canvasSize.width, canvasSize.height);
        g2.drawImage(offScreenImage, 0, 0, null);

        if (hoveringObject != null) {
            Object old = g2.getRenderingHint(RenderingHints.KEY_ANTIALIASING);

            Object oldControl =
                g2.getRenderingHint(RenderingHints.KEY_STROKE_CONTROL);

            g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
                RenderingHints.VALUE_STROKE_PURE);

            // This is slower, but otherwise the object looks noticably different.
            if (antialiasEnabled) {
                g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            }

            hoveringObject.paint(g2);

            if (antialiasEnabled) {
                g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, old);
            }

            g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, oldControl);
        }

        if (visualAidPainted && selectedObject != null) {
            selectedObject.paintVisualAid(g2);
        }

        if (handlePaintMode == HANDLE_PAINT_ON) {
            // SELECT? at least do not use getCurrentMode() for fbox handles
            int handleMode = fboxON ? JaxoConstants.SELECT : getCurrentMode();
            canvasGraph.paintHandles(g2, handle, handleMode);
        } else if (handlePaintMode == HANDLE_PAINT_SELECTION) {
            selectedObject.paintHandles(g2, handle, getCurrentMode());
        }

        if (fboxON) {
            paintFaintBox(g2);
        }

        if (pointsON) {
            paintPointsAid(g2);
        }

        g2.dispose();
    }

    /**
     * Print the Canvas.
     *
     * @param gr The graphics context to print to.
     */
    protected void printComponent(Graphics gr) {
        JaxoGraphics2D g = new JaxoGraphics2D((Graphics2D) gr.create());
        g.setPrinting(true);

        g.setColor(getBackground());
        g.fillRect(0, 0, getWidth(), getHeight());

        Insets n = getInsets();
        g.translate(n.left, n.top);

        Dimension d = getCanvasSize();

        paintBackgroundAndGrid(g, 0, 0, d.width, d.height);

        if (antialiasEnabled) {
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        }

        g.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
            RenderingHints.VALUE_STROKE_PURE);

        canvasGraph.paintClipped(g);

        g.dispose();
    }

    /** Create a new offScreenImage (with undefined contents). */
    private void createOffScreenImage() {
        Dimension d = getEffectiveMaximumCanvasSize();
        offScreenImage = createImage(d.width, d.height);
        zoom.setBackground(offScreenImage);
    }

    /** Paint background (unless covered by grid), and grid (if on).
     * This method may change the graphics clip and color.
     */
    private void paintBackgroundAndGrid(JaxoGraphics2D g, int x, int y,
        int width, int height) {
        g.clipRect(x, y, width, height);

        if (!gridPainted || (grid == null)
                || (grid.getTransparency() != Transparency.OPAQUE)) {
            g.setColor(canvasBackground);

            g.fillRect(x, y, width, height);
        }

        if (gridPainted && (grid != null)) {
            grid.setCanvasSize(getEffectiveMaximumCanvasSize());
            grid.paint(g);
        }
    }

    /** Updates the offScreenImage within the given clipping area, or
     * everything if it is null.
     */
    private void updateOffScreenImage(Rectangle oldbb) {
        if (offScreenImage == null) {
            return;
        }

        JaxoGraphics2D g2 =
            new JaxoGraphics2D((Graphics2D) offScreenImage.getGraphics());

        if (antialiasEnabled) {
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        }

        g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
            RenderingHints.VALUE_STROKE_PURE);

        Rectangle damagedBounds;

        if (oldbb == null) {
            damagedBounds =
                new Rectangle(0, 0, offScreenImage.getWidth(null),
                    offScreenImage.getHeight(null));
        } else {
            damagedBounds = oldbb;
        }

        paintBackgroundAndGrid(g2, damagedBounds.x, damagedBounds.y,
            damagedBounds.width, damagedBounds.height);

        g2.clip(damagedBounds);

        if (hoveringObject == null) {
            canvasGraph.getObjectList().paintClipped(g2);
        } else {
            canvasGraph.getObjectList().paintClippedExcept(
                Collections.singleton(hoveringObject), g2);
        }

        g2.dispose();

        repaintBoundingBox(damagedBounds);
    }

    private void repaintHandles(JaxoObject o) {
        repaintBoundingBox(o.getBoundsWithHandles(getHandle()));
    }

    private void repaintObject(JaxoObject o) {
        repaintBoundingBox(o.getBounds());
    }

    // repaint 'r', given in graph coordinates (currently
    // these are always the same as component coordinates)
    private void repaintBoundingBox(Rectangle2D r) {
        Rectangle s = toComponentCoordinates(r.getBounds());

        JaxoGeometry.grow(s, COMPONENT_REPAINT_PADDING);

        repaint(s);
    }

    private void snapPoint(Point p) {
        if (snappingToGrid && (grid != null)) {
            grid.snapPoint(p);
        }
    }

    private void snapObject(JaxoObject o) {
        // don't snap groups because the snap points are calculated from the
        // bounding box, which leads to an offset for objects inside groups
        // need to solve this more generally?
        if (snappingToGrid && (grid != null) && !(o instanceof JaxoGroup)) {
            Point p = new Point(o.getX(), o.getY());
            Point q = new Point(p);

            grid.snapPoint(q);

            o.moveBy(q.x - p.x, q.y - p.y);
        }
    }

    /** Determines which objects should be selected as belonging to a group (ie, whose
     * handles are to be painted gray), when the mouse is dragged withthe middle or
     * right button.
     * @param p The point in graph coordinates
     */
    private void selectObjectsToGroup(Point p) {
        JaxoObject newSelectedOb = null;
        int i = canvasGraph.listSize();

        setSelectedObject(null);

        while ((newSelectedOb == null) && (i >= 1)) {
            i--;

            JaxoObject curOb = canvasGraph.listElementAt(i);

            int grabbedHandle = curOb.getGrabbedHandle(p.x, p.y, handle);

            if (curOb.canBeSelected(grabbedHandle, getCurrentMode())) {
                newSelectedOb = curOb;
                newSelectedOb.setAsMarked(!newSelectedOb.isMarked());
                setSelectedObject(curOb);
                repaintHandles(newSelectedOb);
            }
        }
    }

    /** Select the object when the mouse is clicked or pressed by comparing
     * to see if the mouse event occured in a handle
     * of an object. Loop through the current list of objects from the
     * end to the beginning. If the mouse is in the handle of more
     * than one object, then the top object will be selected.
     * @param p The point in graph coordinates
     * @param editmode The current edit mode
     */
    private void selectObject(Point p, int editmode) {
        JaxoObject newSelectedOb = null;
        int i = canvasGraph.listSize();

        while ((newSelectedOb == null) && (i >= 1)) {
            i--;

            JaxoObject curOb = canvasGraph.listElementAt(i);

            if (curOb != null) {
                int grabbedHandle = curOb.getGrabbedHandle(p.x, p.y, handle);

                if (curOb.canBeSelected(grabbedHandle, editmode)) {
                    newSelectedOb = curOb;
                    setSelectedObject(newSelectedOb);
                    if (editmode == JaxoConstants.RESIZE) {
                        curHandle = grabbedHandle;
                        setStartCoordinates();
                    }
                }
            }
        }
        setSelectedObject(newSelectedOb);
    }

    /** Set the fixed point for resizing an object.
     * This is always the point opposite to the grabbed handle
     * (Usually the center point (x,y), except for lines it
     * may be the end point (x+relw,y+relh) if the origin is grabbed.
     * For boxes and groups it can also be one of the diagonal points.)
     */
    private void setStartCoordinates() {
        if (selectedObject == null) {
            return;
        }

        JaxoExtendedObject selectedOb = (JaxoExtendedObject) selectedObject;

        fixed.x = selectedOb.getX();
        fixed.y = selectedOb.getY();
        if (curHandle == JaxoExtendedObject.SELECT_P1) {
            fixed.x = selectedOb.getX2();
            fixed.y = selectedOb.getY2();
        } else if (curHandle == JaxoExtendedObject.SELECT_DX) {
            fixed.x = selectedOb.getX2();
        } else if (curHandle == JaxoExtendedObject.SELECT_DY) {
            fixed.y = selectedOb.getY2();
        }
    }

    /** The action to be taken when the mouse is clicked on the canvas.
     * @param e The corresponding mouse event.
     */
    public void mouseClicked(MouseEvent e) {
        int currentMode = thePanel.getMode();

        // Zoom mode -> return
        if (currentMode == JaxoConstants.ZOOM) {
            return;
        }

        // fbox mode -> return
        if (fboxON) {
            return;
        }

        if (currentMode == JaxoConstants.SELECT && JaxoUtils.isButton2(e)) {
            canvasGraph.setAsMarked(false);
            repaint(); // for handles
        }
    }

    /** The action to be taken when the mouse is pressed on the canvas.
     * @param e The corresponding mouse event.
     */
    public void mousePressed(MouseEvent e) {
        int currentMode = thePanel.getMode();

        if (currentMode == JaxoConstants.ZOOM) {
            return;
        }

        if (fboxON) {
            return;
        }

        Point p = toGraphCoordinates(e.getPoint());

        if (JaxoUtils.isButton2(e)) {
            setHandlePaintMode(HANDLE_PAINT_OFF);
        } else if (JaxoUtils.isButton3(e)) {
            // For group, can only start in drag
            if (currentMode != JaxoConstants.SELECT) {
                startFaintBox(p);
            }
        } else if (JaxoUtils.isButton1(e)) {
            if (currentMode == JaxoConstants.STANDBY) {
                return;
            } else if ((currentMode == JaxoConstants.MOVE)
                    || (currentMode == JaxoConstants.RESIZE)) {
                selectObject(p, currentMode);

                if (selectedObject != null) {
                    setHandlePaintMode(HANDLE_PAINT_OFF);
                    setVisualAidPainted(true);

                    if (selectedObject instanceof JaxoGroup) {
                        setCachedGroup((JaxoGroup) selectedObject);
                        gHandle = curHandle;
                        rgbox = ((JaxoGroup) selectedObject).getBounds();
                    }

                    setHoveringObject(selectedObject);

                    if (currentMode == JaxoConstants.MOVE) {
                        snapObject(selectedObject);
                    }

                    scrollTimer.start();
                }
            } else if (currentMode == JaxoConstants.DUPLICATE) {
                selectObject(p, JaxoConstants.MOVE);

                if (selectedObject != null) {
                    setHandlePaintMode(HANDLE_PAINT_OFF);
                    setVisualAidPainted(true);

                    JaxoObject copyOb = selectedObject.copy();
                    canvasGraph.addObject(copyOb);
                    setSelectedObject(copyOb);

                    snapObject(copyOb);
                    setHoveringObject(copyOb);

                    scrollTimer.start();
                }
            } else if (JaxoConstants.isSelectMode(currentMode)) {
                if (currentMode == JaxoConstants.SELECT) {
                    selectObjectsToGroup(p);
                    selectInside = (selectedObject != null) && selectedObject.isMarked();
                } else {
                    selectObject(p, JaxoConstants.MOVE);
                    selectInside = true;

                    if (selectedObject != null) {
                        selectedObject.setAsMarked(true);
                        repaintHandles(selectedObject);
                    }
                }
                return;
            } else {
                Point q = new Point(p);

                snapPoint(q);

                JaxoObject newob = createNewObject(q);

                if (newob == null) {
                    if (pointsON) {
                        scrollTimer.start();
                        repaint(); // for points
                    }
                } else {
                    // Add new object to the list and set it to be selected object
                    canvasGraph.addObject(newob);
                    setSelectedObject(newob);

                    setHoveringObject(newob);

                    //text and latex objects are special, as they have no Canvas release event: back-up them here.
                    if ((currentMode == JaxoConstants.TEXT)
                            || (currentMode == JaxoConstants.LATEX)) {
                        // use setHoveringObject/dropHoveringObject anyway because
                        // they will repaint nicely
                        dropHoveringObject();
                        thePanel.commitRepeatableGraphChanges();
                        thePanel.resetMode();
                    } else {
                        setHandlePaintMode(HANDLE_PAINT_OFF);
                        setVisualAidPainted(true);
                        // assume, for the moment, that the points are in order
                        curHandle = newob.getPointCount() - 1;
                        setStartCoordinates();

                        scrollTimer.start();
                    }

                    revalidateCanvas();
                    repaint();
                }
            }
        }

        last.setLocation(p);
        lastOnScreen.setLocation(e.getPoint());
        SwingUtilities.convertPointToScreen(lastOnScreen, this);
    }

    /** The action to be taken when the mouse is dragged on the canvas.
     * @param e The corresponding mouse event.
     */
    public void mouseDragged(MouseEvent e) {
        Point p = toGraphCoordinates(e.getPoint());

        setMouseLocation(p);

        lastDraggedEvent = e;

        int currentMode = thePanel.getMode();

        if (currentMode == JaxoConstants.ZOOM) {
            return;
        }

        if (pointsON) {
            Point q = ((Point) points.get(points.size() - 1));

            q.setLocation(p);

            snapPoint(q);

            pointsAidLocation.setLocation(q);

            repaint();
        } else {
            if (JaxoUtils.isButton3(e)) {
                if (fboxON || (currentMode == JaxoConstants.SELECT)) {
                    dragFaintBox(p);
                }
            } else if (JaxoUtils.isButton1(e) && (selectedObject != null)) {
                if ((getCurrentMode() == JaxoConstants.RESIZE)
                        || JaxoConstants.isNewObjectMode(getCurrentMode())) {
                    if (PUT_DRAGGED_OBJECTS_TO_FOREGROUND) {
                        canvasGraph.foreground(selectedObject);
                    }

                    Point q = new Point(p);

                    snapPoint(q);

                    if (selectedObject instanceof JaxoGroup) {
                        swapGroups((JaxoGroup) selectedObject,
                            (JaxoGroup) cachedGroup);
                        setFixedPointAndScaleFactor(gHandle, q.x);
                        JaxoGroup group = (JaxoGroup) selectedObject;
                        group.rescaleObject(fixed.x, fixed.y, newRf);

                        // Switch under the hood
                        if (hoveringObject == selectedObject) {
                            hoveringObject = group;
                        }
                    }

                    if (curHandle >= 0) {
                        selectedObject.setX(curHandle, q.x);
                        selectedObject.setY(curHandle, q.y);
                    } else {
                        if (curHandle == JaxoExtendedObject.SELECT_DX) {
                            selectedObject.setLocation(q.x, fixed.y);
                            ((JaxoExtendedObject) selectedObject).setRelWAndH(fixed.x
                                - q.x, q.y - fixed.y);
                        } else if (curHandle == JaxoExtendedObject.SELECT_DY) {
                            selectedObject.setLocation(fixed.x, q.y);
                            ((JaxoExtendedObject) selectedObject).setRelWAndH(q.x
                                - fixed.x, fixed.y - q.y);
                        }
                    }

                    repaint();
                } else if ((getCurrentMode() == JaxoConstants.MOVE)
                        || (getCurrentMode() == JaxoConstants.DUPLICATE)) {
                    if (PUT_DRAGGED_OBJECTS_TO_FOREGROUND) {
                        canvasGraph.foreground(selectedObject);
                    }

                    Point m = new Point(last);
                    Point q = new Point(p);

                    snapObject(selectedObject);

                    snapPoint(m);
                    snapPoint(q);
                    selectedObject.moveBy(q.x - m.x, q.y - m.y);

                    repaint();
                } else if (JaxoConstants.isSelectMode(currentMode)) {
                    if (selectedObject != null) {
                        int grabbedHandle =
                            selectedObject.getGrabbedHandle(p.x, p.y, handle);

                        boolean inside =
                            (grabbedHandle != -1)
                            && selectedObject.canBeSelected(grabbedHandle, getCurrentMode());

                        if ((inside == selectInside) != selectedObject.isMarked()) {
                            selectedObject.setAsMarked(inside == selectInside);
                            repaintHandles(selectedObject);
                        }
                    }
                }
            }
        }

        last.setLocation(p);
        lastOnScreen.setLocation(e.getPoint());
        SwingUtilities.convertPointToScreen(lastOnScreen, this);
    }

    /** The action to be taken when the mouse is released on the canvas.
     * @param e The corresponding mouse event.
     */
    public void mouseReleased(MouseEvent e) {
        int currentMode = thePanel.getMode();

        if (currentMode == JaxoConstants.ZOOM) {
            return;
        }

        Point p = toGraphCoordinates(e.getPoint());

        newRf = 1.0f; // why here???

        if (JaxoUtils.isButton2(e)) {
            updateMode(getCurrentMode());
        } else if (JaxoUtils.isButton3(e)) {
            if (JaxoUtils.isDoubleClick(e)) {
                stopFaintBox();
                stopPointMouseAction();
                stopModeDragAction();

                setSelectedObject(canvasGraph.getNearestObject(p.x, p.y));
                showEditPanelFor(selectedObject);
            } else if (fboxON || getCurrentMode() == JaxoConstants.SELECT) {
                // Show the corresponding possible actions for the marked objects
                boolean canPaste = canPasteGraphFromSystemClipboard();

                if (canvasGraph.containsMarkedObjects()) {
                    fboxPopup.setMenuItemEnabled(1, canPaste);
                    fboxPopup.setMenuItemEnabled(2,
                        canvasGraph.containsMarkedGroups());
                    fboxPopup.setMenuItemEnabled(3,
                        canvasGraph.hasMoreMarkedObjectsThan(1));

                    fboxPopup.normalPopup();
                    fboxPopup.show(this, e.getX() + 2, e.getY() + 2);
                } else if ((!fboxON || fboxArea > 750) && canPaste) {
                    if (!fboxON) { // prepare pasting location
                        fboxON = true;
                        fboxOrigin.setLocation(p);
                        fboxLocation.setLocation(p);
                    }
                    fboxPopup.setMenuItemEnabled(1, true);
                    fboxPopup.onlyPastePopup();
                    fboxPopup.show(this, e.getX() + 2, e.getY() + 2);
                } else {
                    // Otherwise called later by popup
                    stopFaintBox();
                }
            }
        } else if (JaxoUtils.isButton1(e) && !fboxON) {
            finishModeDragAction();
        }

        if (pointsON) {
            mouseMoved(e);
        }
    }

    private void finishModeDragAction() {
        finishModeDragAction(false);
    }

    private void stopModeDragAction() {
        finishModeDragAction(true);
    }

    private void finishModeDragAction(boolean cancel) {
        if (fboxON) {
            return;
        }

        lastDraggedEvent = null;
        scrollTimer.stop();

        boolean resetMode = false;
        int currentMode = getCurrentMode();

        if ((currentMode == JaxoConstants.MOVE)
                || (currentMode == JaxoConstants.DUPLICATE)
                || (currentMode == JaxoConstants.RESIZE)) {
            if (selectedObject != null) {
                dropHoveringObject();
                setVisualAidPainted(false);
                revalidateCanvas();

                thePanel.commitGraphChanges();

                resetMode = true;
            }
        } else if (JaxoConstants.isParticleMode(currentMode)
                || (currentMode == JaxoConstants.BOX)
                || (currentMode == JaxoConstants.BLOB)
                || (currentMode == JaxoConstants.ZIGZAG)
                || (currentMode == JaxoConstants.VERTEX)) {
            if (selectedObject != null) {
                boolean empty = false;

                // Do not allow newly created objects to have size (0, 0)
                if (selectedObject.getPointCount() != 1) {
                    empty = (selectedObject.getWidth() == 0)
                            && (selectedObject.getHeight() == 0);
                }

                if (empty) {
                    canvasGraph.delete(selectedObject);
                } else {
                    thePanel.commitRepeatableGraphChanges();
                }

                dropHoveringObject();
                setVisualAidPainted(false);
                revalidateCanvas();

                resetMode = true;
            }
        } else if (JaxoConstants.isSelectMode(currentMode)) {
            if ((selectedObject != null) && selectedObject.isMarked() && !cancel) {
                if (currentMode == JaxoConstants.UNGROUP) {
                    if (selectedObject instanceof JaxoGroup) {
                        ungroupSelectedObject();
                    }

                    resetMode = true;
                } else if (currentMode == JaxoConstants.COLOR) {
                    selectedObject.setAsMarked(false);
                    setHandlePaintMode(HANDLE_PAINT_SELECTION);
                    showColorChooser(selectedObject);
                    resetMode = true;
                } else if (currentMode == JaxoConstants.DELETE) {
                    canvasGraph.delete(selectedObject);
                    thePanel.commitGraphChanges();
                    // Assume that the bounding box is valid
                    markImageInvalid(selectedObject.getBounds());
                    revalidateCanvas();

                    resetMode = true;
                } else if (currentMode == JaxoConstants.BACKGROUND) {
                    canvasGraph.background(selectedObject);
                    thePanel.commitGraphChanges();
                    // would not the selected object suffice?
                    markImageInvalid(canvasGraph.getBounds());

                    resetMode = true;
                } else if (currentMode == JaxoConstants.FOREGROUND) {
                    canvasGraph.foreground(selectedObject);
                    thePanel.commitGraphChanges();
                    // see above
                    markImageInvalid(canvasGraph.getBounds());
                    resetMode = true;
                } else if (currentMode == JaxoConstants.EDIT) {
                    selectedObject.setAsMarked(false);
                    showEditPanelFor(selectedObject);
                }

                if (currentMode != JaxoConstants.SELECT) {
                    // for these, selection stays marked
                    if (selectedObject != null) {
                        selectedObject.setAsMarked(false);
                        repaintHandles(selectedObject);
                    }
                }
            }

            selectInside = false;
        }

        if (resetMode) {
            thePanel.resetMode();
        }
    }

    /**
     * Process the event when the mouse exits the canvas. Does nothing.
     *
     * @param e The mouse event to process.
     */
    public void mouseExited(MouseEvent e) {
        setMouseLocation(null);
    }

    /**
     * Process the event when the mouse enters the canvas. Does nothing.
     *
     * @param e The mouse event to process.
     */
    public void mouseEntered(MouseEvent e) {
        setMouseLocation(toGraphCoordinates(e.getPoint()));
    }

    /**
     * Process the event when the mouse is moved on the canvas.
     *
     * @param e The mouse event to process.
     */
    public void mouseMoved(MouseEvent e) {
        Point p = toGraphCoordinates(e.getPoint());

        setMouseLocation(p);

        if (pointsON) {
            pointsAidLocation.setLocation(p);

            snapPoint(pointsAidLocation);

            repaint();
        }
    }

    /**
     * Sets the current mouse location in graph coordinates.
     * Notifies any registered PropertyChangeListeners.
     *
     * @param value The new mouse coordinates.
     */
    public void setMouseLocation(Point value) {
        if ((value == null) ? (mouseLocation != null)
                                : (!value.equals(mouseLocation))) {
            Point old = mouseLocation;

            mouseLocation = (value == null) ? null : (Point) value.clone();

            firePropertyChange("Jaxo.mouseLocation", old,
                (mouseLocation == null) ? null : mouseLocation.clone());
        }
    }

    /** Start faint box at 'p'. If already active, restart at 'p'.
     * @param p The point to start.
     */
    private void startFaintBox(Point p) {
        fboxON = true;
        fboxOrigin.setLocation(p);
        fboxLocation.setLocation(p);

        updateFaintBoxContents();
        repaint();
    }

    /** Drag faint box to 'p', starting if 'p' if not already on.
     * @param p The point to start.
     */
    private void dragFaintBox(Point p) {
        if (!fboxON) {
            startFaintBox(p);
        }

        fboxLocation.setLocation(p);

        // To avoid interference with right-double-click (flickering),
        // these only appear when actually having dragged (even if to the same position),
        // not immediately
        setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
        setHandlePaintMode(HANDLE_PAINT_ON);
        setVisualAidPainted(false);

        updateFaintBoxContents();
        repaint();
    }

    /** Stop faint box if active; returning to the current mode (initial state). */
    private void stopFaintBox() {
        if (fboxON) {
            fboxON = false;
            fboxArea = 0;

            canvasGraph.setAsMarked(false);
            thePanel.setMode(thePanel.getMode());

            repaint();
        }
    }

    /** Make (exactly) the objects contained with the 'faintBoxBounds'
     * 'marked'. Does not repaint anything.
     */
    private void updateFaintBoxContents() {
        Rectangle faintbox = getFaintBoxBounds();

        this.fboxArea = faintbox.width * faintbox.height;

        for (int i = 0; i < canvasGraph.listSize(); i++) {
            JaxoObject jaxoOb = canvasGraph.listElementAt(i);
            Rectangle bounds = jaxoOb.getBounds();
            JaxoGeometry.grow(bounds, FAINT_BOX_PADDING);

            if (faintbox.contains(bounds)) {
                jaxoOb.setAsMarked(true);
            } else {
                jaxoOb.setAsMarked(false);
            }
        }
    }

    /** Bounds of the faintBox. */
    private Rectangle getFaintBoxBounds() {
        int width = fboxLocation.x - fboxOrigin.x;
        int height = fboxLocation.y - fboxOrigin.y;

        Rectangle faintbox;

        if (height < 0) {
            if (width < 0) {
                faintbox =
                    new Rectangle(fboxLocation.x, fboxLocation.y, -width,
                        -height);
            } else {
                faintbox =
                    new Rectangle(fboxOrigin.x, fboxLocation.y, width, -height);
            }
        } else {
            if (width < 0) {
                faintbox =
                    new Rectangle(fboxLocation.x, fboxOrigin.y, -width, height);
            } else {
                faintbox =
                    new Rectangle(fboxOrigin.x, fboxOrigin.y, width, height);
            }
        }

        return faintbox;
    }

    /** Paints a light gray box that is used for grouping objects.
     * @param g2 The graphics context to paint
     */
    private void paintFaintBox(JaxoGraphics2D g2) {
        Rectangle f = getFaintBoxBounds();
        g2.setColor(FBOX_FILL_COLOR);
        g2.fill(f);
        g2.draw(f);
        g2.setColor(FBOX_LINE_COLOR);
        g2.setStroke(JaxoGraphics2D.DEFAULT_HELP_STROKE);
        g2.draw(f);
    }

    /** Sets how to paint handles.
     * @param value One of the HANDLE_PAINT constants
     */
    private void setHandlePaintMode(int value) {
        if (handlePaintMode != value) {
            handlePaintMode = value;
            repaint();
        }
    }

    /** Sets whether the visual aid should be painted.
     * @param value True if the visual aid should be drawn, false otherwise.
     */
    private void setVisualAidPainted(boolean value) {
        if (visualAidPainted != value) {
            this.visualAidPainted = value;

            if (selectedObject != null) {
                repaint();
            }
        }
    }

    /** Setup the main panel glassPane to allow swallowing MouseEvents outside
     * the JaxoCanvas.
     */
    private void setupPointMouseAction() {
        getInputMap().put(KeyStroke.getKeyStroke("ESCAPE"),
            "stopPointMouseAction");
        getActionMap().put("stopPointMouseAction",
            new AbstractAction() {
                private static final long serialVersionUID = 75264711556226147L;

                public void actionPerformed(ActionEvent e) {
                    stopPointMouseAction();
                }

                public boolean isEnabled() {
                    return pointsON;
                }
            });

        addFocusListener(new FocusAdapter() {
                public void focusLost(FocusEvent e) {
                    stopPointMouseAction();
                }
            });

        thePanel.setGlassPane(new JComponent() {
                private static final long serialVersionUID = 75264711556226147L;

                {
                    setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
                    addMouseListener(new MouseAdapter() {
                            public void mouseClicked(MouseEvent e) {
                                // Click outside stops
                                stopPointMouseAction();
                            }
                        });
                }

                public boolean contains(int x, int y) {
                    Point p =
                        SwingUtilities.convertPoint(this, new Point(x, y),
                            thePanel.getContentPane());
                    Component c = thePanel.getContentPane().findComponentAt(p);

                    // Allow the canvas itself
                    if (c == JaxoCanvas.this) {
                        return false;
                    }

                    // Allow components within the JScrollPane
                    while (!(c instanceof JScrollPane) && (c != null)) {
                        c = c.getParent();
                    }

                    if ((c != null)
                            && (
                                ((JScrollPane) c).getViewport().getView() == JaxoCanvas.this
                            )) {
                        return false;
                    }

                    // disallow the rest
                    return true;
                }
            });
    }

    /** When drawing arc objects etc.: start click-and-mouse mode.
     * This is called from createNewObject.
     */
    private void startPointMouseAction() {
        if (!pointsON) {
            pointsON = true;
            pointsAidLocation.setLocation((Point) points.get(0));
            requestFocusInWindow();
            thePanel.getGlassPane().setVisible(true);
            revalidateCanvas();
            repaint();
        }
    }

    /** When drawing arc objects etc.: stop click-and-mouse mode.
     * This is called from createNewObject (when the new object has
     * been built), and also otherwise to extraordinarily stop
     * click-and-move mode.
     * This is public because now the main panel does some things for the canvas.
     */
    private void stopPointMouseAction() {
        if (pointsON) {
            lastDraggedEvent = null;
            scrollTimer.stop();
            pointsON = false;
            points.clear();
            thePanel.getGlassPane().setVisible(false);
            revalidateCanvas();
            repaint();
        }
    }

    private void paintPointsAid(JaxoGraphics2D g2) {
        g2.setColor(JaxoGraphics2D.DEFAULT_HELP_COLOR);
        g2.setStroke(JaxoGraphics2D.DEFAULT_HELP_STROKE);

        Point first = (Point) points.get(0);

        paintCrossAt(g2, first.x, first.y);

        GeneralPath gp = new GeneralPath();

        gp.moveTo(first.x, first.y);

        for (Iterator i = points.listIterator(1); i.hasNext();) {
            Point p = (Point) i.next();
            gp.lineTo(p.x, p.y);
        }

        gp.lineTo(pointsAidLocation.x, pointsAidLocation.y);

        g2.draw(gp);
    }

    private void paintCrossAt(JaxoGraphics2D g, int x, int y) {
        int length = 4; // make configurable as function of handle size?
        g.drawLine(x - length, y - length, x + length, y + length);
        g.drawLine(x - length, y + length, x + length, y - length);
    }

    private JaxoObject createNewObject(Point q) {
        int currentMode = thePanel.getMode();

        if (currentMode == JaxoConstants.VERTEX) {
            // TODO: this should not be necessary
            currentMode = thePanel.getVertexType();
        }

        // note: we create an object for every mouse click, this would not
        // be necessary if we knew the pointCount for a given currentMode
        // ie without having to instantiate the object
        JaxoObject newob = JaxoObjectFactory.newObject(currentMode);

        if (newob == null) {
            return null;
        }

        int pointCount = newob.getPointCount();
        Point[] location = new Point[pointCount];

        if (pointCount == 1) {
            location[0] = q;
        } else if (pointCount == 2) {
            location[0] = q;
            location[1] = q;
        } else {
            points.add(new Point(q));
            if (points.size() == pointCount) {
                for (int i = 0; i < pointCount; i++) {
                    location[i] = (Point) points.get(i);
                }
                stopPointMouseAction();
            } else {
                startPointMouseAction();
                return null;
            }
        }

        newob.setPoints(location);
        // let preferences override location, eg to set minimum size
        newob.setPreferences();

        if (JaxoConstants.isTextMode(currentMode)) {
            String text = JaxoDialogs.getText(thePanel);
             newob.setParameter("textString", text);
            if (text.length() == 0) {
                newob = null;
            }
        }

        return newob;
    }

    private void showColorChooser(final JaxoObject ob) {
        Rectangle screenBounds = getScreenBounds(ob);
        Location location =
            new Location.RelativeToAvoiding(thePanel, screenBounds);
        int colorMode = JaxoPrefs.getIntPref(JaxoPrefs.PREF_COLORSPACE);

        if (ob instanceof JaxoFillColorObject
                && ((JaxoFillColorObject) ob).isFilled()) {
            final JaxoFillColorObject fillObject = (JaxoFillColorObject) ob;

            colorChooser.setMode(colorMode == JaxoColor.ALL_COLORS_MODE
            ? JaxoColor.ALL_COLORS_MODE : JaxoColor.JAXO_COLORS_MODE);

            if (HOVERING_EDITED_OBJECTS) {
                setHoveringObject(ob);
            }

            colorChooser.setOptional(false);
            colorChooser.setColor(fillObject.getFillColor());

            colorChooser.addChangeListener(new ColorChooserListener(ob) {
                    protected void setColor(Color c) {
                        fillObject.setFillColor(c);
                    }
                });
            colorChooser.show(location);

            if (colorChooser.hasChanged()
                    && JaxoColor.isGrayScale(colorChooser.getColor())) {
                ob.setColor(JaxoColor.BLACK);
            }
        } else if (ob instanceof JaxoGroup) {
            colorChooser.setMode(colorMode == JaxoColor.ALL_COLORS_MODE
            ? JaxoColor.ALL_COLORS_MODE : JaxoColor.JAXO_COLORS_NO_GRAYSCALES_MODE);

            final JaxoGroup g = (JaxoGroup) ob;
            final JaxoList oldContents = g.getObjectList();
            final JaxoList tmpContents = oldContents.copyOf();


            if (HOVERING_EDITED_OBJECTS) {
                setHoveringObject(g);
            }

            colorChooser.setOptionalColor(g.getColor());

            colorChooser.addChangeListener(new ColorChooserListener(g) {
                    protected void setColor(Color c) {
                        if (c == null) {
                            g.setObjectList(oldContents);
                        } else {
                            g.setObjectList(tmpContents);
                            g.setColor(c);
                        }
                    }
                });
            colorChooser.show(location);

            // Always restore real contents
            g.setObjectList(oldContents);

            if (colorChooser.hasChanged() && colorChooser.getColor() != null) {
                // /Now/ setColor also for contents
                g.setColor(colorChooser.getColor());
            }
        } else {
            colorChooser.setMode(colorMode == JaxoColor.ALL_COLORS_MODE
            ? JaxoColor.ALL_COLORS_MODE
            : ((ob instanceof JaxoPSText)
                ? JaxoColor.JAXO_COLORS_MODE
                : JaxoColor.JAXO_COLORS_NO_GRAYSCALES_MODE));

            if (HOVERING_EDITED_OBJECTS) {
                setHoveringObject(ob);
            }

            colorChooser.setOptional(false);
            colorChooser.setColor(ob.getColor());

            colorChooser.addChangeListener(new ColorChooserListener(ob));
            colorChooser.show(location);
        }

        if (HOVERING_EDITED_OBJECTS) {
            dropHoveringObject();
        } else {
            // Assume that the bounding box is valid - really should be
            markImageInvalid(ob.getBounds());
        }

        if (colorChooser.hasChanged()) {
            thePanel.commitGraphChanges();
        }
    }

    // Paint 'ob' on top of the background image. This assumes that
    // 'ob' was previously hovering (i.e. is not already on the background
    // image), and that it is in the foreground of the graph - CALL IT ONLY
    // IF YOU ARE REALLY SURE, otherwise the background image may be corrupted.
    // (This is just an optimization to repainting the complete background
    // image within the bounds of 'ob'.)
    private void putObjectOnOffScreenImage(JaxoObject ob) {
        JaxoGraphics2D g2 =
            new JaxoGraphics2D((Graphics2D) offScreenImage.getGraphics());

        if (antialiasEnabled) {
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        }

        g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
            RenderingHints.VALUE_STROKE_PURE);

        ob.paint(g2);
        g2.dispose();

        repaintBoundingBox(ob.getBounds());
    }

    private void setCachedGroup(JaxoGroup group) {
        cachedGroup = group.copy();
    }

    /* Methods for rescaling groups by dragging. */
    private void swapGroups(JaxoGroup scaledGroup, JaxoGroup copyGroup) {
        canvasGraph.delete(scaledGroup);
        JaxoObject scalableGroup = copyGroup.copy();
        canvasGraph.addObject(scalableGroup);
        setSelectedObject(scalableGroup);
    }

    private void setFixedPointAndScaleFactor(int selectedHandle, int xpos) {
        if (selectedHandle == JaxoExtendedObject.SELECT_P1) {
            fixed.x = rgbox.x + rgbox.width;
            fixed.y = rgbox.y + rgbox.height;
            if (xpos < fixed.x) {
                newRf = Math.abs((float) (xpos - fixed.x) / rgbox.width);
            }
        } else if (selectedHandle == JaxoExtendedObject.SELECT_DY) {
            fixed.x = rgbox.x;
            fixed.y = rgbox.y + rgbox.height;
            if (xpos > fixed.x) {
                newRf = Math.abs((float) (xpos - fixed.x) / rgbox.width);
            }
        } else if (selectedHandle == JaxoExtendedObject.SELECT_P2) {
            fixed.x = rgbox.x;
            fixed.y = rgbox.y;
            if (xpos > fixed.x) {
                newRf = Math.abs((float) (xpos - fixed.x) / rgbox.width);
            }
        } else if (selectedHandle == JaxoExtendedObject.SELECT_DX) {
            fixed.x = rgbox.x + rgbox.width;
            fixed.y = rgbox.y;
            if (xpos < fixed.x) {
                newRf = Math.abs((float) (xpos - fixed.x) / rgbox.width);
            }
        }
        if (newRf < 0.1f) {
            newRf = 0.1f;
        }
    }




    /** Resets the preferred size of the canvas, fast
        (avoid calling canvasGraph.getBounds(), but use a cached value).
        This assumes only the 'hoveringObject' or the 'pointsAidLocation' may
        have changed bounds.
     */
    private void revalidateCanvasFast() {
        if (cachedGraphBounds == null) {
            revalidateCanvas();
        } else {
            continueRevalidate(new Rectangle(cachedGraphBounds));
        }
    }

    /** Resets the preferred size of the canvas.
     *  This is public because now the main panel does some things for the canvas.
     */
    private void revalidateCanvas() {
        Rectangle bBox;

        if (hoveringObject == null) {
            bBox = canvasGraph.getBounds();
        } else {
            bBox = canvasGraph.getBoundsExcept(
                Collections.singleton(hoveringObject));
        }

        if (bBox == null) {
            bBox = new Rectangle();
        }

        cachedGraphBounds = new Rectangle(bBox);

        continueRevalidate(bBox);
    }

    // continue revalidateCanvas(Fast). 'bBox' is the graph bounds,
    // without hoveringObject or pointsAidLocation.
    private void continueRevalidate(Rectangle bBox) {
        Dimension maximum = getEffectiveMaximumCanvasSize();

        if (hoveringObject != null) {
            bBox.add(hoveringObject.getBounds());
        }

        if (pointsON) {
            for (Iterator i = points.iterator(); i.hasNext();) {
                bBox.add((Point) i.next());
            }
            bBox.add(pointsAidLocation);
        }

        int graphWidth = (bBox.x + bBox.width) + GRAPH_PADDING;
        int graphHeight = (bBox.y + bBox.height) + GRAPH_PADDING;

        int newWidth =
            valueBetween(minimumCanvasSize.width, graphWidth, maximum.width);
        int newHeight =
            valueBetween(minimumCanvasSize.height, graphHeight, maximum.height);

        Insets n = getInsets();

        newWidth += (n.left + n.right);
        newHeight += (n.top + n.bottom);

        Dimension d = getPreferredSize();

        if ((d.width != newWidth) || (d.height != newHeight)) {
            setPreferredSize(new Dimension(newWidth, newHeight));
            revalidate();
        }
    }

    // 'value' put to the closest allowed value in [minimum, maximum]
    // requires: minimum <= maximum
    private static int valueBetween(int minimum, int value, int maximum) {
        return Math.max(minimum, Math.min(value, maximum));
    }

    /**
     * Canvas size. This is equal to the (inner) component size unless
     * because of viewport size tracking, the component has become
     * larger than the effective maximum canvas size.
     * @return The canvas size.
     */
    public Dimension getCanvasSize() {
        Dimension result = getSize();
        Insets n = getInsets();
        Dimension maximum = getEffectiveMaximumCanvasSize();

        result.width =
            Math.min(maximum.width, result.width - n.left - n.right);
        result.height =
            Math.min(maximum.height, result.height - n.top - n.bottom);

        return result;
    }

    /**
     * Minimum size of the canvas (even if there are no objects
     * or they have a smaller bounding box). The default value is (0,0).
     * @return The minimum canvas size.
     */
    public Dimension getMinimumCanvasSize() {
        return (Dimension) minimumCanvasSize.clone();
    }

    /**
     * Sets the minimum canvas size.
     * @param value The minimum canvas size.
     */
    public void setMinimumCanvasSize(Dimension value) {
        if (!value.equals(minimumCanvasSize)) {
            Dimension old = getEffectiveMaximumCanvasSize();

            minimumCanvasSize.setSize(value);

            updateEffectiveMaximumCanvasSize(old);

            // always needed here, may grow
            revalidateCanvas();
        }
    }

    /**
     * Maximum size of the canvas (even if there are objects with a
     * larger bounding box). The default value is larger than (0,0).
     * If the maximum canvas size is smaller (in either direction)
     * than the minimum canvas size, the latter wins.
     * @return The maximum canvas size.
     */
    public Dimension getMaximumCanvasSize() {
        return (Dimension) maximumCanvasSize.clone();
    }

    /**
     * Sets the maximum canvas size.
     * @param value The maximum canvas size.
     */
    public void setMaximumCanvasSize(Dimension value) {
        if (!value.equals(maximumCanvasSize)) {
            Dimension old = getEffectiveMaximumCanvasSize();

            maximumCanvasSize.setSize(value);

            updateEffectiveMaximumCanvasSize(old);
        }
    }

    private void updateEffectiveMaximumCanvasSize(Dimension oldValue) {
        // The current offscreen image size is currently always
        // the effective maximum canvas size, so the
        // image may need to be recreated now.
        if (!oldValue.equals(getEffectiveMaximumCanvasSize())) {
            if (offScreenImage != null) {
                offScreenImage.flush();
                offScreenImage = null;
            }
            revalidateCanvas();
            repaint();
        }
    }

    private Dimension getEffectiveMaximumCanvasSize() {
        Dimension result = new Dimension(maximumCanvasSize);

        result.width = Math.max(result.width, minimumCanvasSize.width);
        result.height = Math.max(result.height, minimumCanvasSize.height);

        return result;
    }

    /**
     * The current handle.
     * @return The current handle.
     */
    public JaxoHandle getHandle() {
        return handle;
    }

    /**
     * Sets the handle.
     * @param newHandle The handle to set.
     */
    public void setHandle(JaxoHandle newHandle) {
        if (newHandle != getHandle()) {
            handle = newHandle;
            repaint();
        }
    }

    /** Just delegates to JaxoMainPanel.getMode().
     * @return The current mode of the main panel.
     */
    private int getCurrentMode() {
        return thePanel.getMode();
    }

    /** Applies the preferences. */
    public void applyPrefs() {
        // nothing to do for now
    }

    /**
     * Puts the current clipboard to the system clipboard.
     */
    public void copyMarkedObjects() {
        putToSystemClipboard(getClipboard());
    }

    /**
     * Gets the current clipboard.
     * @return A JaxoGraph holding all the objects that are currently
     * on the clipboard, or an empty graph, if the clipboard is empty.
     */
    public JaxoGraph getClipboard() {
        return new JaxoGraph(canvasGraph.getCopyOfMarkedObjects());
    }

    /** Paste the objects currently on the clipboard. */
    public void pasteFromClipboard() {
        JaxoGraph g = getSystemClipboardGraph();

        if (g != null) {
            paste(g);
        }

        dropHoveringObject();
        setSelectedObject(null);
    }

    /**
     * Paste a graph.
     *
     * @param g The graph to paste.
     */
    private void paste(JaxoGraph g) {
        int[] delta = getPastingDelta(g);
        for (int k = 0; k < g.listSize(); k++) {
            JaxoObject obj = g.listElementAt(k).copy();
            obj.moveBy(delta[0], delta[1]);
            canvasGraph.addObject(obj);
        }

        thePanel.commitRepeatableGraphChanges(g.listSize());
        markImageInvalid();
        revalidateCanvas();
    }

    private int[] getPastingDelta(JaxoGraph g) {
        int[] delta = {0, 0};

        if (fboxON) {
            Rectangle bbox = g.getBounds();
            Rectangle f = getFaintBoxBounds();

            delta[0] = f.x - bbox.x;
            delta[1] = f.y - bbox.y;
        }

        return delta;
    }

    /** Deletes all 'marked' objects from the current canvas graph. */
    public void deleteMarkedObjects() {
        Rectangle oldBounds = canvasGraph.getBounds();
        canvasGraph.deleteMarkedObjects();
        thePanel.commitGraphChanges();
        markImageInvalid(oldBounds);
        revalidateCanvas();
    }

    /** Ungroup (one level) all marked objects. */
    public void ungroupMarkedObjects() {
        if (canvasGraph.ungroupMarkedObjects()) {
            thePanel.commitGraphChanges();

            if (handlePaintMode != HANDLE_PAINT_OFF) {
                repaint();
            }
        }
    }

    /** If the selected object is a group, ungroup it. */
    public void ungroupSelectedObject() {
        if (canvasGraph.ungroup(selectedObject)) {
            if (handlePaintMode != HANDLE_PAINT_OFF) {
                repaint();
            }

            setSelectedObject(null); // group has gone
            thePanel.commitGraphChanges();
        }
    }

    /** Groups the currently marked objects. */
    public void groupMarkedObjects() {
        if (canvasGraph.groupMarkedObjects()) {
            if (handlePaintMode != HANDLE_PAINT_OFF) {
                repaint();
            }

            // Marks (only) adding the group as repeatable!
            thePanel.commitRepeatableGraphChanges();
        }
    }

    // update marked object on mode change.
    // This implementation almost always clears
    private void updateMarked(int newMode) {
        if (newMode != JaxoConstants.SELECT
                && canvasGraph.hasMoreMarkedObjectsThan(0)) {
            canvasGraph.setAsMarked(false);
            repaint();
        }
    }


    /** Resets canvas parameters for the given mode.
     * Sets the cursor, and determines whether to draw
     * visualAid and handles.
     * @param mode The mode to adjust to.
     */
    public void updateMode(int mode) {
        stopPointMouseAction();
        dropHoveringObject();
        setSelectedObject(null);

        if (mode == JaxoConstants.STANDBY) {
            setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
            setHandlePaintMode(HANDLE_PAINT_OFF);
            setVisualAidPainted(false);
        } else if (mode == JaxoConstants.ZOOM) {
            setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
            setHandlePaintMode(HANDLE_PAINT_OFF);
            setVisualAidPainted(false);
        } else if (JaxoConstants.isParticleMode(mode)) {
            setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
            setHandlePaintMode(HANDLE_PAINT_OFF);
            setVisualAidPainted(false);
        } else if (JaxoConstants.isMiscMode(mode)) {
            if ((mode == JaxoConstants.TEXT) || (mode == JaxoConstants.LATEX)) {
                setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
            } else {
                setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
            }
            setHandlePaintMode(HANDLE_PAINT_OFF);
            setVisualAidPainted(false);
        } else if (JaxoConstants.isEditMode(mode)) {
            if ((mode == JaxoConstants.MOVE) || (
                        mode == JaxoConstants.RESIZE
                    ) || (mode == JaxoConstants.DUPLICATE)) {
                setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
            } else {
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            }
            setHandlePaintMode(HANDLE_PAINT_ON);
            setVisualAidPainted(false);
        }
    }

    /** Clears the canvas. */
    public void clear() {
        dropHoveringObject();
        getCanvasGraph().clear();
        thePanel.commitGraphChanges();
        setSelectedObject(null);
        markImageInvalid();
        revalidateCanvas();
    }

    /**
     * Moves the currently selected objects into either fore- or background.
     *
     * @param backGround True for moving into background, false for foreground.
     */
    public void moveSelection(boolean backGround) {
        dropHoveringObject();
        setSelectedObject(null);
        if (backGround) {
            getCanvasGraph().objectsToBackground();
        } else {
            getCanvasGraph().objectsToForeground();
        }
        thePanel.commitGraphChanges();
        markImageInvalid();
    }

    private void showEditPanelFor(JaxoObject o) {
        if (o == null) {
            return;
        }

        setHandlePaintMode(HANDLE_PAINT_SELECTION);

        if (HOVERING_EDITED_OBJECTS) {
            setHoveringObject(o);
        }

        JaxoObjectEditPanel p = new JaxoOptionsPanel(o);
        p.addChangeListener(new EditPanelListener());

        Rectangle screenBounds = getScreenBounds(o);

        Location l =
            new Location.RelativeToAvoiding(thePanel, screenBounds);

        p.show(thePanel, l);

        if (HOVERING_EDITED_OBJECTS) {
            dropHoveringObject();
        }

        if (p.hasChanged()) {
            thePanel.commitGraphChanges();
            revalidateCanvas();
        }

        thePanel.resetMode();
    }

    /////////////////////////////////////////////////////////////////////
    // AWT datatransfer - should really be somewhere else?
    /////////////////////////////////////////////////////////////////////
    private void systemClipboardUnavailable() {
        getToolkit().beep();
    }

    /** Does the system clipboard contains a JaxoGraph? */
    private boolean canPasteGraphFromSystemClipboard() {
        Clipboard c = getSystemClipboard();

        try {
            // 1.5 would have isDataFlavorAvailable
            Transferable t = c.getContents(this);

            return (t != null) && t.isDataFlavorSupported(GRAPH_FLAVOR);
        } catch (IllegalStateException e) {
            return false;
        }
    }

    /** Graph on the system clipboard, or 'null' if none. */
    private JaxoGraph getSystemClipboardGraph() {
        Clipboard c = getSystemClipboard();

        try {
            Transferable t = c.getContents(this);

            return (JaxoGraph) t.getTransferData(GRAPH_FLAVOR);
        } catch (IllegalStateException e) {
            systemClipboardUnavailable();
            return null;
        } catch (UnsupportedFlavorException e) {
            return null;
        } catch (IOException e) {
            return null;
        }
    }

    /** Make 'g' the contents of the system clipboard. */
    private void putToSystemClipboard(JaxoGraph g) {
        final JaxoGraph graph = g.copyOf();

        Transferable t =
            new Transferable() {
                public DataFlavor[] getTransferDataFlavors() {
                    return new DataFlavor[]{GRAPH_FLAVOR};
                }

                public boolean isDataFlavorSupported(DataFlavor f) {
                    return f.equals(GRAPH_FLAVOR);
                }

                public Object getTransferData(DataFlavor f)
                    throws UnsupportedFlavorException, IOException {
                    if (!isDataFlavorSupported(f)) {
                        throw new UnsupportedFlavorException(f);
                    }

                    return graph.copyOf();
                }
            };

        try {
            getSystemClipboard().setContents(t, null);
        } catch (IllegalStateException e) {
            systemClipboardUnavailable();
        }
    }

    private Clipboard getSystemClipboard() {
        if (fakeSystemClipboard != null) {
            return fakeSystemClipboard;
        }

        try {
            return getToolkit().getSystemClipboard();
        } catch (SecurityException e) {
            fakeSystemClipboard = new Clipboard("JaxoDraw");
            return fakeSystemClipboard;
        }
    }


    /////////////////////////////////////////////////////////////////////
    // Canonical implementation of Scrollable, except for unit increment.
    /////////////////////////////////////////////////////////////////////

    /**
     * Return the preferred size of the viewport.
     *
     * @return The preferred size of the viewport.
     */
    public Dimension getPreferredScrollableViewportSize() {
        Dimension result = getMinimumCanvasSize();

        JaxoGeometry.grow(result, getInsets());

        return result;
    }

    /**
     * Return the unit increment for scrolling.
     *
     * @param visibleRect the visible view area.
     * @param orientation Either SwingConstants.VERTICAL
     *      or SwingConstants.HORIZONTAL.
     * @param direction less than zero to scroll up/left,
     *      greater than zero for down/right.
     * @return Returns 6.
     */
    public int getScrollableUnitIncrement(Rectangle visibleRect,
        int orientation, int direction) {
        // make configurable?
        return 6;
    }

    /**
     * Return the unit increment for block scrolling.
     *
     * @param visibleRect the visible view area.
     * @param orientation Either SwingConstants.VERTICAL
     *      or SwingConstants.HORIZONTAL.
     * @param direction less than zero to scroll up/left,
     *      greater than zero for down/right.
     * @return Returns the width/height of visibleRect depending on orientation.
     */
    public int getScrollableBlockIncrement(Rectangle visibleRect,
        int orientation, int direction) {
        return (orientation == SwingConstants.HORIZONTAL) ? visibleRect.width
                                                          : visibleRect.height;
    }

    /**
     * Return true if a viewport should force the width of the scrollable.
     *
     * @return true if the parent's width is larger than
     *      the width of the preferred size.
     */
    public boolean getScrollableTracksViewportWidth() {
        return getParent() instanceof JViewport
        && (getParent().getWidth() > getPreferredSize().width);
    }

    /**
     * Return true if a viewport should force the height of the scrollable.
     *
     * @return true if the parent's height is larger than
     *      the height of the preferred size.
     */
    public boolean getScrollableTracksViewportHeight() {
        return getParent() instanceof JViewport
        && (getParent().getHeight() > getPreferredSize().height);
    }

    /**
     * Return the zoom of this canvas.
     *
     * @return the zoom of this canvas.
     */
    public JaxoZoom getZoom() {
        return zoom;
    }

    /**
     * Set the zoom for this canvas.
     *
     * @param newZoom the zoom to set.
     */
    public void setZoom(JaxoZoom newZoom) {
        this.zoom = newZoom;
    }

    /** ColorChooser listeners calls setColor for every ChangeEvent
     * and then repaints the object given at construction time.
     * It removes itself automatically once the color choosing is
     * done.
     */
    private class ColorChooserListener implements ChangeListener {
        private final JaxoObject object;

        public ColorChooserListener(JaxoObject o) {
            object = o;
        }

        protected final JaxoObject object() {
            return object;
        }

        public void stateChanged(ChangeEvent e) {
            setColor(colorChooser.getColor());

            // Assume that the bounding box is valid
            // This is safer, as the bounding box does not
            // change when changing the color
            if (HOVERING_EDITED_OBJECTS) {
                repaintBoundingBox(object.getBounds());
            } else {
                markImageInvalid(object.getBounds());
            }

            if (!colorChooser.isAdjusting()) {
                colorChooser.removeChangeListener(this);
            }
        }

        protected void setColor(Color c) {
            object.setColor(c);
        }
    }

    private class EditPanelListener implements ChangeListener {

        public void stateChanged(ChangeEvent evt) {
            repaintObject();
        }

        private void repaintObject() {
            /** Does not work unless bounding box is up-to-date
            Rectangle bounds = oldBounds.union(newBounds);
            if (HOVERING_EDITED_OBJECTS) {
                repaintBoundingBox(bounds);
            } else {
                markImageInvalid(bounds);
            }
            */

            // Legacy
            if (HOVERING_EDITED_OBJECTS) {
                repaint();
            } else {
                // quite inefficient
                markImageInvalid();
            }
        }
    }
}
