/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

import net.sf.jaxodraw.graph.JaxoGraph;
import net.sf.jaxodraw.graph.JaxoSaveGraph;
import net.sf.jaxodraw.gui.grid.JaxoDefaultGrid;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectList;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoGeometry;
import net.sf.jaxodraw.util.JaxoPrefs;


/** One tab of the JaxoTabbedPane.
 * @since 2.0
 */
public class JaxoTab {
    // Events
    private EventListenerList listeners;
    private ChangeEvent event;

    // Components
    private JScrollPane root;
    private Point position;

    // Graph and state
    private JaxoGraph tabGraph;
    private String tabTitle;
    private final JaxoDefaultGrid tabGrid;
    private int tabMode;
    private boolean snappingToGrid;
    private boolean saved;
    private boolean isUsed;
    private JaxoCanvas canvas;

    // Undo
    private List backupList;
    private int repeatCount;
    private int undoCounter;
    private int backupPointer;
    private boolean redoFlag;

    /** Constructor. */
    public JaxoTab() {
        this(new JaxoGraph());
    }

    /**
     * Constructor.
     * @param g The graph for the tab.
     */
    public JaxoTab(JaxoGraph g) {
        this(g, new JaxoDefaultGrid(16, JaxoDefaultGrid.TYPE_RECTANGULAR));
    }

    /**
     * Constructor.
     * @param g A grid for the tab.
     */
    public JaxoTab(JaxoDefaultGrid g) {
        this(new JaxoGraph(), g);
    }

    /**
     * Constructor.
     * @param g A graph.
     * @param grid A grid.
     */
    public JaxoTab(JaxoGraph g, JaxoDefaultGrid grid) {
        this.listeners = new EventListenerList();

        this.root = new JScrollPane();
        root.putClientProperty("JaxoDraw.JaxoTab", this);

        this.tabGraph = g;

        this.tabTitle = "";

        this.backupList =
            new ArrayList(JaxoPrefs.getIntPref(JaxoPrefs.PREF_UNDODEPTH));
        this.undoCounter = 0;
        this.backupPointer = 0;
        this.redoFlag = false;

        this.backupList.add(0, tabGraph.getSaveGraph().copyOf());
        incrementBackupPointer();

        this.position = root.getViewport().getViewPosition();
        this.tabMode = JaxoConstants.STANDBY;

        this.saved = true;

        this.tabGrid = grid;
    }

    /**
     * Determine if a component is a JaxoTab.
     * @param c The component to test.
     * @return The component as a JaxoTab, or null if it was not a JaxoTab.
     */
    public static JaxoTab asJaxoTab(Component c) {
        return (c instanceof JComponent)
        ? (JaxoTab) ((JComponent) c).getClientProperty("JaxoDraw.JaxoTab") : null;
    }

    // This really should not subclass JScrollPane (nor Component).
    // Internally, all it setup properly, but outside it is still
    // assumed that the selected component is the JaxoTab itself, so
    // leave it for the moment.

    /**
     * Returns the root component of this tab.
     * @return The root component.
     */
    public final JComponent getRoot() {
        return root;
    }

    /** Returns this tab's graph.
     * @return This tab's graph.
     */
    public final JaxoGraph getTabGraph() {
        return tabGraph;
    }

    /** Sets this tab's graph.
     * @param theGraph The new graph.
     */
    public final void setTabGraph(JaxoGraph theGraph) {
        this.tabGraph = theGraph;

        if (canvas != null) {
            canvas.setCanvasGraph(tabGraph);
        }
    }

    /** Returns this tab's title.
     * @return This tab's title.
     */
    public final String getTabTitle() {
        return tabTitle;
    }

    /** Sets this tab's title.
     * @param theTitle The new title.
     */
    public final void setTabTitle(String theTitle) {
        this.tabTitle = theTitle;
    }

    /**
     * ChangeEvents will be fired when a graph change is performed,
     * basically this means when the properties "canUndo", "canRedo",
     * "isSaved" and "saveFileName" may have changed.
     * @param l The listener to add.
     */
    public void addChangeListener(ChangeListener l) {
        listeners.add(ChangeListener.class, l);
    }

    /**
     * Removes a change listener.
     * @param l The listener to remove.
     */
    public void removeChangeListener(ChangeListener l) {
        listeners.remove(ChangeListener.class, l);
    }

    /** Notifies all listeners of state changes. */
    protected void fireStateChanged() {
        Object[] pairs = listeners.getListenerList();

        for (int i = pairs.length - 2; i >= 0; i -= 2) {
            if (pairs[i] == ChangeListener.class) {
                if (event == null) {
                    event = new ChangeEvent(this);
                }
                ((ChangeListener) pairs[i + 1]).stateChanged(event);
            }
        }
    }

    /** Redraws the given canvas with this tab's graph.
     * @param value The canvas to update.
     */
    public void setCanvas(JaxoCanvas value) {
        if (value != canvas) {
            canvas = value;

            if (canvas == null) {
                position = root.getViewport().getViewPosition();
                root.setViewportView(null);
            } else {
                canvas.setCanvasGraphAndGrid(tabGraph, tabGrid);
                canvas.setSnappingToGrid(snappingToGrid);

                root.setViewportView(canvas);
                root.getViewport().setViewPosition(position);
            }

            root.revalidate();
        }
    }

    /** Returns the current grid.
     * @return The grid.
     */
    public JaxoDefaultGrid getGrid() {
        return this.tabGrid;
    }

    /** Switches snapping to the grid on/off.
     * @param value Boolean that sets the grid on/off.
     */
    public final void setSnappingToGrid(boolean value) {
        if (value != snappingToGrid) {
            snappingToGrid = value;

            if (canvas != null) {
                canvas.setSnappingToGrid(value);
            }
        }
    }

    /** Determines whether snapping to the grid is currently on.
     * @return True if the grid is on.
     */
    public final boolean isSnappingToGrid() {
        return this.snappingToGrid;
    }

    /** Sets the size of the grid to the given value.
     * @param gs The grid size to be set.
     */
    public final void setGridSize(int gs) {
        tabGrid.setGridSize(gs);
    }

    /** Returns the current size of the grid.
     * @return The grid size.
     */
    public final int getGridSize() {
        return tabGrid.getGridSize();
    }

    /** Sets the type of the grid.
     * @param type The type of the grid to be set.
     */
    public final void setGridType(int type) {
        tabGrid.setGridType(type);
    }

    /** Returns the current type of the grid.
     * @return  The type of the grid.
     */
    public final int getGridType() {
        return tabGrid.getGridType();
    }

    /** Sets the style of the grid.
     * @param style The style of the grid to be set.
     */
    public final void setGridStyle(int style) {
        tabGrid.setGridStyle(style);
    }

    /** Returns the current style of the grid.
     * @return  The style of the grid.
     */
    public final int getGridStyle() {
        return tabGrid.getGridStyle();
    }

    /** Sets the color of the grid.
     * @param color The color of the grid to be set.
     */
    public final void setGridColor(Color color) {
        tabGrid.setGridColor(color);
    }

    /** Returns the current color of the grid.
     * @return The color of the grid.
     */
    public final Color getGridColor() {
        return tabGrid.getGridColor();
    }

    /** Commit changes to 'tabGraph' to the undo history. */
    public void commitGraphChanges() {
        commitGraph(0);
    }

    /** Commit changes to 'tabGraph' to the undo history,
     * where the change is the addition of the last object
     * that should be repeatable.
     */
    public void commitRepeatableGraphChanges() {
        commitRepeatableGraphChanges(1);
    }

    /**
     * Commit changes to 'tabGraph' to the undo history,
     * where the change is the addition of 'count' last objects
     * that should be repeatable.
     * @param count The number of changes to commit.
     */
    public void commitRepeatableGraphChanges(int count) {
        commitGraph(count);
    }

    /** Adds a new element to the backup list at the position
    * specified by the current value of the eventCounter.
    * @param count number of objects added at the end for repeatable change.
    */
    private void commitGraph(int count) {
        setRedoFlag(false);

        repeatCount = count;

        int pos = getBackupPointer();

        if (pos < backupList.size()) {
            backupList.subList(pos, backupList.size()).clear();
        }

        int maxSize = JaxoPrefs.getIntPref(JaxoPrefs.PREF_UNDODEPTH);

        if (backupList.size() >= maxSize) {
            backupList.subList(0, backupList.size() - maxSize + 1).clear();
        }

        backupList.add(tabGraph.getSaveGraph().copyOf());

        resetUndoCounter();
        incrementBackupPointer();

        setUnsaved();

        fireStateChanged();
    }

    /**
     * Sets the save file name.
     * @param value The name to set.
     */
    public void setSaveFileName(String value) {
        tabGraph.setSaveFileName(value);

        fireStateChanged();
    }

    /**
     * Returns the current save file name.
     * @return The save file name of the current tab.
     */
    public final String getSaveFileName() {
        return tabGraph.getSaveFileName();
    }

    /** Clear undo history. Start with a new history, as if
     * newly constructed.
     */

    // TODO: This is still a mess
    public void clearBackupList() {
        boolean wasSaved = isSaved();

        backupList.clear();
        backupList.add(tabGraph.getSaveGraph().copyOf());
        resetUndoCounter();
        setBackupPointer(1);
        setRedoFlag(false);
        repeatCount = 0;

        setUnsaved();

        setSaved(wasSaved);

        fireStateChanged();
    }

    /**
     * Starts a new graph for this tab. This should be equivalent with
     * re-starting, eg it is not un-doable and the new status is saved.
     */
    public void newGraph() {
        setTabGraph(new JaxoGraph());
        commitGraphChanges();
        setSaved(true);
        clearBackupList();
    }

    /**
     * Undo last operation.
     */
    public void undoMove() {
        repeatCount = 0;
        incrementUndoCounter();
        setRedoFlag(true);
        int lastGraphPosition = backupList.size() - undoCounter;
        setBackupPointer(lastGraphPosition);
        setSaveGraph(((JaxoSaveGraph) backupList.get(lastGraphPosition - 1))
            .copyOf());

        setUnsaved();

        fireStateChanged();
    }

    private void setSaveGraph(JaxoSaveGraph g) {
        tabGraph.setSaveGraph(g);

        if (canvas != null) {
            canvas.markImageInvalid();
        }
    }

    /**
     * Redo last operation.
     */
    public void redoMove() {
        if (isRedoFlag()) {
            decrementUndoCounter();
            int lastGraphPosition = backupList.size() - undoCounter;
            setBackupPointer(lastGraphPosition);

            setSaveGraph(((JaxoSaveGraph) backupList.get(lastGraphPosition - 1))
                .copyOf());

            if (lastGraphPosition > (backupList.size() - 1)) {
                setRedoFlag(false);
            }

            setUnsaved();

            fireStateChanged();
        } else {
            repeatChange();
        }
    }

    private void repeatChange() {
        int size = tabGraph.getObjectList().size();

        List l = JaxoGraph.copyFrom(
            new JaxoObjectList(tabGraph.getObjectList().subList(size - repeatCount,
                    size)));

        for (Iterator i = l.iterator(); i.hasNext();) {
            tabGraph.addObject((JaxoObject) i.next());
        }

        if (canvas != null) {
            canvas.markImageInvalid(JaxoGeometry.getBounds(l));
        }

        commitRepeatableGraphChanges(repeatCount);

        setUnsaved();

        fireStateChanged();
    }

    /** Sets the tab mode.
     * @param mode The tab mode.
     */
    public void setTabMode(int mode) {
        this.tabMode = mode;
    }

    /** Returns the tab mode.
    * @return The tab mode.
    */
    public int getTabMode() {
        return tabMode;
    }

    /** Indicates whether this tab has been used.
     * @return True, if the canvas has been modified, false otherwise.
     */
    public boolean hasBeenUsed() {
        return this.isUsed;
    }

    /**
     * Sets the tab saved / not saved.
     * @param value True for saved.
     */
    public void setSaved(boolean value) {
        if (saved != value) {
            saved = value;
            fireStateChanged();
        }
    }

    /**
     * Determines if the graph is saved.
     * This is internally cleared at each change (also undo/redo).
     * @return True if the tab is saved.
     */
    public boolean isSaved() {
        return saved;
    }

    private void setUnsaved() {
        isUsed = true;
        saved = false;
    }

    private void incrementUndoCounter() {
        this.undoCounter++;
    }

    private void decrementUndoCounter() {
        this.undoCounter--;
    }

    private void resetUndoCounter() {
        this.undoCounter = 0;
    }

    private int getBackupPointer() {
        return backupPointer;
    }

    private void setBackupPointer(int fup) {
        this.backupPointer = fup;
    }

    private void incrementBackupPointer() {
        this.backupPointer++;
    }

    private void setRedoFlag(boolean bool) {
        this.redoFlag = bool;
    }

    private boolean isRedoFlag() {
        return this.redoFlag;
    }

    /**
     * Determines if undo is possible.
     * @return True if undo is possible.
     */
    public boolean canUndo() {
        return (backupList.size() - undoCounter) > 1;
    }

    /**
     * Determines if redo is possible.
     * @return True if redo is possible.
     */
    public boolean canRedo() {
        return (backupPointer < backupList.size())
        || (!redoFlag && (repeatCount != 0));
    }
}
