/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.panel;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.jaxodraw.gui.JaxoTab;
import net.sf.jaxodraw.gui.grid.JaxoDefaultGrid;
import net.sf.jaxodraw.gui.panel.button.JaxoColorButton;
import net.sf.jaxodraw.gui.swing.spinner.JaxoFixedJSpinner;
import net.sf.jaxodraw.gui.swing.spinner.JaxoSpinnerComboBox;
import net.sf.jaxodraw.gui.swing.spinner.JaxoSpinnerIntModel;
import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoLanguage;
import net.sf.jaxodraw.util.JaxoLocalized;


/** Responsible for displaying the grid options bar.
 * @since 2.0
 */
public class JaxoGridBar extends JPanel implements ItemListener,
    PropertyChangeListener, ChangeListener, JaxoLocalized {
    private static final long serialVersionUID = 7526471155622776147L;
    private static List gridSizes;

    static {
        gridSizes = new ArrayList();

        gridSizes.add(new Integer(6));
        gridSizes.add(new Integer(8));
        gridSizes.add(new Integer(10));
        gridSizes.add(new Integer(12));
        gridSizes.add(new Integer(14));
        gridSizes.add(new Integer(18));
        gridSizes.add(new Integer(24));
        gridSizes.add(new Integer(36));

        gridSizes = Collections.unmodifiableList(gridSizes);
    }

    private JComboBox chooseGridType;
    private JaxoSpinnerComboBox chooseGridSize;
    private JaxoSpinnerIntModel gridSize;
    private JCheckBox gridCheckBox;
    private JCheckBox snapCheckBox;
    private final JaxoMainPanel thePanel;
    private JaxoColorChooser colorChooser;
    private JaxoColorButton gridColorButton;
    private JaxoColorButton canvasBackgroundButton;
    private JComboBox chooseGridStyle;
    private JLabel gridTypeLabel;
    private JLabel gridStyleLabel;
    private JLabel gridSizeLabel;
    private JLabel gridColorLabel;
    private JLabel canvasBackgroundLabel;
    private transient boolean setting;

    /** Creates a new instance of JaxoGridBar.
     * @param panel An instance of the main panel.
     */
    public JaxoGridBar(JaxoMainPanel panel) {
        super(new GridBagLayout());
        this.thePanel = panel;

        initComponents();

        updateLanguage();

        new GridStateListener().setEnabled(true);
    }

    /** List of numbers of default grid sizes to display to the user.
     * @return List
     */
    public static List getGridSizeDefaults() {
        return gridSizes;
    }

    private void initComponents() {
        JPanel gOnOffPanel = getGridOnOffPanel();
        JPanel gSnapOnOffPanel = getSnapOnOffPanel();
        JPanel gTypePanel = getGridTypePanel();
        JPanel gStylePanel = getGridStylePanel();
        JPanel gColorPanel = getGridColorPanel();
        JPanel gSizePanel = getGridSizePanel();
        JPanel canvasBackgroundPanel = getCanvasBackgroundPanel();

        this.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));

        GridBagConstraints sc = new GridBagConstraints();

        sc.gridx = 0;
        sc.gridy = 0;
        this.add(gOnOffPanel, sc);

        sc.gridx = 1;
        sc.gridy = 0;
        this.add(gSnapOnOffPanel, sc);

        sc.gridx = 2;
        sc.gridy = 0;
        this.add(gTypePanel, sc);

        sc.gridx = 3;
        sc.gridy = 0;
        this.add(gStylePanel, sc);

        sc.gridx = 4;
        sc.gridy = 0;
        this.add(gColorPanel, sc);

        sc.gridx = 5;
        sc.gridy = 0;
        this.add(gSizePanel, sc);

        sc.gridx = 6;
        sc.gridy = 0;
        this.add(canvasBackgroundPanel, sc);

        //Bogus component used to allign everything on the left
        Component glue = Box.createHorizontalGlue();

        sc.gridx = 7;
        sc.gridy = 0;
        sc.weightx = 1;
        this.add(glue, sc);
    }

    private JPanel getGridOnOffPanel() {
        JPanel gridCheckBoxPanel = new JPanel();

        gridCheckBox = new JCheckBox();

        gridCheckBoxPanel.add(gridCheckBox);

        gridCheckBox.addActionListener(thePanel);
        gridCheckBox.setActionCommand(JaxoConstants.getModeAsString(
                JaxoConstants.GRID_ONOFF));

        return gridCheckBoxPanel;
    }

    private JPanel getSnapOnOffPanel() {
        JPanel snapCheckBoxPanel = new JPanel();

        snapCheckBox = new JCheckBox();

        snapCheckBox.setSelected(false);
        snapCheckBoxPanel.add(snapCheckBox);

        snapCheckBox.addActionListener(thePanel);
        snapCheckBox.setActionCommand(JaxoConstants.getModeAsString(
                JaxoConstants.GRID));

        return snapCheckBoxPanel;
    }

    private JPanel getGridTypePanel() {
        chooseGridType = new JComboBox(getGridCombTypes());

        chooseGridType.addItemListener(this);

        JPanel gridTypePanel = new JPanel();

        gridTypeLabel = new JLabel("", JLabel.TRAILING);
        gridTypePanel.add(gridTypeLabel);
        gridTypePanel.add(chooseGridType);

        return gridTypePanel;
    }

    private String[] getGridCombTypes() {
        String[] gridCombTypes =
        {
            JaxoLanguage.translate("Rectangular"),
            JaxoLanguage.translate("Hexagonal")
        };

        return gridCombTypes;
    }

    private String[] getGridStyles() {
        String[] styles =
        {
            JaxoLanguage.translate("Dot"), JaxoLanguage.translate("Cross"),
            JaxoLanguage.translate("Line"), JaxoLanguage.translate("Honeycomb")
        };

        return styles;
    }

    private JPanel getGridStylePanel() {
        chooseGridStyle = new JComboBox(getGridStyles());

        updateGridStyleWidth();

        String type = (String) chooseGridType.getSelectedItem();
        if (JaxoLanguage.translate("Rectangular").equals(type)) {
            chooseGridStyle.removeItemAt(JaxoDefaultGrid.STYLE_LINE_HONEYCOMB);
        }

        chooseGridStyle.addItemListener(this);

        JPanel gridStylePanel = new JPanel();

        gridStyleLabel = new JLabel("", JLabel.TRAILING);
        gridStylePanel.add(gridStyleLabel);
        gridStylePanel.add(chooseGridStyle);

        return gridStylePanel;
    }

    private JPanel getGridColorPanel() {
        JPanel gridColorPanel = new JPanel();

        gridColorButton = new JaxoColorButton();
        colorChooser = new JaxoColorChooser(gridColorPanel);
        gridColorButton.addPropertyChangeListener(this);
        gridColorButton.setChooser(colorChooser);
        gridColorButton.setChooserMode(JaxoColor.ALL_COLORS_MODE);

        gridColorLabel = new JLabel("", JLabel.TRAILING);

        gridColorPanel.add(gridColorLabel);
        gridColorPanel.add(gridColorButton);

        return gridColorPanel;
    }

    private JPanel getCanvasBackgroundPanel() {
        JPanel result = new JPanel();

        canvasBackgroundButton = new JaxoColorButton();
        canvasBackgroundButton.addPropertyChangeListener(this);
        canvasBackgroundButton.setChooser(colorChooser);
        canvasBackgroundButton.setChooserMode(JaxoColor.ALL_COLORS_MODE);

        canvasBackgroundLabel = new JLabel("", JLabel.TRAILING);
        result.add(canvasBackgroundLabel);
        result.add(canvasBackgroundButton);

        return result;
    }

    private JPanel getGridSizePanel() {
        JPanel pgrid = new JPanel();

        gridSizeLabel = new JLabel("", JLabel.TRAILING);
        pgrid.add(gridSizeLabel);

        gridSize = new JaxoSpinnerIntModel(1, 76, 1, 0);
        gridSize.addChangeListener(this);

        chooseGridSize =
            new JaxoSpinnerComboBox(new JaxoFixedJSpinner(gridSize),
                getGridSizeDefaults());

        pgrid.add(chooseGridSize);

        return pgrid;
    }

    /**
      * @param e The ItemEvent
      */
    public final void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() != ItemEvent.SELECTED) {
            return;
        }

        Object list = e.getSource();

        if (list.equals(chooseGridType)) {
            if (chooseGridType.getSelectedItem().equals(JaxoLanguage.translate(
                            "Rectangular"))) {
                if (!setting) {
                    thePanel.actionPerformed(new ActionEvent(this, JaxoConstants.RECTANGULAR_GRID,
                            JaxoConstants.getModeAsString(JaxoConstants.RECTANGULAR_GRID)));
                }
                if (chooseGridStyle.getItemCount() == (
                            JaxoDefaultGrid.STYLE_LINE_HONEYCOMB + 1
                        )) {
                    chooseGridStyle.removeItemAt(JaxoDefaultGrid.STYLE_LINE_HONEYCOMB);
                }
                gridSize.setIntStepSize(1);
            } else {
                if (!setting) {
                    thePanel.actionPerformed(new ActionEvent(this, JaxoConstants.HEXAGONAL_GRID,
                            JaxoConstants.getModeAsString(JaxoConstants.HEXAGONAL_GRID)));
                }
                if (chooseGridStyle.getItemCount() < (
                            JaxoDefaultGrid.STYLE_LINE_HONEYCOMB + 1
                        )) {
                    chooseGridStyle.insertItemAt(JaxoLanguage.translate(
                            "Honeycomb"), JaxoDefaultGrid.STYLE_LINE_HONEYCOMB);
                }
                gridSize.setIntStepSize(2);
            }
        } else if (list.equals(chooseGridStyle) && !setting) {
            int style = getStyleFromIndex(chooseGridStyle.getSelectedIndex());
            thePanel.actionPerformed(new ActionEvent(this, style,
                            JaxoConstants.getModeAsString(style)));
        }
    }

    private int getStyleFromIndex(int selectedIndex) {
        int mode = -1;

        switch (selectedIndex) {
            case 0 :
                mode = JaxoConstants.GRID_STYLE_DOT;
                break;
            case 1 :
                mode = JaxoConstants.GRID_STYLE_CROSS;
                break;
            case 2 :
                mode = JaxoConstants.GRID_STYLE_LINE;
                break;
            case 3 :
                mode = JaxoConstants.GRID_STYLE_LINE_HONEYCOMB;
                break;
            default: break;
        }

        return mode;
    }

    // move into main panel?

    /**
     * Applies a property change event.
     *
     * @param e the change event.
     */
    public final void propertyChange(PropertyChangeEvent e) {
        if (setting) {
            return;
        }

        if (e.getPropertyName().equals("color")) {
            if (e.getSource() == gridColorButton) {
                thePanel.setGridColor(gridColorButton.getColor());
            } else if (e.getSource() == canvasBackgroundButton) {
                thePanel.setCanvasBackground(canvasBackgroundButton.getColor());
            }
        }
    }

    /**
      * @param e The ChangeEvent
      */
    public final void stateChanged(ChangeEvent e) {
        if (setting) {
            return;
        }

        Object source = e.getSource();
        if (source == gridSize) {
            thePanel.setGridSize(gridSize.getIntValue());
        }
    }

    /** Enable/Disable the gridBar according to the status of the
     * gridOn checkBox.
     *@param enable the status of the gridBar to be set
     **/
    private void enableGridBar(boolean enable) {
        snapCheckBox.setEnabled(enable);
        chooseGridType.setEnabled(enable);
        gridTypeLabel.setEnabled(enable);
        chooseGridStyle.setEnabled(enable);
        gridStyleLabel.setEnabled(enable);
        gridColorButton.setEnabled(enable);
        gridColorLabel.setEnabled(enable);
        chooseGridSize.setEnabled(enable);
        gridSizeLabel.setEnabled(enable);
    }

    private void setGridOnOff(boolean onoff) {
        this.gridCheckBox.setSelected(onoff);
        enableGridBar(onoff);
    }

    private void setSnapOnOff(boolean onoff) {
        this.snapCheckBox.setSelected(onoff);
    }

    private void setCanvasBackground(Color value) {
        canvasBackgroundButton.setColor(value);
    }

    private void updateTabSettings(JaxoTab t) {
        updateGridSettings((t == null) ? JaxoDefaultGrid.newDefaultGrid()
                                       : t.getGrid());
    }

    /** Called when changing a tab, to reflect in the grid panel
     * the correct grid settings in the gridBar
     */
    private void updateGridSettings(JaxoDefaultGrid tabGrid) {
        setting = true;

        try {
            chooseGridType.setSelectedIndex(tabGrid.getGridType());
            chooseGridStyle.setSelectedIndex(tabGrid.getGridStyle());
            gridSize.setIntValue(tabGrid.getGridSize());

            gridColorButton.setColor(tabGrid.getGridColor());
        } finally {
            setting = false;
        }
    }

    /** {@inheritDoc} */
    public final void updateLanguage() {
        gridTypeLabel.setText(JaxoLanguage.label("JaxoGridBar.type"));
        gridSizeLabel.setText(JaxoLanguage.label("JaxoGridBar.size"));
        gridStyleLabel.setText(JaxoLanguage.label("JaxoGridBar.style"));
        gridColorLabel.setText(JaxoLanguage.label("JaxoGridBar.color"));
        canvasBackgroundLabel.setText(JaxoLanguage.label(
                "JaxoGridBar.canvasBackground"));
        gridCheckBox.setText(JaxoLanguage.translate("JaxoGridBar.gridOn"));
        snapCheckBox.setText(JaxoLanguage.translate("JaxoGridBar.snapOn"));

        setting = true;

        try {
            int selectedStyle = chooseGridStyle.getSelectedIndex();
            int selectedType = chooseGridType.getSelectedIndex();

            chooseGridType.removeAllItems();
            String[] gridCombTypes = getGridCombTypes();
            for (int i = 0; i < gridCombTypes.length; i++) {
                chooseGridType.addItem(gridCombTypes[i]);
            }
            chooseGridType.setSelectedIndex(selectedType);

            chooseGridStyle.removeAllItems();
            String[] gridStyles = getGridStyles();
            for (int i = 0; i < gridStyles.length; i++) {
                chooseGridStyle.addItem(gridStyles[i]);
            }

            String type = (String) chooseGridType.getSelectedItem();
            if (JaxoLanguage.translate("Rectangular").equals(type)) {
                chooseGridStyle.removeItemAt(JaxoDefaultGrid.STYLE_LINE_HONEYCOMB);
            }
            chooseGridStyle.setSelectedIndex(selectedStyle);

            updateGridStyleWidth();
        } finally {
            setting = false;
        }
    }

    private void updateGridStyleWidth() {
        int width = -1;
        int widestIndex = -1;

        String[] allGridStyles = getGridStyles();
        for (int i = 0; i < allGridStyles.length; i++) {
            chooseGridStyle.setPrototypeDisplayValue(allGridStyles[i]);
            int currentWidth = chooseGridStyle.getPreferredSize().width;

            if (currentWidth > width) {
                width = currentWidth;
                widestIndex = i;
            }
        }

        chooseGridStyle.setPrototypeDisplayValue(allGridStyles[widestIndex]);
    }

    private class GridStateListener implements PropertyChangeListener,
        ChangeListener {
        private boolean enabled;

        public final boolean isEnabled() {
            return enabled;
        }

        public void setEnabled(boolean value) {
            if (value != enabled) {
                enabled = value;

                if (enabled) {
                    thePanel.addPropertyChangeListener(this);
                    JaxoTab t = thePanel.getSelectedTab();

                    if (t != null) {
                        t.getGrid().addChangeListener(this);
                    }

                    updateTabSettings(t);

                    setGridOnOff(thePanel.isGridOn());
                    setSnapOnOff(thePanel.isSnap());
                    setCanvasBackground(thePanel.getCanvasBackground());
                } else {
                    thePanel.removePropertyChangeListener(this);
                    JaxoTab t = thePanel.getSelectedTab();

                    if (t != null) {
                        t.getGrid().removeChangeListener(this);
                    }
                }
            }
        }

        public void propertyChange(PropertyChangeEvent e) {
            if (e.getPropertyName().equals("Jaxo.selectedTab")) {
                JaxoTab old = (JaxoTab) e.getOldValue();

                if (old != null) {
                    old.getGrid().removeChangeListener(this);
                }

                JaxoTab t = (JaxoTab) e.getNewValue();

                if (t != null) {
                    t.getGrid().addChangeListener(this);
                }

                updateTabSettings(t);
            } else if (e.getPropertyName().equals("Jaxo.gridOn")) {
                setGridOnOff(thePanel.isGridOn());
            } else if (e.getPropertyName().equals("Jaxo.snap")) {
                setSnapOnOff(thePanel.isSnap());
            } else if (e.getPropertyName().equals("Jaxo.canvasBackground")) {
                setCanvasBackground(thePanel.getCanvasBackground());
            }
        }

        public void stateChanged(ChangeEvent e) {
            updateGridSettings((JaxoDefaultGrid) e.getSource());
        }
    }
}
