/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.panel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JWindow;
import javax.swing.event.MouseInputAdapter;

import net.sf.jaxodraw.graph.JaxoGraph;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.text.JaxoLatexText;
import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoDictionary;


/** Show tooltips if the mouse moves over JaxoLatexText objects
 * on the JaxoCanvas in the focused window.
 * Note: For simplicity, this implements listener methods directly.
 * Do not call or override.
 * @since 2.0
 */
public class JaxoLatexTextToolTip extends MouseInputAdapter
    implements WindowFocusListener {
    private final JaxoDictionary language = new JaxoDictionary(JaxoLatexTextToolTip.class);

    private final JaxoMainPanel main;
    private boolean enabled;
    private final JWindow window;
    private final JLabel label;

    /**
     * Constructor.
     * @param mainPanel Instance of the main panel.
     */
    public JaxoLatexTextToolTip(JaxoMainPanel mainPanel) {
        this.main = mainPanel;

        label = new JLabel();
        label.setBorder(BorderFactory.createLineBorder(JaxoColor.BLACK));
        label.setBackground(new Color(250, 250, 200));
        label.setOpaque(true);

        window = new JWindow(main);
        window.setContentPane(label);
        window.setFocusableWindowState(false);
    }

    /**
     * If not enabled, do nothing. Default is false.
     * @return True if currently enabled.
     */
    public final boolean isEnabled() {
        return enabled;
    }

    /**
     * If not enabled, do nothing. Default is false.
     * @param value True to enable.
     */
    public void setEnabled(boolean value) {
        if (enabled == value) {
            return;
        }

        enabled = value;

        if (enabled) {
            main.addWindowFocusListener(this);
            main.getCanvas().addMouseListener(this);
            main.getCanvas().addMouseMotionListener(this);
        } else {
            main.removeWindowFocusListener(this);
            main.getCanvas().removeMouseListener(this);
            main.getCanvas().removeMouseMotionListener(this);

            window.setVisible(false);
        }
    }

    /**
     * Action when window gains focus. Does nothing.
     *
     * @param e the window event.
     */
    public void windowGainedFocus(WindowEvent e) {
        // nothing to do
    }

    /**
     * Action when window looses focus. Hide tooltip window.
     *
     * @param e  the window event.
     */
    public void windowLostFocus(WindowEvent e) {
        window.setVisible(false);
    }

    /**
     * Process the event when the mouse enters the canvas.
     * Perform action, just in case.
     *
     * @param e The mouse event to process.
     */
    public final void mouseEntered(MouseEvent e) {
        mouseMoved(e);
    }

    /**
     * The action to be taken when the mouse is moved on the canvas.
     * Checks for Latex texts and shows the latex panel.
     * @param e The mouse event.
     */
    public final void mouseMoved(MouseEvent e) {
        if (!main.isFocused()) {
            return;
        }

        JaxoGraph g = main.getCanvas().getCanvasGraph();

        // Note: It is assumed that this graph shares the
        // coordinate system with the event source (canvas),
        // e.g., no Insets on Canvas
        for (int i = 0; i < g.listSize(); i++) {
            JaxoObject o = g.listElementAt(i);

            if (o instanceof JaxoLatexText) {
                JaxoLatexText text = (JaxoLatexText) o;
                if (text.isRolledOver(e.getX(), e.getY())) {
                    label.setText(language.message("texString%0",
                            text.getTextString()));

                    Point p = e.getComponent().getLocationOnScreen();

                    p.x += e.getX();
                    p.y += e.getY();

                    // offset Window somewhat
                    // (also necessary if a separate Window is used -
                    // if the mouse enters the Window, it is lost here)
                    Dimension d = window.getPreferredSize();

                    p.x -= (d.width / 3);
                    p.y -= (d.height * 4);

                    window.setBounds(p.x, p.y, d.width, d.height);
                    window.setVisible(true);
                    return;
                }
            }
        }

        window.setVisible(false);

    }

    /**
     * Process the event when the mouse exits the canvas. Hide window.
     *
     * @param e The mouse event to process.
     */
    public final void mouseExited(MouseEvent e) {
        window.setVisible(false);
    }

    /**
     * Process the event when the mouse is pressed on the canvas.
     * Hide window on MOVE/DUPLICATE.
     *
     * @param e The mouse event to process.
     */
    public final void mousePressed(MouseEvent e) {
        if ((main.getMode() == JaxoConstants.MOVE)
                || (main.getMode() == JaxoConstants.DUPLICATE)) {
            window.setVisible(false);
        }
    }

    /**
     * Process the event when the mouse is dragged on the canvas.
     * Hide window on MOVE/DUPLICATE.
     *
     * @param e The mouse event to process.
     */
    public final void mouseDragged(MouseEvent e) {
        if ((main.getMode() == JaxoConstants.MOVE)
                || (main.getMode() == JaxoConstants.DUPLICATE)) {
            window.setVisible(false);
        }
    }
}
