/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.panel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import net.sf.jaxodraw.gui.JaxoCanvas;
import net.sf.jaxodraw.gui.JaxoDialogs;
import net.sf.jaxodraw.gui.JaxoTab;
import net.sf.jaxodraw.gui.grid.JaxoDefaultGrid;
import net.sf.jaxodraw.gui.launch.JaxoLauncher;
import net.sf.jaxodraw.gui.menu.JaxoMenuBar;
import net.sf.jaxodraw.gui.panel.button.JaxoButtonPanel;
import net.sf.jaxodraw.io.JaxoPreview;
import net.sf.jaxodraw.io.exports.JaxoExportLatexPS;
import net.sf.jaxodraw.plugin.JaxoPluginInfo;
import net.sf.jaxodraw.util.JaxoPrefs;
import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoInfo;
import net.sf.jaxodraw.util.JaxoLanguage;
import net.sf.jaxodraw.util.JaxoLocalized;
import net.sf.jaxodraw.util.JaxoLog;
import net.sf.jaxodraw.util.JaxoLooknFeel;
import net.sf.jaxodraw.util.JaxoUtils;


/** The main panel: holds the menu bar, the button panel, the flag panel and the
 * canvas. Responsible for the interaction between those components.
 * @since 2.0
 */
public final class JaxoMainPanel extends JFrame implements ActionListener,
    JaxoLocalized {
    private static final long serialVersionUID = 7526471155622776147L;
    /** The button panel of this JaxoMainPanel. */
    private final JaxoButtonPanel buttonPanel;

    /** The menu bar of this JaxoMainPanel. */
    private final JaxoMenuBar jaxomenubar;

    /** The tool bar of this JaxoMainPanel. */
    private final JaxoToolBar toolBar;

    /** The flag panel of this JaxoMainPanel. */
    private final JaxoFlags flagpanel;

    /** The Grid panel of this JaxoMainPanel. */
    private final JaxoGridBar gridBar;
    private int currentMode = JaxoConstants.STANDBY;
    private int vertexType = JaxoConstants.VERTEXT1;
    private JaxoPrefsPanel preferencesPanel;
    private JaxoPluginManagerPanel pluginManagerPanel;

    // Watch file mode
    private boolean watchMode;

    /** The panel that holds the tabs*/
    private final JaxoTabbedCanvasPanel canvasPanel;

    /** A boolean variable that indicates whether the drawing/dragging should be done
     * with the grid or not.
     */
    private boolean snap;

    /** The Canvas where all the objects will be drawn */
    private final JaxoCanvas theCanvas;

    private final JaxoPreview preview;
    private boolean canUndo;
    private boolean canRedo;

    /** The initialisation method of the panel: sets up the menus, the button panels,
     * the flag panel and the canvas. Start by showing one tab with an empty
       graph.
     */
    public JaxoMainPanel() {
        // First set the right look and feel - so that all components are
        // created with the right one - otherwise the layout may get confused.
        applyLookAndFeel(getLookAndFeelClassName(), false);

        // Also set the right locale early.
        JaxoLooknFeel.applyLocale(JaxoLanguage.locale());

        this.buttonPanel = new JaxoButtonPanel();
        buttonPanel.setDefault(getDefaultMode());
        buttonPanel.addActionListener(this);
        this.addPropertyChangeListener(buttonPanel);

        this.toolBar = new JaxoToolBar();
        toolBar.addActionListener(this);
        this.addPropertyChangeListener("Jaxo.watchMode", toolBar);

        this.jaxomenubar = new JaxoMenuBar();
        jaxomenubar.addActionListener(this);
        this.addPropertyChangeListener(jaxomenubar);

        this.theCanvas = new JaxoCanvas(this);

        // need to set these early for size calculations
        theCanvas.setMinimumCanvasSize(new Dimension(JaxoPrefs.getIntPref(
                    JaxoPrefs.PREF_SCREENSIZEX),
                JaxoPrefs.getIntPref(JaxoPrefs.PREF_SCREENSIZEY)));
        theCanvas.setMaximumCanvasSize(JaxoInfo.SCREEN_SIZE);

        this.addWindowFocusListener(new WindowAdapter() {
            public void windowLostFocus(WindowEvent e) {
                theCanvas.setMouseLocation(null);
            }
        });

        this.preview = new JaxoPreview(this);

        this.canvasPanel = new JaxoTabbedCanvasPanel(this);
        canvasPanel.initTabs();

        new JaxoLatexTextToolTip(this).setEnabled(true);

        this.flagpanel = new JaxoFlags(getSelectedTab());
        this.addPropertyChangeListener(flagpanel);
        theCanvas.addPropertyChangeListener(flagpanel);
        this.gridBar = new JaxoGridBar(this);

        JPanel bars = getBarsPanel(toolBar, gridBar);

        // The final layout
        ((JPanel) getContentPane()).setBorder(JaxoButtonPanel.RAISED_ETCHED_BORDER);
        getContentPane().add(bars, BorderLayout.PAGE_START);
        getContentPane().add(canvasPanel.getRoot(), BorderLayout.CENTER);
        setJMenuBar(jaxomenubar);
        getContentPane().add(buttonPanel, BorderLayout.LINE_START);
        getContentPane().add(flagpanel, BorderLayout.PAGE_END);

        setupInitialState();

        applyPrefs();

        setIconImage(JaxoUtils.newImage("frame.png"));
        setTitle(JaxoInfo.VERSION);

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

        this.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    fileEvent(JaxoConstants.QUIT);
                }
            });

        resetMode();
    }

    /**
     * Returns a JaxoPluginInfo object that contains information about
     * available plugins at runtime.
     * @return The JaxoPluginInfo object.
     */
    public JaxoPluginInfo getPluginInfo() {
        return JaxoPluginInfo.getPluginInfo();
    }

    private String getLookAndFeelClassName() {
        return JaxoPrefs.getStringPref(JaxoPrefs.PREF_LOOKNFEEL);
    }

    /**
     * Open given file names, reusing an current unused tab.
     *
     * @param inputFileNames Array of file names to be opened.
     */
    public void openOrImport(String[] inputFileNames) {
        for (int i = 0; i < inputFileNames.length; i++) {
            JaxoLog.info(JaxoLanguage.message("Reading_file__0", inputFileNames[i]));

            canvasPanel.openOrImport(inputFileNames[i]);
        }

        // TODO: calculate the default mode, and just set that, before
        // creating any tabs, and JaxoTabbedCanvasPanel should push mode
        // to new tabs
        // For now: Do it by hand (and use the current mode)
        for (int i = canvasPanel.getTabCount() - 1; i >= 0; --i) {
            canvasPanel.getTabAt(i).setTabMode(getMode());
        }
    }

    /** Return the current drawing/editing mode.
     * @return The current drawing/editing mode.
     */
    public int getMode() {
        return currentMode;
    }

    /** Sets the current drawing/edit mode.
     * @param newMode The drawing/edit mode to be set.
     */
    public void setMode(int newMode) {
        if (newMode != currentMode) {
            int old = currentMode;
            currentMode = newMode;

            firePropertyChange("Jaxo.mode", old, currentMode);
        }

        theCanvas.updateMode(currentMode);
    }

    /** Returns an instance of the canvas.
     * @return The canvas.
     */
    public JaxoCanvas getCanvas() {
        return theCanvas;
    }

    /** Update all sub-components with the new language.
     * @param languageString The new language to set.
     */
    public void setLanguage(String languageString) {
        String old = JaxoPrefs.getStringPref(JaxoPrefs.PREF_LANGUAGE);
        if (!languageString.equals(old)) {
            Dimension oldPreferredSize = getPreferredSize();

            JaxoPrefs.setStringPref(JaxoPrefs.PREF_LANGUAGE, languageString);
            updateLanguage();
            firePropertyChange("Jaxo.language", old, languageString);

            // Increase size to fit the preferred size, but only if the
            // window was not shrunk by hand (independently for each
            // direction)
            Dimension d = getSize();
            Dimension s = getPreferredSize();

            if (d.width >= oldPreferredSize.width) {
                d.width = Math.max(d.width, s.width);
            }
            if (d.height >= oldPreferredSize.height) {
                d.height = Math.max(d.height, s.height);
            }

            setSize(d.width, d.height);
        }
    }

    /** {@inheritDoc} */
    public void updateLanguage() {
        jaxomenubar.updateLanguage();
        gridBar.updateLanguage();
        toolBar.updateLanguage();
        buttonPanel.updateLanguage();
        flagpanel.updateLanguage();
        canvasPanel.updateLanguage();
        theCanvas.updateLanguage();

        if (preferencesPanel != null) {
            preferencesPanel.dispose();
            preferencesPanel = null;
        }

        if (pluginManagerPanel != null) {
            pluginManagerPanel.updateLanguage();
        }

        JaxoLooknFeel.applyLocale(JaxoLanguage.locale());
    }

    /** Processes the corresponding events whenever a particle button has been pressed.
     * @param i An integer specifying the particle button that was pressed.
     */
    private void particleEvent(int i) {
        setMode(i);
    }

    /** Called when a misc item has been chosen from the button panel.
     * Presses the corresponding button in the button panel, sets
     * the mode in the flag panel, sets the current mode and shows all handles.
     * Finally, makes sure that no object is selected.
     * @param i Integer specifying the misc mode.
     */
    private void miscEvent(int i) {
        setMode(i);
    }

    /** Called when an edit item has been chosen from the menu bar, the button panel or
     * via key shortcuts. Presses the corresponding button in the button panel, sets
     * the mode in the flag panel, sets the current mode and shows all handles.
     * Finally, makes sure that no object is selected.
     * @param i Integer specifying the edit mode.
     */
    private void editEvent(int i) {
        setMode(i);
    }

    /** Called when a file item has been chosen from the menu , the tool bar or
     * via key shortcuts. Calls the corresponding methods in the JaxoFileMenu.
     * @param i Integer specifying the file event.
     */
    private void fileEvent(int i) {
        canvasPanel.fileEvent(i);
    }

    /** Called when the vertex type has been changed.
     * @param i Integer specifying the file event.
     */
    private void vertexTypeEvent(int i) {
        setVertexType(i);
    }

    /** Called when an option is chosen from the options menu.
     * @param i Integer specifying the option event.
     */
    private void optionEvent(int i) {
        switch (i) {
            case JaxoConstants.TOOLBAR:
                setToolBarVisible(!JaxoPrefs.getBooleanPref(
                        JaxoPrefs.PREF_SHOWTOOL));
                break;
            case JaxoConstants.STATUSBAR:
                setStatusBarVisible(!JaxoPrefs.getBooleanPref(
                        JaxoPrefs.PREF_SHOWSTATUS));
                break;
            case JaxoConstants.GRIDBAR:
                setGridBarVisible(!JaxoPrefs.getBooleanPref(
                        JaxoPrefs.PREF_SHOWGRIDBAR));
                break;
            case JaxoConstants.ANTIALIAS:
                setAntialiasEnabled(!JaxoPrefs.getBooleanPref(
                        JaxoPrefs.PREF_ANTIALIAS));
                break;
            case JaxoConstants.ARROW:
                setWithArrow(!JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_ARROW));
                break;
            case JaxoConstants.GRID_ONOFF:
                setGridOn(!JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_GRIDONOFF));
                break;
            case JaxoConstants.SNAP_ONOFF:
                setSnap(!JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SNAPONOFF));
                break;
            case JaxoConstants.LOOKNFEEL:
                final JaxoLookAndFeelPanel laf = new JaxoLookAndFeelPanel();
                laf.setToUIManagerValue();

                JaxoDialogs.showMiniDialog(this,
                    JaxoLanguage.translate("Look_And_Feel"), laf.getRoot(),
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            JaxoPrefs.setStringPref(JaxoPrefs.PREF_LOOKNFEEL,
                                laf.getValue());
                            applyLookAndFeel(getLookAndFeelClassName(), true);
                        }
                    });


                break;
            case JaxoConstants.PREFERENCES:
                if (preferencesPanel == null) {
                    preferencesPanel = new JaxoPrefsPanel(this);
                    preferencesPanel.addComponentListener(new ComponentAdapter() {
                        public void componentHidden(ComponentEvent e) {
                            applyPrefs();
                        }
                    });
                }
                preferencesPanel.setFromPrefs();

                preferencesPanel.setVisible(true);
                break;
            case JaxoConstants.PLUGINS:
                if (pluginManagerPanel == null) {
                    pluginManagerPanel = new JaxoPluginManagerPanel(this);
                }
                pluginManagerPanel.setVisible(true);
                break;
            default:
                break;
        }
    }

    /**
     * Called when a button in the grid panel has been pressed.
     * @param i Integer specifying the grid event.
     */
    private void gridEvent(int i) {
        switch (i) {
            case JaxoConstants.QUIT:
                fileEvent(JaxoConstants.QUIT);
                break;
            case JaxoConstants.ZOOM:
                setMode(i);
                break;
            case JaxoConstants.GRID:
                setSnap(!isSnap());
                break;
            case JaxoConstants.RECTANGULAR_GRID:
                setGridType(JaxoDefaultGrid.TYPE_RECTANGULAR);
                break;
            case JaxoConstants.HEXAGONAL_GRID:
                setGridType(JaxoDefaultGrid.TYPE_HEXAGONAL);
                break;
            case JaxoConstants.GRID_STYLE_DOT:
                setGridStyle(JaxoDefaultGrid.STYLE_DOT);
                break;
            case JaxoConstants.GRID_STYLE_CROSS:
                setGridStyle(JaxoDefaultGrid.STYLE_CROSS);
                break;
            case JaxoConstants.GRID_STYLE_LINE:
                setGridStyle(JaxoDefaultGrid.STYLE_LINE);
                break;
            case JaxoConstants.GRID_STYLE_LINE_HONEYCOMB:
                setGridStyle(JaxoDefaultGrid.STYLE_LINE_HONEYCOMB);
                break;
            default:
                break;
        }
    }

    /** Called when an action item has been chosen from the menu bar, the button panel or
     * via key shortcuts. An action item calls for immediate action, without
     * further input, currently there is Undo, Clear, Paste, Refresh and Preview
     * (Quit is a file event).
     * @param i Integer specifying the action mode. Currently one of
     * JaxoConstants.UNDO, JaxoConstants.REDO, JaxoConstants.CLEAR, JaxoConstants.PASTE,
     * JaxoConstants.REFRESH or JaxoConstants.PREVIEW.
     */
    private void actionEvent(int i) {

        switch (i) {
            case JaxoConstants.UNDO:
                getSelectedTab().undoMove();
                break;
            case JaxoConstants.REDO:
                getSelectedTab().redoMove();
                break;
            case JaxoConstants.CLEAR:
                theCanvas.clear();

                break;
            case JaxoConstants.CUT:
                theCanvas.copyMarkedObjects();
                theCanvas.deleteMarkedObjects();

                break;
            case JaxoConstants.SCOPY:
                theCanvas.copyMarkedObjects();
                break;
            case JaxoConstants.SGROUP:
                theCanvas.groupMarkedObjects();
                break;
            case JaxoConstants.SUNGROUP:
                theCanvas.ungroupMarkedObjects();
                break;
            case JaxoConstants.SFORE:
                theCanvas.moveSelection(false);

                break;
            case JaxoConstants.SBACK:
                theCanvas.moveSelection(true);

                break;
            case JaxoConstants.PASTE:
                theCanvas.pasteFromClipboard();

                break;
            case JaxoConstants.REFRESH:
                theCanvas.refresh();

                break;
            case JaxoConstants.PREVIEW:
                JaxoExportLatexPS l = new JaxoExportLatexPS();
                l.setParentComponent(this);
                l.setGraph(theCanvas.getCanvasGraph());
                l.preview(preview.copy(), isWatchMode());
                break;
            case JaxoConstants.WATCHFILE:
                setWatchMode(!isWatchMode());
                break;
            case JaxoConstants.RENAME_TAB:
                canvasPanel.renameTab();
                break;
            case JaxoConstants.DESCRIBE:
                String s =
                    JaxoDialogs.getDescription(this, theCanvas.getCanvasGraph().getDescription());
                if (s != null) {
                    theCanvas.getCanvasGraph().setDescription(s);
                    commitGraphChanges();
                }
                break;
            case JaxoConstants.MOVE_GRAPH:
                JaxoMoveGraph g = new JaxoMoveGraph(this, theCanvas);
                g.show();
                if (g.hasChanged()) {
                    commitGraphChanges();
                }
                break;
            case JaxoConstants.PACKAGE:
                JaxoLatexPackage latexPackage =
                    new JaxoLatexPackage(this, theCanvas.getCanvasGraph().getPackageList());
                if (!latexPackage.wasCancelled()) {
                    commitGraphChanges();
                }
                break;
            case JaxoConstants.LATEX_PREVIEW_SELECTION:
                JaxoExportLatexPS lps = new JaxoExportLatexPS();
                lps.setParentComponent(this);
                lps.setGraph(theCanvas.getClipboard());
                lps.preview(preview.copy(), isWatchMode());
                break;
            default:
                break;
        }

        // Should none of these cause the default action?
        setMode(currentMode);
    }

    /** Called when a help item has been chosen from the menu , the tool bar or
     * via key shortcuts. Calls the corresponding methods in the JaxoHelpMenu.
     * @param mode The mode specifying the help event.
     */
    private void helpEvent(int mode) {
        switch (mode) {
            case JaxoConstants.ABOUT:
                about();
                break;
            case JaxoConstants.USR_GUIDE:
                usrGuide();
                break;
            case JaxoConstants.SYS_INFO:
                sysInfo();
                break;
            case JaxoConstants.MAC_README:
                macReadme();
                break;
            default:
                break;
        }
    }

    /**
     * Default preview to be used. Should not be modified directly
     * - instead, obtain an own copy().
     * @return A preview.
     */
    public JaxoPreview getPreview() {
        return preview;
    }

    private void setupInitialState() {
        toolBar.setVisible(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SHOWTOOL));
        flagpanel.setVisible(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SHOWSTATUS));
        gridBar.setVisible(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SHOWGRIDBAR));
        theCanvas.setGridPainted(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_GRIDONOFF));
        theCanvas.setAntialiasEnabled(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_ANTIALIAS));
        theCanvas.setCanvasBackground(JaxoColor.getColor(JaxoPrefs.getStringPref(
            JaxoPrefs.PREF_CANVASBACKGROUND), JaxoColor.ALL_COLORS_MODE));
        firePropertyChange("Jaxo.gridOn", false, isGridOn());
        firePropertyChange("Jaxo.snap", false, isSnap());
    }

    /** Applies the preferences. */
    public void applyPrefs() {
        setLanguage(JaxoPrefs.getStringPref(JaxoPrefs.PREF_LANGUAGE));

        setToolBarVisible(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SHOWTOOL));
        setStatusBarVisible(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SHOWSTATUS));
        setGridBarVisible(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SHOWGRIDBAR));
        setGridOn(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_GRIDONOFF));
        setSnap(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_SNAPONOFF));
        setAntialiasEnabled(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_ANTIALIAS));
        setWithArrow(JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_ARROW));

        setDefaultAction(JaxoPrefs.getIntPref(JaxoPrefs.PREF_DEFAULTACTION));

        setCanvasBackground(JaxoColor.getColor(JaxoPrefs.getStringPref(
            JaxoPrefs.PREF_CANVASBACKGROUND), JaxoColor.ALL_COLORS_MODE));

        theCanvas.applyPrefs();
        theCanvas.setMinimumCanvasSize(new Dimension(JaxoPrefs.getIntPref(
                    JaxoPrefs.PREF_SCREENSIZEX),
                JaxoPrefs.getIntPref(JaxoPrefs.PREF_SCREENSIZEY)));
        theCanvas.setMaximumCanvasSize(JaxoInfo.SCREEN_SIZE);

        applyLookAndFeel(getLookAndFeelClassName(), false);
    }

    private void applyLookAndFeel(String lookAndFeelClassName, boolean warning) {
        boolean success = JaxoLooknFeel.applyLookAndFeel(lookAndFeelClassName);
        if (!success && warning) {
            String message =
                    JaxoLooknFeel.info(lookAndFeelClassName).getName()
                    + JaxoLanguage.translate(
                        "_Look_and_Feel_is_not_supported_on_this_platform!");
                JaxoDialogs.showErrorDialog(null, message);
        }
    }

    /**
     * Returns the background color of the canvas.
     * @return The canvas background color.
     */
    public Color getCanvasBackground() {
        return theCanvas.getCanvasBackground();
    }

    /**
     * Sets the background color of the canvas.
     * @param value The color to set.
     */
    public void setCanvasBackground(Color value) {
        if (!value.equals(getCanvasBackground())) {
            Color old = getCanvasBackground();

            JaxoPrefs.setStringPref(JaxoPrefs.PREF_CANVASBACKGROUND,
                JaxoColor.getColorName(value, JaxoColor.ALL_COLORS_MODE));

            theCanvas.setCanvasBackground(value);

            firePropertyChange("Jaxo.canvasBackground", old,
                getCanvasBackground());
        }
    }

    /** Switches the toolBar on or off.
     * @param show A boolean variable that indicates whether the toolBar is visible or not.
     */
    public void setToolBarVisible(boolean show) {
        if (show != toolBar.isVisible()) {
            JaxoPrefs.setBooleanPref(JaxoPrefs.PREF_SHOWTOOL, show);
            toolBar.setVisible(show);

            firePropertyChange("Jaxo.toolBarVisible", !show, show);
        }
    }

    /** Switches the status bar on or off.
     * @param show A boolean variable that indicates whether the status bar is visible or not.
     */
    public void setStatusBarVisible(boolean show) {
        if (show != flagpanel.isVisible()) {
            JaxoPrefs.setBooleanPref(JaxoPrefs.PREF_SHOWSTATUS, show);
            flagpanel.setVisible(show);

            firePropertyChange("Jaxo.statusBarVisible", !show, show);
        }
    }

    /** Switches the gridBar on or off.
     * @param show A boolean variable that indicates whether the toolBar is visible or not.
     */
    public void setGridBarVisible(boolean show) {
        if (show != gridBar.isVisible()) {
            JaxoPrefs.setBooleanPref(JaxoPrefs.PREF_SHOWGRIDBAR, show);
            gridBar.setVisible(show);

            firePropertyChange("Jaxo.gridBarVisible", !show, show);
        }
    }

    /** Switches antialiasing on or off.
     * @param on A boolean variable that indicates whether antialising is on or not.
     */
    public void setAntialiasEnabled(boolean on) {
        if (on != theCanvas.isAntialiasEnabled()) {
            JaxoPrefs.setBooleanPref(JaxoPrefs.PREF_ANTIALIAS, on);

            firePropertyChange("Jaxo.antialiasEnabled", !on, on);

            theCanvas.setAntialiasEnabled(on);
        }
    }

    /** Switches the grid on or off.
     * @param on A boolean variable that indicates whether the grid is on or not.
     */
    private void setGridOn(boolean on) {
        if (on != isGridOn()) {
            if (!on) {
                setSnap(false);
            }

            JaxoPrefs.setBooleanPref(JaxoPrefs.PREF_GRIDONOFF, on);
            theCanvas.setGridPainted(on);

            firePropertyChange("Jaxo.gridOn", !on, on);
        }
    }

    /** Determines whether to draw arrows on lines, arcs and loops.
     * @param on A boolean variable that indicates whether to draw arrows on
     * lines, arcs and loops.
     */
    private void setWithArrow(boolean on) {
        if (on != JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_ARROW)) {
            JaxoPrefs.setBooleanPref(JaxoPrefs.PREF_ARROW, on);

            firePropertyChange("Jaxo.withArrow", !on, on);
        }
    }

    /** Sets the current type of vertex.
     * @param value An integer specifying the vertex type.
     */
    private void setVertexType(int value) {
        if (value != vertexType) {
            int old = vertexType;

            vertexType = value;

            firePropertyChange("Jaxo.vertexType", old, value);
        }
    }

    /** Returns the current vertex type.
     * @return The current vertex type.
     */
    public int getVertexType() {
        return vertexType;
    }

    /** Sets the snap mode.
     * @param setSnap The new snap mode.
     */
    public void setSnap(boolean setSnap) {
        boolean newSnap = setSnap;
        if (!isGridOn()) {
            newSnap = false;
        }

        if (snap != newSnap) {
            this.snap = newSnap;

            getSelectedTab().setSnappingToGrid(newSnap);

            firePropertyChange("Jaxo.snap", !newSnap, newSnap);
            JaxoPrefs.setBooleanPref(JaxoPrefs.PREF_SNAPONOFF, newSnap);
        }
    }

    /** Returns the current snap mode.
     * @return The current snap mode.
     */
    public boolean isSnap() {
        return snap;
    }

    /**
     * Indicates whether watch mode is active or not.
     * @return True when watch mode is active, false otherwise.
     */
    public boolean isWatchMode() {
        return watchMode;
    }

    /**
     * Sets the current watch mode.
     * @param value True, if watch mode is to be switched on, false
     * otherwise.
     */
    private void setWatchMode(boolean value) {
        if (watchMode != value) {
            this.watchMode = value;

            firePropertyChange("Jaxo.watchMode", !value, value);
        }
    }

    private void setCanUndo(boolean value) {
        if (value != canUndo) {
            canUndo = value;

            firePropertyChange("Jaxo.canUndo", !value, value);
        }
    }

    private void setCanRedo(boolean value) {
        if (value != canRedo) {
            canRedo = value;

            firePropertyChange("Jaxo.canRedo", !value, value);
        }
    }

    /** Revert to the the defaultMode, or re-initialize the current
     * mode if the defaultMode is none.
     */
    public void resetMode() {
        int newMode = getDefaultMode();

        if (newMode == JaxoConstants.STANDBY) {
            newMode = currentMode;
        }

        setMode(newMode);
    }

    /**
     * Returns the current default mode.
     * @return The current default mode.
     */
    public int getDefaultMode() {
        int defAction = JaxoPrefs.getIntPref(JaxoPrefs.PREF_DEFAULTACTION);
        return JaxoConstants.defaultActionToMode(defAction);
    }

    /**
     * Sets the default action. If value is not a valid default mode,
     * it is set to JaxoConstants.DEF_NONE.
     * @param value The default action to set.
     */
    public void setDefaultAction(int value) {
        int newValue = value;

        if (!JaxoConstants.isDefaultMode(newValue)) {
            newValue = JaxoConstants.DEF_NONE;
        }

        int old = JaxoPrefs.getIntPref(JaxoPrefs.PREF_DEFAULTACTION);

        if (newValue != old) {
            int oldMode = getDefaultMode();

            JaxoPrefs.setIntPref(JaxoPrefs.PREF_DEFAULTACTION, newValue);

            firePropertyChange("Jaxo.defaultAction", old, newValue);
            firePropertyChange("Jaxo.defaultMode", oldMode, getDefaultMode());
        }
    }

    /**
     * Check if the grid is currently switched on in the preferences.
     * @return True if grid is on.
     */
    public boolean isGridOn() {
        return JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_GRIDONOFF);
    }

    /**
     * forward to selected JaxoTab.
     * @return The grid size of the current tab.
     */
    public int getGridSize() {
        return getSelectedTab().getGridSize();
    }

    /**
     * forward to selected JaxoTab and update canvas dimensions.
     * @param value The size to set.
     */
    public void setGridSize(int value) {
        getSelectedTab().setGridSize(value);
    }

    /**
     * set default: currently changes grid size of all tabs.
     * @param value The size to set.
     */
    public void setDefaultGridSize(int value) {
        for (int i = canvasPanel.getTabCount() - 1; i >= 0; --i) {
            canvasPanel.getTabAt(i).setGridSize(value);
        }
    }

    /**
     * forward to selected JaxoTab.
     * @return The grid type of the current tab.
     */
    public int getGridType() {
        return getSelectedTab().getGridType();
    }

    /**
     * forward to selected JaxoTab.
     * @param value The type to set.
     */
    public void setGridType(int value) {
        getSelectedTab().setGridType(value);
    }

    /**
     * forward to selected JaxoTab.
     * @param style The style to set.
     */
    public void setGridStyle(int style) {
        getSelectedTab().setGridStyle(style);
    }

    /**
     * forward to selected JaxoTab.
     * @param color The color to set.
     */
    public void setGridColor(Color color) {
        getSelectedTab().setGridColor(color);
    }

    /**
     * set default: currently changes grid type of all tabs.
     * @param value The type to set.
     */
    public void setDefaultGridType(int value) {
        for (int i = canvasPanel.getTabCount() - 1; i >= 0; --i) {
            canvasPanel.getTabAt(i).setGridType(value);
        }
    }

    /**
     * forward to JaxoTabbedCanvasPanel.
     * @return The current tab.
     */
    public JaxoTab getSelectedTab() {
        return canvasPanel.getSelectedTab();
    }

    /**
     * forward to JaxoTabbedCanvasPanel.
     */
    public void commitGraphChanges() {
        getSelectedTab().commitGraphChanges();
    }

    /** forward to selected JaxoTab. */
    public void commitRepeatableGraphChanges() {
        getSelectedTab().commitRepeatableGraphChanges();
    }

    /**
     * forward to selected JaxoTab.
     * @param count The count of changes.
     */
    public void commitRepeatableGraphChanges(int count) {
        getSelectedTab().commitRepeatableGraphChanges(count);
    }

    /** Adds a new entry to the file menu, with a shortcut to
     * a recently opened graph.
     * @param saveFileName The full path to the saved graph.
     */
    public void addRecentFile(String saveFileName) {
        firePropertyChange("Jaxo.recentFile", null, saveFileName);
    }

    /**
     * Removes the given file from the list of recent files.
     * @param saveFileName The file to remove.
     */
    public void removeRecentFile(String saveFileName) {
        firePropertyChange("Jaxo.recentFile", saveFileName, null);
    }

    /**
     * Performs an action in reply to an action event on either
     * the menu bar, tool bar, grid bar, tabbed canvas panel or button panel.
     * NOTE: This should be the central method where all input events
     * (except drawing events on the Canvas) should be processed.
     * The MainPanel is the only component that has direct access
     * to all sub-components, and therefore it's easiest to trigger
     * identical events from different input sources.
     *
     * <p>This method throws an Exception if the source of the ActionEvent
     * is not a Component.</p>
     *
     * @param evt The action event that triggers an action.
     * The ActionCommand of the event has to be an integer, encapsulated
     * as a String, that identifies the event according to the modes
     * in {@link JaxoConstants}.
     */
    public void actionPerformed(ActionEvent evt) {
        Component source = (Component) evt.getSource();
        int mode = JaxoConstants.getModeAsInt(evt.getActionCommand());

        if (mode == JaxoConstants.RECENT_FILE) {
            AbstractButton sourceButton = (AbstractButton) source;
            String fileName = sourceButton.getName();

            JaxoTab t = canvasPanel.getTabWithSaveFileName(fileName);

            if (t != null) {
                canvasPanel.setSelectedTab(t);
                addRecentFile(fileName);
                return;
            }

            canvasPanel.openOrImport(fileName);

            t = canvasPanel.getTabWithSaveFileName(fileName);

            if (t == null) {
                removeRecentFile(fileName);
            }
        } else {
            // source is not used anywhere, why do we need that?
            /*
            Component c = source;

            while ((c != null) && !c.isShowing()) {
                if (c instanceof JPopupMenu) {
                    c = ((JPopupMenu) c).getInvoker();
                } else {
                    c = c.getParent();
                }
            }
            */

            performAction(mode);
        }
    }

    /**
     * Performs the action corresponding to the given mode.
     * @param mode A mode as defined in {@link JaxoConstants}.
     * @param source The source of the action.
     */
    private void performAction(int mode) {
        if (JaxoConstants.isZoomFactorMode(mode)) {
            theCanvas.getZoom().setZoomFactorFor(mode);
        } else if (JaxoConstants.isFileMode(mode)) {
            fileEvent(mode);
        } else if (JaxoConstants.isActionMode(mode)) {
            actionEvent(mode);
        } else if (JaxoConstants.isHelpMode(mode)) {
            helpEvent(mode);
        } else if (JaxoConstants.isGridMode(mode)) {
            gridEvent(mode);
        } else if (JaxoConstants.isEditMode(mode)) {
            editEvent(mode);
        } else if (JaxoConstants.isParticleMode(mode)) {
            particleEvent(mode);
        } else if (JaxoConstants.isMiscMode(mode)) {
            miscEvent(mode);
        } else if (JaxoConstants.isVertexMode(mode)) {
            vertexTypeEvent(mode);
        } else if (JaxoConstants.isLanguageMode(mode)) {
            setLanguage(JaxoLanguage.getLanguageFor(mode));
        } else if (JaxoConstants.isOptionMode(mode)) {
            optionEvent(mode);
        } else if (JaxoConstants.isDefaultMode(mode)) {
            setDefaultAction(mode);
        }
    }

    /**
     * Dispose the main frame and call JaxoLauncher.shutdown(exitStatus).
     *
     * @param exitStatus The exit status code.
     */
    public void shutdown(int exitStatus) {
        if (exitStatus == 0) {
            try {
                JaxoPrefs.saveRecentFiles();
            } catch (Exception e) {
                JaxoLog.debug(e);
                JaxoDialogs.showI18NErrorDialog(
                    this, "Cannot_save_preferences!");
            }
        }

        setVisible(false);
        dispose();

        JaxoLauncher.shutdown(exitStatus);
    }

    // Called from JaxoTabbedCanvasPanel

    /**
     * Updates the tab when it changes.
     * @param old The old tab.
     * @param t The new tab.
     */
    public void updateSelectedTab(JaxoTab old, JaxoTab t) {
        if (old != null) {
            old.setSnappingToGrid(isSnap());
            old.setTabMode(getMode());
        }

        firePropertyChange("Jaxo.selectedTab", old, t);

        tabChanged(t);

        if (t == null) {
            setSnap(false);
            setMode(JaxoConstants.STANDBY);
        } else {
            setSnap(t.isSnappingToGrid());
            setMode(t.getTabMode());
        }
    }

    /**
     * Adjusts the main panel to a changed tab.
     * @param t The old tab.
     */
    public void tabChanged(JaxoTab t) {
        if (t == null) {
            setCanUndo(false);
            setCanRedo(false);
        } else {
            setCanUndo(t.canUndo());
            setCanRedo(t.canRedo());
        }
    }

    private JPanel getBarsPanel(JaxoToolBar tB, JaxoGridBar gB) {
        JPanel barP = JaxoDialogs.newPageBoxLayoutPanel();

        barP.add(tB);
        barP.add(gB);

        return barP;
    }

    /** Pops a small dialog with some information about the current
     * version of JaxoDraw.
     */
    private void about() {
        String message = JaxoInfo.about(getPluginInfo().availablePlugins());
        String title = JaxoDialogs.windowTitle("About");

        JaxoDialogs.showInfoDialog(this, message, title);
    }

    /** Opens the User Guide of JaxoDraw either in a custom browser
     * (if set in the Preferences) or in an internal browser.
     */
    private void usrGuide() {
        JaxoUserGuide usrGuide = new JaxoUserGuide(this);
        usrGuide.show(getPreview().copy());
    }

    /** Pops up a dialog with some information about
     * the current system environment.
     */
    private void sysInfo() {
        String message = JaxoInfo.sysInfo();
        String title = JaxoDialogs.windowTitle("System_Info");

        JaxoDialogs.showInfoDialog(this, message, title);
    }

    /** Opens a README file for Mac OS X usage in an internal browser. */
    private void macReadme() {
        JaxoPreview p = getPreview().copy();

        p.showURLInternally(
                Thread.currentThread().getContextClassLoader().getResource(
                "resources/MacIssues.html"));
    }
}
