/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.panel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.util.HashMap;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.jaxodraw.graph.JaxoGraph;
import net.sf.jaxodraw.gui.JaxoDialogs;
import net.sf.jaxodraw.gui.JaxoTab;
import net.sf.jaxodraw.gui.grid.JaxoDefaultGrid;
import net.sf.jaxodraw.gui.menu.popup.JaxoCanvasPopupMenu;
import net.sf.jaxodraw.gui.panel.button.JaxoButtonPanel;
import net.sf.jaxodraw.gui.swing.JaxoClosableTabbedPane;
import net.sf.jaxodraw.io.JaxoIO;
import net.sf.jaxodraw.io.JaxoPrint;
import net.sf.jaxodraw.plugin.JaxoImportPlugin;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoInfo;
import net.sf.jaxodraw.util.JaxoDictionary;
import net.sf.jaxodraw.util.JaxoLocalized;
import net.sf.jaxodraw.util.JaxoSoundsHandler;
import net.sf.jaxodraw.util.JaxoUtils;


/** A tabbed panel.
 * @since 2.0
 */
public class JaxoTabbedCanvasPanel extends MouseAdapter
    implements JaxoLocalized {
    private static final JaxoDictionary LANGUAGE
        = new JaxoDictionary(JaxoTabbedCanvasPanel.class);
    private final JaxoMainPanel thePanel;
    private final JaxoCanvasPopupMenu canvasPopup;
    private final JPanel root;
    private final JaxoIO io;
    private final JaxoClosableTabbedPane tabbedPane;
    private JaxoTab selectedTab;
    private final Icon modifiedIcon;
    private final Icon unmodifiedIcon;
    private final ChangeListener tabListener;
    private JaxoExportPanel exportPanel;
    private JaxoImportPanel importPanel;
    private final Map nameIndex = new HashMap();
    private final Map tabInfos = new HashMap();

    /** Constructor: adds MouseListener and ChangeListener to the tabbedPane.
     * @param panel Instance of the main panel
     */
    public JaxoTabbedCanvasPanel(JaxoMainPanel panel) {
        this.thePanel = panel;

        unmodifiedIcon = JaxoUtils.newImageIcon("unmodified.png");
        modifiedIcon = JaxoUtils.newImageIcon("modified.png");

        io = new JaxoIO(thePanel);

        this.root = new JPanel(new BorderLayout(), false);
        root.setBorder(JaxoButtonPanel.RAISED_ETCHED_BORDER);

        this.tabbedPane = new JaxoClosableTabbedPane();
        root.add(tabbedPane);

        this.canvasPopup = new JaxoCanvasPopupMenu(panel);

        tabbedPane.addClosingListener(new JaxoClosableTabbedPane.ClosingListener() {
                public void closing(JaxoClosableTabbedPane.ClosingEvent e) {
                    setSelectedTabIndex(e.getIndex());
                    JaxoTabbedCanvasPanel.this.closing();
                }
            });
        tabbedPane.addMouseListener(this);

        tabbedPane.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    updateSelectedTab();
                }
            });

        tabListener =
            new ChangeListener() {
                    public void stateChanged(ChangeEvent e) {
                        JaxoTab t = (JaxoTab) e.getSource();

                        updateSaveState(t);
                        thePanel.tabChanged(t);
                    }
                };
    }

    // instantiate lazily to improve start-up time
    private void initExportPanel() {
        this.exportPanel =
            new JaxoExportPanel(thePanel,
                thePanel.getPluginInfo().getExportFormats(),
                thePanel.getPreview());
    }

    private void initImportPanel() {
        this.importPanel = new JaxoImportPanel(
                thePanel, thePanel.getPluginInfo().getImportFormats());
    }

    /** Initialize tabs (show a default untitled tab if there is not). */
    public void initTabs() {
        initTabs("");
    }

    /**
     * Initialize tabs (open 'initialFile', or, if it fails, and there is
     * no tab, show a default untitled tab.
     * @param initialFileName The file to open.
     */
    public void initTabs(String initialFileName) {
        if (initialFileName.length() > 0) {
            open(initialFileName);
        }

        if (getTabCount() == 0) {
            aNewT();
        }
    }

    /**
     * Root component displaying current tabs and a way to
     * switch between them.
     * @return The root component.
     */
    public final JComponent getRoot() {
        return root;
    }

    //////////////////////////////////////////////////////////////////////
    // List-like interface for tabs (ad hoc)

    /**
     * Gets the current number of tabs.
     * @return The current number of tabs.
     */
    public final int getTabCount() {
        return tabbedPane.getTabCount();
    }

    /**
     * Returns the tab at given index.
     * @param index The index of tab to get.
     * @return The tab
     */
    public final JaxoTab getTabAt(int index) {
        return JaxoTab.asJaxoTab(tabbedPane.getComponentAt(index));
    }

    /**
     * Get the index of the given tab.
     * @param t The tab.
     * @return The index of the tab.
     */
    public int getTabIndex(JaxoTab t) {
        return tabbedPane.indexOfComponent(t.getRoot());
    }

    /**
     * Return the currently active tab.
     * @return The active tab.
     */
    public final JaxoTab getSelectedTab() {
        return JaxoTab.asJaxoTab(tabbedPane.getSelectedComponent());
    }

    /**
     * Returns the index of the currently active tab.
     * @return The index of the tab.
     */
    public final int getSelectedTabIndex() {
        return tabbedPane.getSelectedIndex();
    }

    /**
     * Sets the given tab as currently selected.
     * @param value True for selected.
     */
    public void setSelectedTab(JaxoTab value) {
        tabbedPane.setSelectedComponent((value == null) ? null : value.getRoot());
    }

    /**
     * Sets the tab at given index as currently active.
     * @param value The index.
     */
    public void setSelectedTabIndex(int value) {
        tabbedPane.setSelectedIndex(value);
    }

    /**
     * (An arbitrary) Tab with the given saveFileName, or 'null' if
     * none exists.
     * @param name The save file name of the tab.
     * @return The tab with given name, or 'null' if if name does not exist.
     */
    public final JaxoTab getTabWithSaveFileName(String name) {
        for (int i = getTabCount() - 1; i >= 0; --i) {
            if (getTabAt(i).getTabGraph().getSaveFileName().equals(name)) {
                return getTabAt(i);
            }
        }

        return null;
    }

    private void updateSaveState(JaxoTab t) {
        int index = getTabIndex(t);

        tabbedPane.setClosableIconAt(index,
            t.isSaved() ? unmodifiedIcon : modifiedIcon);
        tabbedPane.setFontStyleAt(index,
            (t.getSaveFileName().length() > 0) ? Font.BOLD : Font.PLAIN);
    }

    private void updateSelectedTab() {
        JaxoTab t = getSelectedTab();

        if (t != selectedTab) {
            JaxoTab old = selectedTab;

            if (selectedTab != null) {
                selectedTab.setCanvas(null);
            }

            selectedTab = t;

            if (selectedTab != null) {
                selectedTab.setCanvas(thePanel.getCanvas());
            }

            thePanel.updateSelectedTab(old, t);
        }
    }

    private TabInfo info(JaxoTab t) {
        return (TabInfo) tabInfos.get(t);
    }

    private String getName(JaxoTab t) {
        TabInfo n = info(t);

        return (n == null) ? null : n.getName();
    }

    private boolean isUntitled(JaxoTab t) {
        return info(t).isUntitled();
    }

    private void setUntitled(JaxoTab t, boolean value) {
        info(t).setUntitled(value);
    }

    private void setNameAndIndex(JaxoTab t, String name) {
        String old = getName(t);

        if ((old == null) ? (name == null) : old.equals(name)) {
            return;
        }

        TabInfo n = info(t);

        if ((n != null) && (n.getName() != null)) {
            int count = 0;
            int max = -1;

            for (int i = getTabCount() - 1; i >= 0; --i) {
                TabInfo n2 = info(getTabAt(i));

                if (n.getName().equals(n2.getName())) {
                    if (n2 != n) {
                        max = Math.max(max, n2.getIndex());
                    }

                    count++;
                }
            }

            if (count < 2) {
                nameIndex.remove(n.getName());
            } else {
                nameIndex.put(n.getName(), new Integer(max));
            }
        }

        Integer count;

        if (name == null) {
            count = new Integer(0);
        } else {
            count = (Integer) nameIndex.get(name);

            if (count == null) {
                count = new Integer(0);
            } else {
                count = new Integer(count.intValue() + 1);
            }

            nameIndex.put(name, count);
        }

        TabInfo nn = info(t);

        if (nn == null) {
            nn = new TabInfo();
            tabInfos.put(t, nn);
        }

        nn.setName(name);
        nn.setIndex(count.intValue());

        updateTabTitle(t);
    }

    // Update tab titled from name and index
    private void updateTabTitle(JaxoTab t) {
        TabInfo n = info(t);

        if (n.getIndex() > 0) {
            t.setTabTitle(LANGUAGE.message(
                    "tabTitle%0No%1", n.getName(),
                    new Integer(n.getIndex())));
        } else {
            t.setTabTitle(LANGUAGE.message(
                    "tabTitle%0", n.getName()));
        }
    }

    // Update
    private void pushTabTitle(JaxoTab t) {
        int index = getTabIndex(t);

        tabbedPane.setClosableTitleAt(index, t.getTabTitle());
    }

    /** {@inheritDoc} */
    public final void updateLanguage() {
        for (int i = getTabCount() - 1; i >= 0; --i) {
            JaxoTab t = getTabAt(i);

            TabInfo n = info(t);

            if (n.isUntitled()) {
                setNameAndIndex(t,
                    LANGUAGE.value("untitled"));
            } else {
                updateTabTitle(t);
            }

            tabbedPane.setClosableTitleAt(i, t.getTabTitle());
        }

        canvasPopup.updateLanguage();
        if (exportPanel != null) {
            exportPanel.updateLanguage();
        }
    }

    /** Executes the methods defined as file events in {@link JaxoConstants}.
     * @param i Integer specifying the file event.
     */
    public final void fileEvent(int i) {
        switch (i) {
            case JaxoConstants.NEWG:
                aNewG();
                break;
            case JaxoConstants.NEWT:
                aNewT();
                break;
            case JaxoConstants.OPEN:
                open();
                break;
            case JaxoConstants.IMPORT:
                importGraph();
                break;
            case JaxoConstants.CLOSE:
                closing();
                break;
            case JaxoConstants.SAVE:
                save();
                break;
            case JaxoConstants.SAVE_AS:
                saveAs();
                break;
            case JaxoConstants.SAVE_SELECTION_AS:
                save(true, thePanel.getCanvas().getClipboard(),
                    LANGUAGE.value("selection"));
                break;
            case JaxoConstants.EXPORT:
                export();
                break;
            case JaxoConstants.EXPORT_SELECTION:
                export(thePanel.getCanvas().getClipboard(),
                    LANGUAGE.value("selection"));
                break;
            case JaxoConstants.PRINT:
                print();
                break;
            case JaxoConstants.QUIT:
                quit();
                break;
            default:
                break;
        }
    }

    /** Brings up a dialog to rename the current tab. */
    public void renameTab() {
        JaxoTab t = getSelectedTab();

        String oldName = isUntitled(t) ? "" : getName(t);

        String newName = JaxoDialogs.getNewTabName(tabbedPane, oldName);

        if ((newName.length() != 0) && !(newName.equals(oldName))) {
            setUntitled(t, false);
            renameTab(t, newName);
        }
    }

    // Add tab 't' with desired 'name' (which will be indexed).
    // If reuse, close the current tab if unused (effectively reusing it)
    private void addTab(JaxoTab t, String name, boolean untitled, boolean use) {
        t.setTabMode(thePanel.getDefaultMode());

        boolean reuse = use;

        if (reuse) {
            reuse =
                (selectedTab != null) && !selectedTab.hasBeenUsed()
                && (
                    JaxoIO.shortName(selectedTab.getTabGraph().getSaveFileName())
                          .length() == 0
                );
        }

        JaxoTab oldSelectedTab = selectedTab;

        setNameAndIndex(t, name);
        setUntitled(t, untitled);

        t.addChangeListener(tabListener);

        tabbedPane.addClosableTab(t.getTabTitle(), t.getRoot());

        updateSaveState(t);

        setSelectedTab(t);

        if (reuse) {
            removeTab(oldSelectedTab);
        }
    }

    /** Rename the name of 't' to 'newName' (indexed). */
    private void renameTab(JaxoTab t, String newName) {
        setNameAndIndex(t, newName);

        pushTabTitle(t);
    }

    /** Starts a new graph.
     * If the current graph is not saved, a warning message is displayed.
     */
    private void aNewG() {
        JaxoTab t = getSelectedTab();

        if (!t.isSaved()) {
            Object[] options =
            {
                LANGUAGE.value(
                    "newGraph.continue"),
                LANGUAGE.value("newGraph.cancel")
            };

            int choice =
                showTabQuestionDialog("newGraph.title",
                    "newGraph.message%0", t, options);

            if (choice == 0) {
                t.newGraph();
            }
        }
    }

    /** Create a new canvas tab. */
    private void aNewT() {
        addTab(new JaxoTab(JaxoDefaultGrid.newDefaultGrid()),
            LANGUAGE.value("untitled"), true,
            false);
    }

    /** Brings up a file chooser dialog to open a file saved in a previous session. */
    private void open() {
        JaxoGraph newGraph = io.open();
        open(newGraph);
    }

    /**
     * Open given file in a new tab.
     * @param fileName The file to open.
     */
    public void open(String fileName) {
        open(io.open(fileName));
    }

    /**
     * Open given file (JaxoDraw or plugin based on extension) in a new tab.
     *
     * @param fileName The file to open or import.
     */
    public void openOrImport(String fileName) {
        if (JaxoIO.extension(fileName).equals(JaxoInfo.EXTENSION)) {
            open(io.open(fileName));
        } else {
            JaxoImportPlugin plugin =
                    thePanel.getPluginInfo().getImportForExtension(
                    JaxoIO.extension(fileName));
            if (plugin != null) {
                open(plugin.importGraph(fileName));
            }
        }
    }

    private void importGraph() {
        if (importPanel == null) {
            initImportPanel();
        } else {
            importPanel.setImports(thePanel.getPluginInfo().getImportFormats());
        }

        importPanel.show();

        open(importPanel.getImportedGraph());
    }

    /** Closes the current tab (removing it from the tab panel)
     * If the tab has been modified, ask the user whether to save
     * first or to cancel.
     */
    private void closing() {
        JaxoTab t = getSelectedTab();

        if (!t.isSaved()) {
            Object[] options =
            {
                LANGUAGE.value("close.save"),
                LANGUAGE.value("close.dontSave"),
                LANGUAGE.value("close.cancel")
            };

            int choice =
                showTabQuestionDialog("close.title",
                    "close.message%0", t, options);

            if (choice == 2) {
                return;
            }

            if (choice == 0) {
                save();

                if (!t.isSaved()) {
                    return;
                }
            }
        }

        close();
    }

    private void close() {
        JaxoTab t = getSelectedTab();

        if (getTabCount() == 1) {
            setNameAndIndex(t, null);
            aNewT();
        }

        removeTab(t);
    }

    private int showTabQuestionDialog(String titleKey, String messageKey,
        JaxoTab t, Object[] options) {
        return JOptionPane.showOptionDialog(thePanel,
            LANGUAGE.message(messageKey, t.getTabTitle()),
            JaxoDialogs.windowTitle(LANGUAGE, titleKey), JOptionPane.DEFAULT_OPTION,
            JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
    }

    /** Removes the given tab and refreshes the new one. */
    private void removeTab(JaxoTab t) {
        setNameAndIndex(t, null);
        tabInfos.remove(t);
        tabbedPane.remove(t.getRoot());

        t.removeChangeListener(tabListener);

        updateSelectedTab();
    }

    /** If the current JaxoGraph has no save file name associated to it, brings up a file
     * chooser dialog to save the current JaxoGraph to a file. Otherwise saves it to
     * the default file.
     */
    private void save() {
        save(false);
    }

    /** Brings up a file chooser dialog to save the current JaxoGraph to a file. */
    private void saveAs() {
        save(true);
    }

    private void save(boolean saveas) {
        boolean as = saveas;
        boolean suggestUntitled = true; // Suggest "Untitled" as file name?

        JaxoTab t = getSelectedTab();
        String saveFileName = t.getTabGraph().getSaveFileName();
        String shortName = JaxoIO.shortName(saveFileName);

        // If the tab name has been modified, always show save-as
        // for clarity
        if (isUntitled(t) || !JaxoIO.baseName(shortName).equals(getName(t))) {
            as = true;
        }

        if (as) {
            io.saveAs(t.getTabGraph(), t.getTabTitle(),
                (isUntitled(t) && !suggestUntitled) ? "" : getName(t));
        } else {
            io.save(t.getTabGraph(), t.getTabTitle(), shortName);
        }

        saveFileName = t.getTabGraph().getSaveFileName();

        t.setSaveFileName(saveFileName);

        shortName = JaxoIO.shortName(saveFileName);

        if (saveFileName.length() != 0) {
            thePanel.addRecentFile(saveFileName);

            setUntitled(t, false);

            renameTab(t, JaxoIO.baseName(shortName));

            t.setSaved(t.getTabGraph().isSaved());
        }
    }

    // Overlaoad the previous method with the graph that needs to be saved and title
    // (a selection of the current graph made by using the faint box)
    private void save(boolean as, JaxoGraph g, String title) {
        if (as) {
            io.saveAs(g, title, "");
        } else {
            io.save(g, title, "");
        }
    }

    /** Brings up a dialog that allows to export the current JaxoGraph to several file
     * formats: postscript portrait and landscape, EPS and LaTex are supported.
     */
    private void export() {
        JaxoTab t = getSelectedTab();
        export(t.getTabGraph(), t.getTabTitle());
    }

    // Overlaoad the previous method with the graph that needs to be saved and a title
    // (used explicitly for a selection of the current graph made by using the faint box)
    private void export(JaxoGraph g, String title) {
        if (exportPanel == null) {
            initExportPanel();
        }
        // TODO: watchFile mode should get dynamically set
        // this is not updated if the mode changed while the export panel is open
        exportPanel.setNewPreviewFrame(!thePanel.isWatchMode());
        exportPanel.setExports(thePanel.getPluginInfo().getExportFormats());
        exportPanel.export(g, thePanel.getCanvas().getCanvasSize(), title);
    }

    /** Brings up the standard Java print dialog to print the current JaxoGraph directly
     * to a printer or to a postscript file.
     */
    private void print() {
        Dimension d = thePanel.getCanvas().getCanvasSize();
        JaxoPrint p = new JaxoPrint();
        JaxoTab t = getSelectedTab();

        p.print(thePanel, t.getTabGraph(), d, t.getTabTitle());
    }

    /** Exits JaxoDraw, looping over unsaved tabs and asking for saving/discard changes*/
    private void quit() {
        boolean dontQuit = false;
        boolean saveAll = false;
        int unsavedCount = 0;

        for (int i = getTabCount() - 1; i > -1; i--) {
            JaxoTab t = getTabAt(i);

            if (!t.isSaved()) {
                unsavedCount++;
            }
        }

        Object[] options;

        if (unsavedCount > 1) {
            options =
                new Object[]{
                    LANGUAGE.value(
                        "quit.saveAll"),
                    LANGUAGE.value("quit.save"),
                    LANGUAGE.value(
                        "quit.dontSave"),
                    LANGUAGE.value("quit.cancel")
                };
        } else {
            options =
                new Object[]{
                    LANGUAGE.value("quit.save"),
                    LANGUAGE.value(
                        "quit.dontSave"),
                    LANGUAGE.value("quit.cancel")
                };
        }

        for (int i = getTabCount() - 1; (i >= 0) && !dontQuit; --i) {
            JaxoTab t = getTabAt(i);

            if (!t.isSaved()) {
                new Thread(new JaxoSoundsHandler(
                        "resources/sounds/unsaved_changes.wav")).start();
                setSelectedTabIndex(i);

                int choice;

                if (saveAll) {
                    choice = 1;
                } else {
                    choice =
                        showTabQuestionDialog("quit.title",
                            "quit.message%0", t, options);

                    if (options.length == 3) {
                        choice++;
                    }

                    if (choice == 0) {
                        saveAll = true;
                    }
                }

                switch (choice) {
                    case 0:
                    case 1:
                        save();
                        if (!t.isSaved()) {
                            return;
                        }
                        break;
                    case 2:
                        break;
                    case 3:
                        dontQuit = true;
                        break;
                    default:
                        break;
                }
            }
        }

        if (!dontQuit) {
            thePanel.shutdown(0);
        }
    }

    /**
     * Opens the new graph 'graph' (without opening the fileChooser dialog)
     * in a new tab, or in the current tab if it can be reused.
     *
     * @param graph The graph to open.
     */
    private void open(JaxoGraph graph) {
        if (graph == null) {
            return;
        }

        String shortName = JaxoIO.shortName(graph.getSaveFileName());

        addTab(new JaxoTab(graph, JaxoDefaultGrid.newDefaultGrid()),
            JaxoIO.baseName(shortName), false, true);

        thePanel.addRecentFile(graph.getSaveFileName());
    }

    /**
     * Processes the right-click to show the popup menu on the canvas tab.
     * @param e The mouse event.
     */
    public final void mousePressed(MouseEvent e) {
        if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) {
            // Make sure it is clear (both programmatically and for the
            // user) which tab the popup refers to
            int index = tabbedPane.indexAtLocation(e.getX(), e.getY());
            if (index != -1) {
                tabbedPane.setSelectedIndex(index);
            }
            canvasPopup.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    private static class TabInfo {
        private String name;
        private int index;
        private boolean untitled;

        /** Constructor: TabInfo with null name. */
        public TabInfo() {
            this(null);
        }

        /** Constructor.
         * @param nname The name of this TabInfo.
         */
        public TabInfo(String nname) {
            this.name = nname;
        }

        /** Returns the name of this TabInfo.
         * @return The name of this TabInfo.
         */
        public String getName() {
            return name;
        }

        /** Sets the name of this TabInfo.
         * @param nname The name of this TabInfo.
         */
        public void setName(String nname) {
            this.name = nname;
        }

        /** Returns the index of this TabInfo.
         * @return The index of this TabInfo.
         */
        public int getIndex() {
            return index;
        }

        /** Sets the index of this TabInfo.
         * @return The index of this TabInfo.
         */
        public void setIndex(int ind) {
            this.index = ind;
        }

        /** Determines whether this TabInfo is untitled.
         * @return True if this TabInfo is untitled.
         */
        public boolean isUntitled() {
            return untitled;
        }

        /** Sets this TabInfo as untitled.
         * @return True if this TabInfo should be untitled.
         */
        public void setUntitled(boolean ut) {
            this.untitled = ut;
        }
    }
}
