/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.panel;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Rectangle;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;

import java.net.URLDecoder;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.xml.transform.Source;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;

import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import net.sf.jaxodraw.gui.JaxoDialogs;
import net.sf.jaxodraw.io.JaxoPreview;
import net.sf.jaxodraw.util.JaxoInfo;
import net.sf.jaxodraw.util.JaxoDictionary;
import net.sf.jaxodraw.util.JaxoLog;

/** Responsible for creating and locating the User Guide. */
public class JaxoUserGuide {
    private static final JaxoDictionary LANGUAGE = new JaxoDictionary(JaxoUserGuide.class);

    private static final String USER_GUIDE_DIR = JaxoInfo.SETTINGS_DIR + "usrGuide/";
    private static final String WEB_URL = JaxoInfo.WEB_SITE + "userguide.html";


    private final File lockFile = new File(USER_GUIDE_DIR, ".lock");
    private final File indexFile = new File(USER_GUIDE_DIR, "index.html");

    private final Component parent;

    /**
     * Constructor that takes a parent component.
     * @param parentc The parent component for this UserGuide.
     */
    public JaxoUserGuide(Component parentc) {
        this.parent = parentc;
    }

    /**
     * Shows the User Guide in the given Preview.
     * @param p The preview to use.
     */
    public void show(JaxoPreview p) {
        URL indexPage = localIndexPage();

        if (indexPage == null) {
            String msg = LANGUAGE.message("installErrorVisit%0", WEB_URL);
            JaxoDialogs.showErrorDialog(parent, msg);
        } else {
            p.setParentComponent(parent);
            p.browseURL(indexPage);
        }
    }

    /**
     * Checks if the User Guide exists locally and if not, tries to re-create it.
     * @return True if the local version of the User Guide was created
     * without errors, and is therefore browsable.
     */
    public boolean isLocallyBrowsable() {
        boolean ready = false;
        File userGuideDir = new File(USER_GUIDE_DIR);

        if (userGuideDir.isDirectory()) {
            // this implies that it exists
            if (lockFile.exists()) {
                // there was a problem last time, try to re-create
                JaxoLog.debug("Lock file detected, re-creating User Guide!");
                deleteDir(userGuideDir);
                ready = create(userGuideDir);
            } else {
                if (indexFile.exists()) {
                    ready = true;
                } else {
                    // directory is there but no index file, re-create everything
                    JaxoLog.debug("No index file found, re-creating User Guide!");
                    deleteDir(userGuideDir);
                    ready = create(userGuideDir);
                }
            }
        } else {
            // create
            ready = create(userGuideDir);
        }

        return ready;
    }

    /**
     * Returns a URL to the index page of the local copy of the User Guide.
     * @return Null if isLocallyBrowsable() returns false, or if another
     * error occurred; the local URL otherwise.
     */
    public URL localIndexPage() {
        URL indexPage = null;

        try {
            if (isLocallyBrowsable()) {
                indexPage = indexFile.toURI().toURL();
            }
        } catch (MalformedURLException e) {
            JaxoLog.debug(e);
        }

        return indexPage;
    }

      //
     // private
    //

    private boolean create(File targetDir) {
        BuildProgressDialog progress = new BuildProgressDialog(parent, 0, 100);
        progress.setValue(0);
        progress.pack();
        progress.setVisible(true);

        String sourceDirName = "resources/usrGuide/xdocs/";

        if (!prepareTargetDir(targetDir)) {
            return false;
        }

        int value = 10;
        progress.setValue(value);

        File sourceDir = new File(Thread.currentThread().
                getContextClassLoader().getResource(sourceDirName).getFile());
        String sourceDirPath = sourceDir.getPath();

        // handle whitespace on windows
        try {
            sourceDirPath = URLDecoder.decode(sourceDirPath, "utf-8");
        } catch (UnsupportedEncodingException e) {
            // cannot happen
            JaxoLog.debug("utf-8 not supported?", e);
        }

        int bang = sourceDirPath.indexOf(".jar!");
        boolean inJar = (bang != -1);

        if (inJar) {
            // we're inside a jar, copy source to the file system
            // path is of the form "file:path_to_jarfile.jar!path_do_file"
            String jarName = sourceDirPath.substring(5, bang + 4);
            sourceDir = new File(targetDir, "src/");
            sourceDirPath = sourceDir.getPath();

            try {
                copyFolderFromJar(jarName, sourceDirName, sourceDir);
            } catch (IOException e) {
                JaxoLog.debug(e);
                return false;
            }
        }

        File xslFile = new File(sourceDir, "style/style.xsl");

        File[] xmlFiles = sourceDir.listFiles();

        if (xmlFiles == null) {
            JaxoLog.debug("No source files found to transform User Guide!");
            JaxoLog.debug("Not found: " + sourceDirPath);
            return false;
        }

        int nof = xmlFiles.length;
        int step = Math.round(90.f / nof);

        String outFileName;

        try {
            for (int j = 0; j < nof; j++) {
                File xmlFile = xmlFiles[j];
                String filename = xmlFile.getName();
                String ext = "";
                String base = "";
                int i = filename.lastIndexOf('.');

                if ((i > 0) && (i < (filename.length() - 1))) {
                    ext = filename.substring(i + 1).toLowerCase();
                    base = filename.substring(0, i);
                }

                if (!"navigation.xml".equals(filename)
                        && !"pdf-navigation.xml".equals(filename)) {
                    if ("xml".equals(ext)) {
                        outFileName = USER_GUIDE_DIR.concat(base) + ".html";
                        xsl(xmlFile, new File(outFileName), xslFile);
                    } else {
                        outFileName = USER_GUIDE_DIR.concat(filename);
                        copyDirectory(new File(sourceDir, filename), new File(outFileName));
                    }
                }

                value = value + step;
                progress.setValue(value);
            }
        } catch (TransformerException e) {
            JaxoLog.debug(e);
            SourceLocator locator = e.getLocator();
            JaxoLog.debug("column: " + locator.getColumnNumber());
            JaxoLog.debug("line: " + locator.getLineNumber());
            JaxoLog.debug("publicId: " + locator.getPublicId());
            JaxoLog.debug("systemId: " + locator.getSystemId());
            return false;
        } catch (IOException e) {
            JaxoLog.debug(e);
            return false;
        }

        if (inJar && !deleteDir(sourceDir)) {
            // cannot delete temporary file but UG was created: ignore
            JaxoLog.debug("Could not delete temporary file: "
                + sourceDir.getAbsolutePath());
        }

        progress.setValue(100);

        lockFile.delete();

        progress.dispose();

        return true;
    }

    private void xsl(File inFile, File outFile, File xslFile)
        throws IOException, TransformerException {

        InputStream is = null;
        OutputStream os = null;
        InputStream xis = null;

        try {
            is = new FileInputStream(inFile);
            os = new FileOutputStream(outFile);
            xis = new FileInputStream(xslFile);

            // baseDir for xsl:include, xsl:import, etc.
            String baseDir = xslFile.getParent();
            MyResolver resolver = new MyResolver(baseDir);

            TransformerFactory factory = TransformerFactory.newInstance();
            Templates template = factory.newTemplates(new StreamSource(xis));
            Transformer xformer = template.newTransformer();

            xformer.setURIResolver(resolver);
            xformer.setParameter("JDtitle", "JaxoDraw");
            xformer.setParameter("JDversion", JaxoInfo.VERSION_NUMBER);
            xformer.setOutputProperty("doctype-public",
                "-//W3C//DTD XHTML 1.0 Transitional//EN");
            xformer.setOutputProperty("doctype-system",
                "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd");

            xformer.transform(new StreamSource(is), new StreamResult(os));
        } finally {
            if (is != null) {
                is.close();
            }
            if (os != null) {
                os.close();
            }
            if (xis != null) {
                xis.close();
            }
        }
    }

    private boolean prepareTargetDir(File targetDir) {
        boolean succeed = false;
        try {
            if (targetDir.exists()) {
                JaxoLog.debug("Directory " + targetDir + " will be recreated!");
                succeed = deleteDir(targetDir);
                if (!succeed) {
                    JaxoLog.debug(targetDir + " could not be deleted!");
                    return false;
                }
            }

            succeed = targetDir.mkdirs();
            if (!succeed) {
                JaxoLog.debug(targetDir + " could not be created!");
                return false;
            }

            succeed = lockFile.createNewFile();
            if (!succeed) {
                JaxoLog.debug("Lock file could not be created!");
                return false;
            }
        } catch (SecurityException e) {
            JaxoLog.debug(e);
            return false;
        } catch (IOException e) {
            JaxoLog.debug(e);
            return false;
        }

        return true;
    }

    private void copyDirectory(File srcDir, File dstDir)
        throws IOException {
        if (srcDir.isDirectory()) {
            if (!dstDir.exists()) {
                dstDir.mkdir();
            }

            String[] children = srcDir.list();
            for (int i = 0; i < children.length; i++) {
                copyDirectory(new File(srcDir, children[i]),
                    new File(dstDir, children[i]));
            }
        } else {
            copyFile(srcDir, dstDir);
        }
    }

    private void copyFile(File src, File dst) throws IOException {
        InputStream in = null;
        OutputStream out = null;
        try {
            in = new FileInputStream(src);
            out = new FileOutputStream(dst);
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
        } finally {
            if (in != null) {
                in.close();
            }
            if (out != null) {
                out.close();
            }
        }
    }

    private boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        return dir.delete();
    }

    private boolean copyFolderFromJar(String jarName, String folderName, File fDest)
        throws IOException {

        JarFile z = new JarFile(jarName);
        Enumeration entries = z.entries();

        while (entries.hasMoreElements()) {
            JarEntry entry = (JarEntry) entries.nextElement();
            String entryName = entry.getName();

            if (entryName.startsWith(folderName)) {
                File f = new File(fDest,
                    entryName.substring(folderName.length(), entryName.length()));

                if (entry.isDirectory()) {
                    if (!f.mkdirs()) {
                        JaxoLog.debug("could not create directory: "
                            + f.getAbsolutePath());
                        return false;
                    }
                } else if (!f.exists() && !copyFileFromJar(entryName, f)) {
                    JaxoLog.debug("copyFileFromJar failed!!");
                    return false;
                }
            }
        }

        return true;
    }

    private boolean copyFileFromJar(String sResource, File fDest)
        throws IOException {

        if (sResource == null || fDest == null) {
            return false;
        }

        InputStream sIn = null;
        OutputStream sOut = null;

        try {
            int nLen = 0;
            sIn = Thread.currentThread().getContextClassLoader().
                    getResourceAsStream(sResource);

            if (sIn == null) {
                throw new IOException("Error copying from jar"
                + "(" + sResource + " to " + fDest.getPath() + ")");
            }

            sOut = new FileOutputStream(fDest);
            byte[] bBuffer = new byte[1024];

            while ((nLen = sIn.read(bBuffer)) > 0) {
                sOut.write(bBuffer, 0, nLen);
            }

            sOut.flush();
        } finally {
            if (sIn != null) {
                sIn.close();
            }
            if (sOut != null) {
                sOut.close();
            }
        }

        return fDest.exists();
    }

    private static class MyResolver implements URIResolver {
        private final String basePath;

        public MyResolver(String path) {
            this.basePath = path;
        }

        public Source resolve(String href, String base) {
            StringBuffer path = new StringBuffer(this.basePath);
            path.append(File.separator).append(href);

            File file = new File(path.toString());

            if (file.exists()) {
                return new StreamSource(file);
            }

            return null;
        }
    }

    private static class BuildProgressDialog extends JFrame {
        private static final long serialVersionUID = 7526471155622776147L;
        private final JProgressBar progressBar;
        private final JPanel contentPane = new JPanel();

        public BuildProgressDialog(Component parent, int min, int max) {
            setUndecorated(true);
            setLocationRelativeTo(parent);

            this.progressBar = new JProgressBar(min, max);

            JLabel label = new JLabel(LANGUAGE.value("creatingUserGuide"));

            contentPane.setLayout(new BorderLayout());
            contentPane.add(label, BorderLayout.NORTH);
            contentPane.add(progressBar, BorderLayout.SOUTH);
            contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

            this.setContentPane(contentPane);
        }

        public void setValue(int value) {
            progressBar.setValue(value);

            Rectangle rect = this.getBounds();
            rect.x = 0;
            rect.y = 0;
            contentPane.paintImmediately(rect);
        }
    }

}
