/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.panel.edit;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;

import net.sf.jaxodraw.gui.swing.spinner.JaxoFixedJSpinner;


/** A panel to change the x,y and radius parameters of an object.
 * @since 2.0
 */
public class Jaxo3PointsEditPanel extends JaxoEditPanel {
    private static final long serialVersionUID = 7526471155622776147L;
    private transient JSpinner spx1;
    private transient JSpinner spx2;
    private transient JSpinner spx3;
    private transient JSpinner spy1;
    private transient JSpinner spy2;
    private transient JSpinner spy3;

    /** Constructor: returns a 3Points edit panel with
     * default value 0 for all variables.
     */
    public Jaxo3PointsEditPanel() {
        Point p = new Point();
        init(new Point[] {p, p, p});
    }

    /** Constructor: returns a 3Points edit panel with
     * given values for the variables.
     * @param points The initial points for this panel.
     */
    public Jaxo3PointsEditPanel(Point[] points) {
        init(points);
    }

    /** {@inheritDoc} */
    public void addEditPanelListener(JaxoEditPanelListener listener) {
        if (listener != null) {
            spx1.addChangeListener(listener);
            spx2.addChangeListener(listener);
            spx3.addChangeListener(listener);
            spy1.addChangeListener(listener);
            spy2.addChangeListener(listener);
            spy3.addChangeListener(listener);
        }
    }

      //
     // private
    //

    private void init(Point[] p) {
        // Dimension options
        Dimension size = new Dimension(120, 20);
        GridLayout layout = new GridLayout(0, 2);

        JPanel dimSPanel = new JPanel();
        GridBagLayout gb = new GridBagLayout();
        GridBagConstraints sc = new GridBagConstraints();
        dimSPanel.setLayout(gb);

        sc.gridx = 0;
        sc.gridy = 0;
        dimSPanel.add(getPx1Panel(p, size, layout), sc);

        sc.gridx = 0;
        sc.gridy = 1;
        sc.insets = new Insets(3, 0, 3, 0);
        dimSPanel.add(getPy1Panel(p, size, layout), sc);

        sc.gridx = 1;
        sc.gridy = 0;
        dimSPanel.add(getPx2Panel(p, size, layout), sc);

        sc.gridx = 1;
        sc.gridy = 1;
        sc.insets = new Insets(3, 0, 3, 0);
        dimSPanel.add(getPy2Panel(p, size, layout), sc);

        sc.gridx = 2;
        sc.gridy = 0;
        dimSPanel.add(getPx3Panel(p, size, layout), sc);

        sc.gridx = 2;
        sc.gridy = 1;
        sc.insets = new Insets(3, 0, 3, 0);
        dimSPanel.add(getPy3Panel(p, size, layout), sc);

        setLineBoxLayout();

        TitledBorder dimSTitle =
            createI18NBorder("Jaxo3PointsEditPanel.title");

        this.setBorder(dimSTitle);
        this.add(dimSPanel);

    }

    private JPanel getPx1Panel(Point[] p, Dimension size, GridLayout layout) {
        JPanel px1 = new JPanel();
        px1.setLayout(layout);

        SpinnerNumberModel modelX1 = new SpinnerNumberModel();
        modelX1.setValue(new Integer(p[0].x));

        JLabel lx1 = createLabel("X1: ");
        px1.add(lx1);

        this.spx1 = new JaxoFixedJSpinner(modelX1);
        spx1.setName("spx");
        px1.add(spx1);
        px1.setPreferredSize(size);

        return px1;
    }

    private JPanel getPx2Panel(Point[] p, Dimension size, GridLayout layout) {
        JPanel px2 = new JPanel();
        px2.setLayout(layout);

        SpinnerNumberModel modelX2 = new SpinnerNumberModel();
        modelX2.setValue(new Integer(p[1].x));
        JLabel lx2 = createLabel("X2: ");
        px2.add(lx2);

        this.spx2 = new JaxoFixedJSpinner(modelX2);
        spx2.setName("spx2");
        px2.add(spx2);
        px2.setPreferredSize(size);

        return px2;
    }

    private JPanel getPx3Panel(Point[] p, Dimension size, GridLayout layout) {
        JPanel px3 = new JPanel();
        px3.setLayout(layout);

        SpinnerNumberModel modelX3 = new SpinnerNumberModel();
        modelX3.setValue(new Integer(p[2].x));
        JLabel lx3 = createLabel("X3: ");
        px3.add(lx3);

        this.spx3 = new JaxoFixedJSpinner(modelX3);
        spx3.setName("spx3");
        px3.add(spx3);
        px3.setPreferredSize(size);

        return px3;
    }

    private JPanel getPy1Panel(Point[] p, Dimension size, GridLayout layout) {
        JPanel py1 = new JPanel();
        py1.setLayout(layout);

        SpinnerNumberModel modelY1 = new SpinnerNumberModel();
        modelY1.setValue(new Integer(p[0].y));
        JLabel ly1 = createLabel("Y1: ");
        py1.add(ly1);

        this.spy1 = new JaxoFixedJSpinner(modelY1);
        spy1.setName("spy");
        py1.add(spy1);
        py1.setPreferredSize(size);

        return py1;
    }

    private JPanel getPy2Panel(Point[] p, Dimension size, GridLayout layout) {
        JPanel py2 = new JPanel();
        py2.setLayout(layout);

        SpinnerNumberModel modelY2 = new SpinnerNumberModel();
        modelY2.setValue(new Integer(p[1].y));
        JLabel ly2 = createLabel("Y2: ");
        py2.add(ly2);

        this.spy2 = new JaxoFixedJSpinner(modelY2);
        spy2.setName("spy2");
        py2.add(spy2);
        py2.setPreferredSize(size);

        return py2;
    }

    private JPanel getPy3Panel(Point[] p, Dimension size, GridLayout layout) {
        JPanel py3 = new JPanel();
        py3.setLayout(layout);

        SpinnerNumberModel modelY3 = new SpinnerNumberModel();
        modelY3.setValue(new Integer(p[2].y));
        JLabel ly3 = createLabel("Y3: ");
        py3.add(ly3);

        this.spy3 = new JaxoFixedJSpinner(modelY3);
        spy3.setName("spy3");
        py3.add(spy3);
        py3.setPreferredSize(size);

        return py3;
    }
}
