/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.panel.edit;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.lang.ref.WeakReference;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.jaxodraw.object.JaxoArrow;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;

import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.JaxoLanguage;
import net.sf.jaxodraw.util.JaxoPrefs;
import net.sf.jaxodraw.util.JaxoUtils;
import net.sf.jaxodraw.util.Location;


/** A parent class for all option panels.
 * @since 2.0
 */
public class JaxoOptionsPanel extends JPanel implements JaxoObjectEditPanel {
    private static List rotationAngles;

    private static final long serialVersionUID = 7526471155622776147L;

    static {
        rotationAngles = new ArrayList(4);
        rotationAngles.add(new Integer(-135));
        rotationAngles.add(new Integer(-90));
        rotationAngles.add(new Integer(-45));
        rotationAngles.add(new Integer(0));
        rotationAngles.add(new Integer(45));
        rotationAngles.add(new Integer(90));
        rotationAngles.add(new Integer(135));
        rotationAngles.add(new Integer(180));

        rotationAngles = Collections.unmodifiableList(rotationAngles);
    }

    private final GridBagConstraints panelConstraints;
    private JaxoEditPanelListener internalListener;
    private JaxoObject object;
    private JaxoObject backup;
    private String title;
    private Icon icon;
    private ChangeEvent event;

    /**
     * Constructor: initializes the panel.
     *
     * @param ob The object to be edited by this panel.
     */
    public JaxoOptionsPanel(JaxoObject ob) {
        super(new GridBagLayout());
        panelConstraints = new GridBagConstraints();

        panelConstraints.insets = new Insets(5, 5, 5, 5);

        setObject(ob);

//      Not yet: The panels have to be checked whether they support it properly
//        panelConstraints.weightx = 1;
//        panelConstraints.weighty = 1;
//        panelConstraints.fill = GridBagConstraints.BOTH;
    }

    /** List of numbers of default rotation angles to display to the user.
     * @return List
     */
    public static List getRotationAngleDefaults() {
        return rotationAngles;
    }

    /** {@inheritDoc} */
    public void addChangeListener(ChangeListener l) {
        listenerList.add(ChangeListener.class, l);
    }

    /**
     * Removes a change listener.
     * @param l The listener to remove.
     */
    public void removeChangeListener(ChangeListener l) {
        listenerList.remove(ChangeListener.class, l);
    }

    /**
     * Notifies all listeners of a state change.
     */
    protected void fireStateChanged() {
        Object[] pairs = listenerList.getListenerList();

        for (int i = pairs.length - 2; i >= 0; i -= 2) {
            if (pairs[i] == ChangeListener.class) {
                if (event == null) {
                    event = new ChangeEvent(this);
                }
                ((ChangeListener) pairs[i + 1]).stateChanged(event);
            }
        }
    }

    /** Set the object that is being edited.
     * @param value the object that is being edited.
     */
    public final void setObject(JaxoObject value) {
        if (value != object) {
            object = value;
            backup = object.copy();

            internalListener =
                new JaxoOptionsPanelListener(object,
                    new ChangeListener() {
                        public void stateChanged(ChangeEvent e) {
                            fireStateChanged();
                        }
                    });

            updatePanels();
        }
    }

    /** The object that is being edited.
     * @return the currently edited object.
     */
    public final JaxoObject getObject() {
        return object;
    }

    /** {@inheritDoc} */
    public final void add3PointsPanel(Point[] points, int gridx, int gridy) {
        add3PointsPanel(points, gridx, gridy, 1);
    }

    /** {@inheritDoc} */
    public final void add3PointsPanel(Point[] points, int gridx, int gridy,
        int gridwidth) {
        Jaxo3PointsEditPanel panel = new Jaxo3PointsEditPanel(points);
        addPanel(panel, gridx, gridy, gridwidth);
    }

    /** {@inheritDoc} */
    public final void add4PointsPanel(Point[] points, int gridx, int gridy) {
        add4PointsPanel(points, gridx, gridy, 1);
    }

    /** {@inheritDoc} */
    public final void add4PointsPanel(Point[] points, int gridx,
        int gridy, int gridwidth) {
        Jaxo4PointsEditPanel panel = new Jaxo4PointsEditPanel(points);
        addPanel(panel, gridx, gridy, gridwidth);
    }

    /** {@inheritDoc} */
    public final void addPositionPanel(int x, int y, int gridx, int gridy) {
        JaxoPositionEditPanel panel = new JaxoPositionEditPanel(x, y);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void add2PointsPanel(Point[] points, int gridx, int gridy) {
        Jaxo2PointsEditPanel panel = new Jaxo2PointsEditPanel(points);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addXYRPanel(int x, int y, int r, int gridx, int gridy) {
        JaxoXYREditPanel panel = new JaxoXYREditPanel(x, y, r);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addCenterSizePanel(int x, int y, int w, int h, int gridx,
        int gridy) {
        JaxoCenterSizeEditPanel panel = new JaxoCenterSizeEditPanel(x, y, w, h);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addDashPanel(float dash, int gridx, int gridy) {
        JaxoDashEditPanel panel = new JaxoDashEditPanel(dash);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addStrokePanel(float width, int gridx, int gridy) {
        JaxoStrokeEditPanel panel = new JaxoStrokeEditPanel(width);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addReScalePanel(int gridx, int gridy) {
        JaxoScaleEditPanel finalScalePanel = new JaxoScaleEditPanel();
        addPanel(finalScalePanel, gridx, gridy);
    }

    /** {@inheritDoc} */
   public final void addWigglePanel(int amp, int wiggles, int gridx, int gridy) {
        JaxoWiggleEditPanel panel = new JaxoWiggleEditPanel(amp, wiggles);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addWigglePanel(int amp, int gridx, int gridy) {
        JaxoWiggleEditPanel wigglePanel = new JaxoWiggleEditPanel(amp);
        addPanel(wigglePanel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addSymmPanel(boolean symm, int gridx, int gridy) {
        JaxoSymmEditPanel symPanel = new JaxoSymmEditPanel(symm);
        addPanel(symPanel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addStretchingPanel(boolean noFreqStretching, int gridx, int gridy) {
        JaxoStretchingEditPanel panel = new JaxoStretchingEditPanel(noFreqStretching);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addRotationPanel(int angle, int gridx, int gridy) {
        JaxoRotationEditPanel panel = new JaxoRotationEditPanel(angle);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addArrowPanel(boolean arrow, boolean flip, float oldPosition,
        int gridx, int gridy) {
        JaxoArrowEditPanel arrowPanel = new JaxoArrowEditPanel(arrow, flip, oldPosition);
        addPanel(arrowPanel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addArrowOptionsEditPanel(JaxoArrow arrow, int gridx,
        int gridy) {
        addPanel(arrow.getEditPanel(), gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addDoubleLinePanel(boolean dLine, float dlSep, int gridx,
        int gridy) {
        JaxoDLEditPanel dlPanel = new JaxoDLEditPanel(dLine, dlSep);
        addPanel(dlPanel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addLineColorPanel(Color color, int gridx, int gridy) {
        int type = TYPE_LINE_COLOR;
        JaxoColorEditPanel panel = new JaxoColorEditPanel(type, false, color);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addFillLineColorPanels(Color color, final Color fillColor,
        boolean isFilled, int fillX, int fillY, int lineX, int lineY) {
        boolean disable = JaxoColor.isGrayScale(fillColor);

        JaxoColorEditPanel linePanel =
            new JaxoColorEditPanel(TYPE_LINE_COLOR, disable, color);

        addPanel(linePanel, lineX, lineY);

        JaxoColorEditPanel fillPanel =
            new JaxoColorEditPanel(false, fillColor, isFilled);

        addPanel(fillPanel, fillX, fillY);

        final WeakReference weakLinePanel = new WeakReference(linePanel);
        final WeakReference weakFillPanel = new WeakReference(fillPanel);

        addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JaxoColorEditPanel l =
                    (JaxoColorEditPanel) weakLinePanel.get();
                JaxoColorEditPanel f =
                    (JaxoColorEditPanel) weakFillPanel.get();

                if (l == null || f == null) {
                    removeChangeListener(this);
                } else {
                    if (JaxoPrefs.getIntPref(JaxoPrefs.PREF_COLORSPACE)
                    == JaxoColor.ALL_COLORS_MODE) {
                        l.setEnabled(true);
                    } else {
                        boolean isGrayScale = JaxoColor.isGrayScale(f.getColor());

                        if (isGrayScale) {
                            l.setColor(JaxoColor.BLACK);
                        }
                        l.setEnabled(!isGrayScale);
                    }
                }
            }
        });
    }

    /** {@inheritDoc} */
    public final void addColorPanel(Color color, int type, int gridx, int gridy) {
        JaxoColorEditPanel colorPanel =
                new JaxoColorEditPanel(type, false, color);
        addPanel(colorPanel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addTextPanel(String text, int gridx, int gridy) {
        JaxoTextEditPanel textPanel = new JaxoTextEditPanel(text);
        addPanel(textPanel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addPSFontPanel(Font font, int gridx, int gridy,
        int gridwidth) {
        JaxoPSFontPanel fontPanel = new JaxoPSFontPanel(font);
        addPanel(fontPanel, gridx, gridy, gridwidth);
    }

    /** {@inheritDoc} */
    public final void addLatexAlignPanel(int allign, int gridx, int gridy) {
        JaxoLatexAlignPanel panel = new JaxoLatexAlignPanel(allign);
        addPanel(panel, gridx, gridy);
    }

    /** {@inheritDoc} */
    public final void addLatexFontSizePanel(int size, int gridx, int gridy) {
        JaxoLatexFontSizePanel panel = new JaxoLatexFontSizePanel(size);
        addPanel(panel, gridx, gridy);
    }

/////////////////////////////////////////////////////////

    /** Adds the current listeners to the given subpanel
     * and adds it at the given position.
     * @param panel The panel to add.
     * @param gridx The x position of the subpanel.
     * @param gridy The y position of the subpanel.
     */
    private void addPanel(JaxoEditPanel panel, int gridx, int gridy) {
        addPanel(panel, gridx, gridy, 1);
    }

    /**
     * Adds the current listeners to the given subpanel
     * and adds it at the given position. If panel is null, does nothing.
     *
     * @param panel The panel to add.
     * @param gridx The x position of the subpanel.
     * @param gridy The y position of the subpanel.
     * @param gridwidth The width of the subpanel.
     */
    private void addPanel(JaxoEditPanel panel, int gridx, int gridy,
        int gridwidth) {

        if (panel == null) {
            return;
        }

        panel.addEditPanelListener(internalListener);

        panelConstraints.gridx = gridx;
        panelConstraints.gridy = gridy;
        panelConstraints.gridwidth = gridwidth;
        add(panel, panelConstraints);
    }

    /** {@inheritDoc} */
    public final void show(Component parent, Location l) {
        showDialog(parent, l);
    }

    /** {@inheritDoc} */
    public boolean hasChanged() {
        return (backup != null) && !backup.isCopy(object);
    }

    /** {@inheritDoc} */
    public void setTitleAndIcon(String newTitle, String newIcon) {
        this.title = JaxoLanguage.translate(newTitle);
        this.icon = JaxoUtils.newImageIcon(newIcon);
    }

    /** Reset 'object' (and GUI) to initial value. */
    protected void resetObject() {
        object.setState(backup);
        updatePanels();
    }

    private void updatePanels() {
        removeAll();
        object.prepareEditPanel(this);
        revalidate();
        repaint();
        fireStateChanged();
    }

    /**
     * Show a dialog for this panel.
     * The current implementation blocks until the dialog is closed again.
     *
     * @param parent the parent component.
     * @param l the location.
     */
    protected final void showDialog(Component parent, Location l) {
        JButton reset = new JButton(JaxoLanguage.translate("Reset"));

        reset.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    resetObject();
                }
            });

        final Object[] optionButtons = {
            JaxoLanguage.translate("Accept"),
            reset,
            JaxoLanguage.translate("Cancel")
        };

        final JOptionPane p =
            new JOptionPane(this, JOptionPane.PLAIN_MESSAGE,
                JOptionPane.OK_CANCEL_OPTION, icon, optionButtons,
                optionButtons[0]);

        final JDialog d = p.createDialog(parent, title);

        l.setLocation(d);
        d.setVisible(true);
        d.dispose();

        if (!optionButtons[0].equals(p.getValue())) {
            object.setState(backup);
            fireStateChanged();
        }
    }
}
