/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.gui.swing;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;

import java.util.EventListener;
import java.util.EventObject;

import javax.swing.DefaultButtonModel;
import javax.swing.Icon;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;

import net.sf.jaxodraw.util.JaxoUtils;



/** Extension of JTabbedPane with closable tabs.
 *
 * USAGE:
 * <ul>
 * <li>Only use addClosableTab (not addTab, insertTab)
 * <li>Do not use setIconAt, setDisabledIconAt, setTitleAt, but new methds
 * with "closable" in the name. For a disabled icon, use a custom icon that
 * knows how to paint itself disabled.
 * <li>getIconAt, getDisabledIconAt, getTitleAt, are not meaningful.
 * </ul>
 * For general-purpose use, this implementation lacks support for disabled state
 * both of tabs and of the tab pane), support for RTL orientation and
 * improved accessibility code (showing the
 * titles as accessible names). Neither is of importance for JaxoDraw at the moment.
 * @since 2.0
 */
// GTK: does not paint decent focus rectangle - its own fault
public class JaxoClosableTabbedPane extends JTabbedPane {
    private static final long serialVersionUID = 7526471155622776147L;
    // Some insets are needed - otherwise the focus border may be too close
    // to the close button, which looks ugly
    private static final Insets TAB_INSETS = new Insets(3, 0, 3, 0);
    private final IndexButtonModel button;
    private final IndexButtonModel tabRollover;
    private final CloseButtonListener closeButtonListener;

    // all of these icons must have the same size
    private final Icon defaultIcon; // default
    private final Icon selectedIcon; // selected tab (unless rollover or pressed)
    private final Icon rolloverIcon; // rollover (even for unselected tab), unless pressed
    private final Icon pressedIcon; // pressed (but not armed)
    private final Icon pressedArmedIcon; // pressed (and armed)

    /**
     * Constructor.
     */
    public JaxoClosableTabbedPane() {
        defaultIcon = JaxoUtils.newImageIcon("closabletab_notselected.png");
        selectedIcon = JaxoUtils.newImageIcon("closabletab_selected.png");
        rolloverIcon = JaxoUtils.newImageIcon("closabletab_close.png");
        pressedIcon = JaxoUtils.newImageIcon("closabletab_selected.png");
        pressedArmedIcon = JaxoUtils.newImageIcon("closabletab_close.png");

        tabRollover = new IndexButtonModel();
        button = new IndexButtonModel();

        closeButtonListener = new CloseButtonListener();

        // Do NOT enable scroll mode
        addMouseListener(closeButtonListener);
        addMouseMotionListener(closeButtonListener);
    }

    /** {@inheritDoc} */
    public void addNotify() {
        super.addNotify();

        Window w = SwingUtilities.getWindowAncestor(this);

        if (w != null) {
            w.addWindowFocusListener(closeButtonListener);
        }
    }

    /** {@inheritDoc} */
    public void removeNotify() {
        super.removeNotify();

        Window w = SwingUtilities.getWindowAncestor(this);

        if (w != null) {
            w.removeWindowFocusListener(closeButtonListener);
        }

        closeButtonListener.reset();
    }

    /** {@inheritDoc} */
    public void updateUI() {
        super.updateUI();

        // just in case
        updateAllCombinedIcons();
    }

    /** {@inheritDoc} */
    public void setFont(Font font) {
        boolean changed = font != getFont();

        super.setFont(font);

        if (changed) {
            updateAllCombinedIcons();
        }
    }

    /**
     * {@inheritDoc} DO NOT USE. Use any of the addClosableTab methods.
     */
    public void insertTab(String title, Icon icon, Component component, String tip,
        int index) {
        super.insertTab(title, icon, component, tip, index);

        tabAdded(index);
    }

    /** {@inheritDoc} */
    public void removeTabAt(int index) {
        super.removeTabAt(index);

        tabRemoved(index);
    }

    // Icon size may change, e.g., if the font is changed
    private void updateAllCombinedIcons() {
        for (int i = getTabCount() - 1; i >= 0; --i) {
            CombinedIcon old = getCombinedIconAt(i);

            setIconAt(i, (Icon) old.copy());
        }
    }

    private CombinedIcon getCombinedIconAt(int index) {
        return (CombinedIcon) getIconAt(index);
    }

    /**
     * Is the tab at 'index' closable by the user?
     * @param index The index of the tab to check.
     * @return True if the tab is closeable.
     */
    public boolean isClosableAt(int index) {
        return getCombinedIconAt(index).isClosable();
    }

    /**
     * Sets the tab at index as closeable.
     * @param index The index of the tab.
     * @param value True if the tab should be closeable.
     */
    public void setClosableAt(int index, boolean value) {
        if (value != isClosableAt(index)) {
            setIconAt(index, getCombinedIconAt(index).cloneWithClosable(value));
        }
    }

    /**
     * Font style (PLAIN, BOLD, ITALIC, BOLD|ITALIC) of tab title
     * at 'index'. The default is plain.
     * @param index The index of the tab.
     * @return The font style.
     */
    public int getFontStyleAt(int index) {
        return getCombinedIconAt(index).getFontStyle();
    }

    /**
     * Sets the font style of the tab at index.
     * @param index The index of the tab.
     * @param value The font style to set.
     */
    public void setFontStyleAt(int index, int value) {
        if (value != getFontStyleAt(index)) {
            setIconAt(index, getCombinedIconAt(index).cloneWithFontStyle(value));
        }
    }

    /**
     * Inserts a tab.
     * @param title The tab title.
     * @param c The component to be displayed when this tab is clicked.
     */
    public void addClosableTab(String title, Component c) {
        addClosableTab(getTabCount(), title, null, c);
    }

    /**
     * Inserts a tab.
     * @param title The tab title.
     * @param n An icon for the tab.
     * @param c The component to be displayed when this tab is clicked.
     */
    public void addClosableTab(String title, Icon n, Component c) {
        addClosableTab(getTabCount(), title, n, c);
    }

    /**
     * Inserts a tab.
     * @param index The tab index.
     * @param title The tab title.
     * @param c The component to be displayed when this tab is clicked.
     */
    public void addClosableTab(int index, String title, Component c) {
        addClosableTab(index, title, null, c);
    }

    /** Add a tab at 'index' with given title and icon,
     * showing the given component. By default, the tab is closable.
     * This method (or one of the delegates) must be used to add new tabs.
     * @param index The tab index.
     * @param title The tab title.
     * @param n An icon for the tab.
     * @param c The component to be displayed when this tab is clicked.
     */
    public void addClosableTab(int index, String title, Icon n, Component c) {
        insertTab(null, new CombinedIcon(index, title, n), c, null, index);
    }

    /**
     * Use instead of setTitleAt.
     * @param index The index of the tab.
     * @param value The title to set.
     */
    public void setClosableTitleAt(int index, String value) {
        String old = getClosableTitleAt(index);
        if ((value == null) ? (old != null) : (!value.equals(old))) {
            setIconAt(index, getCombinedIconAt(index).cloneWithTitle(value));
        }
    }

    /**
     * Use instead of getTitleAt.
     * @param index The index of the tab.
     * @return The title of the tab at index.
     */
    public String getClosableTitleAt(int index) {
        return getCombinedIconAt(index).getTitle();
    }

    /**
     * Use instead of setIconAt.
     * @param index The index of the tab.
     * @param value The icon to set.
     */
    public void setClosableIconAt(int index, Icon value) {
        if (value != getClosableIconAt(index)) {
            setIconAt(index, getCombinedIconAt(index).cloneWithIcon(value));
        }
    }

    /**
     * Use instead of getIconAt.
     * @param index The index of the tab.
     * @return The icon of the tab.
     */
    public Icon getClosableIconAt(int index) {
        return getCombinedIconAt(index).getIcon();
    }

    /**
     * Standard listener pattern.
     * @param l The listener to add.
     */
    public void addClosingListener(ClosingListener l) {
        listenerList.add(ClosingListener.class, l);
    }

    /**
     * Standard listener pattern.
     * @param l The listener to remove.
     */
    public void removeClosingListener(ClosingListener l) {
        listenerList.remove(ClosingListener.class, l);
    }

    /**
     * Standard listener pattern.
     * @param index The index of the tab.
     */
    protected void fireClosing(int index) {
        Object[] pairs = listenerList.getListenerList();

        ClosingEvent e = null;

        for (int i = pairs.length - 2; i >= 0; i -= 2) {
            if (pairs[i] == ClosingListener.class) {
                if (e == null) {
                    e = new ClosingEvent(this, index);
                }
                ((ClosingListener) pairs[i + 1]).closing(e);
            }
        }
    }

    /** Programmatically cause the same effect (event) as
     * if the user made a closing action at index. The index
     * is required to be closable.
     * @param index The tab to close.
     */
    public void closingTab(int index) {
        if (!isClosableAt(index)) {
            throw new IllegalArgumentException("Not closable " + index);
        }

        fireClosing(index);
    }

    //////////////////
    // Adjust indices.
    private void tabRemoved(int index) {
        closeButtonListener.tabRemoved(index);

        for (int i = index; i < getTabCount(); i++) {
            --getCombinedIconAt(i).index;
        }
    }

    private void tabAdded(int index) {
        closeButtonListener.tabAdded(index);

        for (int i = index + 1; i < getTabCount(); i++) {
            getCombinedIconAt(i).index++;
        }
    }

    // update the rollover tab: 'index' has changed to 'value', and there
    // can be at most one rollover tab
//  COMMENT OUT TO SEE ROLLOVER:
//    private void updateRolloverAt(int index, boolean value) {
//        setBackgroundAt(index, value ? java.awt.Color.blue : null);
//    }

    /** Standard listener pattern. */
    public interface ClosingListener extends EventListener {
        /**
         * The user has requested to close a tab.
         * @param e The closing event.
         */
        void closing(ClosingEvent e);
    }

    private static class IndexButtonModel extends DefaultButtonModel {
        private static final long serialVersionUID = 7526471155622776147L;
        private int index;

        public IndexButtonModel() {
            index = -1;
        }

        public void setIndex(int value) {
            index = value;
        }

        public void shiftIndex(int value) {
            index += value;
        }

        public void insertIndex(int value) {
            if (index >= value) {
                index++;
            }
        }

        public void removeIndex(int value) {
            if (index == value) {
                index = -1;
            } else if (index > value) {
                --index;
            }
        }

        public int getIndex() {
            return index;
        }

        public void reset() {
            setIndex(-1);
        }
    }

    /** Standard listener pattern. */
    public static class ClosingEvent extends EventObject {
        private static final long serialVersionUID = 7526471155622776147L;
        private final int index;

        /**
         * A closing event.
         * @param source The source.
         * @param ind The index of the tab.
         */
        public ClosingEvent(JaxoClosableTabbedPane source, int ind) {
            super(source);

            this.index = ind;
        }

        /**
         * Index that was requested to be closed.
         * @return The index of the tab.
         */
        public final int getIndex() {
            return index;
        }
    }

    // It is assumed that 'button.index' is either -1 or the
    // same as 'tabRollover.index'.
    private class CloseButtonListener extends MouseInputAdapter
        implements WindowFocusListener {
        private int pressedCount;

        public void windowGainedFocus(WindowEvent e) {
            // nop
        }

        public void windowLostFocus(WindowEvent e) {
            // modal dialog cleanup, otherwise things are
            // active even if the window does not /have/ focus
            reset();
        }

        public void reset() {
            setRollover(-1, false);
            pressedCount = 0;
        }

        // Adjust index on tab changes
        // ignore that tab bounds may shift
        public void tabRemoved(int index) {
            tabRollover.removeIndex(index);
            button.removeIndex(index);
        }

        public void tabAdded(int index) {
            tabRollover.insertIndex(index);
            button.insertIndex(index);
        }

        private void repaintAt(IndexButtonModel m) {
            if (m.getIndex() != -1) {
                repaint(getBoundsAt(m.getIndex()));
            }
        }

        private boolean isContainedInButton(int index, Point p) {
            return (index != -1)
            && getCombinedIconAt(index).isContainedInButton(p);
        }

        private void setRollover(int index, boolean inButton) {
            if (index == -1) {
                repaintAt(tabRollover);
                //if (tabRollover.getIndex() != -1) {
                //    updateRolloverAt(tabRollover.getIndex(), false);
                //}
                tabRollover.reset();
                button.reset();
            } else {
                boolean noChange =
                    tabRollover.isRollover()
                    && (tabRollover.getIndex() == index)
                    && (inButton == button.isRollover());

                if (!noChange) {
                    tabRollover.setRollover(true);
                    //if (tabRollover.getIndex() != -1) {
                    //    updateRolloverAt(tabRollover.getIndex(), false);
                    //}
                    tabRollover.setIndex(index);
                    //updateRolloverAt(tabRollover.getIndex(), true);

                    button.setIndex(inButton ? index : (-1));
                    button.setRollover(inButton);

                    repaintAt(tabRollover);
                }
            }
        }

        public void mouseMoved(MouseEvent e) {
            if (pressedCount == 0) {
                int index = indexAtLocation(e.getX(), e.getY());

                boolean inButton = isContainedInButton(index, e.getPoint());

                setRollover(index, inButton);
            } else {
                setRollover(-1, false);
            }
        }

        public void mousePressed(MouseEvent e) {
            if (pressedCount == 0) {
                mouseMoved(e);
            } else {
                mouseDragged(e);
            }

            pressedCount++;

            if (!SwingUtilities.isLeftMouseButton(e)) {
                return;
            }

            if (button.getIndex() != -1) {
                button.setPressed(true);
                button.setArmed(true);
                repaintAt(button);
            }
        }

        public void mouseDragged(MouseEvent e) {
            if (!button.isPressed()) {
                mouseMoved(e);
                return;
            }

            int index = indexAtLocation(e.getX(), e.getY());

            boolean armed =
                (index == button.getIndex())
                && isContainedInButton(index, e.getPoint());

            if (armed != button.isArmed()) {
                button.setArmed(armed);

                repaintAt(button);
            }
        }

        public void mouseReleased(MouseEvent e) {
            mouseDragged(e);

            --pressedCount;

            if (SwingUtilities.isLeftMouseButton(e) && button.isPressed()) {
                if (button.isArmed()) {
                    fireClosing(button.getIndex());
                }

                repaintAt(button);

                button.reset();
            }

            if (isNoButtonDown(e)) {
                mouseMoved(e);
            }
        }

        private boolean isNoButtonDown(MouseEvent e) {
            return (
                e.getModifiersEx()
                & (
                    MouseEvent.BUTTON1_DOWN_MASK
                    | MouseEvent.BUTTON2_DOWN_MASK
                    | MouseEvent.BUTTON3_DOWN_MASK
                )
            ) == 0;
        }

        public void mouseEntered(MouseEvent e) {
            if (pressedCount > 0) { // drag
                mouseDragged(e);
            } else if (isNoButtonDown(e)) { // move
                mouseMoved(e);
            }

            // else foreign entered
        }

        public void mouseExited(MouseEvent e) {
            if (pressedCount > 0) { // drag
                mouseDragged(e);
            } else if (isNoButtonDown(e)) { // move
                mouseMoved(e);
            }

            // else foreign exited
        }
    }

    // Combined Icon that paints the real Icon, the title, and the button
    // This is immutable because it is not really possible for an icon to
    // change size. Use cloneWithXYZ methods to obtain a new CombinedIcon
    // with a changed property.
    private class CombinedIcon implements Icon, Cloneable {
        // modified by tabAdded/tabRemoved in the main class.
        private int index;

        // Visible properties
        private String title;
        private Icon icon;
        private boolean closable;
        private int fontStyle;

        // Icon/Text gap is modelled as a width of a space
        // (Icon and or Text)/Button gap is modelled as two spaces
        // gap is lazily initialized when first needed
        private int gap;
        private Rectangle lastButtonBounds;

        public CombinedIcon(int ind, String text, Icon ico) {
            this.closable = true;
            this.index = ind;
            this.icon = ico;
            this.fontStyle = Font.PLAIN;
            this.title = text;
            this.gap = -1;

        }

        public final int getFontStyle() {
            return fontStyle;
        }

        public final String getTitle() {
            return title;
        }

        public final Icon getIcon() {
            return icon;
        }

        public final boolean isClosable() {
            return closable;
        }

        public Object clone() throws CloneNotSupportedException {
            CombinedIcon clone = (CombinedIcon) super.clone();

            clone.gap = -1;
            clone.lastButtonBounds = null;

            return clone;
        }

        public Object copy() {
            try {
                return clone();
            } catch (CloneNotSupportedException ex) {
                throw new Error(ex);
            }
        }

        public Icon cloneWithFontStyle(int value) {
            CombinedIcon clone = (CombinedIcon) copy();

            clone.fontStyle = value;

            return clone;
        }

        public Icon cloneWithIcon(Icon value) {
            CombinedIcon clone = (CombinedIcon) copy();

            clone.icon = value;

            return clone;
        }

        public Icon cloneWithTitle(String value) {
            CombinedIcon clone = (CombinedIcon) copy();

            clone.title = value;

            return clone;
        }

        public Icon cloneWithClosable(boolean value) {
            CombinedIcon clone = (CombinedIcon) copy();

            clone.closable = value;

            return clone;
        }

        public boolean isContainedInButton(Point p) {
            if (!closable) {
                return false;
            }

            if (lastButtonBounds == null) {
                return false;
            }

            return lastButtonBounds.contains(p);
        }

        protected Font getTabFont() {
            Font f = getFont();
            if (f.getStyle() != fontStyle) {
                f = f.deriveFont(fontStyle);
            }
            return f;
        }

        public void paintIcon(Component c, Graphics gg, int xx, int yy) {
            Graphics g = gg.create();
            int x = xx;
            int y = yy;

            g.setColor(getForegroundAt(index));
            g.setFont(getTabFont());

            x += TAB_INSETS.left;
            y += TAB_INSETS.top;

            int height = getIconHeight() - TAB_INSETS.top - TAB_INSETS.bottom;

            FontMetrics m = g.getFontMetrics();

            if (gap == -1) {
                gap = m.charWidth(' ');
            }

            // Horizontal
            int titleWidth = (title == null) ? 0 : m.stringWidth(title);
            int iconTitleGap = ((icon == null) || (title == null)) ? 0 : gap;
            int titleButtonGap =
                ((iconTitleGap != 0) && closable) ? (2 * gap) : 0;

            // Vertical
            int titleHeight = m.getHeight();
            int iconHeight = (icon == null) ? 0 : icon.getIconHeight();

            int shift = (titleHeight - iconHeight) / 2;

            int textDY = (shift < 0) ? shift : 0;
            int iconDY = (shift > 0) ? shift : 0;

            int textIconDY =
                Math.max(0, height - Math.max(titleHeight, iconHeight)) / 2;

            if (icon != null) {
                icon.paintIcon(c, g, x, y + iconDY + textIconDY);

                x += icon.getIconWidth();
            }

            if (title != null) {
                x += iconTitleGap;

                g.setColor(getForegroundAt(index));
                g.setFont(getTabFont());

                g.drawString(title, x, y + m.getAscent() + textDY + textIconDY);

                x += titleWidth;
            }

            if (closable) {
                x += titleButtonGap;

                Dimension d =
                    new Dimension(defaultIcon.getIconWidth(),
                        defaultIcon.getIconHeight());

                Icon paintedButtonIcon;

                if (index == button.getIndex()) {
                    if (button.isPressed()) {
                        paintedButtonIcon =
                            button.isArmed() ? pressedArmedIcon : pressedIcon;
                    } else {
                        paintedButtonIcon = rolloverIcon;
                    }
                } else if (index == getSelectedIndex()) {
                    paintedButtonIcon = selectedIcon;
                } else {
                    paintedButtonIcon = defaultIcon;
                }

                if (lastButtonBounds == null) {
                    lastButtonBounds = new Rectangle();
                }

                lastButtonBounds.setBounds(x, y + ((height - d.height) / 2),
                    d.width, d.height);

                paintedButtonIcon.paintIcon(c, g, x,
                    y + ((height - d.height) / 2));
            }

            g.dispose();
        }

        public int getIconWidth() {
            FontMetrics m = getFontMetrics(getTabFont());

            if (gap == -1) {
                gap = m.charWidth(' ');
            }

            int titleWidth = (title == null) ? 0 : m.stringWidth(title);
            int iconWidth = (icon == null) ? 0 :  icon.getIconWidth();
            int iconTitleGap = ((icon == null) || (title == null)) ? 0 : gap;
            int buttonWidth = closable ? defaultIcon.getIconWidth() : 0;
            int titleButtonGap =
                ((iconTitleGap != 0) && closable) ? (2 * gap) : 0;

            return TAB_INSETS.left + TAB_INSETS.right + iconWidth
            + iconTitleGap + titleWidth + titleButtonGap + buttonWidth;
        }

        public int getIconHeight() {
            FontMetrics m = getFontMetrics(getTabFont());

            int titleHeight = m.getHeight();
            int iconHeight = (icon == null) ? 0 : icon.getIconHeight();

            // Use button height for uniform layout even if not closable
            return TAB_INSETS.top + TAB_INSETS.bottom
            + Math.max(defaultIcon.getIconHeight(),
                Math.max(titleHeight, iconHeight));
        }
    }
}
