/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.io;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import java.util.Locale;

import javax.print.PrintService;
import javax.print.PrintServiceLookup;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.Copies;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.standard.OrientationRequested;

import javax.swing.JOptionPane;

import net.sf.jaxodraw.graph.JaxoGraph;
import net.sf.jaxodraw.gui.JaxoDialogs;
import net.sf.jaxodraw.util.JaxoDictionary;
import net.sf.jaxodraw.util.JaxoLog;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** Responsible for printing directly to a printer.
 * @since 2.0
 */
public class JaxoPrint implements Printable {
    private final JaxoDictionary language = new JaxoDictionary(JaxoPrint.class);
    private JaxoGraph graph;
    private double scale = 1.d;
    private Dimension canvasD;

    /** The actual printing routine.
     * @param g The specified graphics context.
     * @param pf The specified page format.
     * @param pageIndex The specified page index.
     * @return Either Printable.PAGE_EXISTS or Printable.NO_SUCH_PAGE.
     */
    public final int print(Graphics g, PageFormat pf, int pageIndex) {
        double transX = pf.getImageableX();
        double transY = pf.getImageableY();

        double scaleX = pf.getImageableWidth() / canvasD.width;
        double scaleY = pf.getImageableHeight() / canvasD.height;

        if (pageIndex == 0) {
            JaxoGraphics2D g2 = new JaxoGraphics2D((Graphics2D) g);
            g2.setPrinting(true);

            scale = scaleX;

            if (scaleY < scaleX) {
                scale = scaleY;
            }

            if (scale > 1.) {
                scale = 1.d;
            }

            g2.translate(transX, transY);
            g2.scale(scale, scale);

            graph.paint(g2);

            return Printable.PAGE_EXISTS;
        } else {
            return Printable.NO_SUCH_PAGE;
        }
    }

    private void showPrintError(Component parent, String text) {
        JOptionPane.showMessageDialog(parent, text,
            JaxoDialogs.windowTitle(language, "errorTitle"),
            JOptionPane.ERROR_MESSAGE);
    }

    /** The method to be called from the printing menu.
     * @param parent parent Component for dialogs.
     * @param newGraph The JaxoGraph to be printed.
     * @param d The current dimension of the canvas.
     * @param name Name for the job
     */
    public final void print(Component parent, JaxoGraph newGraph, Dimension d,
        String name) {
        this.graph = newGraph;
        this.canvasD = d;

        PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
        aset.add(OrientationRequested.LANDSCAPE);
        aset.add(new Copies(1));
        aset.add(new JobName(name, null));

        PrinterJob pj = PrinterJob.getPrinterJob();
        pj.setPrintable(this);

        PrintService[] services = PrinterJob.lookupPrintServices();

        PrintService defaultService =
            PrintServiceLookup.lookupDefaultPrintService();

        PrintService chosenService =
            (services.length > 0) ? services[0] : defaultService;

        if (chosenService == null) {
            showPrintError(parent,
                language.value("noPrinters"));
        } else {
            Locale oldLocale = Locale.getDefault();

            try {
                pj.setPrintService(chosenService);

                Locale.setDefault(language.locale());

                if (pj.printDialog(aset)) {
                    pj.print(aset);
                }
            } catch (PrinterException e) {
                JaxoLog.debug(e);
                showPrintError(parent,
                    language.message("print%0Failed", name));
            } finally {
                Locale.setDefault(oldLocale);
            }
        }
    }
}
