/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.io.exports;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.Printable;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import java.util.Calendar;
import java.util.NoSuchElementException;

import javax.print.Doc;
import javax.print.DocFlavor;
import javax.print.DocFlavor.BYTE_ARRAY;
import javax.print.DocFlavor.SERVICE_FORMATTED;
import javax.print.DocPrintJob;
import javax.print.PrintException;
import javax.print.SimpleDoc;
import javax.print.StreamPrintService;
import javax.print.StreamPrintServiceFactory;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.OrientationRequested;
import javax.print.event.PrintJobAdapter;
import javax.print.event.PrintJobEvent;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.jaxodraw.gui.JaxoDialogs;
import net.sf.jaxodraw.io.JaxoPreview;
import net.sf.jaxodraw.plugin.JaxoPluginExecutionException;
import net.sf.jaxodraw.util.JaxoInfo;
import net.sf.jaxodraw.util.JaxoLanguage;
import net.sf.jaxodraw.util.JaxoLog;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** The class that is responsible for exporting to postscript output: portrait,
 * landscape or EPS are possible options. Asks for an export file name.
 * @since 2.0
 */
public class JaxoExportPS extends JaxoExport implements Printable {
    /** An integer specifying the postscript portrait mode. */
    public static final int PS_PS = 2;

    /** An integer specifying the postscript EPS mode. */
    public static final int PS_EPS = 4;

    // private
    private int mode;
    private float transX;
    private float transY;
    private double scale;
    private double paperHeight;
    private boolean landscape;
    private JPanel panel;
    private JLabel orientationLabel;
    private JComboBox orientation;

    /** The unique plugin id. */
    private static final String PLUGIN_ID = JaxoExportPS.class.getName();

    /** Constructor.
     * @param psmode PS_PS or PS_EPS.
     */
    public JaxoExportPS(int psmode) {
        setMode(psmode);
    }

    /** Returns the landscape mode.
     * @return True for landscape, false for portrait..
     */
    public final boolean isLandscape() {
        return landscape;
    }

    /** Sets the landscape mode.
     * @param value True for landscape, false for portrait.
     */
    public void setLandscape(boolean value) {
        landscape = value;
    }

    /** Sets the export mode.
     * @param value The new export mode (PS_XYZ constant).
     */
    public final void setMode(int value) {
        mode = value;
    }

    /** Returns the current export mode.
     * @return The current export mode.
     */
    public final int getMode() {
        return mode;
    }

    /** {@inheritDoc} */
    public final String getFormatName() {
        return JaxoLanguage.translate("JaxoExportPS.formatName."
            + getFileExtension());
    }

    /** {@inheritDoc} */
    public final String getFileExtension() {
        return (getMode() == PS_EPS) ? "eps" : "ps";
    }

    /** {@inheritDoc} */
    public final String getFileExtensionDescription() {
        return JaxoLanguage.translate("JaxoExportPS.fileDescription."
            + getFileExtension());
    }

    /** {@inheritDoc} */
    public String description() {
        return JaxoLanguage.translate("JaxoExportPS.description."
            + getFileExtension());
    }

    /** {@inheritDoc} */
    public String pluginId() {
        return PLUGIN_ID;
    }

    /** {@inheritDoc} */
    public String getShortName() {
        return "ps-export";
    }

    /** {@inheritDoc} */
    protected void exportTo(String fileName)
        throws JaxoPluginExecutionException {
        try {
            if (getMode() == PS_EPS) {
                doExportEPS(fileName);
            } else {
                doExportPS(fileName);
            }
        } catch (IOException ie) {
            throw new JaxoPluginExecutionException(
                errorDialogMessage(fileName), ie, this);
        }
    }

    private void doExportPS(String fileName) throws IOException {
        String tempFileName = ".temp.ps";
        File tempFile = new File(tempFileName);
        tempFile.deleteOnExit();

        writePS(tempFileName);

        BufferedReader inbr = null;
        BufferedWriter outbr = null;
        try {
            inbr = new BufferedReader(new FileReader(tempFileName));
            outbr = new BufferedWriter(new FileWriter(fileName));

            Calendar rightNow = Calendar.getInstance();

            String firstLine = "%!PS-Adobe-3.0";
            String thirdLine =
                "%%Creator: " + JaxoInfo.USER_NAME + " with "
                + JaxoInfo.VERSION;
            String forthLine = "%%Title: " + getGraph().getDescription();
            String fifthLine =
                "%%CreationDate: " + rightNow.get(Calendar.DAY_OF_MONTH) + "/"
                + (rightNow.get(Calendar.MONTH) + 1) + "/"
                + rightNow.get(Calendar.YEAR);

            String str;
            boolean isFirstLine = true;

            while ((str = inbr.readLine()) != null) {
                if (isFirstLine) {
                    outbr.write(firstLine, 0, firstLine.length());
                    outbr.newLine();
                    outbr.write(thirdLine, 0, thirdLine.length());
                    outbr.newLine();
                    outbr.write(forthLine, 0, forthLine.length());
                    outbr.newLine();
                    outbr.write(fifthLine, 0, fifthLine.length());
                    outbr.newLine();
                    isFirstLine = false;
                } else {
                    outbr.write(str, 0, str.length());
                    outbr.newLine();
                }
            }
        } finally { // make sure the streams are closed
            close(inbr);
            close(outbr);
        }
    }

    private void doExportEPS(String fileName) throws IOException {
        String tempFileName = ".temp.eps";
        File tempFile = new File(tempFileName);
        tempFile.deleteOnExit();

        writePS(tempFileName);

        BufferedReader inbr = null;
        BufferedWriter outbr = null;
        try {
            inbr = new BufferedReader(new FileReader(tempFileName));
            outbr = new BufferedWriter(new FileWriter(fileName));

            Rectangle bb = getPSBoundingBox().getBounds();

            Calendar rightNow = Calendar.getInstance();

            String firstLine = "%!PS-Adobe-3.0 EPSF-3.0";
            String secondLine =
                "%%BoundingBox: "
                + Math.round((float) Math.floor(bb.getMinX())) + " "
                + Math.round((float) Math.floor(bb.getMinY())) + " "
                + Math.round((float) Math.ceil(bb.getMaxX())) + " "
                + Math.round((float) Math.ceil(bb.getMaxY()));
            String thirdLine =
                "%%Creator: " + JaxoInfo.USER_NAME + " with "
                + JaxoInfo.VERSION;
            String forthLine = "%%Title: " + getGraph().getDescription();
            String fifthLine =
                "%%CreationDate: " + rightNow.get(Calendar.DAY_OF_MONTH) + "/"
                + (rightNow.get(Calendar.MONTH) + 1) + "/"
                + rightNow.get(Calendar.YEAR);

            String str;
            String setpagedevice = "setpagedevice";
            boolean isFirstLine = true;

            while ((str = inbr.readLine()) != null) {
                if (isFirstLine) {
                    outbr.write(firstLine, 0, firstLine.length());
                    outbr.newLine();
                    outbr.write(secondLine, 0, secondLine.length());
                    outbr.newLine();
                    outbr.write(thirdLine, 0, thirdLine.length());
                    outbr.newLine();
                    outbr.write(forthLine, 0, forthLine.length());
                    outbr.newLine();
                    outbr.write(fifthLine, 0, fifthLine.length());
                    outbr.newLine();
                    isFirstLine = false;
                } else {
                    /*
                        check whether the line contains
                        the postscript command "setpagedevice",
                        which is not allowed in EPS

                        other forbidden commands:
                        banddevice      exitserver    quit
                        clear           framedevice   renderbands
                        cleardictstack  grestoreall   setglobal
                        copypage        initclip      setpagedevice
                        erasepage       initgraphics  setshared
                        initmatrix      startjob
                    */
                    if (str.indexOf(setpagedevice) != -1) {
                        str = str.replaceAll(setpagedevice, "pop");
                    }
                    outbr.write(str, 0, str.length());
                    outbr.newLine();
                }
            }
        } finally {
            close(inbr);
            close(outbr);
        }
    }

    /** The actual printing routine required by the Printable interface.
     * @param g The specified graphics context.
     * @param pf The specified page format.
     * @param pageIndex The specified page index.
     * @return Either Printable.PAGE_EXISTS or Printable.NO_SUCH_PAGE.
     */
    public final int print(Graphics g, PageFormat pf, int pageIndex) {
        transX = (float) pf.getImageableX();
        transY = (float) pf.getImageableY();
        paperHeight = pf.getHeight();

        double scaleX = pf.getImageableWidth() / getCanvasSize().width;
        double scaleY = pf.getImageableHeight() / getCanvasSize().height;

        if (pageIndex == 0) {
            JaxoGraphics2D g2 = new JaxoGraphics2D((Graphics2D) g);

            g2.setPrinting(true);

            scale = scaleX;

            if (scaleY < scaleX) {
                scale = scaleY;
            }

            if (scale > 1.) {
                scale = 1.d;
            }

            g2.translate((double) transX, (double) transY);
            g2.scale(scale, scale);
            g2.clipRect(0, 0, getCanvasSize().width, getCanvasSize().height);

            getGraph().paint(g2);

            return Printable.PAGE_EXISTS;
        } else {
            return Printable.NO_SUCH_PAGE;
        }
    }

    private Rectangle getPSBoundingBox() {
        Rectangle result = new Rectangle();

        Rectangle bBox =
            getGraph().getBounds(new Rectangle(0, 0, getCanvasSize().width,
                    getCanvasSize().height));

        if (bBox == null) {
            bBox = new Rectangle();
        }

        if (landscape) {
            result.setFrameFromDiagonal((bBox.y * scale) + transX,
                (bBox.x * scale) + transX,
                ((bBox.y + bBox.height) * scale) + transX,
                ((bBox.x + bBox.width) * scale) + transX);
        } else {
            result.setFrameFromDiagonal((bBox.x * scale) + transX,
                paperHeight - (((bBox.y + bBox.height) * scale) + transY),
                ((bBox.x + bBox.width) * scale) + transX,
                paperHeight - ((bBox.y * scale) + transY));
        }

        return result;
    }

    /** The class that is responsible for exporting to postscript output:
     * portrait, landscape or EPS are possible options.
     * The export file name is given by fileName.
     * @param fileName The export file name.
     */
    protected final void writePS(String fileName) {
        DocFlavor flavor = SERVICE_FORMATTED.PRINTABLE;
        String psMimeType = BYTE_ARRAY.POSTSCRIPT.getMimeType();
        StreamPrintServiceFactory[] factories =
            StreamPrintServiceFactory.lookupStreamPrintServiceFactories(flavor,
                psMimeType);

        if (factories.length == 0) {
            setFailure(new NoSuchElementException("StreamPrintServiceFactory"));
            showErrorDialog(JaxoLanguage.translate("No_suitable_factories_found!"));
        } else {
            FileOutputStream fos = null;
            try {
                // Create a file for the exported postscript
                fos = new FileOutputStream(fileName);

                // Create a Stream printer for Postscript
                StreamPrintService sps = factories[0].getPrintService(fos);

                // Create and call a Print Job
                DocPrintJob job = sps.createPrintJob();

                PrintRequestAttributeSet aset =
                    new HashPrintRequestAttributeSet();

                aset.add(landscape ? OrientationRequested.LANDSCAPE
                                   : OrientationRequested.PORTRAIT);

                Doc doc = new SimpleDoc(this, flavor, null);
                PrintJobWatcher pjDone = new PrintJobWatcher(job);
                job.print(doc, aset);
                pjDone.waitForDone();

            } catch (PrintException pe) {
                setFailure(pe);
                showErrorDialog(errorDialogMessage(fileName));
            } catch (IOException ie) {
                setFailure(ie);
                showErrorDialog(errorDialogMessage(fileName));
            } finally {
                close(fos);
            }
        }
    }

    /** {@inheritDoc} */
    public void preview(JaxoPreview p, boolean sameWindow) {
        if (p.canShowPostscript()) {
            String previewFileName =
                (getMode() == PS_EPS) ? "Jaxo_tmp.eps" : "Jaxo_tmp.ps";

            File previewFile = new File(JaxoPreview.USER_DIR, previewFileName);
            boolean previewFileExists = previewFile.exists();
            previewFile.deleteOnExit();

            try {
                exportTo(previewFileName);
            } catch (JaxoPluginExecutionException e) {
                setFailure(e); // this logs in debug mode
                if (e.getMessage() != null) {
                    showErrorDialog(e.getMessage());
                }
                return;
            }

            boolean newWindow = (!sameWindow || (sameWindow && !previewFileExists));

            // p.setTitle(.) -- postscript is still always external
            if (newWindow) {
                p.showPostscript(previewFile);
            }
        } else {
            p.showMessageForPostscriptViewer();

        }
    }

    /** {@inheritDoc} */
    public void commitConfiguration() {
        // by default: do nothing.
    }

    /** {@inheritDoc}
     * @return A panel to configure the postscript orientation.
     */
    public JComponent getConfigurationPanel() {
        if (panel == null) {
            panel = new JPanel(new GridBagLayout(), false);

            GridBagConstraints labels = new GridBagConstraints();
            GridBagConstraints components = new GridBagConstraints();

            labels.anchor = GridBagConstraints.LINE_END;
            labels.gridx = 0;
            labels.gridy = 0;
            components.anchor = GridBagConstraints.LINE_START;
            components.gridx = 1;
            components.gridy = 0;
            components.weightx = 1;

            orientationLabel =
                new JLabel(JaxoLanguage.label("JaxoExportPS.orientation"));
            panel.add(orientationLabel, labels);

            this.orientation =
                new JComboBox(new Object[]{
                        JaxoLanguage.translate(
                            "JaxoExportPS.orientation.portrait"),
                        JaxoLanguage.translate(
                            "JaxoExportPS.orientation.landscape")
                    });

            orientation.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        landscape = orientation.getSelectedIndex() == 1;
                    }
                });

            panel.add(orientation, components);

            labels.gridy++;
            components.gridy++;

        }

        return panel;
    }

    /** {@inheritDoc} */
    public final void updateLanguage() {
        if (orientationLabel != null) {
            orientationLabel.setText(JaxoLanguage.label(
                    "JaxoExportPS.orientation"));
            this.orientation =
                new JComboBox(new Object[]{
                        JaxoLanguage.translate(
                            "JaxoExportPS.orientation.portrait"),
                        JaxoLanguage.translate(
                            "JaxoExportPS.orientation.landscape")
                    });
        }
    }

    class PrintJobWatcher {
        // true if it is safe to close the print job's input stream
        private boolean done;

        PrintJobWatcher(DocPrintJob job) {
            // Add a listener to the print job
            job.addPrintJobListener(new PrintJobAdapter() {
                    public final void printJobCanceled(PrintJobEvent pje) {
                        allDone();
                    }

                    public final void printJobCompleted(PrintJobEvent pje) {
                        allDone();
                    }

                    public final void printJobFailed(PrintJobEvent pje) {
                        allDone();
                    }

                    public final void printJobNoMoreEvents(PrintJobEvent pje) {
                        allDone();
                    }

                    void allDone() {
                        synchronized (PrintJobWatcher.this) {
                            done = true;
                            PrintJobWatcher.this.notifyAll();
                        }
                    }
                });
        }

        public final void waitForDone() {
            synchronized (this) {
                try {
                    while (!done) {
                        super.wait();
                    }
                } catch (InterruptedException e) {
                    setFailure(e);
                    JaxoLog.debug(e);
                    JaxoDialogs.showI18NErrorDialog(null, "Print_job_aborted!");
                }
            }
        }
    }
}
