/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.io.imports;

import java.awt.geom.Point2D;

import net.sf.jaxodraw.object.arrow.JaxoDefaultArrow;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoArrow;
import net.sf.jaxodraw.object.JaxoObjectFactory;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoPrefs;


/**
 * Extension to JaxoAxodraw that adds Bezier imports.
 *
 * @since 2.0
 */
public class JaxoAxodraw4J extends JaxoAxodraw {
    private static final String[] KNOWN_COMMANDS =
    {
        "\\Bezier", "\\DashBezier", "\\Line", "\\Arc", "\\PhotonArc",
        "\\Photon", "\\GluonArc", "\\GlueArc", "\\Gluon"
    };
    private static final int N_OF_COMMANDS = KNOWN_COMMANDS.length;
    private String string = "";
    private int index;

    /** Constructor: just calls super(). */
    public JaxoAxodraw4J() {
        super();
        removePackage("axodraw");
        addPackage("axodraw4j");
    }

    /**
     * Returns a JaxoObject corresponding to a axodraw command.
     *
     * @param newString One line of a latex file
     * @return A JaxoObject the corresponds to the axodraw command
     * of the input line, or null, if nothing corresponds
     */
    public JaxoObject getJaxoObject(String newString) {
        this.string = newString;

        JaxoObject newob = null;

        // first check if it's a new axodraw4j command
        boolean valid = validJaxo4JCommand(string);

        if (valid) {
            if (index == -10) { // Fermionic Bezier
                newob = newFBezier();
            } else if (index == -9) { // Scalar Bezier
                newob = newSBezier();
            } else if (index == -8) { // Ghost Bezier
                newob = newGBezier();
            } else if (index == -7) { // Photon Bezier
                newob = newPBezier();
            } else if (index == -6) { // Gluon Bezier
                newob = newGlBezier();
            } else if (index == -5) { // Fermion/Scalar/Ghost Lines
                newob = newA4JLine();
            } else if (index == -4) { // Fermion/Scalar/Ghost Arcs/Loops
                newob = newA4JArc();
            } else if (index == -3) { // Photon line (for double option)
                newob = newA4JPLine();
            } else if (index == -2) { // Gluon Line (for double option)
                newob = newA4JGlLine();
            } else if (index == -1) { // Photon Arcs/Loops (for double option)
                newob = newA4JPArc();
            } else if (index == 0) { // Gluon Arcs/Loops (for double option)
                newob = newA4JGlArc();
            } else if (index == 1) {
                newob = newVertexT4();
            }
        } else {
            // then check if it's an old axodraw command
            newob = super.getJaxoObject(newString);
        }

        return newob;
    }

    private boolean validJaxo4JCommand(String s) {
        int i = 0;

        while (!s.startsWith(KNOWN_COMMANDS[i]) && (i < (N_OF_COMMANDS - 1))) {
            i++;
        }

        if ((i == (N_OF_COMMANDS - 1)) && !s.startsWith(KNOWN_COMMANDS[i])) {
            i++;
        }

        this.index = i;

        if (i == getIndex("\\Bezier")) {
            if (s.indexOf("%JaxoID:FBez") > 0) {
                this.index = -10;
            } else if (s.indexOf("%JaxoID:PBez") > 0) {
                this.index = -7;
            } else if (s.indexOf("%JaxoID:GlBez") > 0) {
                this.index = -6;
            }
        } else if (i == getIndex("\\DashBezier")) {
            if (s.indexOf("%JaxoID:SBez") > 0) {
                this.index = -9;
            } else if (s.indexOf("%JaxoID:GBez") > 0) {
                this.index = -8;
            }
        } else if (i == getIndex("\\Line")) {
           // Need to check here for the cross vertex otherwise it will be broken
            if ((s.indexOf("\\Line", 6) > 0) && (s.indexOf('%') == -1)) {
                this.index = +1;
            } else {
                this.index = -5;
            }
        } else if (i == getIndex("\\Arc")) {
            this.index = -4;
        } else if (i == getIndex("\\Photon")) {
            this.index = -3;
        } else if (i == getIndex("\\Gluon")) {
            this.index = -2;
        } else if (i == getIndex("\\PhotonArc")) {
            this.index = -1;
        } else if (i == getIndex("\\GluonArc") || i == getIndex("\\GlueArc")) {
            this.index = 0;
        }

        return (i < N_OF_COMMANDS);
    }

    private int getIndex(String s) {
        int i = 0;

        while ((KNOWN_COMMANDS[i].compareTo(s) != 0)
                && (i < (N_OF_COMMANDS - 1))) {
            i++;
        }

        if (KNOWN_COMMANDS[i].compareTo(s) == 0) {
            return i;
        } else {
            return N_OF_COMMANDS;
        }
    }

    /**
     * Returns the eight integers x1, y1,x2,y2,x3,y3,x4,y4 contained in a string
     * of the form " # (x1, y1) (x2,y2) (x3,y3) (x4,y4) * " where * can be anything
     * and # doesn't contain any parenthesis.
     *
     * @param str the String.
     * @return the eight ints.
     */
    protected int[] getEightInts(String str) {
        return getEightInts(str, 0);
    }

    /**
     * Returns the eight integers x1, y1,x2,y2,x3,y3,x4,y4 contained in a string
     * of the form " # (x1, y1) (x2,y2) (x3,y3) (x4,y4) * " where * can be anything
     * and # doesn't contain any parenthesis.
     *
     * @param str the String.
     * @param fromIndex the index to start with.
     * @return the eight ints.
     */
    protected int[] getEightInts(String str, int fromIndex) {
        int[] ints = new int[8];

        int start = str.indexOf('(', fromIndex);
        int comma = str.indexOf(',', fromIndex);
        int stop = str.indexOf(')', fromIndex);

        ints[0] =
            Math.round(Float.parseFloat(str.substring(start + 1, comma)));
        ints[1] = Math.round(Float.parseFloat(str.substring(comma + 1, stop)));

        int offset = stop + 1;
        start = str.indexOf('(', offset);
        comma = str.indexOf(',', offset);
        stop = str.indexOf(')', offset);
        ints[2] =
            Math.round(Float.parseFloat(str.substring(start + 1, comma)));
        ints[3] = Math.round(Float.parseFloat(str.substring(comma + 1, stop)));

        offset = stop + 1;
        start = str.indexOf('(', offset);
        comma = str.indexOf(',', offset);
        stop = str.indexOf(')', offset);
        ints[4] =
            Math.round(Float.parseFloat(str.substring(start + 1, comma)));
        ints[5] = Math.round(Float.parseFloat(str.substring(comma + 1, stop)));

        offset = stop + 1;
        start = str.indexOf('(', offset);
        comma = str.indexOf(',', offset);
        stop = str.indexOf(')', offset);
        ints[6] =
            Math.round(Float.parseFloat(str.substring(start + 1, comma)));
        ints[7] = Math.round(Float.parseFloat(str.substring(comma + 1, stop)));

        return ints;
    }

    private int[] getBezierParams(int[] coords) {
        int[] out = {0, 0, 0, 0, 0, 0, 0, 0};

        out[0] = Math.round(coords[0] * getScaleFactor());
        out[1] = Math.round(getBBoxHeight() - (coords[1] * getScaleFactor()));
        out[2] = Math.round(coords[2] * getScaleFactor());
        out[3] = Math.round(getBBoxHeight() - (coords[3] * getScaleFactor()));
        out[4] = Math.round(coords[4] * getScaleFactor());
        out[5] = Math.round(getBBoxHeight() - (coords[5] * getScaleFactor()));
        out[6] = Math.round(coords[6] * getScaleFactor());
        out[7] = Math.round(getBBoxHeight() - (coords[7] * getScaleFactor()));

        return out;
    }

    private String[] getOptions(String str) {

        int start = str.indexOf('[');

        if (start == -1) {
            return new String[] {};
        }

        //A maximum of 12 options can be currently set by JD
        String[] options = new String[20];

        int stop = str.indexOf(']');
        int cindex = str.indexOf(',', start);
        int optnumb = 0;

        while (cindex < stop) {
            options[optnumb] = str.substring(start + 1, cindex);
            start = cindex;
            cindex = str.indexOf(',', start + 1);
            optnumb++;
        }

        options[optnumb] = str.substring(start + 1, stop);

        String[] result = new String[optnumb + 1];
        System.arraycopy(options, 0, result, 0, optnumb + 1);

        return result;
    }

    private boolean isArrow(String[] options) {
        boolean itis = false;
        for (int i = 0; i < options.length; i++) {
            if (options[i].equals("arrow")) {
                itis = true;
            }
        }

        return itis;
    }

    private boolean isDouble(String[] options) {
        boolean itis = false;
        for (int i = 0; i < options.length; i++) {
            if (options[i].equals("double")) {
                itis = true;
            }
        }

        return itis;
    }

    private static boolean contains(String string, String token) {
        return (string.indexOf(token) != -1);
    }

    private float[] getArrowParameters(String[] options) {
        // position/length/width/inset/scale/flip
        float[] arrowParameters = {-1, -1, -1, -1, 1, 0};

        float[] defaults = get4jArrowDefaults(options);

        for (int i = 0; i < options.length; i++) {
            int start = options[i].indexOf('=');
            int stop = options[i].length();

            if (contains(options[i], "arrowpos")) {
                arrowParameters[0] = Float.parseFloat(options[i].
                        substring(start + 1, stop));
            } else if (contains(options[i], "arrowlength")) {
                arrowParameters[1] = Float.parseFloat(options[i].
                        substring(start + 1, stop));
            } else if (contains(options[i], "arrowwidth")) {
                arrowParameters[2] = Float.parseFloat(options[i].
                        substring(start + 1, stop));
            } else if (contains(options[i], "arrowinset")) {
                arrowParameters[3] = Float.parseFloat(options[i].
                        substring(start + 1, stop));
            }  else if (contains(options[i], "arrowscale")) {
                arrowParameters[4] = Float.parseFloat(options[i].
                        substring(start + 1, stop));
            } else if (contains(options[i], "flip")) {
                arrowParameters[5] = 1;
            }
        }

        // preserve default ratio if only one of width/length is given

        if (arrowParameters[1] == -1 && arrowParameters[2] != -1) {
                arrowParameters[1] = 2.5f * arrowParameters[2];
        }

        if (arrowParameters[2] == -1 && arrowParameters[1] != -1) {
                arrowParameters[2] = arrowParameters[1] / 2.5f;
        }

        for (int j = 0; j < arrowParameters.length - 1; j++) {
            if (arrowParameters[j] == -1) {
                arrowParameters[j] = defaults[j];
            }

            if (arrowParameters[4] != 1) {
                arrowParameters[j] = arrowParameters[j] * arrowParameters[4];
            }
        }

        return arrowParameters;
    }

    private float getDashParameter(String[] options) {
        float dashSize = 0;

        for (int i = 0; i < options.length; i++) {
            int start = options[i].indexOf('=');
            int stop = options[i].length();

            if (contains(options[i], "dashsize")) {
                dashSize = Float.parseFloat(options[i].
                        substring(start + 1, stop));
            }
        }

        return dashSize;
    }

    private float getDoubleLineParameter(String[] options) {
        float dlSeparation = -1;

        for (int i = 0; i < options.length; i++) {
            int start = options[i].indexOf('=');
            int stop = options[i].length();

            if (contains(options[i], "sep")) {
                dlSeparation = Float.parseFloat(options[i].
                        substring(start + 1, stop));
            }
        }

        if (dlSeparation == -1) {
            dlSeparation = JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP);
        }

        return dlSeparation;
    }

    private boolean getClockwiseParameter(String[] options) {
        boolean isClock = false;

        for (int i = 0; i < options.length; i++) {
            if (contains(options[i], "clock")) {
                isClock = true;
            }
        }

        return isClock;
    }

    private JaxoObject newFBezier() {
        int[] pars = getBezierParams(getEightInts(string));

        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.FBEZ);
        setBezierPoints(newOb, pars);

        if (string.indexOf('[') > 0) {
            setArrow(newOb);
        } else {
            newOb.setParameter("paintArrow", false);
        }

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());

        return newOb;
    }

    private JaxoObject newSBezier() {
        int[] pars = getBezierParams(getEightInts(string));
        float dash = Float.parseFloat(getOneCurl(string)) * getScaleFactor();


        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.SBEZ);
        setBezierPoints(newOb, pars);

        if (string.indexOf('[') > 0) {
            setArrow(newOb);
        } else {
            newOb.setParameter("paintArrow", false);
        }

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());
        newOb.setParameter("dash", dash);

        return newOb;
    }

    private JaxoObject newGBezier() {
        int[] pars = getBezierParams(getEightInts(string));
        float dash = Float.parseFloat(getOneCurl(string)) * getScaleFactor();

        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.GBEZ);
        setBezierPoints(newOb, pars);

        if (string.indexOf('[') > 0) {
            setArrow(newOb);
        } else {
            newOb.setParameter("paintArrow", false);
        }

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());
        newOb.setParameter("dash", dash);

        return newOb;
    }

    private JaxoObject newPBezier() {
        int[] pars = getEightInts(string, string.indexOf('%'));
        String[] ampwig = getTwoCurls(string);
        String redshift = getOneBrace(string);

        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.PBEZ);
        setBezierPoints(newOb, pars);

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());
        newOb.setParameter("amp", Math.round(Float.parseFloat(ampwig[0])));
        newOb.setParameter("noFreqStretching", Boolean.valueOf(redshift).booleanValue());
        newOb.setParameter("frequencyFromWiggles", Math.round(Float.parseFloat(ampwig[1])));

        return newOb;
    }

    private JaxoObject newGlBezier() {
        int[] pars = getEightInts(string, string.indexOf('%'));
        String[] ampwig = getTwoCurls(string);
        String redshift = getOneBrace(string);

        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.GLBEZ);
        setBezierPoints(newOb, pars);

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());
        newOb.setParameter("amp", Math.round(Float.parseFloat(ampwig[0])));
        newOb.setParameter("noFreqStretching", Boolean.valueOf(redshift).booleanValue());
        newOb.setParameter("frequencyFromWiggles", Math.round(Float.parseFloat(ampwig[1])));

        return newOb;
    }

    private JaxoObject newA4JLine() {

        String[] options = getOptions(string);

        boolean isarr = isArrow(options);
        boolean isdoub = isDouble(options);
        float dsize = getDashParameter(options) * getScaleFactor();

        int[] pars = getLineParams(getFourInts(string), string);

        JaxoObject newOb = null;
        if (dsize == 0) {
            newOb = JaxoObjectFactory.newObject(JaxoConstants.FLINE);
        } else {
            if (dsize > 5.f) {
                newOb = JaxoObjectFactory.newObject(JaxoConstants.SLINE);
            } else {
                newOb = JaxoObjectFactory.newObject(JaxoConstants.GLINE);
            }

            newOb.setParameter("dash", dsize);
        }

        if (isarr) {
            setArrow(newOb);
        } else {
            newOb.setParameter("paintArrow", false);
        }

        if (isdoub) {
            float dlsep = getDoubleLineParameter(options) * getScaleFactor();
            newOb.setParameter("doubleLine", true);
            newOb.setParameter("dLSeparation", dlsep);
        } else {
            newOb.setParameter("doubleLine", false);
            newOb.setParameter("dLSeparation",
                JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP));
        }

        newOb.setParameter("x", pars[0]);
        newOb.setParameter("y", pars[1]);
        newOb.setParameter("relativeWidth", pars[2]);
        newOb.setParameter("relativeHeight", pars[3]);
        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());


        return newOb;
    }

    private JaxoObject newA4JArc() {

        String[] options = getOptions(string);

        boolean isarr = isArrow(options);
        boolean isdoub = isDouble(options);
        float dsize = getDashParameter(options) * getScaleFactor();
        boolean clock = getClockwiseParameter(options);

        String[] st = getArcBrackets(string);
        int phi1 = Math.round(Float.parseFloat(st[3]));
        int phi2 = Math.round(Float.parseFloat(st[4]));

        JaxoObject newOb = null;

        if (Math.abs(phi1 - phi2) >= 360) {
            //This is a loop

            if (dsize == 0) {
                newOb = JaxoObjectFactory.newObject(JaxoConstants.FLOOP);
            } else {
                if (dsize > 5.f) {
                    newOb = JaxoObjectFactory.newObject(JaxoConstants.SLOOP);
                } else {
                    newOb = JaxoObjectFactory.newObject(JaxoConstants.GLOOP);
                }

                newOb.setParameter("dash", dsize);
            }

            int[] pars = getLoopPars(st, string, clock);

            newOb.setParameter("x", pars[0]);
            newOb.setParameter("y", pars[1]);
            newOb.setParameter("relativeWidth", pars[2]);
            newOb.setParameter("relativeHeight", pars[3]);

        } else {
        //This is a genuine arc
            if (dsize == 0) {
                newOb = JaxoObjectFactory.newObject(JaxoConstants.FARC);
            } else {
                if (dsize > 5.f) {
                    newOb = JaxoObjectFactory.newObject(JaxoConstants.SARC);
                } else {
                    newOb = JaxoObjectFactory.newObject(JaxoConstants.GARC);
                }

                newOb.setParameter("dash", dsize);
            }

            setArcPoints(st, clock, newOb);
        }

        //Common to loops and arcs

        if (isarr) {
            setArrow(newOb);
        } else {
            newOb.setParameter("paintArrow", false);
        }

        if (isdoub) {
            float dlsep = getDoubleLineParameter(options) * getScaleFactor();
            newOb.setParameter("doubleLine", true);
            newOb.setParameter("dLSeparation", dlsep);
        } else {
            newOb.setParameter("doubleLine", false);
            newOb.setParameter("dLSeparation",
                JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP));
        }

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());

        return newOb;
    }

    private JaxoObject newA4JPLine() {

        String[] options = getOptions(string);

        boolean isdoub = isDouble(options);

        int[] pars = getLineParams(getFourInts(string), string);
        String[] ampwig = getTwoCurls(string);

        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.PLINE);
        newOb.setParameter("x", pars[0]);
        newOb.setParameter("y", pars[1]);
        newOb.setParameter("relativeWidth", pars[2]);
        newOb.setParameter("relativeHeight", pars[3]);
        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());

        newOb.setParameter("amp", Math.round(
                Float.parseFloat(ampwig[0]) * 2 * getScaleFactor()));

        float nOfWiggles = Float.parseFloat(ampwig[1]);
        boolean symmetric = (Math.round(nOfWiggles) - nOfWiggles) > 0.001;

        newOb.setParameter("symmetric", symmetric);
        newOb.setParameter("frequencyFromWiggles", (int) Math.floor(nOfWiggles));

        if (isdoub) {
            float dlsep = getDoubleLineParameter(options) * getScaleFactor();
            newOb.setParameter("doubleLine", true);
            newOb.setParameter("dLSeparation", dlsep);
        } else {
            newOb.setParameter("doubleLine", false);
            newOb.setParameter("dLSeparation",
                JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP));
        }

        return newOb;
    }

    private JaxoObject newA4JGlLine() {

        String[] options = getOptions(string);
        boolean isdoub = isDouble(options);

        int[] pars = getLineParams(getFourInts(string), string);
        String[] ampwig = getTwoCurls(string);

        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.GLLINE);
        newOb.setParameter("x", pars[0]);
        newOb.setParameter("y", pars[1]);
        newOb.setParameter("relativeWidth", pars[2]);
        newOb.setParameter("relativeHeight", pars[3]);
        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());

        newOb.setParameter("amp", Math.round(
                Float.parseFloat(ampwig[0]) * 2 * getScaleFactor()));

        float nOfWiggles = Float.parseFloat(ampwig[1]) * getScaleFactor();
        newOb.setParameter("frequencyFromWiggles", Math.round(nOfWiggles));

        if (isdoub) {
            float dlsep = getDoubleLineParameter(options) * getScaleFactor();
            newOb.setParameter("doubleLine", true);
            newOb.setParameter("dLSeparation", dlsep);
        } else {
            newOb.setParameter("doubleLine", false);
            newOb.setParameter("dLSeparation",
                JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP));
        }


        return newOb;
    }

    private JaxoObject newA4JPArc() {

        String[] options = getOptions(string);
        boolean isdoub = isDouble(options);
        boolean clock = getClockwiseParameter(options);

        JaxoObject newOb = null;

        String[] st = getArcBrackets(string);
        String[] ampwig = getTwoCurls(string);

        int phi1 = Math.round(Float.parseFloat(st[3]));
        int phi2 = Math.round(Float.parseFloat(st[4]));

        if (Math.abs(phi1 - phi2) >= 360) {
            // this is a loop

            int[] pars = getLoopPars(st, string, false);

            newOb = JaxoObjectFactory.newObject(JaxoConstants.PLOOP);
            newOb.setParameter("x", pars[0]);
            newOb.setParameter("y", pars[1]);
            newOb.setParameter("relativeWidth", -pars[2]);
            newOb.setParameter("relativeHeight", -pars[3]);

        } else {
            // this is a genuine arc

            newOb = JaxoObjectFactory.newObject(JaxoConstants.PARC);
            setArcPoints(st, clock, newOb);

            float nOfWiggles = Float.parseFloat(ampwig[1]);
            boolean symmetric = (Math.round(nOfWiggles) - nOfWiggles) > 0.001;

            newOb.setParameter("symmetric", symmetric);
        }

        // Common to both loops and arcs

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());

        newOb.setParameter("amp", Math.round(
                        Float.parseFloat(ampwig[0]) * 2 * getScaleFactor()));

        float nOfWiggles = Float.parseFloat(ampwig[1]); // * scaleFactor;

        if ((Math.round(nOfWiggles) - nOfWiggles) > 0.001) { //symmetric arc
            nOfWiggles = nOfWiggles - 0.5f;
        }

        newOb.setParameter("frequencyFromWiggles", Math.round(nOfWiggles));

        if (isdoub) {
            float dlsep = getDoubleLineParameter(options) * getScaleFactor();
            newOb.setParameter("doubleLine", true);
            newOb.setParameter("dLSeparation", dlsep);
        } else {
            newOb.setParameter("doubleLine", false);
            newOb.setParameter("dLSeparation",
                JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP));
        }

        return newOb;
    }

    private JaxoObject newA4JGlArc() {

        String[] options = getOptions(string);
        boolean isdoub = isDouble(options);
        boolean clock = getClockwiseParameter(options);

        JaxoObject newOb = null;

        String[] st = getArcBrackets(string);
        String[] ampwig = getTwoCurls(string);

        int phi1 = Math.round(Float.parseFloat(st[3]));
        int phi2 = Math.round(Float.parseFloat(st[4]));

        if (Math.abs(phi1 - phi2) >= 360) {
            //this is a loop

            int[] pars = getLoopPars(st, string, false);

            newOb = JaxoObjectFactory.newObject(JaxoConstants.GLLOOP);
            newOb.setParameter("x", pars[0]);
            newOb.setParameter("y", pars[1]);
            newOb.setParameter("relativeWidth", -pars[2]);
            newOb.setParameter("relativeHeight", -pars[3]);
            newOb.setParameter("amp", Math.round(
                Float.parseFloat(ampwig[0]) * 2 * getScaleFactor()));

        } else {
            //this is a genuine arc

            newOb = JaxoObjectFactory.newObject(JaxoConstants.GLARC);
            setArcPoints(st, clock, newOb);

            newOb.setParameter("amp", Math.round(
                Float.parseFloat(ampwig[0]) * 2 * getScaleFactor()));
        }

        //common to both arcs and loops

        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());

        float nOfWiggles = Float.parseFloat(ampwig[1]) * getScaleFactor();
        newOb.setParameter("frequencyFromWiggles", Math.round(nOfWiggles));

        if (isdoub) {
            float dlsep = getDoubleLineParameter(options) * getScaleFactor();
            newOb.setParameter("doubleLine", true);
            newOb.setParameter("dLSeparation", dlsep);
        } else {
            newOb.setParameter("doubleLine", false);
            newOb.setParameter("dLSeparation",
                JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP));
        }

        return newOb;
    }

    private JaxoObject newVertexT4() { // same as in JaxoAxodraw
        int[] firstl = getFourInts(string);
        int[] secondl = getFourInts(string, string.lastIndexOf("\\Line"));

        int[] pars = getT4Params(firstl, secondl);

        JaxoObject newOb = JaxoObjectFactory.newObject(JaxoConstants.VERTEXT4);
        newOb.setParameter("x", pars[0]);
        newOb.setParameter("y", pars[1]);
        newOb.setParameter("relativeWidth", pars[2]);
        newOb.setParameter("relativeHeight", pars[3]);
        newOb.setParameter("strokeWidth", getStroke());
        newOb.setParameter("color", getColor());

        return newOb;
    }

    private float[] get4jArrowDefaults(String[] options) {
        float[] defaults = {0, 0, 0, 0, 1, 0};
        // position/length/width/inset/scale/flip
        float dlsep = 0;
        if (isDouble(options)) {
            dlsep = getDoubleLineParameter(options);
        }

        defaults[0] = 0.5f; // position
        defaults[2] = 1.2f * (2 + 0.7f * dlsep + getStroke()); // width
        defaults[1] = 2.5f * defaults[2]; // length
        defaults[3] = 0.2f; // inset
        defaults[4] = 1; // scale
        defaults[5] = 0; // flip

        return defaults;
    }

    private void setArcPoints(String[] st, boolean clock, JaxoObject newOb) {
        Point2D[] pts = getArcPoints(st, clock);
        newOb.setParameter("x", (int) Math.round(pts[0].getX()));
        newOb.setParameter("y", (int) Math.round(pts[0].getY()));
        newOb.setParameter("x2", (int) Math.round(pts[1].getX()));
        newOb.setParameter("y2", (int) Math.round(pts[1].getY()));
        newOb.setParameter("x3", (int) Math.round(pts[2].getX()));
        newOb.setParameter("y3", (int) Math.round(pts[2].getY()));
    }

    private void setArrow(JaxoObject newOb) {
        String[] options = getOptions(string);
        float[] arrowParameters = getArrowParameters(options);
        JaxoDefaultArrow arrow = new JaxoDefaultArrow();
        arrow.setArrowLength(arrowParameters[1]);
        arrow.setArrowWidth(arrowParameters[2]);
        arrow.setArrowInset(arrowParameters[3]);
        newOb.setParameter("arrow", JaxoArrow.class, arrow);
        newOb.setParameter("paintArrow", true);
        newOb.setParameter("arrowPosition", arrowParameters[0]);
        if (arrowParameters[5] == 1) {
            newOb.setParameter("flip", true);
        }
    }

    private void setBezierPoints(JaxoObject newOb, int[] pars) {
        newOb.setParameter("x", pars[0]);
        newOb.setParameter("y", pars[1]);
        newOb.setParameter("x2", pars[2]);
        newOb.setParameter("y2", pars[3]);
        newOb.setParameter("x3", pars[4]);
        newOb.setParameter("y3", pars[5]);
        newOb.setParameter("x4", pars[6]);
        newOb.setParameter("y4", pars[7]);
    }
}
