/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.logging;

import java.io.File;
import java.io.IOException;

import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;


/**
 * Default implementation of JaxoLogger.
 *
 * This adds a ConsoleHandler (ie logging to standard error) whose logging
 * level is adjustable, and a FileHandler (logging to a rotating sequence
 * of log files in a given directory) which always logs everything.
 *
 * @since 2.0
 */
public final class JaxoDefaultLogger implements JaxoLogger {

    private static final Logger LOGGER = Logger.getLogger("net.sf.jaxodraw");
    private final JaxoConsoleHandler consoleHandler = new JaxoConsoleHandler();

    /**
     * Constructor. Only adds a JaxoConsoleHandler.
     */
    public JaxoDefaultLogger() {
        this(null);
    }

    /**
     * Constructor. Adds a JaxoConsoleHandler and a JaxoFileHandler.
     *
     * @param logDir The directory where log files should be stored.
     *      If this doesn't exist, it will be created.
     *      If logDir is null, no FileHandler is added.
     */
    public JaxoDefaultLogger(String logDir) {
        // keep the logger level at finest, only set level of handlers
        LOGGER.setLevel(Level.ALL);
        LOGGER.setUseParentHandlers(false);

        LOGGER.addHandler(consoleHandler);

        if (logDir != null) {
            addFileHandler(logDir);
        }
    }


    /**
     * Sets the logging level. Forwards the Level to the ConsoleHandler,
     * the JaxoFileHandler should always log everything.
     *
     * @param level the log level to set.
     */
    public void setLevel(Level level) {
        consoleHandler.setLevel(level);
    }

    /** {@inheritDoc} */
    public Level getLevel() {
        return consoleHandler.getLevel();
    }

    /** {@inheritDoc} */
    public boolean isLoggable(Level level) {
        //return (getLevel().intValue() <= level.intValue());
        return consoleHandler.isLoggable(new LogRecord(level, null));
    }

    /** {@inheritDoc} */
    public void log(Level level, String message, Throwable e) {
        if (e == null) {
            if (message == null) {
                throw new NullPointerException("Nothing to log?");
            } else {
                LOGGER.log(level, message);
            }
        } else {
            LOGGER.log(level, (message == null ? e.getMessage() : message), e);
            if (getLevel().intValue() <= Level.FINE.intValue()) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Formats log records in standard JaxoDraw format.
     */
    public static class LogFormatter extends SimpleFormatter {
        /**
         * Formats the given LogRecord to standard JaxoDraw format.
         *
         * @param rec the LogRecord to format.
         * @return the formatted LogRecord.
         */
        public String format(LogRecord rec) {
            String prefix = "";
            String nl = System.getProperty("line.separator");
            int level = rec.getLevel().intValue();
            if (level == Level.INFO.intValue()) {
                prefix = "[INFO] ";
            } else if (level == Level.FINE.intValue()) {
                prefix = "[DEBUG] ";
            } else if (level == Level.WARNING.intValue()) {
                prefix = "[WARNING] ";
            }

            StringBuffer buf = new StringBuffer(1000);
            String[] lines = formatMessage(rec).split(nl, -1);
            for (int i = 0; i < lines.length; i++) {
                buf.append(prefix);
                buf.append(lines[i]);
                buf.append(nl);
            }
            return buf.toString();
        }
    }

      //
     // private
    //

    // create log dir if it doesn't exist
    private boolean createLogDir(File logDir) throws IOException {
        if (!logDir.exists()) {
            if (logDir.mkdirs()) {
                return true;
            } else {
                throw new IOException("Failed to create log dir: "
                        + logDir.getAbsolutePath());
            }

        }

        return false;
    }

    private void addFileHandler(String logDir) {
        try {
            // make sure log dir exists
            boolean dirCreated = createLogDir(new File(logDir));

            LOGGER.addHandler(new JaxoFileHandler(logDir));

            if (dirCreated) {
                LOGGER.log(Level.WARNING, "Created log directory: " + logDir);
            }
        } catch (IOException ex) {
            LOGGER.log(Level.WARNING, "Failed to initialize FileHandler!", ex);
            if (getLevel().intValue() <= Level.FINE.intValue()) {
                ex.printStackTrace();
            }
        }
    }

}
