/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object;

import java.awt.Color;
import java.awt.Rectangle;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;

import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;

/**
 * Implementation of JaxoList. This should ensure that only JaxoObjects are
 * added as elements by throwing a ClassCastException if something else is
 * attempted.
 *
 * @since 2.0
 */
public class JaxoObjectList extends ArrayList implements JaxoList {
    private static final long serialVersionUID = 314159L;

    private static final String CLASS_CAST_EXCEPTION_MESSAGE =
        "Trying to add non-JaxoObject to JaxoList: ";

      //
     // Constructors
    //

    /**
     * Just calls super().
     */
    public JaxoObjectList() {
        super();
    }

    /**
     * Add all elements of c to this List. Throws a ClassCastException
     * if c contains elements that are not JaxoObjects.
     * @param c the collection whose elements are to be placed into this list.
     */
    public JaxoObjectList(Collection c) {
        super(c.size());
        for (Iterator it = c.iterator(); it.hasNext();) {
            super.add((JaxoObject) it.next());
        }
    }

    /**
     * Just calls super(initialCapacity).
     * @param initialCapacity the initial capacity of the list.
     */
    public JaxoObjectList(int initialCapacity) {
        super(initialCapacity);
    }

      //
     // override methods to ensure only JaxoObjects are added as elements
    //

    /**
     * Add a JaxoObject to this list.
     * A ClassCastException is thrown if o is not a JaxoObject.
     *
     * @param o the object to add.
     * @return true.
     * @see java.util.ArrayList#add(java.lang.Object)
     */
    public boolean add(Object o) {
        return super.add((JaxoObject) o);
    }

    /**
     * Add a JaxoObject to this list.
     * A ClassCastException is thrown if o is not a JaxoObject.
     *
     * @param index index at which the object is to be inserted.
     * @param element the object to add.
     * @see java.util.ArrayList#add(int,java.lang.Object)
     */
    public void add(int index, Object element) {
        super.add(index, (JaxoObject) element);
    }

    /**
     * Add a Collection of JaxoObjects to this list.
     * A ClassCastException is thrown if c contains non-JaxoObjects.
     *
     * @param c the collection to add.
     * @return true if this list is changed as a result of the call.
     * @see java.util.ArrayList#addAll(java.util.Collection)
     */
    public boolean addAll(Collection c) {
        checkCollection(c);
        return super.addAll(c);
    }

   /**
    * Add a Collection of JaxoObjects to this list.
    * A ClassCastException is thrown if c contains non-JaxoObjects.
    *
    * @param index index at which the object is to be inserted.
    * @param c the collection to add.
    * @return true if this list is changed as a result of the call.
    * @see java.util.ArrayList#addAll(int,java.util.Collection)
    */
   public boolean addAll(int index, Collection c) {
        checkCollection(c);
        return super.addAll(index, c);
    }

    private void checkCollection(Collection c) {
        for (Iterator it = c.iterator(); it.hasNext();) {
            Object ob = it.next();
            if (!(ob instanceof JaxoObject)) {
                throw new ClassCastException(CLASS_CAST_EXCEPTION_MESSAGE
                    + ob.getClass().getName());
            }
        }
    }

    /**
     * Replace the object at the given position with the specified object.
     * A ClassCastException is thrown if element is not a JaxoObject.
     *
     * @param index index of the object to be replaced.
     * @param element the object to be stored at the given position.
     * @return the object that was previously at the specified position.
     * @see java.util.ArrayList#set(int,java.lang.Object)
     */
    public Object set(int index, Object element) {
        return super.set(index, (JaxoObject) element);
    }

      //
     // Painting methods
    //


    /** {@inheritDoc} */
    public void paint(JaxoGraphics2D g) {
        for (Iterator i = this.iterator(); i.hasNext();) {
            ((JaxoObject) i.next()).paint(g);
        }
    }

    /** {@inheritDoc} */
    public void paintClipped(JaxoGraphics2D g) {
        for (Iterator i = this.iterator(); i.hasNext();) {
            JaxoObject o = (JaxoObject) i.next();

            Rectangle b = o.getBounds();

            if (((b.width == 0) && (b.height == 0)) // LEGACY
                    || g.hitClip(b.x, b.y, b.width, b.height)) {
                o.paint(g);
            }
        }
    }

    /** {@inheritDoc} */
    public void paintExcept(Collection excludes, JaxoGraphics2D g) {
        for (Iterator i = this.iterator(); i.hasNext();) {
            JaxoObject o = (JaxoObject) i.next();

            if (!excludes.contains(o)) {
                o.paint(g);
            }
        }
    }

    /** {@inheritDoc} */
    public void paintClippedExcept(Collection excludes, JaxoGraphics2D g) {
        for (Iterator i = this.iterator(); i.hasNext();) {
            JaxoObject o = (JaxoObject) i.next();

            if (!excludes.contains(o)) {
                Rectangle b = o.getBounds();

                if (((b.width == 0) && (b.height == 0)) // LEGACY
                        || g.hitClip(b.x, b.y, b.width, b.height)) {
                    o.paint(g);
                }
            }
        }
    }

    /** {@inheritDoc} */
    public JaxoList copyOf() {
        int size = size();
        JaxoList copyObjectList = new JaxoObjectList(size);
        for (int i = 0; i < size; i++) {
            copyObjectList.add(((JaxoObject) get(i)).copy());
        }
        return copyObjectList;
    }

    /** {@inheritDoc} */
    public boolean isCopy(JaxoList list) {
        if (list == null) {
            return false;
        }

        // Two lists are copies if they contain
        // copies of objects at equal positions
        boolean isCopy = false;

        int length = size();
        if (length == list.size()) {
            boolean tmp = true;
            for (int i = 0; i < length; i++) {
                JaxoObject ob1 = (JaxoObject) get(i);
                JaxoObject ob2 = (JaxoObject) list.get(i);
                tmp = tmp && (ob1 == null ? ob2 == null : ob1.isCopy(ob2));
                if (!tmp) {
                    break;
                }
            }
            isCopy = tmp;
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public void moveAllObjects(int deltaX, int deltaY) {
        if ((deltaX == 0) && (deltaY == 0)) {
            return;
        }

        for (int i = 0; i < size(); i++) {
            ((JaxoObject) get(i)).moveBy(deltaX, deltaY);
        }
    }

    /** {@inheritDoc} */
    public void setColor(Color c) {
        for (int i = 0; i < size(); i++) {
            ((JaxoObject) get(i)).setColor(c);
        }
    }

    /** {@inheritDoc} */
    public void move(int index, int newIndex) {
        if (index == newIndex) {
            return;
        }

        JaxoObject o = (JaxoObject) remove(index);
        add(newIndex, o);
    }

    /** {@inheritDoc} */
    public boolean toEnd(JaxoObject object) {
        boolean isLastElement = true;

        // Remove object from current list and append it
        // Start searching at the end - that way repeated
        // foreground() calls will be faster
        int i = lastIndexOf(object);

        if (i != (size() - 1)) {
            remove(i);
            add(object);
            isLastElement = false;
        }

        return isLastElement;
    }

    /** {@inheritDoc} */
    public boolean toFront(JaxoObject object) {
        boolean isFirstElement = true;

        // Remove object from current list and prepend it
        int i = indexOf(object);

        if (i != 0) {
            remove(i);
            add(0, object);
            isFirstElement = false;
        }

        return isFirstElement;
    }

    /** {@inheritDoc} */
    public boolean removeMarkedObjects() {
        boolean modified = false;
        for (int i = size() - 1; i > -1; i--) {
            JaxoObject ob = (JaxoObject) get(i);

            if (ob.isMarked()) {
                modified = remove(ob);
            }
        }
        return modified;
    }

    /** {@inheritDoc} */
    public Rectangle getBounds() {
        Rectangle bBox = null;
        int length = this.size();

        for (int i = 0; i < length; i++) {
            JaxoObject ob = (JaxoObject) get(i);
            Rectangle add = ob.getBounds();
            if (bBox == null) {
                bBox = new Rectangle(add);
            } else {
                bBox.add(add);
            }
        }

        return bBox;
    }

    /** {@inheritDoc} */
    public Rectangle getBoundsExcept(Collection excludes) {
        Rectangle bBox = null;
        int length = this.size();

        for (int i = 0; i < length; i++) {
            JaxoObject ob = (JaxoObject) get(i);

            if (excludes.contains(ob)) {
                continue;
            }

            Rectangle add = ob.getBounds();
            if (bBox == null) {
                bBox = new Rectangle(add);
            } else {
                bBox.add(add);
            }
        }

        return bBox;
    }

    /** {@inheritDoc} */
    public Rectangle intersection(Rectangle inside) {
        Rectangle intersect = null;
        Rectangle bounds = getBounds();
        if ((inside != null) && (bounds != null)) {
            intersect = inside.intersection(bounds);
            if (intersect.isEmpty()) {
                intersect = null;
            }
        }
        return intersect;
    }

}
