/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.arc;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;

import java.io.IOException;
import java.io.ObjectInputStream;

import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.object.JaxoWiggleObject;
import net.sf.jaxodraw.object.line.JaxoGlLine;
import net.sf.jaxodraw.util.JaxoUtils;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** A gluon arc.
 * @since 2.0
 */
public class JaxoGlArc extends JaxoArcObject implements JaxoWiggleObject {
    private static final long serialVersionUID = 314159L;
    private transient float freq;

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        freq = 0.f;
    }

    /** Sets all parameters from the given object to the current one.
     * @param temp The object to copy from.
     */
    public void copyFrom(JaxoGlArc temp) {
        super.copyFrom(temp);
        this.freq = temp.getFrequency();
    }

    /** {@inheritDoc} */
    public void setState(JaxoObject o) {
        copyFrom((JaxoGlArc) o);
    }

    /** {@inheritDoc} */
    public final boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoGlArc) {
            isCopy = super.isCopy(comp);
        }

        return isCopy;

    }

    /** {@inheritDoc} */
    public final void paint(JaxoGraphics2D g2) {
        GeneralPath gp = getObjectPath();
        if (gp != null) {
            g2.setColor(getColor());
            g2.setStroke(getStroke());
            g2.draw(gp);
        }

    }

    /**
     * Returns the bounding box of this object.
     *
     * @return the bounding box of this object.
     */
    public Rectangle getBounds() {
        GeneralPath gp = getObjectPath();
        if (gp == null) {
            // singular arc: return a Rectangle that just contains
            // the three arc points to avoid NPE
            Rectangle r = new Rectangle(getX(), getY(), 0, 0);
            r.add(getX2(), getY2());
            r.add(getX3(), getY3());
            return r;
        } else {
            return getStroke().createStrokedShape(getObjectPath()).getBounds();
        }
    }

    /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        if (isSingular()) {
            return "% Singular GluonArc, ignored!" + JaxoUtils.LINE_SEPARATOR;
        }

        double nwin = (double) getWiggles();

        if (nwin > 1000) {
            return "% Enormous GluonArc, ignored!" + JaxoUtils.LINE_SEPARATOR;
        }

        if (isOneLine()) {
            JaxoGlLine line = new JaxoGlLine();
            line.setLocation(getX(), getY(), getX3(), getY3());
            line.setStrokeWidth(getStrokeWidth());
            line.setAmp(getAmp());
            line.setWiggles(getWiggles());
            line.setDoubleLine(isDoubleLine());
            line.setDLSeparation(getDLSeparation());
            return line.latexCommand(scale, canvasDim);
        }

        double[] par = getArcParameters();
        double cx = par[0] / scale;
        double cy = (canvasDim.height - par[1]) / scale;
        double r = par[2] / scale;
        double ea = par[3] + par[4];
        double sa = par[4];
        boolean clockwise = (par[3] < 0.d);
        double amp = getAmp() / (2.d * scale);

        String command = "";
        String base = "\\GluonArc";
        String options = getOptionsCommand(clockwise);

        command =
            base + options
            + "(" + D_FORMAT.format(cx) + "," + D_FORMAT.format(cy)
            + ")" + "(" + D_FORMAT.format(r) + "," + D_FORMAT.format(sa)
            + "," + D_FORMAT.format(ea) + ")" + "{" + D_FORMAT.format(amp)
            + "}" + "{" + D_FORMAT.format(nwin) + "}";

        return command;
    }

    /** {@inheritDoc} */
    public float getFrequency() {
        return freq;
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency(float frequency) {
        this.freq = frequency;
        setWigglesFromFrequency();
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency() {
        float f = getFrequency();
        double arcLength = getArcLength();
        int n = (int) Math.round(arcLength * f);
        setWiggles(n);
    }

    /** {@inheritDoc} */
    public void setFrequencyFromWiggles(int wiggles) {
        setWiggles(wiggles);
        this.freq = (float) (wiggles / getArcLength());
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        setWigglesFromFrequency(GLUON_FREQ);
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        editPanel.add3PointsPanel(getPoints(), 0, 0, 2);
        editPanel.addDoubleLinePanel(isDoubleLine(), getDLSeparation(), 2, 0);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 1);
        editPanel.addWigglePanel(getAmp(), getWiggles(), 1, 1);
        editPanel.addLineColorPanel(getColor(), 2, 1);

        editPanel.setTitleAndIcon("Gluon_arc_parameters", "gluonarc.png");
    }

      //
     //   private methods
    //

    private GeneralPath getObjectPath() {
        if (isSingular()) {
            return null;
        }

        GeneralPath path = getGeneralPath();
        path.reset();

        if (JaxoUtils.zero(freq)) {
            setFrequencyFromWiggles(getWiggles());
        } else {
            setWigglesFromFrequency();
        }

        int wiggles = getWiggles();

        // cutoff
        if (wiggles > 1000) {
            return null;
        }

        if (isOneLine()) {
            return gluonLine(path);
        }

        double[] par = getArcParameters();
        float cx = (float) par[0];
        float cy = (float) par[1];
        float radius = (float) par[2];
        float oa = (float) Math.abs(Math.toRadians(par[3]));
        float sa = (float) -Math.toRadians(par[4]);
        boolean clockwise = (par[3] < 0.d);
        float amp = 0.5f * getAmp();

        // draw the arc
        if (isDoubleLine()) {
            float ds = getDLSeparation() / 2.f;

            appendFullArc(path, radius + ds, wiggles, amp, sa, oa, cx, cy, clockwise);
            appendFullArc(path, radius - ds, wiggles, amp, sa, oa, cx, cy, clockwise);
        } else {
            appendFullArc(path, radius, wiggles, amp, sa, oa, cx, cy, clockwise);
        }

        return path;
    }

    private void appendFullArc(GeneralPath gp, float rad, int n,
            float a, float sa, float oa, float cx, float cy, boolean clock) {
        float cos = (float) Math.cos(sa);
        float sin = (float) Math.sin(sa);

        float r = rad;

        float absa = Math.abs(a);
        float thetaj;
        float delta = oa / (n + 1);

        float y = -1.f;
        if (clock) {
            y = 1.f;
        }

        // head part
        gp.moveTo(r * cos + cx, r * sin + cy);
        polarCurveTo(gp,
                r + 0.5f * a,
                y * (0.05f * delta),
                (float) ((r + a) / Math.cos(0.45d * delta)),
                y * (0.25f * delta),
                r + a,
                y * (0.7f * delta),
                cos, sin, cx, cy);
        polarCurveTo(gp,
                (float) ((r + a) / Math.cos(0.3d * delta + 0.9d * absa / r)),
                y * (delta + 0.9f * absa / r),
                (float) ((r - a) / Math.cos(0.9d * absa / r)),
                y * (delta + 0.9f * absa / r),
                r - a,
                y * delta,
                cos, sin, cx, cy);

        // middle part
        for (int j = 2; j <= n; j++) {
            thetaj = (j - 1) * delta;
            polarCurveTo(gp,
                    (float) ((r - a) / Math.cos(0.9d * absa / r)),
                    y * (thetaj - 0.9f * absa / r),
                    (float) ((r + a) / Math.cos(0.5d * delta + 0.9d * absa / r)),
                    y * (thetaj - 0.9f * absa / r),
                    r + a,
                    y * (thetaj + 0.5f * delta),
                    cos, sin, cx, cy);
            polarCurveTo(gp,
                    (float) ((r + a) / Math.cos(0.5d * delta + 0.9d * absa / r)),
                    y * (thetaj + delta + 0.9f * absa / r),
                    (float) ((r - a) / Math.cos(0.9d * absa / r)),
                    y * (thetaj + delta + 0.9f * absa / r),
                    r - a,
                    y * (thetaj + delta),
                    cos, sin, cx, cy);
        }

        // tail part
        polarCurveTo(gp,
                (float) ((r - a) / Math.cos(0.9d * absa / r)),
                y * (oa - delta - 0.9f * absa / r),
                (float) ((r + a) / Math.cos(0.3d * delta + 0.9d * absa / r)),
                y * (oa - delta - 0.9f * absa / r),
                r + a,
                y * (oa - 0.7f * delta),
                cos, sin, cx, cy);
        polarCurveTo(gp,
                (float) ((r + a) / Math.cos(0.45d * delta)),
                y * (oa - 0.25f * delta),
                r + 0.5f * a,
                y * (oa - 0.05f * delta),
                r,
                y * oa,
                cos, sin, cx, cy);
    }

    private void polarCurveTo(GeneralPath gp, float r1, float th1, float r2, float th2,
            float r3, float th3, float cos, float sin, float cx, float cy) {
        float x1 = (float) (r1 * Math.cos(th1));
        float y1 = (float) (r1 * Math.sin(th1));
        float x2 = (float) (r2 * Math.cos(th2));
        float y2 = (float) (r2 * Math.sin(th2));
        float x3 = (float) (r3 * Math.cos(th3));
        float y3 = (float) (r3 * Math.sin(th3));

        gp.curveTo(x1 * cos - y1 * sin + cx, x1 * sin + y1 * cos + cy,
                x2 * cos - y2 * sin + cx, x2 * sin + y2 * cos + cy,
                x3 * cos - y3 * sin + cx, x3 * sin + y3 * cos + cy);
    }

    private GeneralPath gluonLine(GeneralPath gp) {
        if (isDoubleLine()) {
            float sep = getDLSeparation() / 2.f;
            appendFullLine(gp, sep);
            appendFullLine(gp, -sep);
        } else {
            appendFullLine(gp, 0.f);
        }

        return gp;
    }

    // should be re-used from GlLine
    private void appendFullLine(GeneralPath gp, float sep) {
        float length = (float) Math.sqrt(((getX3() - getX()) * (getX3() - getX()))
                + ((getY3() - getY()) * (getY3() - getY())));
        int n = getWiggles();
        float delta = length / (n + 1);
        float amp = -0.5f * getAmp();
        float absamp = Math.abs(amp);

        float sin = (getY3() - getY()) / length;
        float cos = (getX3() - getX()) / length;

        gp.moveTo(getX(), getY() + sep);

        float x1 = 0.05f * delta;
        float x2 = 0.25f * delta;
        float x3 = 0.7f * delta;
        float y1 = 0.5f * amp + sep;
        float y2 = amp + sep;
        float y3 = amp + sep;

        curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

        x1 = delta + 0.9f * absamp;
        x2 = delta + 0.9f * absamp;
        x3 = delta;
        y1 = amp + sep;
        y2 = -amp + sep;
        y3 = -amp + sep;

        curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

        float xj;

        for (int j = 1; j < n; j++) {
            xj = j * delta;
            x1 = xj - 0.9f * absamp;
            x2 = xj - 0.9f * absamp;
            x3 = xj + 0.5f * delta;
            y1 = -amp + sep;
            y2 = amp + sep;
            y3 = amp + sep;
            curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

            x1 = xj + 0.9f * absamp + delta;
            x2 = xj + 0.9f * absamp + delta;
            x3 = xj + delta;
            y1 = amp + sep;
            y2 = -amp + sep;
            y3 = -amp + sep;
            curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);
        }

        x1 = length - delta - 0.9f * absamp;
        x2 = length - delta - 0.9f * absamp;
        x3 = length - 0.7f * delta;
        y1 = -amp + sep;
        y2 = amp + sep;
        y3 = amp + sep;
        curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

        x1 = length - 0.25f * delta;
        x2 = length - 0.05f * delta;
        x3 = length;
        y1 = amp + sep;
        y2 = 0.5f * amp + sep;
        y3 = 0.f + sep;
        curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);
    }

    private void curveTo(GeneralPath gp, float x1, float y1,
            float x2, float y2, float x3, float y3, float cos, float sin) {

        gp.curveTo((x1 * cos) - (y1 * sin) + getX(), (x1 * sin) + (y1 * cos) + getY(),
                (x2 * cos) - (y2 * sin) + getX(), (x2 * sin) + (y2 * cos) + getY(),
                (x3 * cos) - (y3 * sin) + getX(), (x3 * sin) + (y3 * cos) + getY());
    }

    // Get the axodraw4j options set for this line

    private String getOptionsCommand(boolean clockwise) {
        StringBuffer buffer = new StringBuffer();

        if (isDoubleLine()) {
            String dlsep = D_FORMAT.format(getDLSeparation());
            buffer.append("double,sep=").append(dlsep);
        }

        if (clockwise) {
            if (buffer.length() > 0) {
                buffer.append(',');
            }

            buffer.append("clock");
        }

        if (buffer.length() > 0) {
            buffer.insert(0, '[');
            buffer.append(']');
        }

        return buffer.toString();
    }

}
