/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.bezier;

import java.awt.geom.Point2D;

import net.sf.jaxodraw.object.Jaxo3PointObject;
import net.sf.jaxodraw.object.JaxoArrow;
import net.sf.jaxodraw.object.JaxoHandle;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoGeometry;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** Abstract class for the implementation of a bezier curve.
 * @since 2.0
 */
public abstract class JaxoBezierObject extends Jaxo3PointObject {
    /** Fourth point (index 3). */
    public static final int SELECT_P4 = 3;
    private static final int POINT_COUNT = 4;

    static {
        setTransient(JaxoBezierObject.class, new String[] {"x4", "y4"});
    }

    /** The fourth points defining this Bezier object.*/
    private int x4;

    /** The fourth points defining this Bezier object.*/
    private int y4;

    /** Indicates whether frequency stretching is allowed for this Bezier. */
    private boolean noFreqStretching = true;

    /** {@inheritDoc}
     * @return 4.
     */
    public int getPointCount() {
        return POINT_COUNT;
    }

    /** {@inheritDoc} */
    public int getX(int index) {
        if (index == SELECT_P4) {
            return x4;
        }
        return super.getX(index);
    }

    /** {@inheritDoc} */
    public int getY(int index) {
        if (index == SELECT_P4) {
            return y4;
        }
        return super.getY(index);
    }

    /** {@inheritDoc} */
    public void setX(int index, int value) {
        if (index == SELECT_P4) {
            x4 = value;
        } else {
            super.setX(index, value);
        }
    }

    /** {@inheritDoc} */
    public void setY(int index, int value) {
        if (index == SELECT_P4) {
            y4 = value;
        } else {
            super.setY(index, value);
        }
    }

    // Bean getter and setter methods

    /** Sets the frequency stretching property of this Bezier.
     * @param nofs True if this Bezier is not frequency stretched.
     */
    public final void setNoFreqStretching(boolean nofs) {
        this.noFreqStretching = nofs;
    }

    /** Returns the frequency stretching property of this Bezier.
     * @return  True if this Bezier is not frequency stretched.
     */
    public final boolean isNoFreqStretching() {
        return noFreqStretching;
    }

    /** Sets the x coordinate of the fourth click point of this Bezier.
     * @param newx4 The x coordinate of the fourth click point of this Bezier.
     */
    public final void setX4(int newx4) {
        this.x4 = newx4;
    }

    /** Returns the x coordinate of the fourth click point of this Bezier.
     * @return The x coordinate of the fourth click point of this Bezier.
     */
    public final int getX4() {
        return x4;
    }

    /** Sets the y coordinate of the fourth click point of this Bezier.
     * @param newy4 The y coordinate of the fourth click point of this Bezier.
     */
    public final void setY4(int newy4) {
        this.y4 = newy4;
    }

    /** Returns the y coordinate of the fourth click point of this Bezier.
     * @return The y coordinate of the fourth click point of this Bezier.
     */
    public final int getY4() {
        return y4;
    }

    /** Sets the points of this Bezier.
     * @param sx1 The x coordinate of the first click point of this Bezier.
     * @param sy1 The y coordinate of the first click point of this Bezier.
     * @param sx2 The x coordinate of the second click point of this Bezier.
     * @param sy2 The y coordinate of the second click point of this Bezier.
     * @param sx3 The x coordinate of the third click point of this Bezier.
     * @param sy3 The y coordinate of the third click point of this Bezier.
     * @param sx4 The x coordinate of the fourth click point of this Bezier.
     * @param sy4 The y coordinate of the fourth click point of this Bezier.
     */
    public final void setBezierPts(int sx1, int sy1, int sx2, int sy2,
        int sx3, int sy3, int sx4, int sy4) {
        set3Pts(sx1, sy1, sx2, sy2, sx3, sy3);
        setX4(sx4);
        setY4(sy4);
    }

    /** {@inheritDoc} */
    public boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoBezierObject) {
            JaxoBezierObject tmp = (JaxoBezierObject) comp;
            if ((tmp.getX4() == getX4()) && (tmp.getY4() == getY4())
                    && (tmp.isNoFreqStretching() == isNoFreqStretching())
                    && super.isCopy(tmp)) {
                isCopy = true;
            }
        }

        return isCopy;
    }

    /** Sets all parameters from the given object to the current one.
     * @param temp The object to copy from.
     */
    public void copyFrom(JaxoBezierObject temp) {
        super.copyFrom(temp);
        setBezierPts(temp.getX(), temp.getY(), temp.getX2(), temp.getY2(),
            temp.getX3(), temp.getY3(), temp.getX4(), temp.getY4());
        setNoFreqStretching(temp.isNoFreqStretching());
    }

    /** {@inheritDoc} */
    public void setState(JaxoObject o) {
        copyFrom((JaxoBezierObject) o);
    }

    /** {@inheritDoc} */
    public final void paintVisualAid(JaxoGraphics2D g2) {
        g2.setColor(JaxoGraphics2D.DEFAULT_HELP_COLOR);
        g2.setStroke(JaxoGraphics2D.DEFAULT_HELP_STROKE);

        g2.drawLine(getX(), getY(), getX2(), getY2());
        g2.drawLine(getX2(), getY2(), getX3(), getY3());
        g2.drawLine(getX3(), getY3(), getX4(), getY4());
        g2.drawLine(getX4(), getY4(), getX(), getY());
    }

    /** {@inheritDoc} */
    public final void paintHandles(JaxoGraphics2D g2, JaxoHandle h,
        int editMode) {
        if (editMode == JaxoConstants.UNGROUP) {
            return;
        }

        boolean secondary = (editMode != JaxoConstants.RESIZE);

        h.paint(g2, getX(), getY(), isMarked(), false);
        h.paint(g2, getX2(), getY2(), isMarked(), secondary);
        h.paint(g2, getX3(), getY3(), isMarked(), secondary);
        h.paint(g2, getX4(), getY4(), isMarked(), false);
    }

    /** {@inheritDoc} */
    public final float smallestDistanceTo(int px, int py) {
        float dist1 = super.smallestDistanceTo(px, py);
        int distX = px - getX4();
        int distY = py - getY4();
        float dist2 = (float) Math.sqrt((distX * distX) + (distY * distY));
        return (dist1 < dist2) ? dist1 : dist2;
    }

    /** {@inheritDoc} */
    public int getGrabbedHandle(int clickX, int clickY, JaxoHandle h) {
        int selected = super.getGrabbedHandle(clickX, clickY, h);

        if (isAround(SELECT_P4, clickX, clickY, h)) {
            selected = SELECT_P4;
        }

        return selected;
    }

    /** {@inheritDoc} */
    public boolean canBeSelected(int handle, int mode) {
        boolean active = ((handle == SELECT_P1) || (handle == SELECT_P4));
        if (mode == JaxoConstants.RESIZE) {
            active = (
                    (handle == SELECT_P1) || (handle == SELECT_P2)
                    || (handle == SELECT_P3) || (handle == SELECT_P4)
                );
        }
        return active;
    }

    /** {@inheritDoc} */
    public final void moveBy(int deltaX, int deltaY) {
        if ((deltaX != 0) || (deltaY != 0)) {
            super.moveBy(deltaX, deltaY);
            setX4(getX4() + deltaX);
            setY4(getY4() + deltaY);
        }
    }

    /** {@inheritDoc} */
    public final void rescaleObject(int orx, int ory, float scale) {
        Point2D p1 = JaxoGeometry.scaledPoint(orx, ory, scale, getX(), getY());
        Point2D p2 =
            JaxoGeometry.scaledPoint(orx, ory, scale, getX2(), getY2());
        Point2D p3 =
            JaxoGeometry.scaledPoint(orx, ory, scale, getX3(), getY3());
        Point2D p4 =
            JaxoGeometry.scaledPoint(orx, ory, scale, getX4(), getY4());
        setBezierPts((int) Math.round(p1.getX()), (int) Math.round(p1.getY()),
            (int) Math.round(p2.getX()), (int) Math.round(p2.getY()),
            (int) Math.round(p3.getX()), (int) Math.round(p3.getY()),
            (int) Math.round(p4.getX()), (int) Math.round(p4.getY()));
    }

    /** Returns the latex (ie axodraw) coordinates of the point passed
     * of this JaxoObject.
     * @param x the x coordinate of the point
     * @param y the y coordinate of the point
     * @param scaleFactor A scalefactor.
     * @param canvasHeight The current height of the canvas.
     * @return A Point2D.
     */
    protected Point2D getLatexPoint(double x, double y, float scaleFactor,
        int canvasHeight) {
        Point2D latexPt = new Point2D.Double();
        latexPt.setLocation(x / scaleFactor, (canvasHeight - y) / scaleFactor);
        return latexPt;
    }

    // return the arrow settings for this Bezier

    protected String getArrowCommand(float scale) {

        String arrowcmd = ""; //Assumes all option default to false in axodraw4j

        float arpos = getArrowPosition(); // The position should be 0.5 ALWAYS, for
                                          // we are putting the arrow as an arrowed line
                                          // layered ovr the actual Bezier. However the line
                                          // is so short that FAPP whatever you have as arpos
                                          // will not matter. It is easier however
                                          // to export the Bezier with the correct arpos.
        arrowcmd = this.getArrow().latexCommand(arpos, scale);

        if (isFlip()) {
            arrowcmd = arrowcmd + ",flip";
        }

        arrowcmd = "[" + arrowcmd + "]";

        return arrowcmd;
    }

    /** {@inheritDoc} */
    public JaxoArrow.Coordinates arrowCoordinates() {
        float arrowPosition = getArrowPosition();

        Point2D point = getPoint(arrowPosition);

        double theta = slope(arrowPosition);

        if (isFlip()) {
            theta = theta + Math.PI;
        }

        return new JaxoArrow.Coordinates(point.getX(), point.getY(), theta);
    }

    /** Returns a point of this Bezier curve at parameter u.
     * @param u the parameter value, needs to be between 0 and 1.
     * @return A Point2D.
     */
    public Point2D getPoint(double u) {
        return JaxoGeometry.bezierPoint(getPoints(), u);
    }

    /** Returns the first derivative of this Bezier curve at parameter u.
     * @param u the parameter value, needs to be between 0 and 1.
     * @return the slope (in radians) at t.
     */
    public double slope(double u) {
        return JaxoGeometry.bezierSlope(getPoints(), u);
    }

    /** Returns the approximate length of this Bezier curve.
     * @return The length of the Bezier curve.
     */
    public final double getBezierLength() {
        return JaxoGeometry.bezierLength(getPoints(), 10.d);
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        setNoFreqStretching(true);
    }
}
