/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.bezier;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/**
 * Implementation of a bezier for fermion lines.
 *
 * @since 2.0
 */
public class JaxoFBezier extends JaxoBezierObject {
    private static final long serialVersionUID = 314159L;

    /** {@inheritDoc} */
    public final boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoFBezier) {
            isCopy = super.isCopy(comp);
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public final void paint(JaxoGraphics2D g2) {
        g2.setColor(getColor());
        g2.setStroke(getStroke());
        g2.draw(getObjectPath());

        if (isPaintArrow()) {
            paintArrow(g2);
        }
    }

    /**
     * Returns the bounding box of this object.
     *
     * @return the bounding box of this object.
     */
    public Rectangle getBounds() {
        // use Area for bounding box calculation instead of gp.getBounds(),
        // see http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4599407
        Rectangle bb =
            getStroke().createStrokedShape(new Area(getObjectPath()))
                .getBounds();

        if (isPaintArrow()) {
            bb.add(getArrow().getBounds(arrowCoordinates()));
        }

        return bb;
    }

    /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;
        String command = "";
        String arrowCommand = "";
        String arrowcmd = "";
        String jdIdentifier = "%JaxoID:FBez";

        double bx1;
        double bx2;
        double by1;
        double by2;
        double t; // time interval
        double k = .005; // time step value for drawing curve

        Point2D latexP1 = getLatexPoint(getX(), getY(), scale, canvasHeight);
        Point2D latexP2 = getLatexPoint(getX2(), getY2(), scale, canvasHeight);
        Point2D latexP3 = getLatexPoint(getX3(), getY3(), scale, canvasHeight);
        Point2D latexP4 = getLatexPoint(getX4(), getY4(), scale, canvasHeight);

        command =
            "\\Bezier" + "(" + D_FORMAT.format(latexP1.getX()) + ","
            + D_FORMAT.format(latexP1.getY()) + ")" + "("
            + D_FORMAT.format(latexP2.getX()) + ","
            + D_FORMAT.format(latexP2.getY()) + ")" + "("
            + D_FORMAT.format(latexP3.getX()) + ","
            + D_FORMAT.format(latexP3.getY()) + ")" + "("
            + D_FORMAT.format(latexP4.getX()) + ","
            + D_FORMAT.format(latexP4.getY()) + ")";

        if (isPaintArrow()) {
            bx1 = (double) getX();
            by1 = (double) getY();

            float arrowPosition = getArrowPosition();
            for (t = k; t <= (1 + k); t += k) {
                // Used to calculate the position of the arrow.
                bx2 = (
                        ((double) getX())
                        + (
                            t * (
                                (-((double) getX()) * 3)
                                + (
                                    t * (
                                        (3 * ((double) getX()))
                                        - (((double) getX()) * t)
                                    )
                                )
                            )
                        )
                    )
                    + (
                        t * (
                            (3 * ((double) getX2()))
                            + (
                                t * (
                                    (-6 * ((double) getX2()))
                                    + (((double) getX2()) * 3 * t)
                                )
                            )
                        )
                    )
                    + (
                        t * t * (
                            (((double) getX3()) * 3)
                            - (((double) getX3()) * 3 * t)
                        )
                    ) + (((double) getX4()) * t * t * t);

                by2 = (
                        ((double) getY())
                        + (
                            t * (
                                (-((double) getY()) * 3)
                                + (
                                    t * (
                                        (3 * ((double) getY()))
                                        - (((double) getY()) * t)
                                    )
                                )
                            )
                        )
                    )
                    + (
                        t * (
                            (3 * ((double) getY2()))
                            + (
                                t * (
                                    (-6 * ((double) getY2()))
                                    + (((double) getY2()) * 3 * t)
                                )
                            )
                        )
                    )
                    + (
                        t * t * (
                            (((double) getY3()) * 3)
                            - (((double) getY3()) * 3 * t)
                        )
                    ) + (((double) getY4()) * t * t * t);

                if ((t > (arrowPosition - (k / 2.f)))
                        && (t < (arrowPosition + (k / 2.f)))) {
                    Point2D startPoint =
                        getLatexPoint(bx1, by1, scale, canvasHeight);
                    Point2D endPoint =
                        getLatexPoint(bx2, by2, scale, canvasHeight);
                    arrowcmd = getArrowCommand(scale);

                    arrowCommand =
                        "\\Line" +  arrowcmd + "("
                        + D_FORMAT.format(startPoint.getX()) + ","
                        + D_FORMAT.format(startPoint.getY()) + ")" + "("
                        + D_FORMAT.format(endPoint.getX()) + ","
                        + D_FORMAT.format(endPoint.getY()) + ")";
                }

                bx1 = bx2;
                by1 = by2;
            }
        }

        if (isPaintArrow()) {
            jdIdentifier = jdIdentifier + arrowcmd;
        }

        return command + arrowCommand + jdIdentifier;
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        editPanel.add4PointsPanel(getPoints(), 0, 0, 2);
        //editPanel.addDoubleLinePanel(bezier, 3, 0);
        editPanel.addLineColorPanel(getColor(), 2, 0);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 1);
        //editPanel.addArrowPanel(isPaintArrow(), isFlip(), false, false, 1, 1);
        //editPanel.addArrowPositionPanel(getArrowPosition(), 2, 1);
        editPanel.addArrowPanel(isPaintArrow(), isFlip(), getArrowPosition(), 1, 1);
        editPanel.addArrowOptionsEditPanel(getArrow(), 2, 1);

        editPanel.setTitleAndIcon("Fermion_bezier_parameters", "bezierf.png");
    }

    private GeneralPath getObjectPath() {
        GeneralPath gp = getGeneralPath();
        gp.reset();

        gp.moveTo(getX(), getY());
        gp.curveTo(getX2(), getY2(), getX3(), getY3(), getX4(), getY4());

        return gp;
    }

}
