/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.fill;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import java.io.IOException;
import java.io.ObjectInputStream;

import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.object.JaxoRotationObject;
import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** A blob object: a general ellypse that can be rotated.
 * @since 2.0
 */
public class JaxoBlob extends JaxoFillObject
    implements JaxoRotationObject {
    private static final long serialVersionUID = 314159L;
    private transient Ellipse2D blob = new Ellipse2D.Float();

    /** The rotation angle of the ellipse */
    private int rotationAngle;

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        blob = new Ellipse2D.Float();
    }

    // Bean getter and setter methods

    /** Returns the rotation angle of this blob.
     * @return The rotation angle of this blob
     */
    public final int getRotationAngle() {
        return rotationAngle;
    }

    /** Sets the rotation angle of this blob.
     * @param newangle The rotation angle of this blob
     */
    public final void setRotationAngle(int newangle) {
        Integer old = new Integer(rotationAngle);
        this.rotationAngle = newangle;
        firePropertyChange("rotationAngle", old, new Integer(rotationAngle));
    }

    /** Sets all parameters from the given object to the current one.
     * @param temp The object to copy from.
     */
    public final void copyFrom(JaxoBlob temp) {
        super.copyFrom(temp);
        setRotationAngle(temp.getRotationAngle());
    }

    /** {@inheritDoc} */
    public void setState(JaxoObject o) {
        copyFrom((JaxoBlob) o);
    }

    /** {@inheritDoc} */
    public final boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoBlob) {
            JaxoBlob temp = (JaxoBlob) comp;
            if ((temp.getRotationAngle() == getRotationAngle()) && super.isCopy(temp)) {
                isCopy = true;
            }
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public final void paintVisualAid(JaxoGraphics2D g2) {
        g2.setColor(JaxoGraphics2D.DEFAULT_HELP_COLOR);
        g2.setStroke(JaxoGraphics2D.DEFAULT_HELP_STROKE);

        int x = getX();
        int y = getY();
        int lp4 = 12;

        g2.drawLine(x - lp4, y, x + lp4, y);
        g2.drawLine(x, y - lp4, x, y + lp4);
    }

    /** {@inheritDoc} */
    public final void paint(JaxoGraphics2D g2) {
        updateShape();

        if (isFilled()) {
            g2.setColor(getFillColor());
            g2.fill(getShape());
        }
        g2.setColor(getColor());
        g2.setStroke(getStroke());
        g2.draw(getShape());
    }

    /**
     * Returns the bounding box of this object.
     *
     * @return the bounding box of this object.
     */
    public Rectangle getBounds() {
        updateShape();

        return getStroke().createStrokedShape(getShape()).getBounds();
    }

    /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;

        Point2D axes = getLaTexAxes(scale);
        if (((int) axes.getX() == 0) && ((int) axes.getY() == 0)) {
            return "%";
        }

        Point2D center = getLaTexCenter(scale, canvasHeight);
        int rotate = getLaTexRotation();
        String command = "";

        if (isFilled()) {
            if (JaxoColor.isGrayScale(getFillColor())) {
                float grayScale = JaxoColor.getGrayScaleFloat(getFillColor());
                command =
                    "\\GOval" + "(" + D_FORMAT.format(center.getX()) + ","
                    + D_FORMAT.format(center.getY()) + ")" + "("
                    + D_FORMAT.format(axes.getY()) + ","
                    + D_FORMAT.format(axes.getX()) + ")" + "("
                    + D_FORMAT.format(rotate) + ")" + "{"
                    + GRAY_SCALE_FORMAT.format(grayScale) + "}";
            } else {
                String tlc;
                if (JaxoColor.isGrayScale(getColor())) {
                    tlc = "Gray";
                } else {
                    tlc = JaxoColor.getColorName(getColor());
                }
                String tfc = JaxoColor.getColorName(getFillColor());
                command =
                    "\\COval" + "(" + D_FORMAT.format(center.getX()) + ","
                    + D_FORMAT.format(center.getY()) + ")" + "("
                    + D_FORMAT.format(axes.getY()) + ","
                    + D_FORMAT.format(axes.getX()) + ")" + "("
                    + D_FORMAT.format(rotate) + ")" + "{" + tlc + "}" + "{"
                    + tfc + "}";
            }
        } else {
            command =
                "\\Oval" + "(" + D_FORMAT.format(center.getX()) + ","
                + D_FORMAT.format(center.getY()) + ")" + "("
                + D_FORMAT.format(axes.getY()) + ","
                + D_FORMAT.format(axes.getX()) + ")" + "("
                + D_FORMAT.format(rotate) + ")";
        }

        return command;
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        setRotationAngle(0);
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        editPanel.addCenterSizePanel(getX(), getY(), getWidth(), getHeight(),
            0, 0);
        editPanel.addRotationPanel(getRotationAngle(), 0, 1);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 2);
        editPanel.addFillLineColorPanels(getColor(), getFillColor(), isFilled(),
            1, 1, 1, 0);

        editPanel.setTitleAndIcon("Blob_parameters", "blob.png");
    }

      //
     // private methods
    //

    private Point2D getLaTexCenter(float scaleFactor, int canvasHeight) {
        return getLaTexLocation(scaleFactor, canvasHeight);
    }

    private Point2D getLaTexAxes(float scaleFactor) {
        Point2D axesVec = new Point2D.Float();
        float wdt;
        float hgt;

        wdt = getWidth() / scaleFactor;
        hgt = getHeight() / scaleFactor;
        axesVec.setLocation(wdt, hgt);

        return axesVec;
    }

    private int getLaTexRotation() {
        int rt = -rotationAngle;

        return rt;
    }

    // Use 'blob' directly if rotation angle is zero
    // -- could also do for 90, 180 etc.
    // Also the calculation of the bounding box can be done
    // directly for any angle, but probably is not worth it.

    // Call updateShape() before
    private Shape getShape() {
        return rotationAngle == 0 ? blob : (Shape) getGeneralPath();
    }

    private void updateShape() {
        blob.setFrame(-getWidth() + getX(), -getHeight() + getY(),
            2 * getWidth(), 2 * getHeight());

        if (rotationAngle != 0) {
            GeneralPath p = getGeneralPath();
            p.reset();
            p.append(blob, false);
            p.transform(AffineTransform.getRotateInstance(
                Math.toRadians(rotationAngle), getX(), getY()));
        }
    }
}
