/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.line;

import java.awt.BasicStroke;
import java.awt.geom.GeneralPath;

import java.io.IOException;
import java.io.ObjectInputStream;

import net.sf.jaxodraw.util.JaxoUtils;


/**
 * A dashed line.
 *
 * @since 2.0
 */
public abstract class JaxoDashLine extends JaxoLineObject {
    private static final long serialVersionUID = 314159L;

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        resetStroke();
    }

    /** {@inheritDoc} */
    protected void resetStroke() {
        if (JaxoUtils.zero(getStrokeWidth()) || JaxoUtils.zero(getDash())) {
            // during initialization, either of them is still null
            // so wait until both are set
            return;
        }

        setStroke(new BasicStroke(getStrokeWidth(), BasicStroke.CAP_ROUND,
                BasicStroke.JOIN_MITER, 10.0f, strokeDashes(), 0.0f));
    }

    /** {@inheritDoc} */
    protected GeneralPath getObjectPath() {
        GeneralPath gp = getGeneralPath();
        gp.reset();
        if (isDoubleLine()) {
            float length = (float) getRadius();
            float sepx = getRelh() / length * getDLSeparation() / 2.f;
            float sepy = getRelw() / length * getDLSeparation() / 2.f;
            gp.moveTo(getX() - sepx, getY() + sepy);
            gp.lineTo(getX2() - sepx, getY2() + sepy);
            gp.moveTo(getX() + sepx, getY() - sepy);
            gp.lineTo(getX2() + sepx, getY2() - sepy);
        } else {
            gp.moveTo(getX(), getY());
            gp.lineTo(getX2(), getY2());
        }
        return gp;
    }

    /**
     * Returns an array that is used as the dash parameter in
     * {@link java.awt.BasicStroke} to paint this object.
     * @return a dash array.
     */
    protected abstract float[] strokeDashes();

    // Get the axodraw4j options set for this line

    /** {@inheritDoc} */
    protected String getAxo4JOptions(float scale) {

        float dashSize = getDash() / scale;
        String optioncmd = "dash,dashsize="
                    + D_FORMAT.format(dashSize);

        if (isPaintArrow()) {
            float arpos = getArrowPosition();
            String arrowcmd = this.getArrow().latexCommand(arpos, scale);

            if (isFlip()) {
                arrowcmd = arrowcmd + ",flip";
            }

            optioncmd = optioncmd + "," + arrowcmd;

            if (isDoubleLine()) {
                optioncmd = optioncmd + ",double,sep="
                    + D_FORMAT.format(this.getDLSeparation());
            }

        } else if (isDoubleLine()) {
            optioncmd = optioncmd + ","
                + "double,sep=" + D_FORMAT.format(this.getDLSeparation());
        }

        optioncmd = "[" + optioncmd + "]";

        return optioncmd;
    }
}
