/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.line;

import java.awt.Dimension;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import java.io.IOException;
import java.io.ObjectInputStream;

import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.object.JaxoWiggleObject;
import net.sf.jaxodraw.util.JaxoUtils;


/** A gluon line.
 * @since 2.0
 */
public class JaxoGlLine extends JaxoLineObject implements JaxoWiggleObject {
    private static final long serialVersionUID = 314159L;
    private transient float freq;

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        freq = 0.f;
    }

    /** Sets all parameters from the given object to the current one.
     * @param temp The object to copy from.
     */
    public void copyFrom(JaxoGlLine temp) {
        super.copyFrom(temp);
        this.freq = temp.getFrequency();
    }

    /** {@inheritDoc} */
    public void setState(JaxoObject o) {
        copyFrom((JaxoGlLine) o);
    }

    /** {@inheritDoc} */
    public boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoGlLine) {
            isCopy = super.isCopy(comp);
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;

        Point2D startPoint = getLaTexStartPoint(scale, canvasHeight);
        Point2D endPoint = getLaTexEndPoint(scale, canvasHeight);

        if (((int) endPoint.getX() == (int) startPoint.getX())
                && ((int) endPoint.getY() == (int) startPoint.getY())) {
            return "%";
        }

        float amplitude = getAmp() / (2.f * scale);
        float windings = (float) getWiggles();
        String command = "";
        String base = "\\Gluon";
        String options = getAxo4JOptions(scale);

        command =
            base + options +  "(" + D_FORMAT.format(startPoint.getX()) + ","
            + D_FORMAT.format(startPoint.getY()) + ")" + "("
            + D_FORMAT.format(endPoint.getX()) + ","
            + D_FORMAT.format(endPoint.getY()) + ")" + "{"
            + D_FORMAT.format(amplitude) + "}" + "{"
            + D_FORMAT.format(windings) + "}";

        return command;
    }

    /** {@inheritDoc} */
    public float getFrequency() {
        return freq;
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency(float frequency) {
        this.freq = frequency;
        setWigglesFromFrequency();
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency() {
        int n = (int) Math.round(length() * getFrequency());
        setWiggles(n);
    }

    /** {@inheritDoc} */
    public void setFrequencyFromWiggles(int wiggles) {
        setWiggles(wiggles);
        this.freq = (float) (wiggles / length());
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        setPaintArrow(false);
        setArrowPosition(0.f);
        setWigglesFromFrequency(GLUON_FREQ);
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        editPanel.add2PointsPanel(getPoints(), 0, 0);
        editPanel.addWigglePanel(getAmp(), getWiggles(), 1, 0);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 1);
        editPanel.addDoubleLinePanel(isDoubleLine(), getDLSeparation(), 1, 1);
        editPanel.addColorPanel(getColor(), JaxoObjectEditPanel.TYPE_LINE_COLOR, 2, 1);

        editPanel.setTitleAndIcon("Gluon_line_parameters", "gluon.png");
    }

      //
     // private methods
    //

    /** {@inheritDoc} */
    protected GeneralPath getObjectPath() {
        GeneralPath gp = getGeneralPath();
        gp.reset();

        if (JaxoUtils.zero(freq)) {
            setFrequencyFromWiggles(getWiggles());
        } else {
            setWigglesFromFrequency();
        }

        if (isDoubleLine()) {
            float sep = getDLSeparation() / 2.f;
            appendFullLine(gp, sep);
            appendFullLine(gp, -sep);
        } else {
            appendFullLine(gp, 0.f);
        }

        return gp;
    }

    // construct a gluon line like axodraw does,
    // see src/doc/algorithms/gluon_algorithm.tex
    private void appendFullLine(GeneralPath gp, float sep) {
            float length = (float) length();
            int n = getWiggles();
            float delta = length / (n + 1);
            float amp = -0.5f * getAmp();
            float absamp = Math.abs(amp);

            float sin = getRelh() / length;
            float cos = getRelw() / length;

            gp.moveTo(getX(), getY() + sep);

            float x1 = 0.05f * delta;
            float x2 = 0.25f * delta;
            float x3 = 0.7f * delta;
            float y1 = 0.5f * amp + sep;
            float y2 = amp + sep;
            float y3 = amp + sep;

            curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

            x1 = delta + 0.9f * absamp;
            x2 = delta + 0.9f * absamp;
            x3 = delta;
            y1 = amp + sep;
            y2 = -amp + sep;
            y3 = -amp + sep;

            curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

            float xj;

            for (int j = 1; j < n; j++) {
                xj = j * delta;
                x1 = xj - 0.9f * absamp;
                x2 = xj - 0.9f * absamp;
                x3 = xj + 0.5f * delta;
                y1 = -amp + sep;
                y2 = amp + sep;
                y3 = amp + sep;
                curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

                x1 = xj + 0.9f * absamp + delta;
                x2 = xj + 0.9f * absamp + delta;
                x3 = xj + delta;
                y1 = amp + sep;
                y2 = -amp + sep;
                y3 = -amp + sep;
                curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);
            }

            x1 = length - delta - 0.9f * absamp;
            x2 = length - delta - 0.9f * absamp;
            x3 = length - 0.7f * delta;
            y1 = -amp + sep;
            y2 = amp + sep;
            y3 = amp + sep;
            curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);

            x1 = length - 0.25f * delta;
            x2 = length - 0.05f * delta;
            x3 = length;
            y1 = amp + sep;
            y2 = 0.5f * amp + sep;
            y3 = 0.f + sep;
            curveTo(gp, x1, y1, x2, y2, x3, y3, cos, sin);
    }

    private void curveTo(GeneralPath gp, float x1, float y1,
            float x2, float y2, float x3, float y3, float cos, float sin) {

        gp.curveTo((x1 * cos) - (y1 * sin) + getX(), (x1 * sin) + (y1 * cos) + getY(),
                (x2 * cos) - (y2 * sin) + getX(), (x2 * sin) + (y2 * cos) + getY(),
                (x3 * cos) - (y3 * sin) + getX(), (x3 * sin) + (y3 * cos) + getY());
    }

    // Get the axodraw4j options set for this line

    /** {@inheritDoc} */
    protected String getAxo4JOptions(float scale) {

        String optioncmd = "";

        if (isDoubleLine()) {
            optioncmd = "[" + "double,sep="
                + D_FORMAT.format(this.getDLSeparation()) + "]";
        }

        return optioncmd;
    }
}
