/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.line;

import java.awt.Dimension;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import java.io.IOException;
import java.io.ObjectInputStream;

import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.object.JaxoSymmetricObject;
import net.sf.jaxodraw.object.JaxoWiggleObject;
import net.sf.jaxodraw.util.JaxoUtils;


/** A photon line.
 * @since 2.0
 */
public class JaxoPLine extends JaxoLineObject implements JaxoWiggleObject,
    JaxoSymmetricObject {
    private static final long serialVersionUID = 314159L;
    private transient float freq;
    private boolean symmetric;

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        freq = 0.f;
    }

    /**
     * Returns the symmetric property of this JaxoPLine.
     * This determines whether the wiggles of this photon
     * line start and end in the same direction.
     *
     * @return The symmetric property of this JaxoPLine.
     */
    public final boolean isSymmetric() {
        return symmetric;
    }

    /**
     * Sets the symmetric property of this line object.
     *
     * @param newSymm The symmetric property of this line object.
     */
    public final void setSymmetric(boolean newSymm) {
        Boolean old = Boolean.valueOf(symmetric);
        this.symmetric = newSymm;
        firePropertyChange("symmetric", old, Boolean.valueOf(symmetric));
    }

    /** Sets all parameters from the given object to the current one.
     * @param temp The object to copy from.
     */
    public void copyFrom(JaxoPLine temp) {
        super.copyFrom(temp);
        this.freq = temp.getFrequency();
        this.symmetric = temp.isSymmetric();
    }

    /** {@inheritDoc} */
    public void setState(JaxoObject o) {
        copyFrom((JaxoPLine) o);
    }

    /** {@inheritDoc} */
    public boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoPLine) {
            JaxoPLine line = (JaxoPLine) comp;
            if ((line.isSymmetric() == isSymmetric()) && super.isCopy(line)) {
                isCopy = true;
            }
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;

        Point2D startPoint = getLaTexStartPoint(scale, canvasHeight);
        Point2D endPoint = getLaTexEndPoint(scale, canvasHeight);

        if (((int) endPoint.getX() == (int) startPoint.getX())
                && ((int) endPoint.getY() == (int) startPoint.getY())) {
            return "%";
        }

        float amplitude = getAmp() / (2.f * scale);
        float wiggles = getLaTexWiggleNumber();
        String command = "";
        String base = "\\Photon";
        String options = getAxo4JOptions(scale);

        command =
            base + options + "(" + D_FORMAT.format(startPoint.getX()) + ","
            + D_FORMAT.format(startPoint.getY()) + ")" + "("
            + D_FORMAT.format(endPoint.getX()) + ","
            + D_FORMAT.format(endPoint.getY()) + ")" + "{"
            + D_FORMAT.format(amplitude) + "}" + "{"
            + D_FORMAT.format(wiggles) + "}";

        return command;
    }

    /** {@inheritDoc} */
    public float getFrequency() {
        return freq;
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency(float frequency) {
        this.freq = frequency;
        setWigglesFromFrequency();
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency() {
        int n = (int) Math.round(length() * getFrequency());
        setWiggles(n);
    }

    /** {@inheritDoc} */
    public void setFrequencyFromWiggles(int wiggles) {
        setWiggles(wiggles);
        this.freq = (float) (wiggles / length());
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        setPaintArrow(false);
        setArrowPosition(0.f);
        setSymmetric(false);
        setWigglesFromFrequency(PHOTON_FREQ);
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        editPanel.add2PointsPanel(getPoints(), 0, 0);
        editPanel.addWigglePanel(getAmp(), getWiggles(), 1, 0);
        editPanel.addSymmPanel(isSymmetric(), 2, 0);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 1);
        editPanel.addDoubleLinePanel(isDoubleLine(), getDLSeparation(), 1, 1);
        editPanel.addColorPanel(getColor(), JaxoObjectEditPanel.TYPE_LINE_COLOR, 2, 1);

        editPanel.setTitleAndIcon("Photon_line_parameters", "photon.png");
    }

      //
     // private methods
    //

    private float getLaTexWiggleNumber() {
        float n = (float) getWiggles();
        if (isSymmetric()) {
            n = n + 0.5f;
        }
        return n;
    }

    /** {@inheritDoc} */
    protected GeneralPath getObjectPath() {
        GeneralPath gp = getGeneralPath();
        gp.reset();

        if (JaxoUtils.zero(freq)) {
            setFrequencyFromWiggles(getWiggles());
        } else {
            setWigglesFromFrequency();
        }

        if (isDoubleLine()) {
            float dlsep = getDLSeparation() / 2.f;
            appendFullLine(gp, dlsep);
            appendFullLine(gp, -dlsep);
        } else {
            appendFullLine(gp, 0.f);
        }

        return gp;
    }

    private void appendFullLine(GeneralPath gp, float sep) {
        float length = (float) getRadius();

        int n = 2 * getWiggles();
        if (isSymmetric()) {
            n = n + 1;
        }

        float ts = length / n;

        float sin = getRelh() / length;
        float cos = getRelw() / length;

        float c = 4.f / 3.f * ts / (float) Math.PI;
        float amp = -2.f / 3.f * getAmp();

        gp.moveTo(getX(), getY() + sep);
        float x0;

        for (int i = 1; i <= n; i++) {
            x0 = (i - 1) * ts;
            curveTo(gp, x0 + c, amp + sep, x0 + ts - c, amp + sep, x0 + ts, sep, cos, sin);
            amp = -amp;
        }
    }

    private void curveTo(GeneralPath gp, float x1, float y1,
            float x2, float y2, float x3, float y3, float cos, float sin) {

        gp.curveTo(x1 * cos - y1 * sin + getX(), x1 * sin + y1 * cos + getY(),
                x2 * cos - y2 * sin + getX(), x2 * sin + y2 * cos + getY(),
                x3 * cos - y3 * sin + getX(), x3 * sin + y3 * cos + getY());
    }

    // Get the axodraw4j options set for this line

    /** {@inheritDoc} */
    protected String getAxo4JOptions(float scale) {

        String optioncmd = "";

        if (isDoubleLine()) {
            optioncmd = "[" + "double,sep="
                + D_FORMAT.format(this.getDLSeparation()) + "]";
        }

        return optioncmd;
    }
}
