/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.loop;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import net.sf.jaxodraw.object.JaxoArrow;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoParticleObject;
import net.sf.jaxodraw.util.JaxoGeometry;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** A general loop object.
 * @since 2.0
 */
public abstract class JaxoLoopObject extends JaxoParticleObject {
    /** Returns an exact copy of this JaxoLoop.
     * @param temp An instance of the loop to copy to (since this is an abstract
     *  class, we have to provide an instance of a sub-class).
     * @return A copy of this JaxoLoop.
     */
    public final JaxoObject copy(JaxoLoopObject temp) {
        temp.copyFrom(this);
        return temp;
    }


    /** {@inheritDoc} */
    public boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoLoopObject) {
            isCopy = super.isCopy(comp);
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public void paint(JaxoGraphics2D g2) {
        GeneralPath gp = getObjectPath();

        if (gp == null) {
            return;
        }

        g2.setColor(getColor());
        g2.setStroke(getStroke());
        g2.draw(gp);

        if (isPaintArrow()) {
            paintArrow(g2);
        }
    }

    /**
     * Returns the bounding box of this object.
     *
     * @return the bounding box of this object.
     */
    public Rectangle getBounds() {
        GeneralPath gp = getObjectPath();
        Rectangle bb;

        if (gp == null) {
            bb = new Rectangle();
            bb.add(getX(), getY());
            bb.add(getX2(), getY2());
            return bb;
        }

        bb = getStroke().createStrokedShape(gp).getBounds();

        if (isPaintArrow()) {
            bb.add(getArrow().getBounds(arrowCoordinates()));
        }

        return bb;
    }

    /** {@inheritDoc} */
    public String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;

        Point2D center = getLaTexCenter(scale, canvasHeight);
        float radius = getLaTexRadius(scale);
        Point2D angles = getLaTexAngles();

        String command = "";
        String base = "\\Arc";
        String options = getAxo4JOptions(scale);

        if ((int) radius == 0) {
            command = "%";
            return command;
        }

        command =
            base + options + "(" + D_FORMAT.format(center.getX()) + ","
            + D_FORMAT.format(center.getY()) + ")" + "("
            + D_FORMAT.format(radius) + ","
            + D_FORMAT.format(angles.getX()) + ","
            + D_FORMAT.format(angles.getY()) + ")";

        return command;
    }

    /** {@inheritDoc} */
    public final void paintVisualAid(JaxoGraphics2D g2) {
        g2.setColor(JaxoGraphics2D.DEFAULT_HELP_COLOR);
        g2.setStroke(JaxoGraphics2D.DEFAULT_HELP_STROKE);

        int x = getX();
        int y = getY();
        int lp4 = 12;

        g2.drawLine(x - lp4, y, x + lp4, y);
        g2.drawLine(x, y - lp4, x, y + lp4);
    }

    /** {@inheritDoc} */
    public JaxoArrow.Coordinates arrowCoordinates() {
        Point2D.Double ap = getLoopPoint(1 - getArrowPosition());

        double theta = Math.atan2(ap.y - getY(), ap.x - getX());
        theta = theta - (Math.PI / 2.d);

        if (isFlip()) {
            theta = theta + Math.PI;
        }

        return new JaxoArrow.Coordinates(ap.x, ap.y, theta);
    }

    /** {@inheritDoc} */
    public final void rescaleObject(int orx, int ory, float scale) {
        int newRelWidth = Math.round(getRelSize().width * scale);
        int newRelHeight = Math.round(getRelSize().height * scale);

        Point2D p = JaxoGeometry.scaledPoint(orx, ory, scale, getX(), getY());

        setLocation((int) Math.round(p.getX()), (int) Math.round(p.getY()));
        setX2(getX() + newRelWidth);
        setY2(getY() + newRelHeight);
        setWiggles(Math.round(getWiggles() * scale));
        setAmp(Math.round(getAmp() * scale));
    }

    /** Returns a center point which is used by the latexCommand method.
     * @param scaleFactor A scale factor.
     * @param canvasHeight The height of the current canvas.
     * @return A Point2D object holding the center point.
     */
    protected Point2D getLaTexCenter(float scaleFactor, int canvasHeight) {
        return getLaTexLocation(scaleFactor, canvasHeight);
    }

    /**
     * Dividing the loop into n equidistant segments of length d,
     * so n * d = 2 * r * PI is the circumference of the loop,
     * this routine returns the n + 1 loop points that are i * d away from P1,
     * where 0 <= i <= n.
     *
     * @param n The number of segments to divide the loop.
     * @return The equidistant points of the loop.
     */
    public final Point2D.Float[] getEquidistantPoints(int n) {
        Point2D.Float p0 = new Point2D.Float(getX(), getY());
        Point2D.Float p1 = new Point2D.Float(getX2(), getY2());

        double radius = getRadius();
        double dphi = (2.d * Math.PI) / n;
        double phi1 = Math.atan2(p1.y - p0.y, p1.x - p0.x);
        float px, py;

        Point2D.Float[] point = new Point2D.Float[n + 1];
        point[0] = p1;

        for (int i = 1; i <= n; i++) {
            px = p0.x + (float) (radius * Math.cos(phi1 + (i * dphi)));
            py = p0.y + (float) (radius * Math.sin(phi1 + (i * dphi)));
            point[i] = new Point2D.Float(px, py);
        }

        return point;
    }

    private Point2D.Double getLoopPoint(double t) {
        double par = JaxoGeometry.curveParameter(t) * 2 * Math.PI;
        double phi1 = Math.atan2(getY2() - getY(), getX2() - getX()) + Math.PI;
        double radius = getRadius();
        double px, py;

        Point2D.Double point = new Point2D.Double();

        px = getX() + (radius * Math.cos(phi1 + par));
        py = getY() + (radius * Math.sin(phi1 + par));
        point.setLocation(px, py);

        return point;
    }

    /** Returns a radius which is used by the latexCommand method.
     * @param scaleFactor A scale factor.
     * @return The latex radius.
     */
    protected float getLaTexRadius(float scaleFactor) {
        return ((float) getRadius() / scaleFactor);
    }

    /**
     * Returns an angle which is used by the latexCommand method.
     * This is basically the relative angle between the center and
     * the second (release) point of the loop.
     *
     * @return A Point2D object with x-component the starting angle
     *      and y-component the starting angle + 360.
     */
    protected Point2D getLaTexAngles() {
        Point2D anglesPoint = new Point2D.Float();

        int sAngle =
            (int) -Math.round((Math.atan2(getRelh(), getRelw()) * 180.f
                ) / Math.PI) + 180;

        int eAngle = sAngle + 360;

        anglesPoint.setLocation(sAngle, eAngle);

        return anglesPoint;
    }

    /**
     * Get the GeneralPath that paints this loop.
     *
     * @return GeneralPath. May be null for an object that cannot be painted.
     */
    protected abstract GeneralPath getObjectPath();

    /**
     * Return the option part of the LaTeX command for this line.
     *
     * @param scale the axodraw4j scale factor.
     *
     * @return the option String.
     */
    protected abstract String getAxo4JOptions(float scale);
}
