/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.loop;

import java.awt.Dimension;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import java.io.IOException;
import java.io.ObjectInputStream;

import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.object.JaxoWiggleObject;
import net.sf.jaxodraw.util.JaxoUtils;


/** A photon loop.
 * @since 2.0
 */
public class JaxoPLoop extends JaxoLoopObject implements JaxoWiggleObject {
    private static final long serialVersionUID = 314159L;
    private transient float freq;

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        freq = 0.f;
    }

    /** Sets all parameters from the given object to the current one.
     * @param temp The object to copy from.
     */
    public void copyFrom(JaxoPLoop temp) {
        super.copyFrom(temp);
        this.freq = temp.getFrequency();
    }

    /** {@inheritDoc} */
    public void setState(JaxoObject o) {
        copyFrom((JaxoPLoop) o);
    }

    /** {@inheritDoc} */
    public final boolean isCopy(JaxoObject comp) {
        boolean isCopy = super.isCopy(comp);

        if (comp instanceof JaxoPLoop) {
            isCopy = super.isCopy(comp);
        }

        return isCopy;
    }

     /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;

        Point2D center = getLaTexCenter(scale, canvasHeight);
        float radius = getLaTexRadius(scale);
        Point2D angles = getLaTexAngles();
        float amplitude = getAmp() / (2.f * scale);
        float wiggles = (float) getWiggles();
        String command = "";
        String base = "\\PhotonArc";
        String options = getAxo4JOptions(scale);

        if ((int) radius == 0) {
            command = "%";
            return command;
        }

        command =
            base + options + "(" + D_FORMAT.format(center.getX()) + ","
            + D_FORMAT.format(center.getY()) + ")" + "("
            + D_FORMAT.format(radius) + ","
            + D_FORMAT.format(angles.getX() - 180) + ","
            + D_FORMAT.format(angles.getY() - 180) + ")" + "{"
            + D_FORMAT.format(amplitude) + "}" + "{"
            + D_FORMAT.format(wiggles) + "}";

        return command;
    }

    /** {@inheritDoc} */
    public float getFrequency() {
        return freq;
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency(float frequency) {
        this.freq = frequency;
        setWigglesFromFrequency();
    }

    /** {@inheritDoc} */
    public void setWigglesFromFrequency() {
        double r = getRadius();
        int n = (int) Math.round((2 * r * Math.PI) * getFrequency());
        setWiggles(n);
    }

    /** {@inheritDoc} */
    public void setFrequencyFromWiggles(int wiggles) {
        setWiggles(wiggles);
        double r = getRadius();
        this.freq = (float) (wiggles / (2 * r * Math.PI));
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        setPaintArrow(false);
        setArrowPosition(0.f);
        setWigglesFromFrequency(PHOTON_FREQ);
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        int r = (int) Math.round(getRadius());

        editPanel.addXYRPanel(getX(), getY(), r, 0, 0);
        editPanel.addWigglePanel(getAmp(), getWiggles(), 1, 0);
        editPanel.addDoubleLinePanel(isDoubleLine(), getDLSeparation(), 2, 0);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 1);
        editPanel.addColorPanel(getColor(), JaxoObjectEditPanel.TYPE_LINE_COLOR, 1, 1);

        editPanel.setTitleAndIcon("Photon_loop_parameters", "photonloop.png");
    }

      //
     // private methods
    //

    /** {@inheritDoc} */
    protected GeneralPath getObjectPath() {
        GeneralPath gp = getGeneralPath();
        gp.reset();

        if (JaxoUtils.zero(freq)) {
            setFrequencyFromWiggles(getWiggles());
        } else {
            setWigglesFromFrequency();
        }

        float radius = (float) getRadius();

        if (radius <= 1.f) {
            return null;
        }

        if (isDoubleLine()) {
            float ds = getDLSeparation() / 2.f;

            appendFullLoop(gp, radius, ds);
            appendFullLoop(gp, radius, -ds);
        } else {
            appendFullLoop(gp, radius, 0.f);
        }

        return gp;
    }

    private void appendFullLoop(GeneralPath gp, float radius, float sep) {
        int n = 2 * getWiggles();
        float delta = (float) (2.d * Math.PI / n);
        float amp = 0.5f * getAmp();
        float r = radius + sep;

        float cos = getRelw() / radius;
        float sin = getRelh() / radius;

        float cp = (float) Math.cos(delta);
        float sp = (float) Math.sin(delta);
        float cp2 = (float) Math.cos(delta / 2.f);
        float sp2 = (float) Math.sin(delta / 2.f);

        gp.moveTo(r * cos + getX(), r * sin + getY());

        float thetaj;
        float cs, sn, beta, tt, amp1;
        float x1, y1, x2, y2, x3, y3;

        for (int j = 1; j <= n; j++) {
            thetaj = (j - 1) * delta;
            amp = -amp;

            cs = (float) Math.cos(thetaj);
            sn = (float) Math.sin(thetaj);

            beta = r * 2.f / amp / n;
            tt = (sp - cp * beta) / (cp + sp * beta);
            amp1 = (8.f * (r + amp) * (beta * cp2 - sp2)
                    - (beta * (4.f + cp) + 3.f * tt * cp - 4.f * sp) * r)
                    / (3.f * (beta - tt));
            x1 = (8.f * (r + amp) * cp2 - (1.f + cp) * r) / 3.f - amp1;
            y1 = (x1 - r) * beta;
            x2 = amp1;
            y2 = (amp1 - r * cp) * tt + r * sp;
            x3 = r * cp;
            y3 = r * sp;

            curveTo(gp, x1 * cs - y1 * sn, x1 * sn + y1 * cs,
                x2 * cs - y2 * sn, x2 * sn + y2 * cs,
                x3 * cs - y3 * sn, x3 * sn + y3 * cs, cos, sin);
        }
    }

    private void curveTo(GeneralPath gp, float x1, float y1,
            float x2, float y2, float x3, float y3, float cos, float sin) {

        gp.curveTo(x1 * cos - y1 * sin + getX(), x1 * sin + y1 * cos + getY(),
                x2 * cos - y2 * sin + getX(), x2 * sin + y2 * cos + getY(),
                x3 * cos - y3 * sin + getX(), x3 * sin + y3 * cos + getY());
    }

    // Get the axodraw4j options set for this line

    /** {@inheritDoc} */
    protected String getAxo4JOptions(float scale) {

        String optioncmd = "";

        if (isDoubleLine()) {
            optioncmd = "[" + "double,sep="
                + D_FORMAT.format(this.getDLSeparation()) + "]";
        }

        return optioncmd;
    }
}
