/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.vertex;

import java.awt.geom.Point2D;

import net.sf.jaxodraw.object.JaxoExtendedObject;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.object.JaxoRotationObject;
import net.sf.jaxodraw.util.JaxoPrefs;
import net.sf.jaxodraw.util.JaxoGeometry;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** A vertex object.
 * @since 2.0
 */
public abstract class JaxoVertex extends JaxoExtendedObject
    implements JaxoRotationObject {
    /**
     * Returns the rotation angle of this vertex.
     *
     * @return The rotation angle of this vertex (in degrees).
     */
    public final int getRotationAngle() {
        double theta = Math.atan2((float) getRelh(), (float) getRelw());
        return ((int) Math.round(Math.toDegrees(theta)));
    }

    /**
     * Sets the rotation angle of this vertex, this only acts on P2.
     *
     * @param angle The rotation angle of this vertex (in degrees).
     */
    public final void setRotationAngle(int angle) {
        double radAngle = Math.toRadians(angle);
        double length =
            Math.sqrt((getWidth() * getWidth()) + (getHeight() * getHeight()));
        int newRelW = (int) Math.round(length * Math.cos(radAngle));
        int newRelH = (int) Math.round(length * Math.sin(radAngle));
        setRelWAndH(newRelW, newRelH);
    }

    /** Returns an exact copy of this JaxoVertex.
     * @param temp An instance of the vertex to copy to.
     * @return A copy of this JaxoVertex.
     */
    public final JaxoObject copy(JaxoVertex temp) {
        temp.copyFrom(this);

        return temp;
    }

    /** {@inheritDoc} */
    public boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoVertex) {
            isCopy = super.isCopy(comp);
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public final void paintVisualAid(JaxoGraphics2D g2) {
        g2.setColor(JaxoGraphics2D.DEFAULT_HELP_COLOR);
        g2.setStroke(JaxoGraphics2D.DEFAULT_HELP_STROKE);

        int x = getX();
        int y = getY();
        int lp4 = 12;

        g2.drawLine(x - lp4, y, x + lp4, y);
        g2.drawLine(x, y - lp4, x, y + lp4);
    }

    /** {@inheritDoc} */
    public final void rescaleObject(int orx, int ory, float scale) {
        int newRelWidth = Math.round(this.getRelSize().width * scale);
        int newRelHeight = Math.round(this.getRelSize().height * scale);

        Point2D newP =
            JaxoGeometry.scaledPoint(orx, ory, scale, getX(), getY());

        setLocation((int) Math.round(newP.getX()),
            (int) Math.round(newP.getY()));
        this.setX2(getX() + newRelWidth);
        this.setY2(getY() + newRelHeight);

        //this.setRelWAndH(newRelWidth, newRelHeight);
    }

    /** Returns a center point which is used by the latexCommand method.
     * @param scaleFactor A scale factor.
     * @param canvasHeight The height of the current canvas.
     * @return A Point2D object holding the center point.
     */
    protected Point2D getLaTexCenter(float scaleFactor, int canvasHeight) {
        return getLaTexLocation(scaleFactor, canvasHeight);
    }

    /** Returns a radius which is used by the latexCommand method.
     * @param scaleFactor A scale factor.
     * @return The latex radius.
     */
    protected float getLaTexRadius(float scaleFactor) {
        float length =
            (float) Math.sqrt(((double) getWidth() * (double) getWidth())
                + ((double) getHeight() * (double) getHeight()));

        length = length / scaleFactor;

        return length;
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        int size = JaxoPrefs.getIntPref(JaxoPrefs.PREF_VERTEXSIZE);
        if (size > 0) {
            setRelWAndH(size, size);
        }
    }

    /**
     * Prepares a simple edit panel to edit coordinates and color.
     *
     * @param editPanel the panel to prepare.
     * @param iconName the name of the icon for the panel.
     */
    protected void prepareGenericPanel(JaxoObjectEditPanel editPanel, String iconName) {
        int r = (int) Math.round(getRadius());

        editPanel.addXYRPanel(getX(), getY(), r, 0, 0);
        editPanel.addColorPanel(getColor(), JaxoObjectEditPanel.TYPE_LINE_COLOR, 1, 0);

        editPanel.setTitleAndIcon("Vertex_parameters", iconName);
    }
}
