/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.vertex;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import java.io.IOException;
import java.io.ObjectInputStream;

import net.sf.jaxodraw.object.JaxoFillColorObject;
import net.sf.jaxodraw.object.JaxoHandle;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** A vertex type 2 object (a cross with a circle).
 * @since 2.0
 */
public class JaxoVertexT2 extends JaxoVertex implements JaxoFillColorObject {
    private static final long serialVersionUID = 314159L;
    private transient Ellipse2D blob = new Ellipse2D.Float();

    /** The fill color of this fillObject. */
    private Color fillColor;
    private boolean filled = true;

    private void readObject(ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        blob = new Ellipse2D.Float();
    }

    // Bean getter and setter methods

    /** {@inheritDoc} */
    public final boolean isFilled() {
        return filled;
    }

    /** {@inheritDoc} */
    public final void setFilled(boolean value) {
        Boolean old = Boolean.valueOf(filled);
        filled = value;
        firePropertyChange("filled", old, Boolean.valueOf(filled));
    }

    /** {@inheritDoc} */
    public final Color getFillColor() {
        return fillColor;
    }

    /** {@inheritDoc} */
    public final void setFillColor(Color color) {
        Color old = fillColor;
        this.fillColor = color;
        firePropertyChange("fillColor", old, fillColor);
    }

    /** Sets all parameters from the given object to the current one.
     * @param temp The object to copy from.
     */
    public final void copyFrom(JaxoVertexT2 temp) {
        super.copyFrom(temp);
        setFillColor(temp.getFillColor());
        setFilled(temp.isFilled());
    }

    /** {@inheritDoc} */
    public void setState(JaxoObject o) {
        copyFrom((JaxoVertexT2) o);
    }

    /** {@inheritDoc} */
    public final boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;

        if (comp instanceof JaxoVertexT2) {
            JaxoVertexT2 vertexT2 = (JaxoVertexT2) comp;
            if (super.isCopy(comp)
                    && vertexT2.getFillColor().equals(getFillColor())
                    && (vertexT2.isFilled() == isFilled())) {
                isCopy = true;
            }
        }

        return isCopy;

    }

    /** {@inheritDoc} */
    public boolean canBeSelected(int handle, int mode) {
        boolean active = ((handle == SELECT_P1) || (handle == SELECT_P2));
        if (mode == JaxoConstants.RESIZE) {
            active = (handle == SELECT_P2);
        }
        return active;
    }

    /** {@inheritDoc} */
    public void paintHandles(JaxoGraphics2D g2, JaxoHandle h, int editMode) {
        if (editMode == JaxoConstants.UNGROUP) {
            return;
        }

        boolean secondary = (editMode == JaxoConstants.RESIZE);

        h.paint(g2, getX(), getY(), isMarked(), secondary);
        h.paint(g2, getX2(), getY2(), isMarked(), false);
    }

    /** {@inheritDoc} */
    public final void paint(JaxoGraphics2D g2) {
        updateBlob();

        if (isFilled()) {
            g2.setColor(getFillColor());
            g2.fill(blob);
        }

        g2.setColor(getColor());
        g2.setStroke(getStroke());
        g2.draw(blob);

        int sepx = (int) Math.round(getRelh() / Math.sqrt(2.d));
        int sepy = (int) Math.round(getRelw() / Math.sqrt(2.d));

        g2.drawLine(getX() + sepx, getY() - sepy, getX() - sepx, getY() + sepy);
        g2.drawLine(getX() - sepy, getY() - sepx, getX() + sepy, getY() + sepx);

    }

    /**
     * Returns the bounding box of this object.
     *
     * @return the bounding box of this object.
     */
    public Rectangle getBounds() {
        updateBlob();

        return getStroke().createStrokedShape(blob).getBounds();
    }

    /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;

        float radius = getLaTexRadius(scale);

        if ((int) radius == 0) {
            return "%";
        }

        Point2D center = getLaTexCenter(scale, canvasHeight);
        Point2D axes = getLaTexAxes(scale);
        String command = "";

        float angle =
            (float) Math.toDegrees(Math.atan2((float) getRelh(),
                    (float) getRelw()));

        String command1 = "";
        String tlc = JaxoColor.getColorName(getColor());

        if (isFilled()) {
            if (JaxoColor.isGrayScale(getFillColor())) {
                float grayScale = JaxoColor.getGrayScaleFloat(getFillColor());
                command1 =
                    "\\GOval" + "(" + D_FORMAT.format(center.getX()) + ","
                    + D_FORMAT.format(center.getY()) + ")" + "("
                    + D_FORMAT.format(axes.getY()) + ","
                    + D_FORMAT.format(axes.getX()) + ")" + "(" + angle + ")"
                    + "{" + GRAY_SCALE_FORMAT.format(grayScale) + "}";
            } else {
                String tfc = JaxoColor.getColorName(getFillColor());
                command1 =
                    "\\COval" + "(" + D_FORMAT.format(center.getX()) + ","
                    + D_FORMAT.format(center.getY()) + ")" + "("
                    + D_FORMAT.format(axes.getY()) + ","
                    + D_FORMAT.format(axes.getX()) + ")" + "(" + angle + ")"
                    + "{" + tlc + "}" + "{" + tfc + "}";
            }
        } else {
            command1 =
                "\\Oval" + "(" + D_FORMAT.format(center.getX()) + ","
                + D_FORMAT.format(center.getY()) + ")" + "("
                + D_FORMAT.format(axes.getY()) + ","
                + D_FORMAT.format(axes.getX()) + ")" + "(" + angle + ")";
        }

        Point2D invec1 =
            new Point2D.Float((float) (center.getX() - (radius * 0.5f)),
                (float) (center.getY() - (radius * 0.5f)));
        Point2D finvec1 =
            new Point2D.Float((float) (center.getX() + (radius * 0.5f)),
                (float) (center.getY() + (radius * 0.5f)));
        Point2D invec2 =
            new Point2D.Float((float) (center.getX() - (radius * 0.5f)),
                (float) (center.getY() + (radius * 0.5f)));
        Point2D finvec2 =
            new Point2D.Float((float) (center.getX() + (radius * 0.5f)),
                (float) (center.getY() - (radius * 0.5f)));

        double theta = (Math.PI / 4) - Math.atan2(getRelh(), getRelw());

        AffineTransform at = new AffineTransform();
        at.rotate(theta, center.getX(), center.getY());

        Point2D invect1 = new Point2D.Float();
        Point2D finvect1 = new Point2D.Float();
        Point2D invect2 = new Point2D.Float();
        Point2D finvect2 = new Point2D.Float();

        invect1 = at.transform(invec1, invect1);
        finvect1 = at.transform(finvec1, finvect1);
        invect2 = at.transform(invec2, invect2);
        finvect2 = at.transform(finvec2, finvect2);

        String command2 =
            "\\Line" + "(" + D_FORMAT.format(invect1.getX()) + ","
            + D_FORMAT.format(invect1.getY()) + ")" + "("
            + D_FORMAT.format(finvect1.getX()) + ","
            + D_FORMAT.format(finvect1.getY()) + ")";

        String command3 =
            "\\Line" + "(" + D_FORMAT.format(invect2.getX()) + ","
            + D_FORMAT.format(invect2.getY()) + ")" + "("
            + D_FORMAT.format(finvect2.getX()) + ","
            + D_FORMAT.format(finvect2.getY()) + ")";

        command = command1 + command2 + command3;

        return command;
    }

    /** {@inheritDoc} */
    public void setPreferences() {
        super.setPreferences();
        setFilled(true);
        setFillColor(JaxoColor.WHITE);
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        int r = (int) Math.round(getRadius());

        editPanel.addXYRPanel(getX(), getY(), r, 0, 0);
        editPanel.addRotationPanel(getRotationAngle(), 0, 1);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 2);
        editPanel.addFillLineColorPanels(getColor(), getFillColor(),
            isFilled(), 1, 1, 1, 0);

        editPanel.setTitleAndIcon("Vertex_parameters", "vertexT2.png");
    }

    private Point2D getLaTexAxes(float scaleFactor) {
        float wdt = (float) getRadius() / scaleFactor;

        return new Point2D.Float(wdt, wdt);
    }

    private void updateBlob() {
        double len = getRadius();

        blob.setFrame(-len + getX(), -len + getY(), 2 * len, 2 * len);
    }
}
