/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.vertex;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import net.sf.jaxodraw.object.JaxoHandle;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectEditPanel;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.graphics.JaxoGraphics2D;


/** A vertex (a cross).
 * @since 2.0
 */
public class JaxoVertexT4 extends JaxoVertex {
    private static final long serialVersionUID = 314159L;

    /** {@inheritDoc} */
    public final boolean isCopy(JaxoObject comp) {
        boolean isCopy = false;
        if (comp instanceof JaxoVertexT4) {
            isCopy = super.isCopy(comp);
        }

        return isCopy;
    }

    /** {@inheritDoc} */
    public boolean canBeSelected(int handle, int mode) {
        boolean active = ((handle == SELECT_P1) || (handle == SELECT_P2));
        if ((mode == JaxoConstants.RESIZE)) {
            active = (handle == SELECT_P2);
        }
        return active;
    }

    /** {@inheritDoc} */
    public void paintHandles(JaxoGraphics2D g2, JaxoHandle h, int editMode) {
        if (editMode == JaxoConstants.UNGROUP) {
            return;
        }

        boolean secondary = (editMode == JaxoConstants.RESIZE);

        h.paint(g2, getX(), getY(), isMarked(), secondary);
        h.paint(g2, getX2(), getY2(), isMarked(), false);
    }

    /** {@inheritDoc} */
    public final void paint(JaxoGraphics2D g2) {
        g2.setColor(getColor());
        g2.setStroke(getStroke());
        g2.draw(getObjectPath());
    }

    /**
     * Returns the bounding box of this object.
     *
     * @return the bounding box of this object.
     */
    public Rectangle getBounds() {
        return getStroke().createStrokedShape(getObjectPath()).getBounds();
    }

    /** {@inheritDoc} */
    public final String latexCommand(float scale, Dimension canvasDim) {
        int canvasHeight = canvasDim.height;

        String command = "";
        Point2D center = getLaTexCenter(scale, canvasHeight);
        float radius = getLaTexRadius(scale);

        if ((int) radius == 0) {
            command = "%";
        } else {
            Point2D invec1 =
                new Point2D.Float((float) (center.getX() - (radius * 0.707f)),
                    (float) (center.getY() - (radius * 0.707f)));
            Point2D finvec1 =
                new Point2D.Float((float) (center.getX() + (radius * 0.707f)),
                    (float) (center.getY() + (radius * 0.707f)));
            Point2D invec2 =
                new Point2D.Float((float) (center.getX() - (radius * 0.707f)),
                    (float) (center.getY() + (radius * 0.707f)));
            Point2D finvec2 =
                new Point2D.Float((float) (center.getX() + (radius * 0.707f)),
                    (float) (center.getY() - (radius * 0.707f)));

            double theta = (Math.PI / 4) - Math.atan2(getRelh(), getRelw());

            AffineTransform at = new AffineTransform();
            at.rotate(theta, center.getX(), center.getY());

            Point2D invect1 = new Point2D.Float();
            Point2D finvect1 = new Point2D.Float();
            Point2D invect2 = new Point2D.Float();
            Point2D finvect2 = new Point2D.Float();

            invect1 = at.transform(invec1, invect1);
            finvect1 = at.transform(finvec1, finvect1);
            invect2 = at.transform(invec2, invect2);
            finvect2 = at.transform(finvec2, finvect2);

            String command1 =
                "\\Line" + "(" + D_FORMAT.format(invect1.getX()) + ","
                + D_FORMAT.format(invect1.getY()) + ")" + "("
                + D_FORMAT.format(finvect1.getX()) + ","
                + D_FORMAT.format(finvect1.getY()) + ")";

            String command2 =
                "\\Line" + "(" + D_FORMAT.format(invect2.getX()) + ","
                + D_FORMAT.format(invect2.getY()) + ")" + "("
                + D_FORMAT.format(finvect2.getX()) + ","
                + D_FORMAT.format(finvect2.getY()) + ")";

            command = command1 + command2;
        }

        return command;
    }

    /** {@inheritDoc} */
    public void prepareEditPanel(JaxoObjectEditPanel editPanel) {
        int r = (int) Math.round(getRadius());

        editPanel.addXYRPanel(getX(), getY(), r, 0, 0);
        editPanel.addStrokePanel(getStrokeWidth(), 0, 1);
        editPanel.addColorPanel(getColor(), JaxoObjectEditPanel.TYPE_LINE_COLOR, 1, 0);
        editPanel.addRotationPanel(getRotationAngle(), 1, 1);

        editPanel.setTitleAndIcon("Vertex_parameters", "vertexT4.png");
    }

    private GeneralPath getObjectPath() {
        GeneralPath gp = getGeneralPath();
        gp.reset();

        int sepx = getRelh();
        int sepy = getRelw();

        gp.moveTo(getX() + sepx, getY() - sepy);
        gp.lineTo(getX() - sepx, getY() + sepy);
        gp.moveTo(getX() - sepy, getY() - sepx);
        gp.lineTo(getX() + sepy, getY() + sepx);

        return gp;
    }
}
