/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.plugin;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import net.sf.jaxodraw.util.JaxoLog;


/**
 * Loads JaxoDraw plugins.
 *
 * @since 2.0
 */
public class JaxoPluginLoader {

    /**
     * Load plugins. Scans the given directory for jar files,
     * each jar file for a class that ends with "JaxoPlugin.class"
     * and if this class implements JaxoPlugin, adds the jar to the classpath.
     *
     * @param pluginDirName the directory to scan for plugins.
     * @return A list of JaxoPlugins that have been loaded.
     */
    public List loadPlugins(String pluginDirName) {
        List plugins = new ArrayList();

        String[] children = getJars(pluginDirName);

        if (children == null) {
            JaxoLog.debug("No plugin directory found, no plugins loaded!");
            return plugins;
        }

        // loop over jars in plugin dir
        for (int i = 0; i < children.length; i++) {
            String filename = new File(pluginDirName, children[i]).toString();
            loadPluginFromJar(filename, plugins);
        }

        // load plugin properties from corresponding files
        for (Iterator it = plugins.iterator(); it.hasNext();) {
            ((JaxoPlugin) it.next()).loadProperties();
        }

        return plugins;
    }

    /**
     * Invalidates the class loader responsible for plugin loading
     * and re-scans the plugin dir, loading any found classes.
     * This is the only way to remove plugins from the class loader.
     *
     * @param pluginDirName the directory to scan for plugins.
     * @return A list of JaxoPlugins that have been loaded.
     * @see #loadPlugins(java.lang.String)
     */
    public List reValidate(String pluginDirName) {
        jarLoader().reset();

        return loadPlugins(pluginDirName);
    }

    /**
     * Loads plugins from a jar file. If the jar contains
     * any classes that end with "JaxoPlugin.class" and that
     * implement JaxoPlugin, the jar is added to the classpath.
     *
     * @param filename absolute path to a jar file that contains a JaxoPlugin.
     *
     * @return A List of JaxoPlugins that have been loaded,
     * or null if there was a problem or the jar didn't contain a JaxoPlugin.
     */
    public List loadPluginsFromJar(String filename) {
        List plugins = new ArrayList();
        loadPluginFromJar(filename, plugins);

        if (plugins.isEmpty()) {
            return null;
        }

        for (Iterator it = plugins.iterator(); it.hasNext();) {
            JaxoPlugin plugin = (JaxoPlugin) it.next();
            plugin.loadProperties();
        }

        return plugins;
    }

    /**
     * Tries to find the jar source file for a JaxoPlugin
     * in the given directory. The search is done via the pluginId() of the
     * plugin, ie the first jar that contains a class whose name matches
     * the pluginId is returned.
     *
     * @param plugin the plugin whose source jar to find.
     * @param pluginDir the directory where jars are listed.
     * @return a File whose jar contains the given plugin,
     *      or null if nothing is found.
     */
    public static File getPluginJar(JaxoPlugin plugin, String pluginDir) {
        String[] children = getJars(pluginDir);

        if (children == null) {
            return null;
        }

        // loop over jars in plugin dir
        for (int i = 0; i < children.length; i++) {
            File file = new File(pluginDir, children[i]);
            JarFile jarFile = getJarFile(file.toString());
            Enumeration entries = jarFile.entries();

            // loop over jar entries
            while (entries.hasMoreElements()) {
                JarEntry entry = (JarEntry) entries.nextElement();
                String name = entry.getName();
                if (name.endsWith("JaxoPlugin.class")) {

                    String clas =
                        name.substring(0, name.lastIndexOf(".class"))
                            .replace('/', '.');
                    if (clas.equals(plugin.pluginId())) {
                        return file;
                    }

                }
            }
        }

        // nothing found
        return null;
    }

      //
     // private
    //

    private JarLoader jarLoader() {
        return JarLoader.getInstance();
    }

    private static JarFile getJarFile(String filename) {
        JarFile jarFile = null;
        try {
            jarFile = new JarFile(filename);
        } catch (IOException e) {
            JaxoLog.debug(filename + " cannot be loaded, ignoring!", e);
            return null;
        }
        return jarFile;
    }

    private void loadPluginFromJar(String filename, List plugins) {
        JarFile jarFile = getJarFile(filename);

        if (jarFile == null) {
            return;
        }

        // check if a jar with the same mainclass is already loaded
        try {
            String mainclass =
                    jarFile.getManifest().getMainAttributes().getValue("Main-Class");

            if (mainclass != null && jarLoader().isLoaded(mainclass)) {
                JaxoLog.warn(mainclass + ": A Main-Class with this name is already loaded! Ignoring.");
                return;
            }

            // TODO: try to load dependencies from manifests Class-Path?
            //String classPath =
            //        jarFile.getManifest().getMainAttributes().getValue("Class-Path");
        } catch (IOException ex) {
            JaxoLog.debug(ex);
            JaxoLog.warn("Failed to load plugin from: " + jarFile.getName());
            return;
        }

        Enumeration entries = jarFile.entries();

        // loop over jar entries
        while (entries.hasMoreElements()) {
            JarEntry entry = (JarEntry) entries.nextElement();
            String name = entry.getName();
            if (name.endsWith("JaxoPlugin.class")) {
                jarLoader().addJar(filename);

                String clas =
                    name.substring(0, name.lastIndexOf(".class"))
                        .replace('/', '.');

                if (jarLoader().isLoaded(clas)) {
                    JaxoLog.warn(clas + ": double loaded!");
                }

                loadPluginFromClass(clas, plugins);
            }
        }
    }

    private void loadPluginFromClass(String clas, List plugins) {
        Class jarLoaderClass = null;

        try {
            jarLoaderClass = jarLoader().loadClass(clas);
        } catch (ClassNotFoundException e) {
            JaxoLog.warn("Plugin class " + clas + " cannot be loaded, ignoring!");
            JaxoLog.debug(e);
            return;
        }

        JaxoLog.debug("Class " + clas + " successfully loaded!");

        Object pluginClass = null;

        try {
            pluginClass = jarLoaderClass.newInstance();
        } catch (IllegalAccessException e) {
            JaxoLog.warn("Plugin class " + clas + " cannot be accessed, ignoring!");
            JaxoLog.debug(e);
            return;
        } catch (InstantiationException e) {
            JaxoLog.warn("Plugin class " + clas + " cannot be instantiated, ignoring!");
            JaxoLog.debug(e);
            return;
        } catch (NoClassDefFoundError e) {
            JaxoLog.warn("Plugin class " + clas + " is missing dependencies, ignoring!");
            JaxoLog.debug(new Exception(e));
            return;
        }

        if (pluginClass instanceof JaxoPlugin) {
            JaxoPlugin plugin = (JaxoPlugin) pluginClass;

            if (plugin.pluginId() == null) {
                JaxoLog.warn(clas + " has null pluginId(), ignoring!");
            } else if (plugin.pluginId().equals(jarLoaderClass.getName())) {
                plugins.add(pluginClass);
                JaxoLog.info(clas + ": Plugin registered!");
            } else {
                JaxoLog.warn(clas + " has invalid pluginId(), ignoring!");
            }
        } else {
            JaxoLog.warn(clas + " is not a JaxoPlugin, ignoring!");
        }
    }

    private static String[] getJars(String pluginDirName) {
        FilenameFilter filter =
            new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    return name.endsWith(".jar");
                }
            };

        return new File(pluginDirName).list(filter);
    }

    /** Load jars into the classpath. */
    private static final class JarLoader extends URLClassLoader {
        private static JarLoader jarLoader;

        private JarLoader() {
            super(new URL[] {});
        }

        public static JarLoader getInstance() {
            synchronized (JarLoader.class) {
                if (jarLoader == null) {
                    jarLoader = new JarLoader();
                }

                return jarLoader;
            }
        }

        public boolean isLoaded(String clazz) {
            return (findLoadedClass(clazz) != null);
        }

        public void reset() {
            // only way to remove classes (plugins) is to gc the classloader
            jarLoader = null;
        }

        public void addJar(String path) {
            String uri = new File(path).toURI().toString();
            String urlPath = "jar:" + uri + "!/";

            JaxoLog.debug("Loading plugin from " + path + "...");

            try {
                addURL(new URL(urlPath));
            } catch (MalformedURLException e) {
                JaxoLog.debug("Could not load " + urlPath + ", ignoring!", e);
            }
        }
    }
}
