/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.util;

import java.util.HashMap;
import java.util.Map;
import java.util.prefs.Preferences;


/** Responsible for importing/exporting user preferences.
 * @since 2.0
 */
public final class JaxoPrefs {

    // Strings:

    /** The preferred HTML browser. */
    public static final String PREF_BROWSER = "prefBROWSER";

    /** The preferred text editor. */
    public static final String PREF_EDITOR = "prefEDITOR";

    /** The preferred postscript viewer. */
    public static final String PREF_PSVIEWER = "prefPSVIEWER";

    /** The preferred Look and Feel (as of 2.0, the Class Name). */
    public static final String PREF_LOOKNFEEL = "prefLOOKNFEEL";

    /** The preferred language. */
    public static final String PREF_LANGUAGE = "prefLANGUAGE";

    /** The preferred grid type. */
    public static final String PREF_GRIDTYPE = "prefGRIDTYPE";

    /** The preferred Object color. */
    public static final String PREF_COLOR = "prefCOLOR";

    /** The preferred Postscript font family. */
    public static final String PREF_PSFAMILY = "prefPSFAMILY";

    /** The preferred Text color. */
    public static final String PREF_TEXTCOLOR = "prefTEXTCOLOR";

    /** The preferred latex path. */
    public static final String PREF_LATEXPATH = "prefLATEXPATH";

    /** The preferred dvips path. */
    public static final String PREF_DVIPSPATH = "prefDVIPSPATH";

    /** The preferred canvas background color. */
    public static final String PREF_CANVASBACKGROUND = "prefCANVASBACKGROUND";

    /** The version of the preferences. */
    public static final String PREF_VERSION = "prefVERSION";

    /** The preferred canvas background color. */
    public static final String PREF_GRIDCOLOR = "prefGRIDCOLOR";

    /** The current user directory. */
    public static final String PREF_USER_DIR = "prefUSERDIR";

    // ints:

    /** The preferred screen size - x dimension. */
    public static final String PREF_SCREENSIZEX = "prefSCREENSIZEX";

    /** The preferred screen size - y dimension. */
    public static final String PREF_SCREENSIZEY = "prefSCREENSIZEY";

    /** The preferred grid size. */
    public static final String PREF_GRIDSIZE = "prefGRIDSIZE";

    /** The preferred amplitude for photon and gluon objects. */
    public static final String PREF_AMPLITUDE = "prefAMPLITUDE";

    /** The preferred Postscript font style. */
    public static final String PREF_PSSTYLE = "prefPSSTYLE";

    /** The preferred Postscript font size. */
    public static final String PREF_PSSIZE = "prefPSSIZE";

    /** The preferred TeX font size. */
    public static final String PREF_TEXSIZE = "prefTEXSIZE";

    /** The preferred TeX alignment. */
    public static final String PREF_TEXALIGN = "prefTEXALIGN";

    /** The preferred postscript text rotation angle. */
    public static final String PREF_PSROTANGLE = "prefPSROTANGLE";

    /** The preferred LaTeX text rotation angle. */
    public static final String PREF_TEXROTANGLE = "prefTEXROTANGLE";

    /** The default action. */
    public static final String PREF_DEFAULTACTION = "prefDEFAULTACTION";

    /** The preferred undo depth. */
    public static final String PREF_UNDODEPTH = "prefUNDODEPTH";

    /** The preferred color space. */
    public static final String PREF_COLORSPACE = "prefCOLORSPACE";

    /** The preferred grid style. */
    public static final String PREF_GRIDSTYLE = "prefGRIDSTYLE";

    /** The preferred default vertex size. */
    public static final String PREF_VERTEXSIZE = "prefVERTEXSIZE";


    // floats:

    /** The preferred line width. */
    public static final String PREF_LINEWIDTH = "prefLINEWIDTH";

    /** The preferred separation of double line objects. */
    public static final String PREF_DLSEP = "prefDLSEP";

    /** The preferred arrow poaition. */
    public static final String PREF_ARROWPOSITION = "prefARROWPOSITION";

    /** The preferred arrow length. */
    public static final String PREF_ARROWLENGTH = "prefARROWLENGTH";

    /** The preferred arrow width. */
    public static final String PREF_ARROWWIDTH = "prefARROWWIDTH";

    /** The preferred arrow inset. */
    public static final String PREF_ARROWINSET = "prefARROWINSET";

    // booleans:

    /** A boolean preference indicating wether to show the tool bar or not. */
    public static final String PREF_SHOWTOOL = "prefSHOWTOOL";

    /** A boolean preference indicating wether to show the status bar or not.*/
    public static final String PREF_SHOWSTATUS = "prefSHOWSTATUS";

    /** A boolean preference indicating wether to use antialias or not.*/
    public static final String PREF_ANTIALIAS = "prefANTIALIAS";

    /** A boolean preference indicating wether to draw arrows or not.*/
    public static final String PREF_ARROW = "prefARROW";

    /** A boolean preference indicating wether the grid is on or off.*/
    public static final String PREF_GRIDONOFF = "prefGRIDONOFF";

    /** A boolean preference indicating wether snapping is on or off.*/
    public static final String PREF_SNAPONOFF = "prefSNAPONOFF";

    /** A boolean preference indicating wether to show the gridbar bar or not. */
    public static final String PREF_SHOWGRIDBAR = "prefSHOWGRIDBAR";

    /** A boolean preference indicating wether to show the start-up splash window or not. */
    public static final String PREF_SHOWSPLASHWINDOW = "prefSHOWSPLASHWINDOW";

    // private
    private static Preference[] preferences;

    private static final Preferences PREFS =
        Preferences.userNodeForPackage(JaxoPrefs.class);

    static {
        String black = JaxoColor.getColorName(JaxoColor.BLACK);
        String curDIR = System.getProperty("user.dir");

        preferences =
            new Preference[]{
                new Preference(PREF_BROWSER, ""),
                new Preference(PREF_EDITOR, ""),
                new Preference(PREF_PSVIEWER, ""),
                new Preference(PREF_LOOKNFEEL,
                    "javax.swing.plaf.metal.MetalLookAndFeel"),
                new Preference(PREF_LANGUAGE, "english"),
                new Preference(PREF_GRIDTYPE, "rectangular"),
                new Preference(PREF_COLOR, black),
                new Preference(PREF_PSFAMILY, "Default"),
                new Preference(PREF_TEXTCOLOR, black),
                new Preference(PREF_LATEXPATH, "latex"),
                new Preference(PREF_DVIPSPATH, "dvips"),
                new Preference(PREF_CANVASBACKGROUND, "r255g255b255"),
                new Preference(PREF_VERSION, "1.4"),
                new Preference(PREF_GRIDCOLOR, black),
                new Preference(PREF_USER_DIR, curDIR),
                new Preference(PREF_DEFAULTACTION, JaxoConstants.DEF_NONE),
                new Preference(PREF_UNDODEPTH, 20),
                new Preference(PREF_COLORSPACE, JaxoColor.ALL_COLORS_MODE),
                new Preference(PREF_GRIDSTYLE, 0),
                new Preference(PREF_VERTEXSIZE, 0),
                new Preference(PREF_SCREENSIZEX, 800),
                new Preference(PREF_SCREENSIZEY, 600),
                new Preference(PREF_GRIDSIZE, 16),
                new Preference(PREF_AMPLITUDE, 15),
                new Preference(PREF_PSSTYLE, 2),
                new Preference(PREF_PSSIZE, 18),
                new Preference(PREF_TEXSIZE, 6),
                new Preference(PREF_TEXALIGN, 2),
                new Preference(PREF_PSROTANGLE, 0),
                new Preference(PREF_TEXROTANGLE, 0),
                new Preference(PREF_LINEWIDTH, 1f),
                new Preference(PREF_DLSEP, 2f),
                new Preference(PREF_ARROWPOSITION, .5f),
                new Preference(PREF_ARROWLENGTH, 5f),
                new Preference(PREF_ARROWWIDTH, 2f),
                new Preference(PREF_ARROWINSET, .2f),
                new Preference(PREF_SHOWTOOL, true),
                new Preference(PREF_SHOWSTATUS, true),
                new Preference(PREF_ANTIALIAS, true),
                new Preference(PREF_ARROW, true),
                new Preference(PREF_GRIDONOFF, true),
                new Preference(PREF_SNAPONOFF, true),
                new Preference(PREF_SHOWGRIDBAR, true),
                new Preference(PREF_SHOWSPLASHWINDOW, true)
            };
    }

    private static Map nameToPreference; // String name -> its Preference
    private static Map values; // String name -> its value (mixed type)

    static {
        nameToPreference = new HashMap();
        values = new HashMap();

        for (int i = 0; i < preferences.length; i++) {
            nameToPreference.put(preferences[i].getName(), preferences[i]);
            values.put(preferences[i].getName(), preferences[i].getDefaultValue());
        }
    }

    /** Defines the maximum numbers of recent files presented in the recent file
     *  menu.
     */
    public static final int MAX_RECENT_FILES = 10;

    /** An array of strings identifying the recently opened files. */
    private static String[] recentKEYS = new String[MAX_RECENT_FILES];

    /** The recently opened files. */
    private static String[] recentFile = new String[MAX_RECENT_FILES];


    static {
        for (int i = 0; i < MAX_RECENT_FILES; i++) {
            String rf = "";
            recentFile[i] = rf;
            rf = "recentFile".concat((new Integer(i + 1)).toString());
            recentKEYS[i] = rf;
        }
    }

    /** Empty private constructor: do not instantiate. */
    private JaxoPrefs() {
        // empty on purpose
    }

    /**
     * Returns an array of recently opened files.
     *
     * @return An array containing the absolute paths of recently opened files
     * or empty Strings.
     */
    public static String[] getRecentFiles() {
        String[] copy = new String[recentFile.length];
        System.arraycopy(recentFile, 0, copy, 0, recentFile.length);
        return copy;
    }

    /**
     * Set the recently opened files to the given array of absolute path names.
     *
     * @param filePaths the absolute path names.
     *      If an entry is null, it will be replaced by an empty String.
     */
    public static void setRecentFiles(String[] filePaths) {
        clearRecentFiles();

        for (int i = 0; i < filePaths.length; i++) {
            String rf = filePaths[i];
            if (rf == null) {
                recentFile[i] = "";
            } else {
                recentFile[i] = rf;
            }
        }
    }

    /**
     * Sets all recent files to empty Strings.
     */
    public static void clearRecentFiles() {
        for (int i = 0; i < MAX_RECENT_FILES; i++) {
            recentFile[i] = "";
        }
    }

    /**
     * Adds the given absolute path name to the list of recntly opened files.
     *
     * @param st an absolute path name.
     */
    public static void addRecentFile(String st) {
        int i = 0;
        for (; i < MAX_RECENT_FILES; i++) {
            if ("".equals(recentFile[i])) {
                recentFile[i] = st;
                break;
            }
        }

        if (i == MAX_RECENT_FILES) {
            System.arraycopy(recentFile, 1, recentFile, 0, recentFile.length - 1);
            recentFile[MAX_RECENT_FILES - 1] = st;
        }

    }

    /**
     * Returns the preference with 'name' as a String, or throws a
     * IllegalArgumentException if it does not correspond to a String.
     *
     * @param name The name of the preference.
     * @return The preference with 'name'.
     */
    public static String getStringPref(String name) {
        if (hasType(name, String.class)) {
            return (String) values.get(name);
        } else {
            throw new IllegalArgumentException("Not String Pref: " + name);
        }
    }

    /**
     * Returns the preference with 'name' as an integer, or throws a
     * IllegalArgumentException if it does not correspond to an int.
     *
     * @param name The name of the preference.
     * @return The preference with 'name'.
     */
    public static int getIntPref(String name) {
        if (hasType(name, Integer.TYPE)) {
            return ((Integer) values.get(name)).intValue();
        } else {
            throw new IllegalArgumentException("Not int Pref: " + name);
        }
    }

    /**
     * Returns the preference with 'name' as a float, or throws a
     * IllegalArgumentException if it does not correspond to a float.
     *
     * @param name The name of the preference.
     * @return The preference with 'name'.
     */
    public static float getFloatPref(String name) {
        if (hasType(name, Float.TYPE)) {
            return ((Float) values.get(name)).floatValue();
        } else {
            throw new IllegalArgumentException("Not float Pref: " + name);
        }
    }

    /**
     * Returns the preference with 'name' as a boolean, or throws a
     * IllegalArgumentException if it does not correspond to a boolean.
     *
     * @param name The name of the preference.
     * @return The preference with 'name'.
     */
    public static boolean getBooleanPref(String name) {
        if (hasType(name, Boolean.TYPE)) {
            return Boolean.TRUE.equals(values.get(name));
        } else {
            throw new IllegalArgumentException("Not boolean Pref: " + name);
        }
    }

    /**
     * Sets the preference with 'name' to the string str.
     *
     * @param name The name of the preference to be set.
     * @param str the preference to be set.
     */
    public static void setStringPref(String name, String str) {
        setPref(name, str);
    }

    /**
     * Sets the preference with 'name' to the int value newPref.
     *
     * @param name The name of the preference to be set.
     * @param newPref The preference to be set.
     */
    public static void setIntPref(String name, int newPref) {
        setPref(name, new Integer(newPref));
    }

    /**
     * Sets the preference with 'name' to the float value of newPref.
     *
     * @param name The name of the preference to be set.
     * @param newPref The preference to be set.
     */
    public static void setFloatPref(String name, float newPref) {
        setPref(name, new Float(newPref));
    }

    /**
     * Sets the preference with 'name' to the boolean value of newPref.
     *
     * @param name The name of the preference to be set.
     * @param newPref The preference to be set.
     */
    public static void setBooleanPref(String name, boolean newPref) {
        setPref(name, Boolean.valueOf(newPref));
    }

    /**
     * State of preferences. The only use is to restore the current state later
     * (after having made temporary changes) with {@link #setState(Object)}.
     *
     * @return Object
     */
    public static Object getState() {
        return new HashMap(values);
    }

    /**
     * Restore the preferences to a previous state.
     *
     *  @param value previous state to be set, obtained from {@link #getState}.
     */
    public static void setState(Object value) {
        Map m = (Map) value;

        for (int i = 0; i < preferences.length; i++) {
            setPref(preferences[i].getName(), m.get(preferences[i].getName()));
        }
    }

    /**
     * Sets the Preferences for the current session from the Preferences node.
     */
    public static void initSessionPrefs() {
        Preferences legacy =  Preferences.userRoot().node(JaxoInfo.USER_HOME);
        setPrefsFrom(legacy);
        setPrefsFrom(PREFS);

        // re-set user dir (or should we keep it across sessions?)
        resetPref(PREF_USER_DIR);

        setStringPref(PREF_VERSION, PREFS.get(PREF_VERSION, "1.3"));

        if (!getStringPref(PREF_VERSION).equals("1.4")) {
            upgradePreferences();
            setStringPref(PREF_VERSION, "1.4");
        }
    }

    private static void setPrefsFrom(Preferences p) {
        for (int j = 0; j < preferences.length; j++) {
            setPrefFromString(preferences[j].getName(),
            p.get(preferences[j].getName(),
            preferences[j].getDefaultValue().toString()));
        }

        for (int j = 0; j < MAX_RECENT_FILES; j++) {
            recentFile[j] = p.get(recentKEYS[j], recentFile[j]);
        }
    }



    /**
     * Re-sets the given Preference to its default value.
     *
     * @param name the name of the preference to re-set.
     */
    public static void resetPref(String name) {
        Preference p = (Preference) nameToPreference.get(name);
        setPref(name, p.getDefaultValue());
    }

    /**
     * Saves the current set of Preferences to the user Preferences node,
     * so they will be used in later sessions.
     */
    public static void savePrefs() {
        for (int j = 0; j < preferences.length; j++) {
            PREFS.put(preferences[j].getName(),
                values.get(preferences[j].getName()).toString());
        }

        for (int j = 0; j < MAX_RECENT_FILES; j++) {
            PREFS.put(recentKEYS[j], recentFile[j]);
        }
    }

    /**
     * Saves the last visited files in the Preferences node.
     */
    public static void saveRecentFiles() {
        for (int j = 0; j < MAX_RECENT_FILES; j++) {
            PREFS.put(recentKEYS[j], recentFile[j]);
        }
    }

      //
     // private
    //

    private static boolean hasType(String name, Class type) {
        Preference p = (Preference) nameToPreference.get(name);

        return (p != null) && (p.getType() == type);
    }

    private static void upgradePreferences() {
        String lookAndFeel = getStringPref(PREF_LOOKNFEEL);

        if ("Metal".equals(lookAndFeel)) {
            lookAndFeel = "javax.swing.plaf.metal.MetalLookAndFeel";
        } else if ("CDE/Motif".equals(lookAndFeel)) {
            lookAndFeel = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
        } else if ("Windows".equals(lookAndFeel)) {
            lookAndFeel = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
        } else if ("Windows Classic".equals(lookAndFeel)) {
            lookAndFeel =
                "com.sun.java.swing.plaf.windows.WindowsClassicLookAndFeel";
        } else if ("Mac OS X".equals(lookAndFeel)) {
            lookAndFeel = "apple.laf.AquaLookAndFeel";
        } else if ("GTK".equals(lookAndFeel) || "GTK+".equals(lookAndFeel)) {
            lookAndFeel = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
        }

        setStringPref(PREF_LOOKNFEEL, lookAndFeel);
    }

    private static void setPrefFromString(String name, String value) {
        Preference p = (Preference) nameToPreference.get(name);

        if (p.getType() == String.class) {
            setPref(name, value);
        } else if (p.getType() == Integer.TYPE) {
            setPref(name, new Integer(value));
        } else if (p.getType() == Float.TYPE) {
            setPref(name, new Float(value));
        } else if (p.getType() == Boolean.TYPE) {
            setPref(name, Boolean.valueOf(value));
        }
    }

    private static void setPref(String name, Object value) {
        if (name.equals(PREF_LANGUAGE)) {
            Object old = values.get(name);
            values.put(name, value);
            if ((old == null) ? (value != null) : (!old.equals(value))) {
                JaxoLanguage.languageChanged(getStringPref(PREF_LANGUAGE));
            }
        } else {
            values.put(name, value);
        }
    }

    private static class Preference {
        private final String name;
        private final Class type;
        private final Object defaultValue;

        public Preference(String nname, String value) {
            this.name = nname;
            this.type = String.class;
            this.defaultValue = value;
        }

        public Preference(String nname, int value) {
            this.name = nname;
            this.type = Integer.TYPE;
            this.defaultValue = new Integer(value);
        }

        public Preference(String nname, float value) {
            this.name = nname;
            this.type = Float.TYPE;
            this.defaultValue = new Float(value);
        }

        public Preference(String nname, boolean value) {
            this.name = nname;
            this.type = Boolean.TYPE;
            this.defaultValue = Boolean.valueOf(value);
        }

        public String getName() {
            return name;
        }

        public Class getType() {
            return type;
        }

        public Object getDefaultValue() {
            return defaultValue;
        }
    }
}
