/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.util.graphics;

import java.awt.Color;
import java.awt.Composite;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Paint;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ImageObserver;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.RenderableImage;

import java.text.AttributedCharacterIterator;

import java.util.Map;


/** Wrapper around an existing Graphics2D.
 * All methods except two create() are delegated.
 * @since 2.0
 */
public class JaxoWrapperGraphics2D extends Graphics2D implements Cloneable {
    private Graphics2D graphics;

    /** Constructor.
     * @param g The initial graphics object.
     */
    protected JaxoWrapperGraphics2D(Graphics2D g) {
        graphics = g;
    }

    /** A clone which wraps the same Graphics2D as this.
     * @return A clone of the Graphics2D.
     */
    protected Object copy() {
        try {
            return super.clone();
        } catch (CloneNotSupportedException e) {
            throw new Error(e);
        }
    }

    /** A clone which wraps the given Graphics2D.
     * @param g The graphics object to clone.
     * @return A clone of the Graphics2D.
     */
    protected JaxoWrapperGraphics2D clone(Graphics2D g) {
        JaxoWrapperGraphics2D result = (JaxoWrapperGraphics2D) copy();
        result.graphics = g;
        return result;
    }

    /** A JaxoWrapperGraphics2D wrapping a Graphics2D obtained
     * from the wrapped Graphics' create().
     * @return A new graphics object.
     */
    public Graphics create() {
        return clone((Graphics2D) graphics.create());
    }

    /** A JaxoWrapperGraphics2D wrapping a Graphics2D obtained
     * from the wrapped Graphics' create(...).
     * @param x The x coordinate.
     * @param y The y coordinate.
     * @param w The width.
     * @param h The height.
     * @return A new graphics object.
     */
    public Graphics create(int x, int y, int w, int h) {
        return clone((Graphics2D) graphics.create(x, y, w, h));
    }

    /// Everything else is delegated

    /** {@inheritDoc} */
    public void dispose() {
        graphics.dispose();
    }

    /** {@inheritDoc} */
    public void setPaintMode() {
        graphics.setPaintMode();
    }

    /** {@inheritDoc} */
    public void translate(int a, int b) {
        graphics.translate(a, b);
    }

    /** {@inheritDoc} */
    public void clearRect(int a, int b, int c, int d) {
        graphics.clearRect(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void clipRect(int a, int b, int c, int d) {
        graphics.clipRect(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void drawLine(int a, int b, int c, int d) {
        graphics.drawLine(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void drawOval(int a, int b, int c, int d) {
        graphics.drawOval(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void drawRect(int a, int b, int c, int d) {
        graphics.drawRect(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void fillOval(int a, int b, int c, int d) {
        graphics.fillOval(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void fillRect(int a, int b, int c, int d) {
        graphics.fillRect(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void setClip(int a, int b, int c, int d) {
        graphics.setClip(a, b, c, d);
    }

    /** {@inheritDoc} */
    public boolean hitClip(int a, int b, int c, int d) {
        return graphics.hitClip(a, b, c, d);
    }

    /** {@inheritDoc} */
    public void copyArea(int a, int b, int c, int d, int e, int f) {
        graphics.copyArea(a, b, c, d, e, f);
    }

    /** {@inheritDoc} */
    public void drawArc(int a, int b, int c, int d, int e, int f) {
        graphics.drawArc(a, b, c, d, e, f);
    }

    /** {@inheritDoc} */
    public void drawRoundRect(int a, int b, int c, int d, int e, int f) {
        graphics.drawRoundRect(a, b, c, d, e, f);
    }

    /** {@inheritDoc} */
    public void fillArc(int a, int b, int c, int d, int e, int f) {
        graphics.fillArc(a, b, c, d, e, f);
    }

    /** {@inheritDoc} */
    public void fillRoundRect(int a, int b, int c, int d, int e, int f) {
        graphics.fillRoundRect(a, b, c, d, e, f);
    }

    /** {@inheritDoc} */
    public void draw3DRect(int a, int b, int c, int d, boolean e) {
        graphics.draw3DRect(a, b, c, d, e);
    }

    /** {@inheritDoc} */
    public void fill3DRect(int a, int b, int c, int d, boolean e) {
        graphics.fill3DRect(a, b, c, d, e);
    }

    /** {@inheritDoc} */
    public void drawBytes(byte[] a, int b, int c, int d, int e) {
        graphics.drawBytes(a, b, c, d, e);
    }

    /** {@inheritDoc} */
    public void drawChars(char[] a, int b, int c, int d, int e) {
        graphics.drawChars(a, b, c, d, e);
    }

    /** {@inheritDoc} */
    public void drawPolygon(int[] a, int[] b, int c) {
        graphics.drawPolygon(a, b, c);
    }

    /** {@inheritDoc} */
    public void drawPolyline(int[] a, int[] b, int c) {
        graphics.drawPolyline(a, b, c);
    }

    /** {@inheritDoc} */
    public void fillPolygon(int[] a, int[] b, int c) {
        graphics.fillPolygon(a, b, c);
    }

    /** {@inheritDoc} */
    public Color getColor() {
        return graphics.getColor();
    }

    /** {@inheritDoc} */
    public void setColor(Color a) {
        graphics.setColor(a);
    }

    /** {@inheritDoc} */
    public void setXORMode(Color a) {
        graphics.setXORMode(a);
    }

    /** {@inheritDoc} */
    public Font getFont() {
        return graphics.getFont();
    }

    /** {@inheritDoc} */
    public void setFont(Font a) {
        graphics.setFont(a);
    }

    /** {@inheritDoc} */
    public FontMetrics getFontMetrics() {
        return graphics.getFontMetrics();
    }

    /** {@inheritDoc} */
    public void drawPolygon(Polygon a) {
        graphics.drawPolygon(a);
    }

    /** {@inheritDoc} */
    public void fillPolygon(Polygon a) {
        graphics.fillPolygon(a);
    }

    /** {@inheritDoc} */
    public Rectangle getClipBounds() {
        return graphics.getClipBounds();
    }

    /** {@inheritDoc} */
    public Shape getClip() {
        return graphics.getClip();
    }

    /** {@inheritDoc} */
    public void setClip(Shape a) {
        graphics.setClip(a);
    }

    /** {@inheritDoc} */
    public FontMetrics getFontMetrics(Font a) {
        return graphics.getFontMetrics(a);
    }

    /** {@inheritDoc} */
    public Rectangle getClipBounds(Rectangle a) {
        return graphics.getClipBounds(a);
    }

    /** {@inheritDoc} */
    public boolean drawImage(java.awt.Image a, int b, int c, int d, int e,
        int f, int g, int h, int i, ImageObserver j) {
        return graphics.drawImage(a, b, c, d, e, f, g, h, i, j);
    }

    /** {@inheritDoc} */
    public boolean drawImage(java.awt.Image a, int b, int c, int d, int e,
        ImageObserver f) {
        return graphics.drawImage(a, b, c, d, e, f);
    }

    /** {@inheritDoc} */
    public boolean drawImage(java.awt.Image a, int b, int c, ImageObserver d) {
        return graphics.drawImage(a, b, c, d);
    }

    /** {@inheritDoc} */
    public boolean drawImage(java.awt.Image a, int b, int c, int d, int e,
        int f, int g, int h, int i, Color j, ImageObserver k) {
        return graphics.drawImage(a, b, c, d, e, f, g, h, i, j, k);
    }

    /** {@inheritDoc} */
    public boolean drawImage(java.awt.Image a, int b, int c, int d, int e,
        Color f, ImageObserver g) {
        return graphics.drawImage(a, b, c, d, e, f, g);
    }

    /** {@inheritDoc} */
    public boolean drawImage(Image a, int b, int c, Color d, ImageObserver e) {
        return graphics.drawImage(a, b, c, d, e);
    }

    /** {@inheritDoc} */
    public void rotate(double a) {
        graphics.rotate(a);
    }

    /** {@inheritDoc} */
    public void scale(double a, double b) {
        graphics.scale(a, b);
    }

    /** {@inheritDoc} */
    public void shear(double a, double b) {
        graphics.shear(a, b);
    }

    /** {@inheritDoc} */
    public void translate(double a, double b) {
        graphics.translate(a, b);
    }

    /** {@inheritDoc} */
    public void rotate(double a, double b, double c) {
        graphics.rotate(a, b, c);
    }

    /** {@inheritDoc} */
    public Color getBackground() {
        return graphics.getBackground();
    }

    /** {@inheritDoc} */
    public void setBackground(Color a) {
        graphics.setBackground(a);
    }

    /** {@inheritDoc} */
    public Composite getComposite() {
        return graphics.getComposite();
    }

    /** {@inheritDoc} */
    public void setComposite(Composite a) {
        graphics.setComposite(a);
    }

    /** {@inheritDoc} */
    public GraphicsConfiguration getDeviceConfiguration() {
        return graphics.getDeviceConfiguration();
    }

    /** {@inheritDoc} */
    public Paint getPaint() {
        return graphics.getPaint();
    }

    /** {@inheritDoc} */
    public void setPaint(Paint a) {
        graphics.setPaint(a);
    }

    /** {@inheritDoc} */
    public RenderingHints getRenderingHints() {
        return graphics.getRenderingHints();
    }

    /** {@inheritDoc} */
    public void clip(Shape a) {
        graphics.clip(a);
    }

    /** {@inheritDoc} */
    public void draw(Shape a) {
        graphics.draw(a);
    }

    /** {@inheritDoc} */
    public void fill(Shape a) {
        graphics.fill(a);
    }

    /** {@inheritDoc} */
    public Stroke getStroke() {
        return graphics.getStroke();
    }

    /** {@inheritDoc} */
    public void setStroke(Stroke a) {
        graphics.setStroke(a);
    }

    /** {@inheritDoc} */
    public FontRenderContext getFontRenderContext() {
        return graphics.getFontRenderContext();
    }

    /** {@inheritDoc} */
    public void drawGlyphVector(GlyphVector a, float b, float c) {
        graphics.drawGlyphVector(a, b, c);
    }

    /** {@inheritDoc} */
    public AffineTransform getTransform() {
        return graphics.getTransform();
    }

    /** {@inheritDoc} */
    public void setTransform(AffineTransform a) {
        graphics.setTransform(a);
    }

    /** {@inheritDoc} */
    public void transform(AffineTransform a) {
        graphics.transform(a);
    }

    /** {@inheritDoc} */
    public void drawString(String a, float b, float c) {
        graphics.drawString(a, b, c);
    }

    /** {@inheritDoc} */
    public void drawString(String a, int b, int c) {
        graphics.drawString(a, b, c);
    }

    /** {@inheritDoc} */
    public void drawString(AttributedCharacterIterator a, float b, float c) {
        graphics.drawString(a, b, c);
    }

    /** {@inheritDoc} */
    public void drawString(AttributedCharacterIterator a, int b, int c) {
        graphics.drawString(a, b, c);
    }

    /** {@inheritDoc} */
    public void addRenderingHints(Map a) {
        graphics.addRenderingHints(a);
    }

    /** {@inheritDoc} */
    public void setRenderingHints(Map a) {
        graphics.setRenderingHints(a);
    }

    /** {@inheritDoc} */
    public boolean hit(Rectangle a, Shape b, boolean c) {
        return graphics.hit(a, b, c);
    }

    /** {@inheritDoc} */
    public void drawRenderedImage(RenderedImage a, AffineTransform b) {
        graphics.drawRenderedImage(a, b);
    }

    /** {@inheritDoc} */
    public void drawRenderableImage(RenderableImage a, AffineTransform b) {
        graphics.drawRenderableImage(a, b);
    }

    /** {@inheritDoc} */
    public void drawImage(BufferedImage a, BufferedImageOp b, int c, int d) {
        graphics.drawImage(a, b, c, d);
    }

    /** {@inheritDoc} */
    public Object getRenderingHint(RenderingHints.Key a) {
        return graphics.getRenderingHint(a);
    }

    /** {@inheritDoc} */
    public void setRenderingHint(RenderingHints.Key a, Object b) {
        graphics.setRenderingHint(a, b);
    }

    /** {@inheritDoc} */
    public boolean drawImage(Image a, AffineTransform b, ImageObserver c) {
        return graphics.drawImage(a, b, c);
    }
}
