/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */

/*
On linux, using jpackage-utils:
Compile like:
    javac -classpath $CLASSPATH:$(build-classpath fop avalon-framework) \
        UserGuide2PDF.java
Execute:
    java -cp $CLASSPATH:$(build-classpath fop avalon-framework batik) \
        UserGuide2PDF
 */

//Java
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

//JAXP
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

// Avalon
import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.avalon.framework.logger.Logger;

//FOP
import org.apache.fop.apps.Driver;
import org.apache.fop.apps.FOPException;
import org.apache.fop.messaging.MessageHandler;


/**
 * This class demonstrates the conversion of an XML file to PDF using
 * JAXP (XSLT) and FOP (XSL-FO).
 */
public class UserGuide2PDF {
    /**
     * Main method.
     * @param args command-line arguments
     */
    public static void main(String[] args) {
        try {
            File baseDir = new File(".");
            File outDir = new File(baseDir, "usrGuide");
            outDir.mkdirs();

            org.apache.fop.configuration.Configuration.put("baseDir",
                outDir.getName());

            File xmlfile = new File(baseDir, "xdocs/pdf-navigation.xml");
            File xsltfile = new File(baseDir, "xdocs/style/project2fo.xslt");
            File fofile = new File(baseDir, "usrGuide.fo");
            File pdffile = new File(outDir, "usrGuide.pdf");

            UserGuide2PDF app = new UserGuide2PDF();
            app.copyDirectory(new File("xdocs/images"),
                new File(outDir, "images"));
            app.copyDirectory(new File("xdocs/style"),
                new File(outDir, "style"));
            app.convertXML2FO(xmlfile, xsltfile, fofile);
            app.convertFO2PDF(fofile, pdffile);
            try {
                fofile.delete();
            } catch (SecurityException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    private void copyDirectory(File srcDir, File dstDir)
        throws IOException {
        if (srcDir.isDirectory()) {
            if (!dstDir.exists()) {
                dstDir.mkdir();
            }

            String[] children = srcDir.list();
            for (int i = 0; i < children.length; i++) {
                copyDirectory(new File(srcDir, children[i]),
                    new File(dstDir, children[i]));
            }
        } else {
            copyFile(srcDir, dstDir);
        }
    }

    private void copyFile(File src, File dst) throws IOException {
        InputStream in = new FileInputStream(src);
        OutputStream out = new FileOutputStream(dst);
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) {
            out.write(buf, 0, len);
        }
        in.close();
        out.close();
    }

    /**
     * Converts an XML file to an XSL-FO file using JAXP (XSLT).
     * @param xml the XML file
     * @param xslt the stylesheet file
     * @param fo the target XSL-FO file
     * @throws IOException In case of an I/O problem
     * @throws TransformerException In case of a XSL transformation problem
     */
    private void convertXML2FO(File xml, File xslt, File fo)
        throws IOException, TransformerException {
        OutputStream out = new FileOutputStream(fo);
        try {
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer =
                factory.newTransformer(new StreamSource(xslt));

            transformer.setParameter("basePath", "../");
            transformer.setParameter("paperType", "A4");
            transformer.setParameter("companyIncName",
                "Daniele Binosi & Lukas Theussl");
            transformer.setParameter("confidential", "false");
            transformer.setParameter("publicationYear", "2003-2006");
            transformer.setParameter("imageDpi", "150");
            transformer.setParameter("companyLogo", "images/top.gif");
            //transformer.setParameter("projectLogo", "images/ZHfermions.gif");
            transformer.setParameter("coverProjectCompany",
                "Daniele Binosi & Lukas Theussl");
            transformer.setParameter("coverProjectName", "JaxoDraw");
            transformer.setParameter("coverType", "User Guide");
            transformer.setParameter("coverVersion", "1.3-2");
            transformer.setParameter("coverDate", "2006");

            Source src = new StreamSource(xml);
            Result res = new StreamResult(out);

            transformer.transform(src, res);
        } finally {
            out.close();
        }
    }

    /**
     * Converts an FO file to a PDF file using FOP
     * @param fo the FO file
     * @param pdf the target PDF file
     * @throws IOException In case of an I/O problem
     * @throws FOPException In case of a FOP problem
     */
    private void convertFO2PDF(File fo, File pdf)
        throws IOException, FOPException {
        OutputStream out = null;

        try {
            // Construct driver and setup output format
            Driver driver = new Driver();
            Logger logger = new ConsoleLogger(ConsoleLogger.LEVEL_DISABLED);
            MessageHandler.setScreenLogger(logger);
            driver.setLogger(logger);
            driver.setRenderer(Driver.RENDER_PDF);

            // Setup output stream.  Note: Using BufferedOutputStream
            // for performance reasons (helpful with FileOutputStreams).
            out = new FileOutputStream(pdf);
            out = new BufferedOutputStream(out);
            driver.setOutputStream(out);

            // Setup JAXP using identity transformer
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer(); // identity transformer

            // Setup input stream
            Source src = new StreamSource(fo);

            // Resulting SAX events (the generated FO) must be piped through to FOP
            Result res = new SAXResult(driver.getContentHandler());

            // Start XSLT transformation and FOP processing
            transformer.transform(src, res);

        } catch (Exception e) {
            e.printStackTrace(System.err);
            System.exit(-1);
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }
}
