/**
 *  Licensed under GPL. For more information, see
 *    http://jaxodraw.sourceforge.net/license.html
 *  or the LICENSE file in the jaxodraw distribution.
 */
package net.sf.jaxodraw.object.line;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;

import net.sf.jaxodraw.JaxoTestUtils;
import net.sf.jaxodraw.io.imports.JaxoAxodraw;
import net.sf.jaxodraw.object.JaxoObject;
import net.sf.jaxodraw.object.JaxoObjectFactory;
import net.sf.jaxodraw.object.loop.JaxoFLoop;
import net.sf.jaxodraw.util.JaxoColor;
import net.sf.jaxodraw.util.JaxoConstants;
import net.sf.jaxodraw.util.JaxoPrefs;


/** Tests JaxoGLine. */
public class JaxoGLineTest extends AbstractJaxoLineObjectTest {
    private JaxoGLine line = new JaxoGLine();

    /** Constructor: just calls super().
     * @param testName The name of this test.
     */
    public JaxoGLineTest(String testName) {
        super(testName);
    }

    /** JUnit setup: initialize the object. */
    protected void setUp() {
        super.setUp();
        init();
    }

    private void init() {
        line.setLocation(100, 100);
        line.setRelWAndH(100, 100);
        line.setStrokeWidth(1.f);
        line.setDash(2.f);
        line.setColor(JaxoColor.GREEN);
        line.setPaintArrow(false);
        line.setArrowPosition(0.5f);
        line.setFlip(false);
        line.setDoubleLine(false);
        line.setDLSeparation(2.f);
    }

    /** {@inheritDoc} */
    public JaxoObject getInstance() {
        return line;
    }

    /** Tests serialization. */
    public void testSerialization() {
        line.setPaintArrow(true);
        line.setFlip(true);
        line.setDash(2.5f);
        line.setDoubleLine(true);
        line.setDLSeparation(4.f);
        JaxoTestUtils.checkSerializedEquality(line);
    }

    /** Tests that a copy of an object is still the same object. */
    public void testCopy() {
        JaxoGLine line2 = new JaxoGLine();
        line2.setState(line);

        line.setPaintArrow(true);
        line.setFlip(true);
        line.setDash(3.f);
        line.setDoubleLine(true);
        line.setDLSeparation(4.f);
        JaxoTestUtils.checkCopyEquality(line);

        assertFalse(line2.isCopy(line));

        line2.setPaintArrow(true);
        line2.setFlip(true);
        line2.setDash(3.f);
        line2.setDoubleLine(true);
        line2.setDLSeparation(4.f);
        assertTrue(line2.isCopy(line));

        JaxoFLoop loop = new JaxoFLoop();
        assertFalse(line.isCopy(loop));

    }

    /** Tests identity. */
    public void testIdentity() {
        JaxoGLine line2 = line;
        assertEquals(line, line2);
    }

    /** Tests the latex command. */
    public void testLatexCommand() {
        Dimension dim = new Dimension(100, 100);

        String expected = "\\Line[dash,dashsize=2](100,0)(200,-100)";
        String actual = line.latexCommand(1.f, dim);
        assertEquals("Latex commands do not match!", expected, actual);

        line.setPaintArrow(true);
        expected = "\\Line[dash,dashsize=2,arrow,arrowpos=0.5,arrowlength=5,arrowwidth=2,arrowinset=0.2](100,0)(200,-100)";
        actual = line.latexCommand(1.f, dim);
        assertEquals("Latex commands do not match!", expected, actual);

        line.setFlip(true);
        expected = "\\Line[dash,dashsize=2,arrow,arrowpos=0.5,arrowlength=5,arrowwidth=2,arrowinset=0.2,flip](100,0)(200,-100)";
        actual = line.latexCommand(1.f, dim);
        assertEquals("Latex commands do not match!", expected, actual);

        // reset to initial state
        line.setPaintArrow(false);
        line.setFlip(false);

        line.setDoubleLine(true);
        line.setDLSeparation(4.f);

        expected = "\\Line[dash,dashsize=2,double,sep=4](100,0)(200,-100)";
        actual = line.latexCommand(1.f, dim);
        assertEquals("Latex commands do not match!", expected, actual);

        line.setFlip(true);
        expected = "\\Line[dash,dashsize=2,double,sep=4](100,0)(200,-100)";
        actual = line.latexCommand(1.f, dim);
        assertEquals("Latex commands do not match!", expected, actual);

        line.setRelWAndH(0, 0);
        expected = "%";
        actual = line.latexCommand(1.f, dim);
        assertEquals("Latex commands do not match!", expected, actual);

        // reset to initial state
        line.setPaintArrow(false);
        line.setFlip(false);
        line.setDoubleLine(false);
        line.setRelWAndH(-100, -100);
        expected = "\\Line[dash,dashsize=2](100,0)(0,100)";
        actual = line.latexCommand(1.f, dim);
        assertEquals("Latex commands do not match!", expected, actual);

    }

    /** Tests axodraw. */
    public void testAxodraw() {
        JaxoAxodraw axo = new JaxoAxodraw();
        JaxoObject ob = axo.getJaxoObject("\\SetWidth{1.0}");
        assertNull(ob);
        String command;

        command = "\\DashLine(100,200)(200,300){2}";
        line = (JaxoGLine) axo.getJaxoObject(command);
        checkLineParameters(line, false);

        command = "\\DashArrowLine(100,200)(200,300){2}";
        line = (JaxoGLine) axo.getJaxoObject(command);
        checkLineParameters(line, false);

        command =
            "\\DashLine(102.12,-191.88)(202.12,-291.88){2}\\DashLine(97.88,-196.12)(197.88,-296.12){2}%%JaxoDrawID:DoubleLine(3)";
        line = (JaxoGLine) axo.getJaxoObject(command);
        checkLineParameters(line, true);

        command =
            "\\DashArrowLine(102.12,-191.88)(202.12,-291.88){2}\\DashArrowLine(97.88,-196.12)(197.88,-296.12){2}%%JaxoDrawID:DoubleLine(3)";
        line = (JaxoGLine) axo.getJaxoObject(command);
        checkLineParameters(line, true);

    }

    private void checkLineParameters(JaxoGLine aline, boolean doubleLine) {
        assertEquals("GLine doesn't have the right color!", JaxoColor.BLACK,
            aline.getColor());
        assertTrue("GLine doesn't have the right stroke!", aline.strokeIs(1.f));
        assertTrue("GLine doesn't have the right dash!", aline.dashIs(2.f));
        assertEquals("Mismatch in GLine doubleLine!", doubleLine,
            aline.isDoubleLine());

        float dlsep = 2.f;
        int x = 100;
        int y = -200;
        if (doubleLine) {
            dlsep = 3.f;
            x = 100;
            y = 194;
        }

        assertEquals("GLine doesn't have the right x!", x, aline.getX());
        assertEquals("GLine doesn't have the right y!", y, aline.getY());
        assertTrue("Mismatch in GLine doubleLine separation!",
            aline.dlSepIs(dlsep));

    }

    /** Tests the getBounds() method. */
    public void testGetBounds() {
        Rectangle bBox = line.getBounds();
        Rectangle expected = new Rectangle(99, 99, 102, 102);
        assertEquals("Bounding box not correct!", expected, bBox);
        line.setPaintArrow(true);
        line.setLocation(10, 10, 20, 10);
        bBox = line.getBounds();
        expected.setRect(9, 7, 10, 6);
        assertEquals("Arc Bounding box not correct!", expected, bBox);
        line.setDoubleLine(true);
        bBox = line.getBounds();
        expected.setRect(9, 7, 10, 6);
        assertEquals("DL Bounding box not correct!", expected, bBox);

        assertNotNull(line.getBounds2D());
    }

    /** Tests createNewObject in JaxoObjectFactory. */
    public void testCreateNewObject() {
        Point p = new Point(50, 50);
        JaxoObject ob =
            JaxoObjectFactory.newObject(JaxoConstants.GLINE, new Point[] {p, p});

        assertTrue("createNewObject did not return a GLine!",
            (ob instanceof JaxoGLine));

        line = (JaxoGLine) ob;
        assertEquals("GLine doesn't have the right x!", 50, line.getX());
        assertEquals("GLine doesn't have the right y!", 50, line.getY());
        assertEquals("GLine doesn't have the right color!",
            JaxoColor.getColor(JaxoPrefs.getStringPref(JaxoPrefs.PREF_COLOR)),
            line.getColor());
        assertTrue("GLine doesn't have the right stroke!",
            line.strokeIs(JaxoPrefs.getFloatPref(JaxoPrefs.PREF_LINEWIDTH)));
        assertEquals("GLine doesn't have the right arrow!",
            JaxoPrefs.getBooleanPref(JaxoPrefs.PREF_ARROW), line.isPaintArrow());
        assertEquals("GLine doesn't have the right flip!", false,
            line.isFlip());
        assertEquals("GLine doesn't have the right double line!", false,
            line.isDoubleLine());
        assertTrue("GLine doesn't have the right dlsep!",
            line.dlSepIs(JaxoPrefs.getFloatPref(JaxoPrefs.PREF_DLSEP)));
        assertTrue("GLine doesn't have the right dash!", line.dashIs(2.f));

    }

    /** {@inheritDoc} */
    public String className() {
        return "JaxoGLine";
    }

    /** {@inheritDoc} */
    public void testJaxo20Compat() {
        JaxoGLine object = (JaxoGLine) instanceFrom20OpenGraph(JaxoGLine.class);
        assertNotNull(object);
        check20Object("opened wrong object!", object);

        object = (JaxoGLine) instanceFrom20ImportGraph(JaxoGLine.class);
        assertNotNull(object);
        check20Object("imported wrong object!", object);
    }

    private void check20Object(String message, JaxoGLine object) {
        assertEquals(message, 39, object.getX());
        assertEquals(message, 197, object.getY());
        assertEquals(message, 116, object.getX2());
        assertEquals(message, 252, object.getY2());

        assertTrue(message, object.isDoubleLine());
        assertEquals(message, 3.f, object.getDLSeparation());
        assertEquals(message, 1.997f, object.getDash());
        assertEquals(message, 0.8f, object.getStrokeWidth());

        assertEquals(message, JaxoColor.CERULEAN, object.getColor());

        assertFalse(message, object.isPaintArrow());
    }
}
